#include <stdio.h>
#include <math.h>
#include "ADGraphLayout1.h"

#ifndef NULL
#define NULL 0
#endif


struct ADNode {
  double x; int y;    /* location */
  int dx, dy;         /* size */
  int dir;            /* lineadjustment (-2 bottom -1 down 0 no 1 up 2 top) */
  int intermediate; 
  int theObject;
  int line, lineindex, nodeindex;
  ADNode * left , * right;
  int leftslant, rightslant;
  int flag, first;
  double leftx, rightx; int ori;
};

struct ADLongarc{ ADNode *first;
                  int yfirst, ylast;
                  double goal;
};

static ADNode* makenode(int xx, int yy, int dxx, int dyy, int ddir)
{ ADNode* n;
  n=new ADNode;
  n->x=double(xx); n->y=yy; n->dx=dxx; n->dy=dyy;
  n->dir=ddir; n->intermediate=0;
  n->line=n->lineindex=n->nodeindex= -1;
  n->left=n->right=NULL;
  return n;
}

static ADNode* makeintermediate(int l, ADNode* lft)
{ ADNode* n;
  n=new ADNode;
  n->x=0.0; n->y=0; n->dx=2; n->dy=0; n->dir=0;
  n->intermediate=1;
  n->line=l;
  n->lineindex=n->nodeindex= -1;
  n->left=lft; n->right=NULL;
  return n;
}

ADGraphLayout::ADGraphLayout(void)
{ nodenumber = 0;
  intermediatenumber = 0;
  allnodes = 0;
  arcnumber = 0;
  linenumber = 0;
  nodes = NULL;
  intermediates = NULL;
  lines = NULL;
  linelengths = NULL;
  leftarcs = NULL;
  rightarcs = NULL;
  shortarcs = NULL;
  leftarcnumber = 0;
  rightarcnumber = 0;
  shortarcnumber = 0;
  graphisok = 0;
  step=-1;
  alpha = 1.0e-2;
  beta = 1.0e-1;
  epsilon = 1.0e-2;
  M=Diag=NULL;
  Delta=X=D=Grad=H = NULL;
  Msize = 0;
  linelinedist = 20;
  nodenodedist = 10;
  nodearcdist = 5;
  arcarcdist = 4;
  arccrossingoffset = 2;
  longarcboundary = 85;
  MAXSTEPS = 1000;
}

ADGraphLayout::~ADGraphLayout(void)
{ ADReset();
}

void ADGraphLayout::setflag(int n)
{ int i;
  for (i=0; i<nodenumber; i++)
    nodes[i]->flag=n;
  for (i=0; i<intermediatenumber; i++)
    intermediates[i]->flag=n;
}

void ADGraphLayout::printnodes(void)
{ int i;
  for (i=0; i<nodenumber; i++)
    if (nodes[i]) 
       printf("%3d: (%e,%d)[%d,%d]\n",i,nodes[i]->x,nodes[i]->y,nodes[i]->dx,nodes[i]->dy);
    else
       printf("%3d: nil\n",i);
}

void ADGraphLayout::printmatrix(char * n, int * ptr, int z, int s)
{ int i, j;
  printf("%s[%d,%d]=\n",n,z,s);
  for (i=0; i<z; i++)
  { for (j=0; j<s; j++)
      printf("%6d ",*ptr++);
    printf("\n");
  }
}

void ADGraphLayout::printvector(char * n, double * ptr, int z)
{ int i, j;
  printf("%s[%d]=\n",n,z);
  for (i=0; i<z; i++)
    printf("%6.0f ",*ptr++);
  printf("\n");
}

/*  in der hoehe sortieren und zeilen erzeugen */

void ADGraphLayout::heightsort(void)
{ int i, h;
  setflag(0);  /* flag: cycle control */
  for (i=0; i<nodenumber; i++)
    nodes[i]->line= -1;
  for (i=0; i<nodenumber; i++)
    if (nodes[i]->dir==2)
      if ((h=setheight(nodes[i]))>linenumber) linenumber=h;
  for (i=0; i<nodenumber; i++)
    if (nodes[i]->line==-1)
      if ((h=setheight(nodes[i]))>linenumber) linenumber=h;
  setflag(0);  /* flag: move control */
  for (i=0; i<nodenumber; i++)
  { if (nodes[i]->dir==2) nodes[i]->line=linenumber;
  }
  for (i=0; i<nodenumber; i++) moveupward(nodes[i]);
  linenumber++;
}

void ADGraphLayout::intermediatenodes(void)
{ int i, j, inter; ADNode * n;
  intermediatenumber=0; 
  for (i=0; i<nodenumber; i++)
  { if ((nodes[i]->left) && (nodes[i]->line - nodes[i]->left->line>1))
    { intermediatenumber+=nodes[i]->line - nodes[i]->left->line-1;
    }
    if ((nodes[i]->right) && (nodes[i]->line - nodes[i]->right->line>1))
    { intermediatenumber+=nodes[i]->line - nodes[i]->right->line-1;
    }
  }
  intermediates=new ADNode* [intermediatenumber];
  allnodes+=intermediatenumber;
  inter=0;
  for (i=0; i<nodenumber; i++)
  { if ((nodes[i]->left) && (nodes[i]->line - nodes[i]->left->line>1))
    { n=nodes[i]->left;
      for (j=nodes[i]->left->line+1; j<nodes[i]->line-1; j++)
        n = intermediates[inter++] = makeintermediate(j,n);
      nodes[i]->left
      = intermediates[inter++]
      = makeintermediate(j,n);
    }
    if ((nodes[i]->right) && (nodes[i]->line - nodes[i]->right->line>1))
    { n=nodes[i]->right;
      for (j=nodes[i]->right->line+1; j<nodes[i]->line-1; j++)
      n = intermediates[inter++] = makeintermediate(j,n);
      nodes[i]->right
      = intermediates[inter++]
      = makeintermediate(j,n);
    }
  }
}

void ADGraphLayout::makelines(void)
{ int i, l;
  lines=new ADNode** [linenumber];
  linelengths=new int [linenumber];
  for (i=0; i<linenumber; i++) linelengths[i]=0;
  for (i=0; i<nodenumber; i++)
    linelengths[nodes[i]->line]++;
  for (i=0; i<intermediatenumber; i++)
    linelengths[intermediates[i]->line]++;
  for (i=0; i<linenumber; i++)
  { lines[i]=new ADNode* [linelengths[i]];
    linelengths[i]=0;
  }
  for (i=0; i<nodenumber; i++)
  { l=nodes[i]->line;
    lines[l][linelengths[l]++]=nodes[i];
  }
  for (i=0; i<intermediatenumber; i++)
  { l=intermediates[i]->line;
    lines[l][linelengths[l]++]=intermediates[i];
  }
} 

void ADGraphLayout::moveupward(ADNode * node)
{ if (node->left && (node->left->dir==1))
  { if ((node->left->flag==0) || (node->left->line>=node->line))
    { node->left->line=node->line-1;
      node->left->flag=1;
      moveupward(node->left);
    }
  }
  if (node->right && (node->right->dir==1))
  { if ((node->right->flag==0) || (node->right->line>=node->line))
    { node->right->line=node->line-1;
      node->right->flag=1;
      moveupward(node->right);
    }
  }
}

int ADGraphLayout::setheight(ADNode * node)
{ int h, height;
  if (node->flag) return -1;
  if (node->line>=0) return node->line;
  node->flag=1; 
  height=0;
  if (node->left)
  { if ((h=setheight(node->left))>=height) height=h+1;
    else if (h<0) node->left=NULL;
  }
  if (node->right)
  { if ((h=setheight(node->right))>=height) height=h+1;
    else if (h<0) node->right=NULL;
  }
  node->line=height;
  node->flag=0;
  return node->line;
}

/*  sortieren der zeilen */

void ADGraphLayout::arrange(void)
{ int i;
  for (i=0; i<linelengths[linenumber-1]; i++)
    lines[linenumber-1][i]->flag=(lines[linenumber-1][i]->dir==2);
  freenodesinline(linenumber-1);
  sortline(linenumber-1);
  for (i=linenumber-1; i>0; i--)
    arrangeline(i);
  setlineindices();
}

void ADGraphLayout::arrangeline(int l)
{ int i;
  if ((l>linenumber-1) || (l<1)) return;
  prepareline(l-1);
  analyselineandsetbelow(l);
  if (evaluateline(l-1))
    freenodesinline(l-1);
  sortline(l-1);
}

void ADGraphLayout::sortline(int l)
{ int i, j; double min; int idx; ADNode * n;
  for (i=0; i<linelengths[l]-1; i++)
  { idx=i; min=lines[l][i]->x;
    for (j=i+1; j<linelengths[l]; j++)
      if (min>lines[l][j]->x)
      { idx=j; min=lines[l][j]->x;
      }
    if (i<idx)
    { n=lines[l][i]; lines[l][i]=lines[l][idx]; lines[l][idx]=n;
    }
    lines[l][i]->x=double(i<<2);
  }
  lines[l][i]->x=double(i<<2);
}

void ADGraphLayout::prepareline(int l)
{ int i; ADNode * n;
  for (i=0; i<linelengths[l]; i++)
  { n=lines[l][i];
    n->flag=0; n->x=0.0;
  }
}

void ADGraphLayout::analyselineandsetbelow(int l)
{ int i; ADNode * n;
  for (i=0; i<linelengths[l]; i++)
  { n=lines[l][i];
    if (n->right)
    { if (n->left)
      { n->right->x+=n->x+1; n->right->flag++;
        n->left->x+=n->x-1; n->left->flag++;
      }
      else
      { n->right->x+=n->x; n->right->flag++;
      }
    }
    else
    { if (n->left)
      { n->left->x+=n->x; n->left->flag++;
      }
    }
  }
}

int ADGraphLayout::evaluateline(int l)
{ int i, h; ADNode * n;
  h=0;
  for (i=0; i<linelengths[l]; i++)
  { n=lines[l][i];
    if (n->flag==0) h++;
    if (n->flag<=1) continue;
    n->x/=(double)(n->flag);
  }
  return h;
}

void ADGraphLayout::freenodesinline(int l)
{ int i, j; double min, v; int d; ADNode * n;
  for (i=0; i<linelengths[l]; i++)
    lines[l][i]->lineindex=0;
  for (i=0; i<linelengths[l]; i++)
  { if (lines[l][i]->flag==0)
    { lines[l][i]->lineindex= -1; continue; }
    min=lines[l][i]->x;
    for (j=0; j<i; j++)
      if ((lines[l][j]->flag) && (min<lines[l][j]->x))
        lines[l][j]->lineindex+=nodenumber;
    for (j=i+1; j<linelengths[l]; j++)
      if ((lines[l][j]->flag) && (min<=lines[l][j]->x))
        lines[l][j]->lineindex+=nodenumber<<1;
  }
  setflag(0);
  for (i=0; i<linelengths[l]; i++)
    if (lines[l][i]->lineindex>=0)
      setincludednodes(lines[l][i],(double)lines[l][j]->lineindex);
  for (i=0; i<linelengths[l]; i++)
    if (lines[l][i]->lineindex<0)
      getbestxvalue(lines[l][i],&v,&d);
}

void ADGraphLayout::setincludednodes(ADNode * n, double v)
{ n->flag++; n->x+=v;
  if (n->left)
  { if (n->right)
    { setincludednodes(n->left,v-1.0);
      setincludednodes(n->right,v+1.0);
    }
    else
    { setincludednodes(n->left,v);
    }
  } else
  { if (n->right)
    { setincludednodes(n->right,v+1.0);
    }
  }
}

void ADGraphLayout::getbestxvalue(ADNode * n, double * x, int * depth)
{ int d1, d2; double v1, v2;
  if (n->flag>0) { *x=n->x/n->flag; *depth=0; return; }
  if (n->flag== -1) { *x=n->x; *depth=0; return; }
  if (n->flag== -2) { *x=n->x; *depth= -1; return; }
  if (n->left)
  { if (n->right)
    { getbestxvalue(n->left,&v1,&d1);
      getbestxvalue(n->right,&v2,&d2);
      if (d1<0)
      { if (d2<0) { n->flag= -2; n->x= -1.0; *depth= -1; return; }
        n->flag= -1; *x=n->x=v2-1.0; *depth=d2+1; return;
      } else
      { if (d2<0)  { n->flag= -1; *x=n->x=v1+1.0; *depth=d1+1; return; }
        if (d1<d2) { n->flag= -1; *x=n->x=v1+1.0; *depth=d1+1; return; }
        if (d1>d2) { n->flag= -1; *x=n->x=v2-1.0; *depth=d2+1; return; }
        n->flag= -1;*x=n->x=(v1+v2)/2.0; *depth=d2+1;
        return;
      }
    }
    else
    { getbestxvalue(n->left,&v1,&d1);
      if (d1<0)
      { n->flag= -2; n->x= -1.0; *depth= -1; return;
      } else
      { n->flag= -1; *x=n->x=v1+1.0; *depth=d1+1; return;
      }
    }
  } else
  { if (n->right)
    { getbestxvalue(n->right,&v2,&d2);
      if (d2<0)
      { n->flag= -2; n->x= -1.0; *depth= -1; return;
      } else
      { n->flag= -1; *x=n->x=v2-1.0; *depth=d2+1; return;
      }
    }
  }
}

void ADGraphLayout::setlineindices(void)
{ int l, i, idx;
  for (l=0; l<linenumber; l++)
    for (i=idx=0; i<linelengths[l]; i++)
    {  lines[l][i]->lineindex=i;
       if (lines[l][i]->intermediate==0) lines[l][i]->nodeindex=idx++;
    }
}

/* behandeln langer kanten */

void ADGraphLayout::longhandling(void)
{ int i, l, la, ra, sa;
  setflag(0);
  for (l=2; l<linenumber; l++)
  { for (i=0; i<linelengths[l]; i++)
    { if (lines[l][i]->intermediate) continue;
      if ((lines[l][i]->left)&&(lines[l][i]->left->intermediate))
      { switch (lines[l][i]->left->flag=longarctest(lines[l][i],1))
        { case  1: rightarcnumber++; break;
          case  0: shortarcnumber++; break;
          case -1: leftarcnumber++; break;
        }
      }
      if ((lines[l][i]->right)&&(lines[l][i]->right->intermediate))
      { switch (lines[l][i]->right->flag=longarctest(lines[l][i],0))
        { case  1: rightarcnumber++; break;
          case  0: shortarcnumber++; break;
          case -1: leftarcnumber++; break;
        }
      }
    }
  }
  leftarcs=new ADLongarc* [leftarcnumber];
  shortarcs=new ADLongarc* [shortarcnumber];
  rightarcs=new ADLongarc* [rightarcnumber];
  la=0; ra=0; sa=0;
  for (l=2; l<linenumber; l++)
  { for (i=0; i<linelengths[l]; i++)
    { if (lines[l][i]->intermediate) continue;
      if ((lines[l][i]->left)&&(lines[l][i]->left->intermediate))
        switch (lines[l][i]->left->flag)
        { case  1: rightarcs[ra++]=longarcregister(lines[l][i],1); break;
          case  0: shortarcs[sa++]=longarcregister(lines[l][i],1); break;
          case -1: leftarcs[la++]=longarcregister(lines[l][i],1); break;
        }
      if ((lines[l][i]->right)&&(lines[l][i]->right->intermediate))
        switch (lines[l][i]->right->flag)
        { case  1: rightarcs[ra++]=longarcregister(lines[l][i],0); break;
          case  0: shortarcs[sa++]=longarcregister(lines[l][i],0); break;
          case -1: leftarcs[la++]=longarcregister(lines[l][i],0); break;
        }
    }
  }
  longarcsort(leftarcs,la,1);
  for (i=0; i<la; i++)
    longarcmoveleft(leftarcs[i]);
  longarcsort(rightarcs,ra,1);
  for (i=0; i<ra; i++)
    longarcmoveright(rightarcs[i]);
  longarcsort(shortarcs,sa,0);
}

int ADGraphLayout::longarctest(ADNode * node, int isleft)
{ int len, i, l, cnt1, cnt2;
  ADNode * n;
  if (isleft) n=node->left;
  else n=node->right;
  len=0;
  while(n->intermediate)
  { len++;
    n=n->left;
  }
  if (len<longarcboundary) return 0;
  l=node->line; cnt1=-1;
  for (i=0; i<linelengths[l]; i++)
    if (lines[l][i]->intermediate==0) cnt1++;
  l=n->line; cnt2=-1;
  for (i=0; i<linelengths[l]; i++)
    if (lines[l][i]->intermediate==0) cnt2++;
  l=((node->nodeindex+n->nodeindex)<<1)-cnt1-cnt2;
  if (l<0) return -1;
  if (l==0) return isleft?-1:1;
  return 1;
}

ADLongarc * ADGraphLayout::longarcregister(ADNode * node, int isleft)
{ ADNode * n; ADLongarc * la;
  int l;
  if (isleft) n=node->left;
  else n=node->right;
  la=new ADLongarc;
  la->first=n; l=la->yfirst=n->line;
  while(n->intermediate)
  { l++;
    n=n->left;
  }
  la->ylast=l-1;
  return la;
}

void ADGraphLayout::longarcsort(ADLongarc ** la,int ln, int all)
{ int i, j, h, min, imin;
  ADLongarc * hptr;
  for (i=0; i<ln-1; i++)
  { min=la[i]->yfirst-la[i]->ylast; imin=i;
    for (j=i+1; j<ln; j++)
      if ((h=la[j]->yfirst-la[j]->ylast)<min)
      { min=h; imin=j;
      }
    if (i!=imin)
    { hptr=la[i]; la[i]=la[imin]; la[imin]=hptr;
    }
  }
  if (all==0) return;
  for (i=0; i<ln-1; i++)
  { min=la[i]->yfirst; imin=i;
    for (j=i+1; j<ln; j++)
      if (min<la[j]->ylast)
      { hptr=la[j]; i++;
        for (h=j; h>i; h--) la[h]=la[h-1];
        la[i]=hptr;
        min=la[i]->yfirst;
      }
  }
}

void ADGraphLayout::longarcmoveleft(ADLongarc * la)
{ int l, i;
  ADNode * n;
  n=la->first;
  while (n->intermediate)
  { l=n->line;
    for (i=n->lineindex; i>0; i--)
    { lines[l][i]=lines[l][i-1];
      lines[l][i]->lineindex++;
    }
    lines[l][0]=n;
    n->lineindex=0;
    n=n->left;
  }
}

void ADGraphLayout::longarcmoveright(ADLongarc * la)
{ int l, i;
  ADNode * n;
  n=la->first;
  while (n->intermediate)
  { l=n->line;
    for (i=n->lineindex; i<linelengths[l]-1; i++)
    { lines[l][i]=lines[l][i+1];
      lines[l][i]->lineindex--;
    }
    lines[l][i]=n;
    n->lineindex=linelengths[l]-1;
    n=n->left;
  }
}


/* ausrichten der knoten */

void ADGraphLayout::adjust(void)
{ int i, h;
  makeM();
  makeDelta();
  makeX0();
  for (i=0; ((h=makenewX())>0) && (i<MAXSTEPS); i++);
  makeStore();
  makeReset();
}


void ADGraphLayout::adjustbystep1(void)
{ int h;
  makeM();
  makeDelta();
  makeX0();
  if ((h=makenewX())<=0)
  { step=-1;
  }
  else printf("step(%d)\n",step);
  makeStore();
}

void ADGraphLayout::adjustbystep2(void)
{ int h;
  if ((h=makenewX())<=0)
  { step=-1;
  }
  else printf("step(%d)\n",step);
  makeStore();
}

void ADGraphLayout::makeM(void)
{ int i, j, l, idx, beg;
  int * ptr; ADNode * n, * n1;
  arcnumber=0;
  for (i=0; i<nodenumber; i++)
    arcnumber+=(nodes[i]->left!=NULL)+(nodes[i]->right!=NULL);
  arcnumber+=intermediatenumber;
  Msize=arcnumber*allnodes;
  ptr=M=new int [Msize];
  for (i=0; i<Msize; i++) *ptr++=0;
  idx=beg=0;
  for (l=linenumber-1;l>=0; l--)
  { for (i=0; i<linelengths[l]; i++)
    { lines[l][i]->flag=idx++;
      lines[l][i]->first=beg;
    }
    beg+=linelengths[l];
  }
  ptr=M;
  for (l=linenumber-1;l>=0; l--)
    for (i=0; i<linelengths[l]; i++)
    { n=lines[l][i];
      if (n->left)
      { for (j=n->first; j<=n->flag; j++)
          *(ptr+j)= -1;
        n1=n->left;
        for (j=n1->first; j<=n1->flag; j++)
          *(ptr+j)= 1;
        ptr+=allnodes;
      }
      if (n->right)
      { for (j=n->first; j<=n->flag; j++)
          *(ptr+j)= -1;
        n1=n->right;
        for (j=n1->first; j<=n1->flag; j++)
          *(ptr+j)= 1;
        ptr+=allnodes;
      }
    }
  ptr=Diag=new int[allnodes];
  for (i=0; i<allnodes; i++) *ptr++=0;
  ptr=M;
  for (i=0; i<arcnumber; i++)
    for (j=0; j<allnodes; j++)
       if (*ptr++) Diag[j]++;
}

void ADGraphLayout::makeDelta(void)
{ int i, l, dd1, dd2, in1, in2, idx;
  Delta=new double [allnodes];
  idx=0;
  for (l=linenumber-1;l>=0; l--)
  { dd1=(lines[l][0]->dx+1)>>1;
    in1=lines[l][0]->intermediate;
    Delta[idx++]=dd1;
    for (i=1; i<linelengths[l]; i++)
    { dd2=(lines[l][i]->dx+1)>>1;
      in2=lines[l][i]->intermediate;
      switch(in1+in2)
      { case 0: Delta[idx++]=dd1+dd2+nodenodedist; break;
        case 1: Delta[idx++]=dd1+dd2+nodearcdist; break;
        case 2: Delta[idx++]=dd1+dd2+arcarcdist; break;
        default: Delta[idx++]=dd1+dd2; break;
      }
      dd1=dd2; in1=in2;
    }
  }
}

void ADGraphLayout::makeX0(void)
{ int i, j;
  int * ptr;
  X=new double [allnodes];
  for (i=0; i<allnodes; i++)
    X[i]=0.0;
  D=new double [arcnumber];
  for (i=0; i<arcnumber; i++)
    D[i]=0.0;
  ptr=M;
  for (i=0; i<arcnumber; i++)
    for (j=0; j<allnodes; (j++,ptr++))
      switch(*ptr)
      { case -1:  D[i]-=Delta[j]; continue;
        case 1:  D[i]+=Delta[j]; continue;
      }
}

#define FASTNULL 1.0e-12

int ADGraphLayout::makenewX(void)
{ int i, j;
  int * ptr;
  double lambda, h1, max;
  max=0.0;
  for (i=0; i<allnodes; i++)
  { ptr=M+i;
    h1=0.0;
    for (j=0; j<arcnumber; (j++,ptr+=allnodes))
      switch(*ptr)
      { case -1:  h1+=D[j]; continue;
        case 1:  h1-=D[j]; continue;
      }
    lambda=h1/Diag[i];
    if (lambda<0.0)
    { if (X[i]<-lambda)
      { lambda=-X[i]; X[i]=0.0;
      }
      else X[i]+=lambda;
      if (max<-lambda) max=-lambda;
    }
    else 
    { X[i]+=lambda;
      if (max<lambda) max=lambda;
    }
    ptr=M+i;
    for (j=0; j<arcnumber; (j++,ptr+=allnodes))
      switch(*ptr)
      { case -1:  D[j]-=lambda; continue;
        case 1:  D[j]+=lambda; continue;
      }
  }
  return (max>beta);
}

void ADGraphLayout::makeStore(void)
{ int i, l, idx; double offset;
  idx=0;
  for (l=linenumber-1;l>=0; l--)
  { offset=0.0;
    for (i=0; i<linelengths[l]; i++)
    { offset=lines[l][i]->x=offset+Delta[idx]+X[idx];
      idx++;
    }
  }
}

void ADGraphLayout::makeReset(void)
{ if (M) delete M;
  if (X) delete X;
  if (D) delete D;
  if (Delta) delete Delta;
  if (Diag) delete Diag;
  M=Diag=NULL;
  X=D=Delta=Grad=H=NULL;
}

/* generelles layout */

void ADGraphLayout::updateY(void)
{ int i, h1, y;
  ysetline(0,0);
  h1=ygetheight(0);
  y=0;
  for (i=1; i<linenumber; i++)
  { y+=h1;
    y+=h1=ygetheight(i);
    y+=linelinedist+ygetcrossings(i)*arccrossingoffset;
    ysetline(i,y);
  }
}

void ADGraphLayout::ysetline(int l, int yy)
{ int i;
  for (i=0; i<linelengths[l]; i++)
    lines[l][i]->y=yy;
}

int ADGraphLayout::ygetheight(int l)
{ int i, h;
  h=0;
  for (i=0; i<linelengths[l]; i++)
    if (lines[l][i]->intermediate==0)
      if (h<lines[l][i]->dy) h=lines[l][i]->dy;
  for (i=0; i<linelengths[l]; i++)
    if (lines[l][i]->intermediate)
      lines[l][i]->dy=h;
  return (h+1)>>1;
}

int ADGraphLayout::ygetcrossings(int l)
{ int i, j, h, m; ADNode * n1, * n2;
  int x1, x2, x3, x4;
  h=0; m=linelengths[l-1];
  for (i=0; i<linelengths[l]-1; i++)
  { n1=lines[l][i];
    if (n1->left) x1=n1->left->lineindex; else x1=-1;
    if (n1->right) x2=n1->right->lineindex; else x2=-1;
    for (j=i+1; j<linelengths[l]; j++)
    { n2=lines[l][j];
      if (n2->left) x3=n2->left->lineindex; else x3=m;
      if (n2->right) x4=n2->right->lineindex; else x4=m;
      h+=(x1>x3)+(x1>x4)+(x2>x3)+(x2>x4);
    }
  }
  return h;
}

void ADGraphLayout::centerXY(void)
{ int i, j, l, ymin, ymax;
  double xmin, xmax, h;
  ymin= -ygetheight(0);
  ymax= lines[linenumber-1][0]->y+ygetheight(linenumber-1);
  xmin=lines[0][0]->x - ((lines[0][0]->dx+1)>>1);
  j=linelengths[0]-1;
  xmax=lines[0][j]->x + ((lines[0][j]->dx+1)>>1);
  for (l=1; l<linenumber; l++)
  { if (xmin > ( h=lines[l][0]->x - ((lines[l][0]->dx+1)>>1) )) xmin=h;
    j=linelengths[l]-1;
    if (xmax < ( h=lines[l][j]->x + ((lines[l][j]->dx+1)>>1) )) xmax=h;
  }
  ymin=(ymin+ymax+1)>>1;
  xmin=(xmin+xmax+1)/2.0;
  for (i=0; i<nodenumber; i++)
  { nodes[i]->x-=xmin; nodes[i]->y-=ymin;
  }
  for (i=0; i<intermediatenumber; i++)
  { intermediates[i]->x-=xmin; intermediates[i]->y-=ymin;
  }
}

void ADGraphLayout::designintervalls(void)
{ int i, l, c; double lx, max; ADNode * n;
  max=lines[0][linelengths[0]-1]->x;
  for (l=1; l<linenumber; l++)
    if (lines[l][linelengths[l]-1]->x > max)
      max=lines[l][linelengths[l]-1]->x;
  for (l=1; l<linenumber; l++)
  { lx=0.0-(intermediatenumber*(arcarcdist+2)+nodearcdist);
    for (i=0;i<linelengths[l]; i++)
    { n=lines[l][i];
      if (n->intermediate)
      { n->leftx=lx;
        lx+=arcarcdist+2;
      }
      else
        lx=n->x+double(((n->dx+3)>>1)+nodearcdist);
    }
    lx=max+(intermediatenumber*(arcarcdist+2)+nodearcdist);
    for (i=linelengths[l]-1;i>=0; i--)
    { n=lines[l][i];
      if (n->intermediate)
      { n->rightx=lx;
        lx-=arcarcdist+2;
      }
      else
        lx=n->x-double(((n->dx+3)>>1)+nodearcdist);
    }
  }
  setflag(-1);
  c=0;
  for (l=0; l<leftarcnumber; l++)
    leftarcs[l]->first->flag=c++;
  for (l=0; l<rightarcnumber; l++)
    rightarcs[l]->first->flag=c++;
  for (l=0; l<shortarcnumber; l++)
    shortarcs[l]->first->flag=c++;
  for (i=0; i<nodenumber; i++)
  { n=nodes[i];
    if ((n->left)&&(n->left->intermediate)
       &&(n->left->flag>=0))
    { c=n->left->flag;
      if (c<leftarcnumber)
        leftarcs[c]->goal=n->x-(n->right?10.0:0.0);
      else if ((c-=leftarcnumber)<rightarcnumber)
        rightarcs[c]->goal=n->x-(n->right?10.0:0.0);
      else
        shortarcs[c-rightarcnumber]->goal=n->x-(n->right?10.0:0.0);
    }
    if ((n->right)&&(n->right->intermediate)
       &&(n->right->flag>=0))
    { c=n->right->flag;
      if (c<leftarcnumber)
        leftarcs[c]->goal=n->x+10.0;
      else if ((c-=leftarcnumber)<rightarcnumber)
        rightarcs[c]->goal=n->x+10.0;
      else 
        shortarcs[c-rightarcnumber]->goal=n->x+10.0;
    }
  }
}

void ADGraphLayout::designtubeintervall(ADNode * n,double xlt,double xrt
                                   ,double * xlb,double * xrb
                                   ,double glt)
{ int intube;
  double xl, xr, gl;
  intube=0;
  if (n->intermediate==0)
  { *xlb=xlt;
    *xrb=xrt;
    n->flag=0;
    return;
  }
  if ((n->leftx<=xrt)&&(n->rightx>=xlt))
  { if (n->leftx<xlt) n->leftx=xlt;
    if (n->rightx>xrt) n->rightx=xrt;
    designtubeintervall(n->left,n->leftx,n->rightx,&xl,&xr,glt);
    n->flag=1;
    n->leftx=*xlb=xl;
    n->rightx=*xrb=xr;
    if (glt<=xl) designnode(n,xl);
    else if (glt>=xr) designnode(n,xr);
    else designnode(n,glt);
    return;
  }
  if (glt<=xl) glt=xl;
  else if (glt>=xr) glt=xr;
  designtubeintervall(n->left,n->leftx,n->rightx,&xl,&xr,glt);
  n->flag=1;
  *xlb=xlt;
  *xrb=xrt;
  n->leftx=xl;
  n->rightx=xr;
  if (glt<=xl) designnode(n,xl);
  else if (glt>=xr) designnode(n,xr);
  else designnode(n,glt);
}


void ADGraphLayout::designtubes(void)
{ int i;
  double xl, xr;
  for (i=0; i<leftarcnumber; i++)
    designtubeintervall(leftarcs[i]->first
                       ,leftarcs[i]->first->leftx,leftarcs[i]->first->rightx
                       ,&xl,&xr,leftarcs[i]->goal);
  for (i=0; i<rightarcnumber; i++)
    designtubeintervall(rightarcs[i]->first
                       ,rightarcs[i]->first->leftx,rightarcs[i]->first->rightx
                       ,&xl,&xr,rightarcs[i]->goal);
  for (i=0; i<shortarcnumber; i++)
    designtubeintervall(shortarcs[i]->first
                       ,shortarcs[i]->first->leftx,shortarcs[i]->first->rightx
                       ,&xl,&xr,shortarcs[i]->goal);
}

void ADGraphLayout::designnode(ADNode * n, double xloc)
{ int i, l; double xr;
  if (n->intermediate==0) return;
  if (n->first) return;
  n->x=xloc;
  n->first=1;
  l=n->line;
  xr=xloc-arcarcdist-2;
  for (i=n->lineindex-1; i>=0; i--)
  { if (lines[l][i]->intermediate==0) break;
    if (lines[l][i]->first) break;
    if (lines[l][i]->rightx<=xr) break;
    lines[l][i]->rightx=xr;
    xr-=arcarcdist+2;
 }
  xr=xloc+arcarcdist+2;
  for (i=n->lineindex+1; i<linelengths[l]; i++)
  { if (lines[l][i]->intermediate==0) break;
    if (lines[l][i]->first) break;
    if (lines[l][i]->leftx>=xr) break;
    lines[l][i]->leftx=xr;
    xr+=arcarcdist+2;
 }
}

void ADGraphLayout::designarcs(void)
{ int i;
   int l; ADNode * n;
  designintervalls();
  setflag(0);
  for (i=0; i<nodenumber; i++)
    nodes[i]->first=0;
  for (i=0; i<intermediatenumber; i++)
    intermediates[i]->first=0;
  designtubes();
}

static double xcor(ADNode * n, int w)
{ if (w) return n->x+((n->dx+1)>>1);
  return n->x-((n->dx+1)>>1);
}

static double ycor(ADNode * n, int w)
{ if (w) return n->y+((n->dy+1)>>1);
  return n->y-((n->dy+1)>>1);
}

static ADNode* bottom(ADNode * n)
{  while (n->intermediate) n=n->left;
   return n;
}
int ADGraphLayout::isseen(ADNode * from, ADNode * to, int offset)
{ double fx, vx, fy, vy, mju;
  int l, i;
  ADNode * n;
  fx=from->x; fy=from->y+((from->dy+1)>>1);
  vy=-(to->x+offset-fx); vx=(to->y-((to->dy+1)>>1)-fy);
  mju=sqrt(vx*vx+vy*vy);
  for( l=from->line+1; l<to->line; l++)
  { for (i=0; i<linelengths[l]; i++)
    { n=lines[l][i];
      if (n->intermediate) continue;
      if (vy<=0)
      { if (   ((xcor(n,0)-fx)*vx+(ycor(n,1)-fy)*vy < mju)
             &&((xcor(n,1)-fx)*vx+(ycor(n,0)-fy)*vy > -mju)
           ) return 0;
      }
      else
      {  if (   ((xcor(n,0)-fx)*vx+(ycor(n,0)-fy)*vy < mju)
             &&((xcor(n,1)-fx)*vx+(ycor(n,1)-fy)*vy > -mju)
           ) return 0;
      }
    }
  }
  return 1;
}

void ADGraphLayout::designslantings(void)
{ int i;
  for (i=0; i<intermediatenumber; i++)
    intermediates[i]->leftslant=intermediates[i]->rightslant=0;
  for (i=0; i<nodenumber; i++)
  { nodes[i]->leftslant=nodes[i]->rightslant=0;
    if ((nodes[i]->left)&&(nodes[i]->left->intermediate))
    { if (nodes[i]->right) 
        nodes[i]->leftslant=isseen(bottom(nodes[i]->left),nodes[i],-10);
      else
        nodes[i]->leftslant=isseen(bottom(nodes[i]->left),nodes[i],0);
   }
   if ((nodes[i]->right)&&(nodes[i]->right->intermediate))
     nodes[i]->rightslant=isseen(bottom(nodes[i]->right),nodes[i],+10);
 }
}


int ADGraphLayout::ADTest(void)
{ int i;
  if (graphisok) return 1;
  graphisok=0;
  if (nodenumber==0) return 0;
  for (i=0; i<nodenumber; i++)
    if (nodes[i]==NULL) return -1;
  graphisok=1;
  return 1;
}

void ADGraphLayout::ADReset(void)
{ int i;
  if (nodes)
  { for (i=0; i<nodenumber; i++)
      delete nodes[i];
    delete nodes;
    nodes=NULL;
  }
  if (intermediates)
  { for (i=0; i<intermediatenumber; i++)
      delete intermediates[i];
    delete intermediates;
    intermediates=NULL;
  }
  if (lines)
  { for (i=0; i<linenumber; i++)
      delete lines[i];
    delete lines;
    lines=NULL;
  }
  if (linelengths)
  { delete linelengths;
    linelengths=NULL;
  }
  makeReset();
  if (leftarcs)
  { for (i=0; i<leftarcnumber; i++)
      delete leftarcs[i];
    delete leftarcs;
    leftarcs=NULL;
  }
  if (rightarcs)
  { for (i=0; i<rightarcnumber; i++)
      delete rightarcs[i];
    delete rightarcs;
    rightarcs=NULL;
  }
  if (shortarcs)
  { for (i=0; i<shortarcnumber; i++)
      delete shortarcs[i];
    delete shortarcs;
    shortarcs=NULL;
  }
  intermediatenumber=nodenumber=linenumber=0;
  allnodes=arcnumber=0;
  leftarcnumber = rightarcnumber = shortarcnumber = 0;
  graphisok=0;
  step=-1;
}

void ADGraphLayout::ADRestart(void)
{ int i;
  if (intermediates)
  { for (i=0; i<intermediatenumber; i++)
      delete intermediates[i];
    delete intermediates;
    intermediates=NULL;
  }
  if (lines)
  { for (i=0; i<linenumber; i++)
      delete lines[i];
    delete lines;
    lines=NULL;
  }
  if (linelengths)
  { delete linelengths;
    linelengths=NULL;
  }
  makeReset();
  if (leftarcs)
  { for (i=0; i<leftarcnumber; i++)
      delete leftarcs[i];
    delete leftarcs;
    leftarcs=NULL;
  }
  if (rightarcs)
  { for (i=0; i<rightarcnumber; i++)
      delete rightarcs[i];
    delete rightarcs;
    rightarcs=NULL;
  }
  if (shortarcs)
  { for (i=0; i<shortarcnumber; i++)
      delete shortarcs[i];
    delete shortarcs;
    shortarcs=NULL;
  }
  intermediatenumber=linenumber=arcnumber=0;
  allnodes=nodenumber;
  leftarcnumber = rightarcnumber = shortarcnumber = 0;
  graphisok=0;
  step=-1;
}

void ADGraphLayout::ADSetNodeNumber(int num)
{ int i;
  if (nodenumber) ADReset();
  allnodes=nodenumber=num;
  nodes=new ADNode* [num];
  for (i=0; i<num; i++) nodes[i]=NULL;
  return;
}

int ADGraphLayout::ADAddNode(int idx, int x, int y, int dx, int dy, int dir)
{ if ((idx<0) || (idx>=nodenumber)) return 0;
 nodes[idx]=makenode(x,y,dx,dy,dir);
 return 1;
}

int ADGraphLayout::ADAddArc(int from, int to, int isleft)
{ if ((from<0) || (from>=nodenumber) || (to<0) || (to>=nodenumber)) return 0;
  if ((nodes[to]==NULL) || (nodes[from]==NULL)) return -1;
  if (isleft) nodes[to]->left=nodes[from];
  else nodes[to]->right=nodes[from];
  return 1;
}

int ADGraphLayout::ADResizeNode(int idx, int x, int y, int dx, int dy)
{ if ((idx<0) || (idx>=nodenumber)) return 0;
  nodes[idx]->x=double(x);
  nodes[idx]->y=y;
  nodes[idx]->dx=dx;
  nodes[idx]->dy=dy;
  return 1;
}

int ADGraphLayout::ADMoveNode(int idx, int newx, int newy)
{ int i, j, k, l, found; ADNode * n;
  if ((idx<0)||(idx>=nodenumber)) return 0;
  found=0;
  if (lines)
  { for (l=0; l<linenumber; l++)
    { for (i=0; i<linelengths[l]; i++)
        if (lines[l][i]==nodes[idx]) { found=1; break; }
      if (found) break;
    }
    if (!found)
    { nodes[idx]->x=double(newx);
      return 1;
    }
    for (j=0; j<linelengths[l]; j++)
      if (newx<lines[l][j]->x) break;
    if (j<i)
    { n=lines[l][i];
      for (k=i; k>j; k--) lines[l][k]=lines[l][k-1];
      lines[l][j]=n;
      nodes[idx]->x=double(newx);
      return 1;
    }
    j--;
    if (j<=i)
    { nodes[idx]->x=double(newx);
      return 1;
    }
    n=lines[l][i];
    for (k=i; k<j; k++) lines[l][k]=lines[l][k+1];
    lines[l][j]=n;
    nodes[idx]->x=double(newx);
    return 1;
  }
  nodes[idx]->x=double(newx);
  return 1;
}

void ADGraphLayout::ADSetParameter(int ll, int nn, int na, int aa, int ac, int las)
{ linelinedist = ll;
  nodenodedist = nn;
  nodearcdist = na;
  arcarcdist = aa;
  arccrossingoffset = ac;
  longarcboundary=las;
}

int ADGraphLayout::ADLayout(int st, int what)
{ int i;
  MAXSTEPS = st;
  if (ADTest()<=0) return 0;
  if (what<=0)
  { ADRestart();
    heightsort();
    intermediatenodes();
    makelines();
  }
  if (what<=1)
  { if (lines==NULL) return -1;
    arrange();
    longhandling();
  }
  if (what<=2)
  { if (lines==NULL) return -1;
    adjust();
    updateY();
    designarcs();
    designslantings();
    centerXY();
  }
  return 1;
}

int ADGraphLayout::ADLayoutByStep(int st, int what)
{ MAXSTEPS = st;
  if (step==-1)
  { if (ADTest()<=0) return 0;
    if (what<=0)
    { ADRestart();
      heightsort();
      intermediatenodes();
      makelines();
    }
    if (what<=1)
    { if (lines==NULL) { makeReset(); return -1; }
      arrange();
    }
    if (what<=2)
    { if (lines==NULL) { makeReset(); return -1; }
      step=0;
      adjustbystep1();
      updateY();
      if (step == -1) { makeReset(); return -2; }
    }
    return 1;
  }
  if (step++>=MAXSTEPS)
  { makeReset();
    designarcs();
    designslantings();
    return 0;
  }
  adjustbystep2();
  if (step == -1)
  { makeReset();
    designarcs();
    designslantings();
    return -2;
  }
  return 1;
}

int ADGraphLayout::ADGetNode(int idx, int *x, int *y, int *l, int *li)
{ int i;
  if ((idx<0) || (idx>nodenumber) || (nodes[idx]==NULL)) return 0;
  *x=nodes[idx]->x;
  *y=nodes[idx]->y;
  *l=nodes[idx]->line;
  *li=nodes[idx]->nodeindex;
  return 1;
}

int ADGraphLayout::ADGetArc(int to, int isleft, int * n, int ** inter)
{ int * ip; ADNode * ptr;
  int y=0; double x;
  *n=0; *inter=NULL;
  if (nodes[to]==NULL) return 0;
  if (isleft)
  { if (   (nodes[to]->left==NULL)
        || (nodes[to]->left->intermediate==0)
        || (nodes[to]->leftslant)
    )
    { *n=0; *inter=NULL; return 2;
    }
    *n=0;
    ptr=nodes[to]->left;
    x=ptr->x-1.0;
    while (ptr->intermediate)
    { if (x!=ptr->x) { (*n)+=2; x=ptr->x; }
      ptr=ptr->left;
    }
    *inter=new int [(*n)<<1];
    ip= *inter+((*n)<<1)-1;
    ptr=nodes[to]->left;
    x=ptr->x;
    *ip-- = ptr->y+((ptr->dy+1)>>1);
    *ip-- = ptr->x;
    while (ptr->intermediate)
    { if (x!=ptr->x)
      { *ip-- = y;
        *ip-- = x;
        x=ptr->x;
        *ip-- = ptr->y+((ptr->dy+1)>>1);
        *ip-- = ptr->x;
      }
      y = ptr->y-((ptr->dy+1)>>1);
      ptr=ptr->left;
    }
    *ip-- = y;
    *ip-- = x;
    return 1;
  }
  else
  { if (   (nodes[to]->right==NULL)
        || (nodes[to]->right->intermediate==0)
        || (nodes[to]->rightslant)
       )
    { *n=0; *inter=NULL; return 2;
    }
    *n=0;
    ptr=nodes[to]->right;
    x=ptr->x-1.0;
    while (ptr->intermediate)
    { if (x!=ptr->x) { (*n)+=2; x=ptr->x; }
      ptr=ptr->left;
    }
    *inter=new int [(*n)<<1];
    ip= *inter+((*n)<<1)-1;
    ptr=nodes[to]->right;
    x=ptr->x;
    *ip-- = ptr->y+((ptr->dy+1)>>1);
    *ip-- = ptr->x;
    while (ptr->intermediate)
    { if (x!=ptr->x)
      { *ip-- = y;
        *ip-- = x;
        x=ptr->x;
        *ip-- = ptr->y+((ptr->dy+1)>>1);
        *ip-- = ptr->x;
      }
      y = ptr->y-((ptr->dy+1)>>1);
      ptr=ptr->left;
    }
    *ip-- = y;
    *ip-- = x;
    return 1;
  }
  return 0;
}
