#define ADBOTTOM   -2
#define ADDOWN     -1
#define ADANYWHERE  0
#define ADUP        1
#define ADTOP       2

#define ADLAYOUT    0
#define ADARRANGE   1
#define ADADJUST    2
#define ADNOTHING   3

#define ADLEFTARC   1
#define ADMIDDLEARC 1
#define ADRIGHTARC  0

struct ADNode;
struct ADLongarc;

class ADGraphLayout {

  int nodenumber ;
  int intermediatenumber ;
  int allnodes ;
  int arcnumber ;
  int linenumber ;
  ADNode** nodes;
  ADNode** intermediates;
  ADNode*** lines;
  int * linelengths;
  ADLongarc ** leftarcs, ** rightarcs, ** shortarcs;
  int leftarcnumber, rightarcnumber, shortarcnumber;
  int graphisok ;
  int * M, * Diag;
  double * Delta, * X, * D, * Grad, * H;
  int Msize;
  int step;


  double alpha, beta, epsilon;

  int linelinedist,
      nodenodedist,
      nodearcdist,
      arcarcdist,
      arccrossingoffset,
      longarcboundary,
      MAXSTEPS;

  void setflag(int n);
  void printnodes(void);
  void printmatrix(char * n, int * ptr, int z, int s);
  void printvector(char * n, double * ptr, int z);
/*  in der hoehe sortieren und zeilen erzeugen */

  void heightsort(void);
  void intermediatenodes(void);
  void makelines(void);

  void moveupward(ADNode * node);
  int setheight(ADNode * node);

/*  sortieren der zeilen */

  void arrange(void);
  void arrangeline(int l);
  void sortline(int l);
  void prepareline(int l);
  void analyselineandsetbelow(int l);
  int evaluateline(int l);
  void freenodesinline(int l);
  void setincludednodes(ADNode * n, double v);
  void getbestxvalue(ADNode * n, double * x, int * depth);
  void setlineindices(void);

/* behandeln langer kanten */

  void longhandling(void);
  int longarctest(ADNode * node, int isleft);
  ADLongarc * longarcregister(ADNode * node, int isleft);
  void longarcsort(ADLongarc ** la,int ln, int all);
  void longarcmoveleft(ADLongarc * la);
  void longarcmoveright(ADLongarc * la);

/* ausrichten der knoten */

  void adjust(void);
  void adjustbystep1(void);
  void adjustbystep2(void);
  void makeM(void);
  void makeDelta(void);
  void makeX0(void);
  int makenewX(void);
  void makeStore(void);
  void makeReset(void);

/* generelles layout */

  void updateY(void);
  void ysetline(int l, int yy);
  int ygetheight(int l);
  int ygetcrossings(int l);
  void centerXY(void);

/* kanten begradigen */

  void designintervalls(void);
  void designtubeintervall(ADNode * n,double xlt,double xrt
                                   ,double * xlb,double * xrb
                                   ,double glt);
  void designtubes(void);
  void designnode(ADNode * n, double xloc);
  void designarcs(void);
  int isseen(ADNode * from, ADNode * to, int offset);
  void designslantings(void);

public :

  ADGraphLayout(void);
  ~ADGraphLayout(void);
  int ADTest(void);
  void ADReset(void);
  void ADRestart(void);
  void ADSetNodeNumber(int num);
  int ADAddNode(int idx, int x, int y, int dx, int dy, int dir);
  int ADAddArc(int from, int to, int isleft);
  int ADResizeNode(int idx, int x, int y, int dx, int dy);
  int ADMoveNode(int idx, int newx, int newy);
  void ADSetParameter(int ll, int nn, int na, int aa, int ac, int las);
  int ADLayout(int steps, int what);
  int ADLayoutByStep(int st, int what);
  int ADGetNode(int idx, int *x, int *y, int *line, int *lineidx);
  int ADGetArc(int to, int isleft, int * n, int ** points);

};
