
/*												    							
   ---------------------------------------------------------------
   tape_translate
   ---------------------------------------------------------------
*/

extern "C" {
#include "dvlparms.h" /* Developers Parameters */
#include "usrparms.h"
#include "oplate.h"
#include "taputil1.h"
#include "taputil2.h"
#include "taputil3.h"
/*#include "tayutil.h"*/
#include "adutilsc.h"
}
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static char * strdup(char * ptr)
{ return strcpy(new char[strlen(ptr)+1],ptr);
}

#define OPCODEMAX 257
#define val_opcode 256

static struct Opdesc{ char * name; int type; } opdesc[OPCODEMAX];
static char * unknownname = "unknown";

#define UNKNOWNTYPE 0
#define Taa 	 1
#define TI 	 2
#define TD 	 3
#define TAR 	 4
#define TDR 	 5
#define TDRR 	 6
#define TARR 	 7
#define TAAR 	 8
#define TDAR 	 9
#define TAadR 	10
#define TAAdR	19
#define TAdR 	11
#define TAaR  	18
#define TDA 	16
#define TAA 	17 
#define Teop	12
#define Teoi	13
#define Teov	14
#define Ti	15

#define MAXNODES 16000

#define NoNode 0
#define ONode 1
#define INode 2
#define DNode 3
#define VNode 4

struct Node {	int opcode;
		Node * arg1, * arg2, * arg3;
		double value;
		int mark;
		int type;
		int index;
		Node(void);
		void print(void);
	    };

Node::Node(void)
{ opcode=0;
  arg1=arg2=arg3=NULL;
  mark=0;
  type=NoNode;
  value=0.0;
  index=-1;
}

void Node::print(void)
{ printf("%4d %d %s ",index,type,opdesc[opcode].name);
  printf("%d ",(arg1!=NULL)+(arg2!=NULL)+(arg3!=NULL));
  if (arg1) printf("%d ",arg1->index);
  if (arg2) printf("%d ",arg2->index);
  if (arg3) printf("%d ",arg3->index);
  if (opcode==val_opcode) printf("%25.19e ",value);
  printf("\n");
}

static Node * nodelist[MAXNODES];

static void registernode(int opcode, int type, int res, int arg1)
{ nodelist[res]->opcode=opcode;
  nodelist[res]->type=type;
  nodelist[res]->arg1=nodelist[arg1];
  nodelist[res]->arg2=NULL;
  nodelist[res]->arg3=NULL;
}

static void registernode(int opcode, int type, int res, int arg1, int arg2)
{ nodelist[res]->opcode=opcode;
  nodelist[res]->type=type;
  nodelist[res]->arg1=nodelist[arg1];
  nodelist[res]->arg2=nodelist[arg2];
  nodelist[res]->arg3=NULL;
}

static void registernode(int opcode, int type, int res, int arg1, int arg2, int arg3)
{ nodelist[res]->opcode=opcode;
  nodelist[res]->type=type;
  nodelist[res]->arg1=nodelist[arg1];
  nodelist[res]->arg2=nodelist[arg2];
  nodelist[res]->arg3=nodelist[arg3];
}

static void registervalue(int opcode, int res, double value)
{ nodelist[res]->opcode=opcode;
  nodelist[res]->type=VNode;
  nodelist[res]->value=value;
  nodelist[res]->arg1=NULL;
  nodelist[res]->arg2=NULL;
  nodelist[res]->arg3=NULL;
}

static int transtable[MAXNODES];
static int maxnode = 0;
static int valuenode = MAXNODES-1;
static int accessnode = 0;;

static int getnode(int node)
{ int i;
  if (node>=accessnode)
  { for (;accessnode<=node; accessnode++)
      transtable[accessnode]=maxnode++;
    if (maxnode>valuenode)
    { fprintf(stderr,"Fatal error: more than %d nodes in the tape\n",MAXNODES);
      exit(1);
    }
  }
  if (nodelist[transtable[node]]==NULL)
    nodelist[transtable[node]]=new Node;
  return transtable[node];
}

static int getnewnode(int node)
{ if (node<accessnode)
  { transtable[node]=maxnode;
    nodelist[maxnode]=new Node;
    maxnode++;
    if (maxnode>valuenode)
    { fprintf(stderr,"Fatal error: more than %d nodes in the tape\n",MAXNODES);
      exit(1);
    }
    return transtable[node];
  }
  return getnode(node);
}

static int getvaluenode(void)
{ if (maxnode>=valuenode)
  { fprintf(stderr,"Fatal error: more than %d nodes in the tape\n",MAXNODES);
    exit(1);
  }
  nodelist[valuenode]=new Node;
  transtable[valuenode]=valuenode;
  return valuenode--;
}

static int getaditionalnode(void)
{ if (maxnode>=valuenode)
  { fprintf(stderr,"Fatal error: more than %d nodes in the tape\n",MAXNODES);
    exit(1);
  }
  nodelist[valuenode]=new Node;
  transtable[valuenode]=valuenode;
  return valuenode--;
}

static void deathnodes(int arg1, int arg2)
{ int i,m;
  if (arg1>arg2) 
  { fprintf(stderr,"Fatal error: death nodes from %d to %d\n",arg1,arg2);
    exit(1);
  }
  if ((arg1>=accessnode)||(arg1>=accessnode))
  { fprintf(stderr,"Fatal error: death nodes from %d to %d\n but accessed nodes less than %d"
           ,arg1,arg2,accessnode);
    exit(1);
  }
  for (i=arg1; i<=arg2; i++)
    transtable[i]=maxnode++;
  if (maxnode>=valuenode)
  { fprintf(stderr,"Fatal error: more than %d nodes in the tape\n",MAXNODES);
    exit(1);
  }
}

#define DEFOP(o,n,t) ptr= #n ; opdesc[o].name=strdup(ptr) ; opdesc[o].type=t;

static void tdinit(void)
{ int i, j; char * ptr;
  for (i=0; i<OPCODEMAX; i++)
  { opdesc[i].name=unknownname;
    opdesc[i].type=UNKNOWNTYPE;
  }

#include "ttranslate.opcodedescription"

  opdesc[val_opcode].name=strdup("Value");
  opdesc[val_opcode].type=Ti;
  for (i=0; i<MAXNODES; i++)
  { transtable[i]=i;
    nodelist[i]=NULL;
 }
}


void tapeopen(short tag)
{  int tape_stats[11];  /* tape stats */

  tapestats(tag,tape_stats);
				    
  set_buf_size(tape_stats[4]);
  init_for_sweep(tag);
}

void tapeanalyse(short tag)
{ int operation;
  int arg1, arg2, arg3, res, size, darg;
  double *d, coval;
  do
  {
    operation=get_op_f();
    switch (opdesc[operation].type){
     case Taa:
	arg1  = get_locint_f();
	arg2  = get_locint_f();
	deathnodes(arg1,arg2);
	break;
     case TI:
	res  = getnode(get_locint_f());
        if (nodelist[res]->type==NoNode)
        { arg1 = getvaluenode();
	  coval=0.0;
	  registervalue(val_opcode,arg1,coval);
	  registernode(operation,INode,res,arg1);
	}
	nodelist[res]->type=INode;
	break;
     case TD:
	darg = get_locint_f();
	arg1  = getnode(darg);
	res  = getnewnode(darg);
	registernode(operation,DNode,res,arg1);
	nodelist[res]->mark=1;
	break;
     case TAR:
	arg1  = getnode(get_locint_f());
	res  = getnewnode(get_locint_f());
	registernode(operation,ONode,res,arg1);
	break;
     case TDR:
	coval=get_val_f();
	arg1 = getvaluenode();
	res  = getnewnode(get_locint_f());
	registervalue(val_opcode,arg1,coval);
	registernode(operation,ONode,res,arg1);
	break;
     case TDRR:
	coval=get_val_f();
	darg = get_locint_f();
	arg1  = getnode(darg);
	arg2 = getvaluenode();
	res  = getnewnode(darg);
	registervalue(val_opcode,arg2,coval);
	registernode(operation,ONode,res,arg1,arg2);
	break;
     case TARR:
	arg2  = getnode(get_locint_f());
	darg = get_locint_f();
	arg1  = getnode(darg);
	res  = getnewnode(darg);
	registernode(operation,ONode,res,arg1,arg2);
	break;
     case TAAR:
	arg1  = getnode(get_locint_f());
	arg2  = getnode(get_locint_f());
	res  = getnewnode(get_locint_f());
	registernode(operation,ONode,res,arg1,arg2);
	break;
     case TDAR:
	coval=get_val_f();
	arg1 = getvaluenode();
	arg2  = getnode(get_locint_f());
	res  = getnewnode(get_locint_f());
	registervalue(val_opcode,arg1,coval);
	registernode(operation,ONode,res,arg1,arg2);
	break;
     case TAadR:
	arg1  = getnode(get_locint_f());
	get_val_f();
	arg2=get_locint_f();
	res  = getnewnode(get_locint_f());
        getnewnode(arg2);
	registernode(operation,ONode,res,arg1);
	break;
     case TAAdR:
	arg1  = getnode(get_locint_f());
	arg2  = getnode(get_locint_f());
	get_val_f();
	res  = getnewnode(get_locint_f());
        getnewnode(arg2);
	registernode(operation,ONode,res,arg1,arg2);
	break;
     case TAdR:
	arg1  = getnode(get_locint_f());
	get_val_f();
	res  = getnewnode(get_locint_f());
	registernode(operation,ONode,res,arg1);
	break;
     case TAaR:
	arg1  = getnode(get_locint_f());
	arg2=get_locint_f();
	res  = getnewnode(get_locint_f());
        getnewnode(arg2);
	registernode(operation,ONode,res,arg1);
	break;
     case TAA:
	arg1  = getnode(get_locint_f());
	arg2  = getnode(get_locint_f());
	res  = getaditionalnode();
	registernode(operation,ONode,res,arg1,arg2);
	nodelist[res]->mark=1;
	break;
     case TDA:
	coval=get_val_f();
	arg2 = getvaluenode();
	registervalue(val_opcode,arg2,coval);
	arg1  = getnode(get_locint_f());
	res  = getaditionalnode();
	registernode(operation,ONode,res,arg1,arg2);
	nodelist[res]->mark=1;
	break;
     case Teop:				  
	get_op_block_f();
	operation=get_op_f(); 
	/* Skip next operation, it's another end_of_op */
	break;
     case Teoi:
	get_loc_block_f();
	break;
     case Teov:				  
	get_val_block_f();
	break;
     case UNKNOWNTYPE:
	fprintf(stderr,"Fatal error: unknown type for operation %s(%d)\n"
               ,opdesc[operation].name,operation);
	exit(1);;
    } /* endswitch */
  } while (operation != end_of_tape);
  end_sweep();
}

static void compress(int what)
{ int i, j;
  if (what)
  { for (i=maxnode-1; i>=0; i--)
      if ((nodelist[i])&&(nodelist[i]->mark))
      { if (nodelist[i]->arg1) nodelist[i]->arg1->mark=1;
        if (nodelist[i]->arg2) nodelist[i]->arg2->mark=1;
        if (nodelist[i]->arg3) nodelist[i]->arg3->mark=1;
      }
    for (i=valuenode+1; i<MAXNODES; i++)
      if ((nodelist[i])&&(nodelist[i]->mark))
      { if (nodelist[i]->arg1) nodelist[i]->arg1->mark=1;
        if (nodelist[i]->arg2) nodelist[i]->arg2->mark=1;
        if (nodelist[i]->arg3) nodelist[i]->arg3->mark=1;
      }
    for ((i=0,j=0); i<maxnode; i++)
      if ((nodelist[i])&&(nodelist[i]->mark))
      { nodelist[j]=nodelist[i];
        nodelist[j]->index=j;
        j++;
      }
    for (i=valuenode+1; i<MAXNODES; i++)
      if ((nodelist[i])&&(nodelist[i]->mark))
      { nodelist[j]=nodelist[i];
        nodelist[j]->index=j;
        j++;
      }
    maxnode=j;
    return;
  }
    for ((i=0,j=0); i<maxnode; i++)
      if (nodelist[i])
      { nodelist[j]=nodelist[i];
        nodelist[j]->index=j;
        j++;
      }
    for (i=valuenode+1; i<MAXNODES; i++)
      if (nodelist[i])
      { nodelist[j]=nodelist[i];
        nodelist[j]->index=j;
        j++;
      }
    maxnode=j;
    return;
}

static void writefile(void)
{ int i;
  printf("%d (nodes)\n",maxnode);
  for (i=0; i<maxnode; i++)
    nodelist[i]->print();
}

int main(int argc, char* argv[])
{ short id; int what;
  if (argc<2)
  { id=1; what=1; }
  else if (argc<3)
  { if ((argv[1][0]=='A')||(argv[1][0]=='a'))
    { id=1; what=0; }
    else
    { id=atoi(argv[1]); what=1; }
  }
  else
  { if ((argv[1][0]=='A')||(argv[1][0]=='a'))
    { id=atoi(argv[2]); what=0; }
    else
    { id=atoi(argv[1]); what=1-((argv[2][0]=='A')||(argv[2][0]=='a')); }
  }

  tdinit();
  tapeopen(id);
  tapeanalyse(id);
  compress(what);
  writefile();
  return 0;
}
