#define _VDABSTRACTWINDOW_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdAbstractWindow.cxx                              *
*  Date: 02-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Declaration of class VdAbstractWindow:               *
*        An abstract window is a XmMainWindow with         *
*        MenuBar, ScrollBars and an DrawingArea for        *
*        graphics. It shall encapsulated into an           *
*        application shell or an dialogshell.              *
*        It is the base class for Magnifier and EditWindow *
*                                                          *
*     References:                                          *
* [XtRM] X Toolkit Intrinsics Reference Manual,            * 
*        Second Edition for X11 Release 4                  *
*        O'Reilly & Associates, Inc.                       *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <X11/StringDefs.h>

extern "C" {
#include <stdlib.h>
}
#include <X11/cursorfont.h>
#include <Xm/MainW.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/CascadeB.h>
#include <strstream.h> 
#include <iostream.h>

#include <Vd/VdAbstractWindow.hxx>
#include <Vd/VdButtonInterface.hxx>

#define DRAW_IN_BACKGROUND

/******* local declaration of class ExitHandling ****/
class ExitHandling: public VdButtonInterface{
	private: 
	    VdShell *theShellPtr ;
	public:
	    int(*checkIt)(const VdFeature &) ;
	    int mode ;
	    ExitHandling(int buttonMode,
		VdShell *shell) ;
	    virtual void activate ( const VdFeature & theWindow
                        );
	} ;


ExitHandling::ExitHandling(int buttonMode,
	VdShell *shell)
{
	if(buttonMode==0) idf=(VdString)"Exit";
	else idf=(VdString)"Close" ;
	mode=buttonMode ;
	theShellPtr=shell ;
	checkIt=NULL ;
}

void ExitHandling::activate( const VdFeature &theIdent)
{
	if (!checkIt) {
	    if(mode==0) 
		exit(0) ;
	    else {
		theShellPtr->handleDestroyed();
		XtDestroyWidget(theShellPtr->getWidget());
	    }
	} else if (checkIt(theIdent)==1) {
	    if(mode==0) 
		exit(0) ;
	    else {
		theShellPtr->handleDestroyed();
		XtDestroyWidget(theShellPtr->getWidget()) ;
	    }
	}
}		

//*************************************************************

void VdAbstractWindow::init(VdShell &shell,
		Boolean withScrollbars)
{
	Arg al[5];
	unsigned int ac;
	theShellPtr = &shell;
	theShellWidget = shell.getWidget();
	theBackgroundPixel = 0; 
		// used only #ifdef DRAW_IN_BACKGROUND
		// initialized in redrawPicture
	ac = 0;
	XtSetArg(al[ac], XmNscrollBarDisplayPolicy, XmSTATIC); ac++;
	XtSetArg(al[ac], XmNscrollingPolicy,XmAPPLICATION_DEFINED); ac++;
	theMainWindow = XmCreateMainWindow( theShellWidget,
			"VisamadMainWindow", al, ac);
	XtManageChild(theMainWindow);

	exitButtonPtr=NULL ;
	exitHandler= new ExitHandling(
			theShellPtr->isMainShell() ? 0 : 1,
			theShellPtr);
	theDrawingArea = XmCreateDrawingArea( theMainWindow,
			"VisamadDrawingArea", NULL, 0);
	XtManageChild(theDrawingArea);

	updateDrawableValues();

	theMenubar = XmCreateMenuBar( theMainWindow,
			"VisamadMenuBar", NULL, 0);
	
	XtManageChild(theMenubar);

	ac = 0;
	theHelpButton = XmCreateCascadeButton(theMenubar, 
		"Help", al, ac);
	ac = 0;
	XtSetArg(al[ac], XmNmenuHelpWidget, theHelpButton); 
	ac++;
	XtSetValues(theMenubar, al, ac);
	XtManageChild(theHelpButton);
	XtAddCallback(theHelpButton, XmNactivateCallback, 
		VdAbstractWindow::helpCB, (XtPointer)this );
#ifdef DRAW_IN_BACKGROUND
	ac = 0;
	XtSetArg(al[ac],XmNbackground, &theBackgroundPixel); ac++;
	XtGetValues(theDrawingArea,al,ac);
#endif
	hasScrollbars = withScrollbars;
	if (withScrollbars) {
	   Arg al[5];
	   unsigned int ac;
	   ac = 0;
	   XtSetArg(al[ac], XmNorientation, XmHORIZONTAL); 
	   ac++;
	   XtSetArg(al[ac], XmNvalue, 0); ac++;
	   XtSetArg(al[ac], XmNsliderSize, 100); ac++;
	   XtSetArg(al[ac], XmNincrement, 2); ac++;
	   theHoriScrollbar 
		= XmCreateScrollBar(theMainWindow,
			"VisamadScrollbar",al,ac);
	   ac = 0;
	   XtSetArg(al[ac], XmNorientation, XmVERTICAL);
	   ac++;
	   XtSetArg(al[ac], XmNvalue, 0); ac++;
	   XtSetArg(al[ac], XmNsliderSize, 100); ac++;
	   XtSetArg(al[ac], XmNincrement, 2); ac++;
	   theVertScrollbar
		= XmCreateScrollBar(theMainWindow,
			"VisamadScrollbar",al,ac);

	   addCB(theHoriScrollbar, valueChanged);
	   addCB(theVertScrollbar, valueChanged);
	} else 
	   theHoriScrollbar = theVertScrollbar = NULL;

	XmMainWindowSetAreas( theMainWindow, theMenubar,
		NULL, theHoriScrollbar, theVertScrollbar,
		theDrawingArea);
	if (hasScrollbars) {
	   XtManageChild(theVertScrollbar);
	   XtManageChild(theHoriScrollbar);
	}
	updateDrawableValues();

//  Resize-Callback darf offensichtlich erst nach XmMainWindowSetAreas
//  installiert werden, sonst erfolgt ein Absturz via motifCB -
//  handleResized - updateDrawableValues - XtGetValues.
	addCB(theDrawingArea,resized);
	addCB(theDrawingArea,exposed);
	addCB(theDrawingArea,input);

	thePicturePtr = NULL;
	theGeometricContextPtr = new VdGeometricContext(
		VdArea(VdPoint(0,0),
		       VdPoint(theDrawableSize.x,
			       theDrawableSize.y)));
	theScrollbarValues.world = 
	   theScrollbarValues.displayed =
		theGeometricContextPtr->getWindowArea();
	updateScrollbars();
	theGAPtr = NULL;
	theIdentification = VdFeature(theShellPtr->getTitle());
	theShellPtr->setWindow(this);
}

VdAbstractWindow::~VdAbstractWindow( void )
{
	if (theGAPtr) {
		delete theGAPtr;
		theGAPtr = NULL;
	}
	if (theGeometricContextPtr) {
		delete theGeometricContextPtr;
		theGeometricContextPtr = NULL;
	}
	if(exitHandler) delete exitHandler ;
}


void VdAbstractWindow::helpCB(Widget widget, 
		XtPointer clientData, XtPointer )
{
	((VdAbstractWindow *)clientData)->handleHelp();
}


void VdAbstractWindow::clear( void )
{
	if (!XtWindow(theDrawingArea)) return;
#ifndef DRAW_IN_BACKGROUND
	XClearWindow(
		theShellPtr->getDisplay(),
		XtWindow(theDrawingArea)
	);
#else
	XSetForeground(
	  theShellPtr->getDisplay(),
	  theGAPtr->getGC(),
	  (Pixel)theBackgroundPixel
	);
	XFillRectangle(
	  theShellPtr->getDisplay(),
	  theGAPtr->getDrawable(),
	  theGAPtr->getGC(),
	  0,0,
	  theDrawableSize.x,
	  theDrawableSize.y
	);
	XSetForeground(
	  theShellPtr->getDisplay(),
	  theGAPtr->getGC(),
	  0
	);
#endif
}

// get the size values from the Widget
void VdAbstractWindow::updateDrawableValues( void )
{
	Arg al[5];
	Cardinal ac;
	Dimension x=0, y=0;
	
	ac = 0;
	XtSetArg(al[ac],XmNwidth, &x); ac++;
	XtSetArg(al[ac],XmNheight, &y); ac++;
	XtGetValues(theDrawingArea,al,ac);
	theDrawableSize = VdVector(x,y);
}


void VdAbstractWindow::setIdent(const VdFeature& ident)
{
	theIdentification = ident;
}

VdFeature VdAbstractWindow::getIdent( void )
{
	return theIdentification;
}

int VdAbstractWindow::checkClose( void )
{
	if (!exitHandler) return 1;
	ExitHandling * ehPtr = (ExitHandling *)exitHandler;
	if (!ehPtr->checkIt) return 1;
	return ehPtr->checkIt(getIdent());
}

void VdAbstractWindow::map( void )
{
	theShellPtr->map();
}

void VdAbstractWindow::unmap( void )
{
	theShellPtr->unmap();
}

void VdAbstractWindow::destroy( void )
{
	if (exitHandler) 
		((ExitHandling *)exitHandler)->checkIt=NULL;
	XtDestroyWidget(theShellPtr->getWidget());
}


void VdAbstractWindow::setDisplayOfWorld(
		VdArea displayed, VdArea world)
{
	if (theGeometricContextPtr) {
	   theGeometricContextPtr->setWorldArea(
					world);
	   world = 
		theGeometricContextPtr->getWorldArea();
	   theGeometricContextPtr->setDisplayedArea(
					displayed);
	   displayed = 
	       theGeometricContextPtr->getDisplayedArea();
	}
	theScrollbarValues.world = world;
	theScrollbarValues.displayed = displayed;
	updateScrollbars();
}

void VdAbstractWindow::getDisplayOfWorld( 
		VdArea& displayed, VdArea& world )
{
	world = theGeometricContextPtr->getWorldArea();
	displayed = 
	      theGeometricContextPtr->getDisplayedArea();
}

void VdAbstractWindow::setWindowType(VdWindowType wt)
{
	if (theGeometricContextPtr) {
	   theGeometricContextPtr->setWindowType(wt);
	}
	windowType = wt;
}

// update the WindowScrollbars layout by the current
// value of TheScrollbarValues

void VdAbstractWindow::updateScrollbars( void )
{
	Arg al[5];
	Cardinal ac;
	int horiValue, horiSize, vertValue, vertSize;

	if (!hasScrollbars) return;
	if (theGeometricContextPtr) {
	   theScrollbarValues.displayed = 
		theGeometricContextPtr->getDisplayedArea();
	   theScrollbarValues.world = 
		theGeometricContextPtr->getWorldArea();
	}

	horiValue = 100 *
		  ( theScrollbarValues.displayed.ul.x 
		    - theScrollbarValues.world.ul.x)
		/ ( theScrollbarValues.world.lr.x 
		    - theScrollbarValues.world.ul.x );
	horiSize = 100 *
		  ( theScrollbarValues.displayed.lr.x 
		    - theScrollbarValues.displayed.ul.x)
		/ ( theScrollbarValues.world.lr.x 
		    - theScrollbarValues.world.ul.x );
	vertValue = 100 *
		  ( theScrollbarValues.world.ul.y 
		    - theScrollbarValues.displayed.ul.y )
		/ ( theScrollbarValues.world.ul.y 
		    - theScrollbarValues.world.lr.y );
	vertSize = 100 *
		  ( theScrollbarValues.displayed.ul.y 
		    - theScrollbarValues.displayed.lr.y )
		/ ( theScrollbarValues.world.ul.y 
		    - theScrollbarValues.world.lr.y );
	if (horiValue<0) horiValue = 0;
	if (vertValue<0) vertValue = 0;
	if (horiSize<0) horiSize = 1;
	if (vertSize<0) vertSize = 1;
	if (horiSize>100) horiSize = 100;
	if (vertSize>100) vertSize = 100;
	if (horiValue+horiSize>100) 
		horiValue = 100 - horiSize;
	if (vertValue+vertSize>100) 
		vertValue = 100 - vertSize;

	// considering special cases to exclude rounding
	// errors
	if ( (theScrollbarValues.world.lr.x
		==theScrollbarValues.displayed.lr.x)
	     && (horiValue+horiSize<100) )
		horiValue=100-horiSize;
	if ( (theScrollbarValues.world.lr.y
		==theScrollbarValues.displayed.lr.y)
	     && (vertValue+vertSize<100) )
		vertValue=100-vertSize;

	ac = 0;
	XtSetArg(al[ac],XmNvalue, horiValue); ac++;
	XtSetArg(al[ac],XmNsliderSize, horiSize); ac++;
	XtSetValues(theHoriScrollbar,al,ac);
	ac = 0;
	XtSetArg(al[ac],XmNvalue,vertValue); ac++;
	XtSetArg(al[ac],XmNsliderSize, vertSize); ac++;
	XtSetValues(theVertScrollbar,al,ac);
}

void VdAbstractWindow::setPicture( VdPicture *pic )
{
	thePicturePtr = pic;
	redrawPicture();
}

void VdAbstractWindow::setPicture( VdPicture *pic,  
		VdGeometricContext& con )
{
	VdPoint size(theDrawableSize.x, theDrawableSize.y);
	thePicturePtr = pic;
	*theGeometricContextPtr = con;
	theGeometricContextPtr->setWindowArea(
				VdArea(VdNullPoint,size));
	// the windowArea of con must be ignored (here)
	// or results an resize
	redrawPicture();
	updateScrollbars();
}

// predicate for XCheckIfEvent detecting expose events
static Bool isExposeEvent(Display* , XEvent *ev, char *)
{
	return (   ev->type == Expose ); 
}
// predicate for XCheckIfEvent detecting input events
static Bool isInputEvent(Display* , XEvent *ev, char *)
{
	return (   ev->type == ButtonPress 
		|| ev->type == ButtonRelease 
		|| ev->type == KeyPress  
		|| ev->type == KeyRelease );
}

void VdAbstractWindow::movePicture( 
		VdPicture * toDraw, VdPicture * toMove,
		VdPoint * worldLocation )
{
	Window theXWindow = XtWindow(getDrawable()), dummy;
	int intDummy;
	unsigned int mask;
	int buttonReleasable=0;

	if (!worldLocation) return;
	VdPoint oldLocation;
	VdPoint newLocation;
	VdVector diff;
	XEvent ev;
#ifdef DRAW_IN_BACKGROUND
	Drawable savedDrawable = theGAPtr->getDrawable();
	theGAPtr->setDrawable((Drawable)theXWindow);
#endif
	{	// detecting if window is not visible
	    Window w; int rev;
	    XGetInputFocus(getDisplay(),&w,&rev);
	    if (w != XtWindow(getShell())) {
		// window has not the focus
		// map window to the top of the window stack
		XMapRaised(getDisplay(), XtWindow(getShell()));
		// wait for first and process all Expose events 
		XIfEvent(getDisplay(),&ev,
			 	isExposeEvent,
				(char *)0);
		XtDispatchEvent(&ev);
		while ( XCheckIfEvent(getDisplay(),&ev,
			 	isExposeEvent,
				(char *)0) )
			XtDispatchEvent(&ev);
		// give input focus to the window
	        XSetInputFocus(getDisplay(),
			XtWindow(getShell()),
			rev, CurrentTime);
	    }
	}

	toMove->prepare();
	toDraw->prepare();
	oldLocation = 
	    theGeometricContextPtr->transPictToWind( *worldLocation );
	newLocation=oldLocation;

	// catch the pointer
	XGrabPointer(getDisplay(),
			theXWindow,
			0,
			PointerMotionMask|ButtonReleaseMask,
			GrabModeAsync,
			GrabModeAsync,
			theXWindow,
 			None,
			CurrentTime
	);		// X11 Reference, p.188
	XWarpPointer(	// pointer on worldLocation
		  getDisplay(), 
		  None,
		  theXWindow,
		  0,0,0,0,
		  oldLocation.x,
		  oldLocation.y
	);  		// X11 Reference, p.201
	XDefineCursor(
		getDisplay(),
		XtWindow(getDrawable()),
		XCreateFontCursor(getDisplay(),XC_fleur)
	);		// X11 Reference, p.180

#ifndef DRAW_IN_BACKGROUND
	theGAPtr->setDrawingFunction(
		VdGraphicAttributes::VdDrawingFunctionXor);
	int cc=0;
#endif

	GC gc = theGAPtr->getGC();

	do {					// query loop
		XQueryPointer( 
			getDisplay(), 
			theXWindow,
			&dummy,
			&dummy,
			&intDummy,
			&intDummy,
			&newLocation.x,
			&newLocation.y,
			&mask
		); 		// X11 Reference, p.83
		if (mask) buttonReleasable=1;
		if ( oldLocation == newLocation ) continue;
		diff =  theGeometricContextPtr->transWindToPict(newLocation)
                      - theGeometricContextPtr->transWindToPict(oldLocation);

#ifndef DRAW_IN_BACKGROUND
		if (cc) // clearing the old picture
		   toDraw->draw( 
			theGeometricContextPtr, 
			*theGAPtr
		   );
		cc++;
#endif
                *worldLocation+=diff;
		toMove->shift( diff );
		toMove->prepare();
		toDraw->prepare();
#ifdef DRAW_IN_BACKGROUND
#ifdef TEST_JH
	cerr << "copyArea (move)------------------------\n";
#endif
	   	XCopyArea(
		    theShellPtr->getDisplay(),
		    savedDrawable,
		    XtWindow(theDrawingArea),
		    gc,
		    0,0,
		    theDrawableSize.x,
		    theDrawableSize.y,
		    0,0
	   	);
#ifdef TEST_JH
	cerr << "ready copying area (move) \n";
#endif
#endif
		toDraw->draw( 
			theGeometricContextPtr, 
			*theGAPtr
		);
		oldLocation = newLocation;
	} while (!buttonReleasable || mask);


	// all hanging input event are to be discarded
	// but not expose/resize events
	while ( XCheckIfEvent(getDisplay(),&ev,
			 isInputEvent,
			(char *)0) );

	XUndefineCursor(
		getDisplay(),
		XtWindow(getDrawable())
	);
	XUngrabPointer( getDisplay(), CurrentTime );
	

#ifdef DRAW_IN_BACKGROUND
	theGAPtr->setDrawable( savedDrawable );
#else
	theGAPtr->setDrawingFunction(
		VdGraphicAttributes::VdDrawingFunctionCopy);
#endif
	redrawPicture(); 
	
}

Display * VdAbstractWindow::getDisplay( void ) 
{ 
	return theShellPtr->getDisplay(); 
}

GC VdAbstractWindow::getGC( void ) 
{
	return theShellPtr->getGC(); 
}

Colormap VdAbstractWindow::getColormap( void ) 
{ 
	return theShellPtr->getColormap(); 
}

Widget VdAbstractWindow::getWidget( void) 
{ 
	return theMainWindow; 
}

Widget VdAbstractWindow::getShell( void) 
{ 
	return theShellWidget; 
}

Widget VdAbstractWindow::getMenubar( void ) 
{
	return theMenubar; 
}

Widget VdAbstractWindow::getDrawable( void) 
{ 
	return theDrawingArea; 
}

void VdAbstractWindow::setGeometricContext( 
		VdGeometricContext& con ) 
{
	*theGeometricContextPtr = con;
	redrawPicture();
}

void VdAbstractWindow::setTitle( const VdString & title )
{
	theShellPtr->setTitle( title.charptr() );
}

void VdAbstractWindow::setDrawableSize( const VdVector& size)
{
	unsigned int w,h,x,y;
	VdVector diff = size - theDrawableSize;
	if (theGeometricContextPtr) {
		VdArea windowArea = 
		   theGeometricContextPtr->getWindowArea();
		windowArea.lr.x += diff.x;
		windowArea.lr.y += diff.y;
		theGeometricContextPtr->
			setWindowArea(windowArea);
	}
	theShellPtr->getGeometry(&w,&h,&x,&y);
	theShellPtr->setGeometry(
		w+diff.x, h+diff.y, x-diff.x/2, y-diff.y/2);
// results an resize event -> updates theDrawableSize
}

void VdAbstractWindow::redrawPicture( int toClear )
{
	if (!XtWindow(theDrawingArea)) return;
		// DrawingArea not yet realized

#ifndef DRAW_IN_BACKGROUND
	if (!theGAPtr) {
	   theGAPtr = new VdGraphicAttributes(
		theShellPtr->getDisplay(),
		XtWindow(theDrawingArea));
	}
	if ( toClear ) clear();
	if (thePicturePtr) {
	   thePicturePtr->prepare();
	   thePicturePtr->draw(
		theGeometricContextPtr,
		*theGAPtr
	   );
	}
#else
// Variante mit Zeichnen im Hintergrund: ------------------
	if (!theGAPtr) {
	   Pixmap pixmap;
#ifdef TEST_JH
	cerr << "create Pixmap (redraw)------------------------\n";
#endif
	   pixmap = XCreatePixmap(
		theShellPtr->getDisplay(),
		XtWindow(theDrawingArea),
		theDrawableSize.x,
		theDrawableSize.y,
		8
	   );
#ifdef TEST_JH
	cerr << "ready createing pixmap (redraw)\n";
#endif
	   theGAPtr = new VdGraphicAttributes(
		theShellPtr->getDisplay(),
		pixmap
	   );
	   XSetGraphicsExposures(
		theShellPtr->getDisplay(),
		theGAPtr->getGC(),
		False
	   );	// in order to generate no expsoe events
		// executing XCopyArea
	}
	
	if ( toClear ) clear();
#ifdef TEST_JH
	cerr << "ready clearing (redraw)\n";
#endif
	if (thePicturePtr) {
	   thePicturePtr->prepare();
	   thePicturePtr->draw(
		theGeometricContextPtr,
		*theGAPtr
	   );
	}
	if (thePicturePtr || toClear) {
#ifdef TEST_JH
	cerr << "copyArea (redraw)----------------------------\n";
#endif
	   XCopyArea(
		theShellPtr->getDisplay(),
		theGAPtr->getDrawable(),
		XtWindow(theDrawingArea),
		theGAPtr->getGC(),
		0,0,
		theDrawableSize.x,
		theDrawableSize.y,
		0,0
	   );
#ifdef TEST_JH
	cerr << "ready copyArea (redraw)\n";
#endif
	}
#endif
}

VdPoint VdAbstractWindow::getDrawableSize( void ) 
{
	return theDrawableSize; 
}

VdPicture *VdAbstractWindow::getPicture( void ) 
{
	return thePicturePtr; 
}

VdWindowType VdAbstractWindow::getWindowType( void ) 
{
	return windowType; 
}

VdGeometricContext & 
	VdAbstractWindow::getGeometricContext( void ) 
{
	return *theGeometricContextPtr; 
}

VdGraphicAttributes * 
	VdAbstractWindow::getGraphicAttributesPtr( void )
{
	return theGAPtr; 
}

VdArea VdAbstractWindow::putAreaInWindow(VdArea & area)
{
	VdVector window = theDrawableSize;
	VdArea adaptedArea = area;
	VdVector areaSize(area.lr.x-area.ul.x, 
			  area.ul.y-area.lr.y);
	int temp;
	if (window.x*areaSize.y > areaSize.x*window.y) {
		// adapting width
		temp =areaSize.x;
		areaSize.x = window.x*areaSize.y/window.y;
		temp = (areaSize.x-temp)/2;
		adaptedArea.ul.x -= temp;
		adaptedArea.lr.x += temp;
	} else {
		// adapting height
		temp =areaSize.y;
		areaSize.y = window.y*areaSize.x/window.x;
		temp = (areaSize.y-temp)/2;
		adaptedArea.ul.y += temp;
		adaptedArea.lr.y -= temp;
	}
	return adaptedArea;
}

static int *colorTable = NULL;

void VdAbstractWindow::getImage( VdPixMap & aPixmap )
{
	XImage *theImage; int i,j,k; unsigned long pixel;
	// constructing the table of colors 
	// colorTable[VISAMADcode]==pixelValue
	if (!colorTable) {
	    colorTable= new int[VdColor::MaxColors];
	    for (i=0;i<VdColor::MaxColors;i++) { 
		VdColor c(i); 
		colorTable[i]=c.Code(); 
	    }
	}
	theImage = XGetImage(getDisplay(),
			     XtWindow(getDrawable()),
			     0,0,
			     theDrawableSize.x,
			     theDrawableSize.y,
			     255l/*planemask*/,ZPixmap);
	aPixmap.setSize(theImage->width,
			theImage->height);
	for (i=0;i<theImage->width;i++)
	    for (j=0;j<theImage->height;j++) {
		pixel=XGetPixel(theImage,i,j);
		// searching the pixel value
		for (k=0;k<VdColor::MaxColors;k++) 
			if (colorTable[k]==pixel) break;
		*( (char *)aPixmap.getArea()
		   +j*theImage->width+i) = k;
	    }
}

// The first two methods have to transform the location

void VdAbstractWindow::callbackButtonPressed( VdEvent *ev )
{
	lastEvent = *ev;
	if (theGeometricContextPtr)
	   lastEvent.location = theGeometricContextPtr->
			transWindToPict(lastEvent.location);
	handleButtonPressed(&lastEvent);
}

void VdAbstractWindow::callbackButtonReleased( VdEvent *ev )
{
	lastEvent = *ev;
	if (theGeometricContextPtr)
	   lastEvent.location = theGeometricContextPtr->
			transWindToPict(lastEvent.location);
	handleButtonReleased(&lastEvent);
}

void VdAbstractWindow::handleExposed( void )
{
	redrawPicture();
}

void VdAbstractWindow::handleResized( void )
{
	VdVector oldSize = theDrawableSize, diff;

	updateDrawableValues(); 
// Vorerst werden alle Resize-Events als Maniulationen der
// rechten unteren Ecke interpretiert!
// Weiterhin mte auch WorldArea angepat werden, falls 
// displayedArea das worldArea berschreitet, und zwar 
// vorlufig auch nach rechts unten

	if (theGeometricContextPtr) {
	  VdArea newWindow;
	  if (thePicturePtr) {
		diff = theDrawableSize -  oldSize ; 
		diff.y = -diff.y;
	   	newWindow = 
		   theGeometricContextPtr->getWindowArea();
	   	newWindow.lr += diff;
	  	theGeometricContextPtr->changeWindowArea(
				newWindow);
	   	theScrollbarValues.displayed = 
		  theGeometricContextPtr->getDisplayedArea();
		if (! (theScrollbarValues.displayed 
			< theScrollbarValues.world)) {
		   if (	theScrollbarValues.displayed.ul.x 
		        < theScrollbarValues.world.ul.x) 
		      theScrollbarValues.world.ul.x
			=theScrollbarValues.displayed.ul.x;
		   if (theScrollbarValues.displayed.ul.y > 
		        theScrollbarValues.world.ul.y) 
		      theScrollbarValues.world.ul.y
			=theScrollbarValues.displayed.ul.y;
		   if (theScrollbarValues.displayed.lr.x > 
		        theScrollbarValues.world.lr.x) 
		      theScrollbarValues.world.lr.x
			=theScrollbarValues.displayed.lr.x;
		   if (theScrollbarValues.displayed.lr.y < 
		        theScrollbarValues.world.lr.y) 
		      theScrollbarValues.world.lr.y
			=theScrollbarValues.displayed.lr.y;
	  	   theGeometricContextPtr->setWorldArea(
				theScrollbarValues.world);
		}
	  } else {
		newWindow = VdArea(VdPoint(0,0),
		       VdPoint(theDrawableSize.x,
			       theDrawableSize.y));
		theGeometricContextPtr->setWorldArea(
				newWindow);
		theGeometricContextPtr->setDisplayedArea(
				newWindow);
		theGeometricContextPtr->setWindowArea(
				newWindow);
		theScrollbarValues.world = 
	   	  theScrollbarValues.displayed =
		     theGeometricContextPtr->getWindowArea();
	   }
	} else {
	   theScrollbarValues.world = 
	      theScrollbarValues.displayed = 
		VdArea(VdNullPoint,
		       VdPoint(theDrawableSize.x,
			       theDrawableSize.y));
	}
	updateScrollbars();

#ifdef DRAW_IN_BACKGROUND
	if (theGAPtr) {
	   Pixmap pixmap=(Pixmap)theGAPtr->getDrawable();
#ifdef TEST_JH
	cerr << "Free Pixmap (expose)\n";
#endif
	   XFreePixmap(theShellPtr->getDisplay(),pixmap);
#ifdef TEST_JH
	cerr << "Create Pixmap (expose)\n";
#endif
	   pixmap = XCreatePixmap(
		theShellPtr->getDisplay(),
		XtWindow(theDrawingArea),
		theDrawableSize.x,
		theDrawableSize.y,
		8
	   );
#ifdef TEST_JH
	cerr << "Ende handleExpose\n";
#endif
	   theGAPtr->setDrawable((Drawable)pixmap);
	}
#endif

//      redrawPicture in Vd_DEFAULT_WINDOW not neccessary.
//	A resize event which enlarge the window causes
//      an expose event.
	if (windowType == Vd_BUBBLE_GUM) redrawPicture();
}

void VdAbstractWindow::handleKeyPressed( VdEvent * )
{
}

void VdAbstractWindow::handleKeyReleased( VdEvent * )
{
	// normally no action. Bind action to the
	// KeyPressed action
}

void VdAbstractWindow::handleButtonPressed( VdEvent * )
{
}

void VdAbstractWindow::handleButtonReleased( VdEvent * )
{
}

void VdAbstractWindow::handleHoriValueChanged ( void )
{
	Arg al[5];
	Cardinal ac;
	VdPoint newUL(theScrollbarValues.displayed.ul);

	int value=0,size=0;
	ac = 0;
	XtSetArg(al[ac],XmNvalue, &value); ac++;
	XtSetArg(al[ac],XmNsliderSize, &size); ac++;
	XtGetValues(theHoriScrollbar,al,ac);

	newUL.x = value *
		( theScrollbarValues.world.lr.x 
		    - theScrollbarValues.world.ul.x ) / 100
		+ theScrollbarValues.world.ul.x;
	theScrollbarValues.displayed +=
		(newUL - theScrollbarValues.displayed.ul);
	if (size+value==100)
		theScrollbarValues.displayed.lr.x
		   =theScrollbarValues.world.lr.x;
	setDisplayOfWorld(theScrollbarValues.displayed,
		theScrollbarValues.world);
		redrawPicture();
}

void VdAbstractWindow::handleVertValueChanged ( void )
{
	Arg al[5];
	Cardinal ac;
	VdPoint newUL(theScrollbarValues.displayed.ul);

	int value=0,size=0;
	ac = 0;
	XtSetArg(al[ac],XmNvalue, &value); ac++;
	XtSetArg(al[ac],XmNsliderSize, &size); ac++;
	XtGetValues(theVertScrollbar,al,ac);

	newUL.y = theScrollbarValues.world.ul.y
		- value *
		  ( theScrollbarValues.world.ul.y 
		    - theScrollbarValues.world.lr.y ) / 100;
	theScrollbarValues.displayed +=
		(newUL - theScrollbarValues.displayed.ul);
	if (size+value==100)
		theScrollbarValues.displayed.lr.y
		   =theScrollbarValues.world.lr.y;
	setDisplayOfWorld(theScrollbarValues.displayed,
		theScrollbarValues.world);
	redrawPicture();
}


void VdAbstractWindow::installExitButton(VdExitMode theMode) 
{
	switch(theMode){
	    case Vd_Exit:
		exitHandler->idf = VdString("Exit");
		break;
	    case Vd_Close:
		exitHandler->idf = VdString("Exit");
		break;
	    case Vd_Default:
		if (theShellPtr->isMainShell())
		    exitHandler->idf = VdString("Exit");
		else
		    exitHandler->idf = VdString("Close");
		break;
	}

	if(exitButtonPtr)  
		exitButtonPtr->update();
	else {
		exitButtonPtr= new VdButton(
				*this,
				exitHandler->idf.charptr(),
				exitHandler
				);
		exitButtonPtr->map();
	}
}

void VdAbstractWindow::setExitChecker(
	int(*theChecker)(const VdFeature &)) 
{
	if(!exitHandler) return ;
	((ExitHandling*)exitHandler)->checkIt=theChecker ;
}

void VdAbstractWindow::handleHelp( void )
{
	VdShell::help(VdFeature("Visamad:GraphicWindow"));
}

void VdAbstractWindow::handleDestroyed( void )
{
}

#undef _VDABSTRACTWINDOW_CXX
