#ifndef _VDABSTRACTWINDOW_HXX
#define _VDABSTRACTWINDOW_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdAbstractWindow.hxx                              *
*  Date: 02-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Declaration of class VdAbstractWindow:               *
*        An abstract window is a XmMainWindow with         *
*        MenuBar, ScrollBars and an DrawingArea for        *
*        graphics. It shall encapsulated into an           *
*        application shell or an dialogshell.              *
*        It is the base class for Magnifier and EditWindow *
*                                                          *
*     Interface                                            *
*         Vd/VdCBmanager.hxx, Vd/VdShell.hxx,              *
*         Vd/VdGeometricContext.hxx, Vd/VdPicture.hxx,     *
*         Vd/VdArea.hxx, Vd/VdWindowConnection.hxx         *
*                                                          *
***********************************************************/
/******************  defined classes  *********************/

class VdAbstractWindow;
class VdWindowConnection;

/******************    dependences    *********************/

#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Vd/VdCBmanager.hxx>
#include <Vd/VdShell.hxx>
#include <Vd/VdGeometricContext.hxx>
#include <Vd/VdPicture.hxx>
#include <Vd/VdArea.hxx>
#include <Vd/VdFeature.hxx>
#include <Vd/VdPixMap.hxx>
#include <Vd/VdEvent.hxx>
#include<Vd/VdButton.hxx>
#include <Vd/VdButtonInterface.hxx>

enum VdExitMode{Vd_Default,Vd_Exit,Vd_Close} ;

class VdAbstractWindow: virtual public VdCBmanager {
   friend class VdShell;
   private:
	Widget theShellWidget,
	       theMainWindow,
	       theMenubar,
	       theHoriScrollbar,
	       theVertScrollbar,
	       theDrawingArea,
	       theHelpButton;
	Pixel theBackgroundPixel;
	VdButtonInterface *exitHandler ;
	VdWindowType windowType;
	VdFeature theIdentification;
	Boolean hasScrollbars;
	VdGraphicAttributes *theGAPtr;
	VdGeometricContext *theGeometricContextPtr;

	VdVector theDrawableSize;
	struct { VdArea world, displayed;
	       } theScrollbarValues;
	VdPicture *thePicturePtr;
	VdEvent lastEvent;

	static void helpCB(Widget widget, 
		XtPointer clientData, XtPointer callData );
	void clear( void ); // clear the drawable
	void updateDrawableValues( void );
	VdAbstractWindow( VdAbstractWindow& aw );
		// not accessible, use == compile error!

   protected:
	void init(VdShell &shell, Boolean withScrollbars);
	void setIdent( const VdFeature& ident);
	VdShell *theShellPtr;
	VdButton *exitButtonPtr ;

   public:
	VdAbstractWindow( VdShell &shell, 
		Boolean withScrollbars = True) {
	   init(shell, withScrollbars); }
	~VdAbstractWindow( void );

	VdFeature getIdent( void );
	int checkClose( void );
	void map( void );
	void unmap( void );
	void destroy( void);
	void setDisplayOfWorld(VdArea displayed,
		VdArea world);
	void getDisplayOfWorld( 
		VdArea& displayedr, VdArea& world );
	void updateScrollbars( void );
	void setWindowType(VdWindowType wt);
	void setPicture( VdPicture *pic );
	void setPicture( VdPicture *pic,  
		VdGeometricContext& con );
	void movePicture( VdPicture * toDraw, VdPicture * toMove,
		VdPoint * initialLocation = NULL);
	Display *getDisplay( void );
	GC getGC( void );
	Colormap getColormap( void );
	Widget getShell( void);
	Widget getWidget( void); // main window
	Widget getMenubar( void );
	Widget getDrawable( void);
	void setGeometricContext( 
			VdGeometricContext& con );
	void setTitle( const VdString & title );
	void setDrawableSize( const VdVector& size);
	virtual void redrawPicture( int toClear=1 );
	VdPoint getDrawableSize( void );
	VdPicture *getPicture( void );
	VdWindowType getWindowType( void );
	VdGeometricContext& 
			getGeometricContext( void );
	VdGraphicAttributes* 
			getGraphicAttributesPtr( void );
	VdArea putAreaInWindow(VdArea & area);
		// constructs an Area of window format which
		// contains the parameter centered
  	void getImage( VdPixMap & aPixmap );
	void installExitButton(VdExitMode theMode) ;
	void setExitChecker(int(*theChecker)(const VdFeature&)); 


protected:		
	virtual void callbackButtonPressed ( VdEvent *ev );
	virtual void callbackButtonReleased ( VdEvent *ev );
	virtual void handleExposed( void );
	virtual void handleResized( void );
	virtual void handleKeyPressed ( VdEvent *ev );
	virtual void handleKeyReleased ( VdEvent *ev );
	virtual void handleButtonPressed ( VdEvent *ev );
	virtual void handleButtonReleased ( VdEvent *ev );
	virtual void handleHoriValueChanged ( void );
	virtual void handleVertValueChanged ( void );
	virtual void handleHelp( void );
	virtual void handleDestroyed( void );
};

#endif   /* _VDABSTRACTWINDOW_HXX */

