#define _VDAREA_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdArea.cxx                                        *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/


#ifdef _HXX_INCLUDE_
#include <limits.hxx> 
#else
#include <limits.h>
#endif

#include <Vd/VdArea.hxx>

#define VdMIN(a,b) (((a)<(b)?(a):(b))) 
#define VdMAX(a,b) (((a)>(b)?(a):(b))) 
 
VdArea::VdArea ( const VdPoint & p1 , const VdPoint & p2 ) 
{  ul.x = VdMIN(p1.x,p2.x);
   lr.x = VdMAX(p1.x,p2.x);
   ul.y = VdMAX(p1.y,p2.y); 
   lr.y = VdMIN(p1.y,p2.y); 
} 
 
 
VdArea::VdArea ( const VdPoint & p, int width, int height) 
{ if (width<0) 
  { ul.x = p.x+width; lr.x = p.x; }
  else
  { lr.x = p.x+width; ul.x = p.x; }
  if (height>0) 
  { ul.y = p.y+height; lr.y = p.y; }
  else
  { lr.y = p.y+height; ul.y = p.y; }
} 
 
VdArea::VdArea ( const void * )
{ ul.x=INT_MAX; ul.y=INT_MIN;
  lr.x=INT_MIN; lr.y=INT_MAX;
}
 
VdArea VdArea::operator| ( const VdArea & a) const 
{ VdPoint p1( VdMIN(ul.x, a.ul.x), VdMAX(ul.y, a.ul.y)), 
          p2( VdMAX(lr.x, a.lr.x), VdMIN(lr.y, a.lr.y)); 
  VdArea h(p1, p2); 
  return h; 
} 
 
VdArea VdArea::operator& ( const VdArea & a) const
{ 
  if(   ((ul.x <= a.lr.x) && (a.ul.x <= lr.x))
     && ((ul.y >= a.lr.y) && (a.ul.y >= lr.y))
    ) 
  { VdPoint p1 ( VdMAX(ul.x, a.ul.x), VdMIN(ul.y, a.ul.y) ), 
            p2 ( VdMIN(lr.x, a.lr.x), VdMAX(lr.y, a.lr.y ) ); 
  return VdArea (p1,p2); 
  } 
  else return VdEmptyArea; 
} 
 
VdArea VdArea::operator+ ( const VdVector & v) const
{ VdArea a; 
  if ((*this == VdEmptyArea) || (*this == VdEntireArea))
  { a.ul = ul; a.lr = lr; }
  else
  { a.ul = ul + v; a.lr = lr + v; }
  return a; 
} 
 
VdArea VdArea::operator- ( const VdVector & v) const
{ VdArea a; 
  if ((*this == VdEmptyArea) || (*this == VdEntireArea))
  { a.ul = ul; a.lr = lr; }
  else
  { a.ul = ul - v; a.lr = lr - v; }
  return a; 
} 
  
VdArea & VdArea::operator|= ( const VdArea & a)
{ ul.x=VdMIN(ul.x, a.ul.x); ul.y=VdMAX(ul.y, a.ul.y); 
  lr.x=VdMAX(lr.x, a.lr.x); lr.y=VdMIN(lr.y, a.lr.y); 
  return *this; 
} 
 
VdArea & VdArea::operator&= ( const VdArea & a)
{ 
  if(   ((ul.x <= a.lr.x) && (a.ul.x <= lr.x))
     && ((ul.y >= a.lr.y) && (a.ul.y >= lr.y))
    ) 
  { ul.x=VdMAX(ul.x, a.ul.x); ul.y=VdMIN(ul.y, a.ul.y);
    lr.x=VdMIN(lr.x, a.lr.x); lr.y=VdMAX(lr.y, a.lr.y); 
  } 
  else *this=VdEmptyArea; 
  return *this;
} 
 
VdArea & VdArea::operator+= ( const VdVector & v)
{ if ((*this != VdEmptyArea) && (*this != VdEntireArea))
  { ul += v; lr += v; }
  return *this; 
} 
 
VdArea & VdArea::operator-= ( const VdVector & v)
{ if ((*this != VdEmptyArea) && (*this != VdEntireArea))
  { ul -= v; lr -= v; }
  return *this; 
} 
 
int VdArea::operator< ( const VdArea & a) const
{ return (   (a.ul.x <= ul.x) && (lr.x <= a.lr.x)
          && (a.ul.y >= ul.y) && (lr.y >= a.lr.y)
         ); 
} 
 
int operator< ( const VdPoint & p, const VdArea & a) 
{ return (   (a.ul.x <= p.x) && (p.x <= a.lr.x)
          && (a.ul.y >= p.y) && (p.y >= a.lr.y)
         ); 
}  

 
int VdArea::operator == ( const VdArea & a) const
{ return (( ul == a.ul) && (lr == a.lr) ); 
} 
  
int VdArea::operator != ( const VdArea & a) const
{ return (( ul != a.ul) || (lr != a.lr) ); 
} 
 
#undef VdMIN
#undef VdMAX

#undef _VDAREA_CXX
