#ifndef _VDAREA_HXX
#define _VDAREA_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdArea.hxx                                        *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdArea;

/******************    dependences    *********************/

#include <Vd/VdPoint.hxx> 
#include <Vd/VdVector.hxx>
 
class VdArea{ 
   public: 
     VdPoint ul,                                // upper left corner 
             lr;                                // lower right corner 
 
 
     VdArea ( const VdPoint & p1 = VdNullPoint, 
              const VdPoint & p2 = VdNullPoint 
            ); 
     VdArea ( const VdArea & a) { ul = a.ul; lr = a.lr;} 
     VdArea ( const VdPoint & ul1, int width, int height); 
     VdArea ( const void * );
 
     VdArea operator| ( const VdArea & a) const;      // Vereinigung 
     VdArea operator& ( const VdArea & a) const;      // Durchschnitt 
     VdArea operator+ ( const VdVector & v) const;    // Verschieben 
     VdArea operator- ( const VdVector & v) const;    // Verschieben 
     VdArea & operator|= ( const VdArea & a);         // Vereinigung 
     VdArea & operator&= ( const VdArea & a);         // Durchschnitt 
     VdArea & operator+= ( const VdVector & v);       // Verschieben 
     VdArea & operator-= ( const VdVector & v);       // Verschieben 
  
     int operator< ( const VdArea & a) const; 
     friend int operator< ( const VdPoint & p, const VdArea & a);
 
     int operator == ( const VdArea & a) const;
     int operator != ( const VdArea & a) const;
}; 

/******************     constants     *********************/ 
 
#ifdef _VDAREA_CXX

   VdArea VdNullArea;
   VdArea VdEmptyArea( (void*)0L );
   VdArea VdEntireArea( VdPoint(INT_MIN,INT_MAX)
                      , VdPoint(INT_MAX,INT_MIN)
                      );
#else
   extern const VdArea VdNullArea;                   // Nullumgebung 
   extern const VdArea VdEmptyArea;                  // leere Umgebung 
   extern const VdArea VdEntireArea;                 // gesamte Ebene 
#endif

#endif /* _VDAREA_HXX */
