#define _VDARROW_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdArrow.C                                         *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdArrow.hxx>

#ifdef _HXX_INCLUDE_
#include <math.hxx>
#include <stdio.hxx>
#else
#include <math.h>
#include <stdio.h>
#endif

VdArrow::VdArrow( const VdPoint & from
              , const VdPoint & to
              ) 
{ startPoint = from;  endPoint = to;
}

VdArrow::VdArrow( const VdPoint  &  from
              , const VdVector &  v
              , int betrag
              ) 
{ startPoint = from; endPoint = from + v.norm(betrag); 
}
 
VdPicture * VdArrow::duplicate(void)
{ VdArrow * pict = new VdArrow;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->startPoint=startPoint;
  pict->endPoint=endPoint;
  return pict;
}

void VdArrow::setStartPoint ( const VdPoint & p )
{ startPoint=p;
}

void VdArrow::setEndPoint ( const VdPoint & p )
{ endPoint=p;
}

VdPoint VdArrow::getStartPoint ( void ) const
{ return startPoint;
}

VdPoint    VdArrow::getEndPoint ( void ) const
{ return endPoint;
}

void VdArrow::prepare ( void )
{ VdArrow::computeFrame();
}

void VdArrow::computeFrame( void )
{ VdArea a(startPoint,endPoint);
  if (autoFrame) frame=a;
}

static double theSuperGagNumber = 6.0/10.0;
// 0.6 kann IBMs xlC nur als 0.0 speichern

void VdArrow::draw( VdGeometricContext * G
                 , VdGraphicAttributes GA 
                 ) 
{  VdVector v = startPoint-endPoint;
   double lv = !v;
   if (lv > 5.0) lv=5.0/lv; else lv=1.0;
   VdPoint p = G->transPictToWind(startPoint+(VdVector)location); 
   VdPoint q = G->transPictToWind(endPoint+(VdVector)location);
   VdVector v1(p-q); 
   VdVector v2(v1.y,-(v1.x));
   v1.x=(int)(lv*v1.x); v1.y=(int)(lv*v1.y);
   lv=theSuperGagNumber*lv;
   v2.x=(int)(lv*v2.x); v2.y=(int)(lv*v2.y);
   VdPoint l = q+v1+v2;
   VdPoint r = q+v1-v2;
   XPoint pnts[5];
   pnts[0].x=p.x;
   pnts[0].y=p.y;
   pnts[1].x=q.x;
   pnts[1].y=q.y;
   pnts[2].x=l.x;
   pnts[2].y=l.y;
   pnts[3].x=r.x;
   pnts[3].y=r.y;
   pnts[4].x=q.x;
   pnts[4].y=q.y;
   GA.setForeground(foreground); 
   GA.setBackground(background); 
   GA.setLineType(lineType); 
 
   XDrawLines( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
               pnts,5,0
             );
} 
 
void VdArrow::printPostScript(FILE* f) 
{  int s=0;
   VdVector v = startPoint-endPoint;
   double lv = !v;
   if (lv > 5.0) lv=5.0/lv; else lv=1.0;
   VdPoint p = startPoint+(VdVector)location; 
   VdPoint q = endPoint+(VdVector)location;
   VdVector v1(p-q); 
   VdVector v2(v1.y,-(v1.x));
   v1.x=(int)(lv*v1.x); v1.y=(int)(lv*v1.y);
   lv=theSuperGagNumber*lv;
   v2.x=(int)(lv*v2.x); v2.y=(int)(lv*v2.y);
   VdPoint l = q+v1+v2;
   VdPoint r = q+v1-v2;
   XPoint pnts[5];
   pnts[0].x=p.x;
   pnts[0].y=p.y;
   pnts[1].x=q.x;
   pnts[1].y=q.y;
   pnts[2].x=l.x;
   pnts[2].y=l.y;
   pnts[3].x=r.x;
   pnts[3].y=r.y;
   pnts[4].x=q.x;
   pnts[4].y=q.y;
   if(  (lineType!=VdLINEUNDEF)
     ||(background!=VdCOLORUNDEF)
    )
  { fprintf(f,"save\n"); s=1; }
  else
  if (foreground!=VdCOLORUNDEF)
  { fprintf(f,"gsave\n"); s=2; }
  if (s)
  { if (lineType!=VdLINEUNDEF) fprintf(f,"%d SLS ",2);
    if (background!=VdCOLORUNDEF)
      fprintf(f,"%d SBC "
             ,(background.code>>4)+((background.code&0xf)<<2)
             );
    if (foreground!=VdCOLORUNDEF)
      fprintf(f,"%d SC "
             ,(foreground.code>>4)+((foreground.code&0xf)<<2)
             );
    fprintf(f,"\n");
  }
  fprintf(f,"%d %d %d %d %d %d %d %d %d %d  AL\n",
          pnts[0].x,pnts[0].y,
          pnts[1].x,pnts[1].y,
          pnts[2].x,pnts[2].y,
          pnts[3].x,pnts[3].y,
          pnts[4].x,pnts[4].y
         );
  switch (s) { case 1 : fprintf(f,"restore\n"); break;
               case 2 : fprintf(f,"grestore\n"); break;
             }
} 

void VdArrow::print( int i)
{ fprintf(stderr,
          " VdArrow"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}


#undef _VDARROW_CXX
