#define _VDBUTTON_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdButton.cxx                                      *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization: experimental version              *
************************************************************
*                                                          *
*                                                          *
*     experimental realization of class VdButton:          *
*                                                          *
*     Methods:	                                           * 
*                                                          *
*                                                          *
*     Interface:                                           *
*                                                          *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdButton.hxx>
#include <Vd/VdAbstractWindow.hxx>

#include <X11/Intrinsic.h>

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>

#include <iostream.h>
#ifdef TEST_JH
#include <stdio.h>
#endif

VdButton::VdButton( VdAbstractWindow & window, 
		String aLabelString,
		VdButtonInterface *handle)
{
	parentWidget = window.getMenubar();
	if (aLabelString) {
	   theLabelString = new char[strlen(aLabelString)+1];
	   strcpy(theLabelString,aLabelString);
	} else {
	   theLabelString = new char[3];
	   strcpy(theLabelString,"[]");
	}
	isCreated = False;
	isSensitive = True;
	theButton = NULL;
	theHandle = handle;
	parentWindow = &window;
#ifdef TEST_JH
	fprintf(stderr,"VdButton::VdButton parentWindow=%s\n",
		parentWindow->getIdent().getString().charptr());
#endif
}

VdButton::~VdButton( void )
{
	delete theLabelString;
	if (isCreated) 
		XtDestroyWidget(theButton);
}

void VdButton::map ( void )
{
	if (!isCreated) {
		isCreated = True;
		theButton = XmCreateCascadeButton(
			parentWidget,
			theLabelString, NULL, 0);
		XtManageChild( theButton );
		addCB(theButton, activated);
		XtSetSensitive(theButton, isSensitive);
		watchDestroying(theButton);
	} else
		XtMapWidget(theButton);
}

void VdButton::unmap ( void )
{
	XtUnmapWidget(theButton);
}

void VdButton::remove( void )
{
	if (isCreated) 
		XtDestroyWidget(theButton);
}

void VdButton::update( void )
{
	char *p;
	if (!theHandle) return;
	p = theHandle->idf.charptr();
	if (0==strcmp(theLabelString,p))
		return; // nothing to do
	setLabel(p);
}

void VdButton::setSensitive( void )
{
	isSensitive = True;
	if (isCreated)
	   XtSetSensitive(theButton, True);
}

void VdButton::setInsensitive( void )
{
	isSensitive = False;
	if (isCreated)
	   XtSetSensitive(theButton, False);
}

void VdButton::setLabel( String newLabel )
{
	Arg al[10];
	Cardinal ac;

	if (newLabel!=theLabelString) {
		if (theLabelString) 
			delete theLabelString;
		theLabelString = 
			new char[strlen(newLabel)];
		strcpy(theLabelString,newLabel);
	}
	if (isCreated) {
		XmString string = XmStringCreate(theLabelString,
			XmSTRING_DEFAULT_CHARSET);
		ac=0;
		XtSetArg(al[ac], XmNlabelString, string); 
		ac++;
		XtSetValues(theButton,al,ac);
		XmStringFree(string);
	}
}

Widget VdButton::getParentWidget( void )
{
	return parentWidget;
}

void VdButton::handleActivated( void )
{
#ifdef TEST_JH
	fprintf(stderr,"VdButton::handleActivated parentWindow=%s\n",
		parentWindow->getIdent().getString().charptr());
#endif
	if (!theHandle) return;
	theHandle->activate(parentWindow->getIdent());
}

void VdButton::handleCascaded( void )
{
// only of interest for derived classes
}

#undef _VDBUTTON_CXX
