#ifndef _VDBUTTON_HXX
#define _VDBUTTON_HXX

/***********************************************************
*                  VISAMAD.defgraph                        *
*             ==========================                   *
*  File: VdButton.hxx                                      *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization: experimental version              *
************************************************************
*                                                          *
*                                                          *
*     experimental declaration of class VdButton:          *
*       VdButton is the base class for all user defined    *
*       Buttons in a Menubar                               *
*                                                          *
*     Methods:	                                           *
*     - handleActivated: should be rewritten to            *
*       implement own behavior of button activation        *
*     - remove: detroys the button and its occurence in    *
*       parent widget (menubar)                            *
*     - setSensitive/setInsensitive: controls whether the  *
*       button can be activated or not                     *
*     - map/unmap                                          *
*                                                          *
*     Interface                                            *
*          VdCBmanager.hxx, X11/Intrinsic.h                *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdButton;

/******************    dependences    *********************/

#include <X11/Intrinsic.h>
#include <Vd/VdCBmanager.hxx>
#include <Vd/VdDestruction.hxx>
#include <Vd/VdButtonInterface.hxx>
#include <Vd/VdAbstractWindow.hxx>

class VdButton: virtual public VdCBmanager,
		public VdDestruction {
   protected:
	VdAbstractWindow * parentWindow;
	Widget theButton;
	Widget parentWidget;
	VdButtonInterface *theHandle;
	String theLabelString;
	Boolean isCreated, isSensitive;
#ifdef __ghs
   public:
	// Green Hills C++ translator can not access
	// a protected destructor i derived classes
	// here VdMagnifierButton.
#else
   protected:
#endif
	~VdButton( void );

   public:
	VdButton( VdAbstractWindow & window,
		String aLabelString,
		VdButtonInterface * handle= NULL);

	virtual void map( void );
	virtual void unmap( void );
	virtual void remove( void ); // explicit deleting
	virtual void update( void );
	void setLabel( String newLabel );
	void setSensitive( void );
	void setInsensitive( void );
	Widget getParentWidget( void );
	virtual void handleActivated( void );
	virtual void handleCascaded( void );
};


#endif  /* _VDBUTTON_HXX */

