#define _VDCBMANAGER_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdCBmanager.cxx     (alias VtCBmanager.cxx)       *
*          The alias is neccessary becaus of the alpha-    *
*          betic order of moduls initializing global       *
*          objects.                                        *
*  Date: 01-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization: experimental version              *
************************************************************
*                                                          *
*                                                          *
*     experimental realization of class VdCBmanager:       *
*                                                          *
*     Methods:	                                           *
*                                                          *
*                                                          *
*     Interface                                            *
*                                                          *
*                                                          *
*     References:                                          *
*        [XMan] Adrian Nye:                                *
*        Xlib programming manual for version 11, release 4 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#define __VDCBMANAGER_CXX
#include <Vd/VdCBmanager.hxx>

#include <X11/keysym.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Vd/VdPoint.hxx>
#include <Vd/VdEventHandling.hxx>

static String resourceStrings[] = {
		"none",
		XmNactivateCallback,
		XmNentryCallback,
		XmNexposeCallback,
		XmNresizeCallback,
		XmNinputCallback,
		XmNvalueChangedCallback,
		XmNcascadingCallback,
		XmNmapCallback,
		XmNunmapCallback,
		XmNarmCallback,
		XmNdisarmCallback
	};

#include <iostream.h>

// All InputEvent handling methods get the Address of
// this structure to evaluate the content.
// Membee 'location' is in Pixel coordinates and has to be
// translated in to world coordinates on the window level
// using the current geometric context!

static VdEvent lastEvent; 
XmAnyCallbackStruct * VdCBmanager::lastStruct;
Widget VdCBmanager::lastWidget;

// The following function gets the character after a 
// KeyPressed or KeyReleased event considering the state
// of the modifier keys

static int extractChar(XKeyEvent *event)
{
	unsigned char ch; 
	int nr;
	static char buffer[10];
	KeySym keySym=0;

	// The following see [XMan, pp.273]
	nr = XLookupString(
		event,
		buffer,
		10,
		&keySym,
		(XComposeStatus *)NULL
	);   // [XMan, pp.273]
	ch = buffer[nr-1]; 

	// location will be undefined

	if (((keySym >= XK_Shift_L)
	     && (keySym <= XK_Hyper_R))
	     || (keySym == XK_Mode_switch)) {
		return 0;     // only modifier key, no VdEvent
	}

        lastEvent.XKeyNum=event->keycode;
        lastEvent.XKeyState=event->state;
	
	if ((keySym >= XK_Select)
	     && (keySym <= XK_Help)) { 
		lastEvent.keyCode = keySym - XK_Select + 1;
		lastEvent.statusCode = Vd_SPECIAL_KEY; 
	} else if ((keySym >= XK_F1)
	     && (keySym <= XK_F35)) {
		lastEvent.keyCode = keySym - XK_F1 + 1;
		lastEvent.statusCode = Vd_FUNCTION_KEY;
	} else if ((keySym >= XK_Home)
	     && (keySym <= XK_Begin)) {
		lastEvent.keyCode = keySym - XK_Home + 1;
		lastEvent.statusCode = Vd_CURSOR_KEY;
	} else if ((keySym >= XK_KP_Space)
	     && (keySym <= XK_KP_9)) {
		lastEvent.keyCode = keySym - XK_KP_Space + 1;
		lastEvent.statusCode = Vd_KEYPAD_KEY;
	} else if (ch) {
		lastEvent.keyCode = ch;  // [XMan, pp.237]
		lastEvent.statusCode = 0; 
	}

	if (event->state & (ShiftMask|LockMask))
			lastEvent.statusCode += Vd_SHIFT_STATE;
	if (event->state & ControlMask)
			lastEvent.statusCode += Vd_CONTROL_STATE;
	if (event->state & Mod2Mask) // Tasten "Zusatzfunktion"
			lastEvent.statusCode += Vd_ALT_STATE;
	if (event->state & Mod1Mask) // Tasten "Gruppenumsch"
			lastEvent.statusCode += Vd_ALT_GR_STATE;
	lastEvent.type=VdFeature("");
	if (lastEvent.statusCode&Vd_ALL_MODIFIER_KEYS)
	{	lastEvent.type+=Vd_KEYSTATE;
		if (event->state & (ShiftMask|LockMask))
			lastEvent.type += Vd_SHIFT_KEYSTATE;
		if (event->state & ControlMask)
			lastEvent.type += Vd_CONTROL_KEYSTATE;
		if (event->state & Mod2Mask) 	
			lastEvent.type += Vd_ALT_KEYSTATE;
		if (event->state & Mod1Mask) 	
			lastEvent.type += Vd_ALT_GR_KEYSTATE;
	}

	return 1; 
}

// The following function gets the pointer position after a 
// ButtonPressed or ButtonReleased event 

static  void extractCoordinates(XButtonEvent *event,
			 int pressed)
{ 
	if (event->button == Button1) {
		lastEvent.type = (pressed ? 
				Vd_LEFT_MOUSE_BUTTON_PRESSED
			      : Vd_LEFT_MOUSE_BUTTON_RELEASED); 
	} else if (event->button == Button2) {
		lastEvent.type = (pressed ? 
				Vd_MIDDLE_MOUSE_BUTTON_PRESSED 
			      : Vd_MIDDLE_MOUSE_BUTTON_RELEASED);
	} else if (event->button == Button3) {
		lastEvent.type = (pressed ? 
				Vd_RIGHT_MOUSE_BUTTON_PRESSED 
			      : Vd_RIGHT_MOUSE_BUTTON_RELEASED);
	} else if (event->button == Button4) {
		lastEvent.type = (pressed ? 
				Vd_LEFT_MIDDLE_MOUSE_BUTTON_PRESSED 
			      : Vd_LEFT_MIDDLE_MOUSE_BUTTON_RELEASED);
	} else if (event->button == Button5) {
		lastEvent.type = (pressed ? 
				Vd_RIGHT_MIDDLE_MOUSE_BUTTON_PRESSED 
			      : Vd_RIGHT_MIDDLE_MOUSE_BUTTON_RELEASED);
	} 

	lastEvent.location = VdPoint(event->x,event->y);
	lastEvent.statusCode = 0; 
	if (event->state & (ShiftMask|LockMask))
		lastEvent.statusCode += Vd_SHIFT_STATE;
	if (event->state & ControlMask)
		lastEvent.statusCode += Vd_CONTROL_STATE;
	if (event->state & Mod2Mask) // Tasten "Zusatzfunktion"
		lastEvent.statusCode += Vd_ALT_STATE;
	if (event->state & Mod1Mask) // Tasten "Gruppenumsch"
		lastEvent.statusCode += Vd_ALT_GR_STATE;

	if (lastEvent.statusCode)
	{	lastEvent.type+=Vd_KEYSTATE;
		if (event->state & (ShiftMask|LockMask))
			lastEvent.type += Vd_SHIFT_KEYSTATE;
		if (event->state & ControlMask)
			lastEvent.type += Vd_CONTROL_KEYSTATE;
		if (event->state & Mod2Mask) 	
			lastEvent.type += Vd_ALT_KEYSTATE;
		if (event->state & Mod1Mask) 	
			lastEvent.type += Vd_ALT_GR_KEYSTATE;
	}

	// keyCode will be undefined
}

static Display * predicateDisplay;
static int predicateType;
Window predicateWindow;
static Bool predicate(Display * d, XEvent * ev, char * arg )
{
	return (
		d == predicateDisplay
		&& ev->type == predicateType
		&& ev->xexpose.window 
		   == predicateWindow
	);
}

void VdCBmanager::motifCB(Widget widget, 
		XtPointer clientData, XtPointer callData )
{
	VdCBmanager *mp = (VdCBmanager *)clientData;
	lastStruct = (XmAnyCallbackStruct *)callData;
	int reason = lastStruct->reason;
	lastWidget = widget;

	Arg al[5];
	unsigned int ac;
	unsigned char orientation;

	switch (reason) {
	case XmCR_ACTIVATE: 
		mp->handleActivated();
		break;
	case XmCR_EXPOSE:    
		if (lastStruct->event->xexpose.count) break;
		// while count>0 further expose events for regions
		// of the window follow
		mp->handleExposed();
		// the expose events for the regions occupied
		// by the old scrollbars can be discarded.
		// The code assumes that at first the drawing
		// area will resized and after this the scrollbars
		// will placed in the main window. The code will
		// fail if at first the scrollbars will be placed and
		// after this for the drawing area only one expose
		// event will be generated!
		predicateDisplay = XtDisplay(widget);
		predicateType = lastStruct->event->type;
		predicateWindow = lastStruct->event->xexpose.window;
		if ( ! XCheckIfEvent( 
				XtDisplay(widget), 
				lastStruct->event, 
				predicate, NULL) ) {
			break;
		}
		XCheckIfEvent( 
			XtDisplay(widget), 
			lastStruct->event, 
			predicate, NULL);
		break;
	case XmCR_RESIZE:
		mp->handleResized();
		break;
	case XmCR_INPUT: 
		{   // block for scoping evPtr
		XEvent *evPtr=((XmDrawingAreaCallbackStruct *)
			      callData)->event; 
		switch (
		// setzt voraus, da es an ein DrawingArea
		// gerichtetes  Ereignis war
		  ((XmDrawingAreaCallbackStruct *)callData)
				->event->type) {
		case KeyPress:
		   if (extractChar((XKeyEvent *)evPtr)) {
		      lastEvent.type = Vd_KEY_RELEASED+lastEvent.type;
		      mp->handleKeyPressed(&lastEvent);
		   }
		   break;
		case KeyRelease:
		   if (extractChar((XKeyEvent *)evPtr)) {
		      lastEvent.type = Vd_KEY_PRESSED+lastEvent.type;
		      mp->handleKeyReleased(&lastEvent);
		   }
		   break;
		case ButtonPress: 
		   extractCoordinates(
			(XButtonEvent *)evPtr, 1);
		   mp->callbackButtonPressed(&lastEvent);
		   break;
		case ButtonRelease:
		   extractCoordinates( 
			(XButtonEvent *)evPtr, 0);
		   mp->callbackButtonReleased(&lastEvent);
		   break;
		};
		}; // end block
		break; 
	case XmCR_VALUE_CHANGED:
		// setzt voraus, da durch XtGetValues nur
		// ein Wert geliefert wird, wenn Resource fr
		// Widget existiert
		ac = 0;   orientation=XmHORIZONTAL & XmVERTICAL;
				// tricky
	   	XtSetArg(al[ac], XmNorientation, &orientation);
		ac++;
		XtGetValues(widget, al, ac);
		switch (orientation) {
		case XmHORIZONTAL:
			mp->handleHoriValueChanged();
			break;
		case XmVERTICAL:
			mp->handleVertValueChanged();
			break;
		default:
			mp->handleValueChanged();
		}
		break;
	case XmCR_CASCADING:
		mp->handleCascaded();
		break;
	case XmCR_MAP:
		mp->handleMapped();
		break;
	case XmCR_UNMAP:
		mp->handleUnmapped();
		break;
	case XmCR_ARM:
		mp->handleArmed();
		break;
	case XmCR_DISARM:
		mp->handleDisarmed();
		break;
	default:  ;    
	}
}

XmAnyCallbackStruct * 
	VdCBmanager::getLastCallbackData( void )
{
	return lastStruct;
}

Widget VdCBmanager::getLastWidgetOfEvent( void )
{
	return lastWidget;
}

void VdCBmanager::addCB(Widget widget, handlerType ht)
{
	XtAddCallback(widget, 
		resourceStrings[ht], 
		(XtCallbackProc)&motifCB, 
		(XtPointer)this );
}

void VdCBmanager::handleActivated( void ) {}
void VdCBmanager::handleExposed( void ) {}
void VdCBmanager::handleResized( void ) {}
void VdCBmanager::handleKeyPressed( VdEvent *ev ) {}
void VdCBmanager::handleKeyReleased( VdEvent *ev ) {}
void VdCBmanager::callbackButtonPressed ( VdEvent *ev ) {}
void VdCBmanager::callbackButtonReleased ( VdEvent *ev ) {}
void VdCBmanager::handleValueChanged( void ) {}
void VdCBmanager::handleHoriValueChanged( void ) {}
void VdCBmanager::handleVertValueChanged( void ) {}
void VdCBmanager::handleCascaded( void ) {}
void VdCBmanager::handleMapped( void ) {}
void VdCBmanager::handleUnmapped( void ) {}
void VdCBmanager::handleArmed( void ) {}
void VdCBmanager::handleDisarmed( void ) {}
#undef _VDCBMANAGER_CXX
