#ifndef _VDCBMANAGER_HXX
#define _VDCBMANAGER_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdCBmanager.hxx                                   *
*  Date: 01-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization: experimental version              *
************************************************************
*                                                          *
*  Achtung: Version ist unvollstndig und es mu mit       *
*           nderungen der Klassendefinition gerechnet     *
*           werden. Vorerst sind nur die Ereignisse        *
*           expose, resize und activate sinnvoll behandel- *
*           bar.                                           *
*                                                          *
*     experimental declaration of class VdCBmanager:       *
*        CBManager should be a virtual base class of all   *
*        classes, in which handling of any events are      *
*        neccessary                                        *
*                                                          *
*     Methods:	                                           *
*                                                          *
*     Interface                                            *
*          X11/Intrinsic.h                                 *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdCBmanager;

/******************    dependences    *********************/

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>

#include <Vd/VdFeature.hxx>
#include <Vd/VdEventHandling.hxx>
 
extern const VdFeature
 	Vd_LEFT_MOUSE_BUTTON_PRESSED;
extern const VdFeature 
	Vd_RIGHT_MOUSE_BUTTON_PRESSED;
extern const VdFeature 
	Vd_MIDDLE_MOUSE_BUTTON_PRESSED;
extern const VdFeature 
	Vd_RIGHT_MIDDLE_MOUSE_BUTTON_PRESSED;
extern const VdFeature 
	Vd_LEFT_MIDDLE_MOUSE_BUTTON_PRESSED;
extern const VdFeature 
	Vd_LEFT_MOUSE_BUTTON_RELEASED;
extern const VdFeature 
	Vd_RIGHT_MOUSE_BUTTON_RELEASED;
extern const VdFeature 
	Vd_MIDDLE_MOUSE_BUTTON_RELEASED;
extern const VdFeature 
	Vd_RIGHT_MIDDLE_MOUSE_BUTTON_RELEASED;
extern const VdFeature 
	Vd_LEFT_MIDDLE_MOUSE_BUTTON_RELEASED;
extern const VdFeature Vd_KEY_PRESSED;
extern const VdFeature Vd_KEY_RELEASED;
extern const VdFeature Vd_KEYSTATE;
extern const VdFeature Vd_SHIFT_KEYSTATE;
extern const VdFeature Vd_CONTROL_KEYSTATE;
extern const VdFeature Vd_ALT_KEYSTATE;
extern const VdFeature Vd_ALT_GR_KEYSTATE;

class VdCBmanager {

protected:
	enum handlerType { 
	none, 
	              // adaquate OSF/Motif notions:
	              // Resource             Reason
	activated,    // XmNactivateCallback, XmCR_ACTIVATE
	entry,        // XmNentryCallback,    XmCR_ACTIVATE
	exposed,      // XmNexposeCallback,   XmCR_EXPOSE
	resized,      // XmNresizeCallback,   XmCR_RESIZE
	input,        // XmNinputCallback,    XmCR_INPUT
	valueChanged, // XmNvalueChangedCallback,
	              //                XmCR_VALUE_CHANGED
	cascaded,     // XmNcascadingCallback, XmCR_CASCADING
	mapped,       // XmNmapCallback,      XmCR_MAP
	unmapped,     // XmNunmapCallback,    XmCR_UNMAP
	armed,        // XmNarmCallback,      XmCR_ARM  
	disarmed      // XmNdisarmCallback,   XmCR_DISARM
	};

	static void motifCB(Widget widget, 
		XtPointer clientData, XtPointer callData );
   private:
	static XmAnyCallbackStruct * lastStruct;
	static Widget lastWidget;

   protected:
	XmAnyCallbackStruct * getLastCallbackData( void );
	Widget getLastWidgetOfEvent( void );

   public:
	// default constructor correct (nothing to do)

	void addCB(Widget w, handlerType ht);

   protected:
	virtual void handleActivated( 
			void );
	virtual void handleExposed( 
			void );
	virtual void handleResized( 
			void );
	virtual void handleKeyPressed( 
			VdEvent *ev );
	virtual void handleKeyReleased( 
			VdEvent *ev );
	virtual void callbackButtonPressed ( 
			VdEvent *ev );
	virtual void callbackButtonReleased ( 
			VdEvent *ev );
	virtual void handleValueChanged(
			void );
	virtual void handleHoriValueChanged(
			void );
	virtual void handleVertValueChanged( 
			void );
	virtual void handleCascaded( 
			void );
	virtual void handleMapped( 
			void );
	virtual void handleUnmapped( 
			void );
	virtual void handleArmed( 
			void );
	virtual void handleDisarmed( 
			void );
};

#endif /* _VDCBMANAGER_HXX */
