#ifndef _VDDESTRUCTION_HXX
#define _VDDESTRUCTION_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdDestruction.hxx                                 *
*  Date: 06-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization: experimental version              *
*                                                          *
************************************************************
*                                                          *
*     experimental declaration of class VdDestruction:     *
*        VdDestruction is a (virtual) base class for all   *
*        classes which contain widgets, if they have to    * 
*        watch destroying the widgets.                     *
*        Writing a own method 'destroy' the                *
*        programmer can garantee that an object is         *
*        handled well if the contained widgets are         * 
*        destroyed by the window manager or by top down    *
*        destroying of higher level widgets.               *
*                                                          *
*                                                          *
*     Interface                                            *
*        X11/Intrinsic.h                                   *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdDestruction;

/******************    dependences    *********************/


#include <X11/Intrinsic.h>


class VdDestruction {

   private:
	static void destroyCB(Widget widget, 
		XtPointer clientData, XtPointer callData );

   public:
	virtual ~VdDestruction( void );

	// The following function installs watching of
	// a DestroyEvent for some widget. This event will
	// call operator delete for the watched object.
	// So derived classes should protect their destructor
	// ensuring that no objects are allocated in the
	// stack.
	// Explicit deleting of such objects is possible
	// only via a method which destroys the widget!
	// Explicit destructing causes problems beacuse of
	// neccessary widget destroying in any destructor
	// (which would destruct the object a second one).
	// Contrary a second widget destroying is harmless
	// (look XtDestroyWidget).
	void watchDestroying(Widget w);
};

#endif /* _VDDESTRUCTION_HXX */
