#define _VDEDITWINDOW_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdEditWindow.cxx                                  *
*  Date: 30-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Declaration of class VdEditWindow:                   *
*                                                          *
*     Interface                                            *
*          VdEditWindow.hxx                                *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdAbstractWindow.hxx>
#include <Vd/VdWindowConnection.hxx>
#include <Vd/VdEditWindow.hxx>
#include <Vd/VdEventManagerInterface.hxx>

extern "C" {
#include <stdlib.h>
}


VdEditWindow::VdEditWindow( VdShell &shell,
                Boolean withScrollbars ) :
	VdAbstractWindow(shell, withScrollbars)
{
	setIdent(shell.getTitle());
	theConnection = NULL;
	theEventInterface = NULL;
}

VdEditWindow::~VdEditWindow( void )
{
	if (theConnection)
		theConnection->removeEditWindow(getIdent());
}

void VdEditWindow::initiateEventHandling( VdEvent *ev )
{
	if ( !theEventInterface 
	     || !getPicture() 
	     || !theConnection) return;
	handleList.theEntirePicture = getPicture();
	ev->type = theEventInterface->getEventPrefix()
			 + ev->type;
	handleList.theEvent = *ev;
	handleList.theEvent.window = getIdent();
	handleList.theEvent.theEntirePicture=handleList.theEntirePicture;
	handleList.selections.reset();
	(getPicture())->collectHandles(&handleList);
	theEventInterface->handleEvent(
		theConnection->getWindowManagerInterface(), 
		&handleList);	
}


VdFeature VdEditWindow::setConnection( 
	VdWindowConnection *wc, VdFeature& ident )
{
// This function will be only called by 
// VdWindowConnection::setMaginifierWindow
	theConnection = wc;
	if (wc)
		setIdent(ident + getIdent());
	else 
		setIdent( (getIdent()).getLast(1));
	return getIdent();
}

VdWindowConnection *VdEditWindow::getConnection( void )
{
	return theConnection;
}

void VdEditWindow::setEventManagerInterface(
		VdEventManagerInterface *interface)
{
	theEventInterface = interface;
}

VdEventManagerInterface *
	VdEditWindow::getEventManagerInterface( void )
{
	return theEventInterface;
}

void VdEditWindow::handleDestroyed( void )
{
	// Behandlung a la collectEventHandling
	if ( !theConnection ) return;
	theConnection->removeEditWindow( getIdent() );
}


void VdEditWindow::handleButtonPressed( 
	VdEvent* ev )
{
	initiateEventHandling(ev);
}

void VdEditWindow::handleButtonReleased( 
	VdEvent* ev )
{
	initiateEventHandling(ev);
}

void VdEditWindow::handleKeyPressed( 
	VdEvent* ev )
{
	initiateEventHandling(ev);
}

void VdEditWindow::handleResized( void )
{
	VdAbstractWindow::handleResized();
	if ( !theConnection 
		|| !theConnection->isActiveEditWindow(this) ) 
	    return;
	VdArea displayed, world;
	getDisplayOfWorld(displayed, world);
	theConnection->changedDisplayOfWorld(displayed,world);
}

void VdEditWindow::handleHoriValueChanged( void )
{
	VdAbstractWindow::handleHoriValueChanged();
	if ( !theConnection 
		|| !theConnection->isActiveEditWindow(this) ) 
	    return;
	VdArea displayed, world;
	getDisplayOfWorld(displayed, world);
	theConnection->movedDisplayOfWorld(displayed,world);
}

void VdEditWindow::handleVertValueChanged( void )
{
	VdAbstractWindow::handleVertValueChanged();
	if ( !theConnection 
		|| !theConnection->isActiveEditWindow(this) ) 
	    return;
	VdArea displayed, world;
	getDisplayOfWorld(displayed, world);
	theConnection->movedDisplayOfWorld(displayed,world);
}

void VdEditWindow::handleHelp( void )
{
	VdShell::help(VdFeature("Visamad:EditWindow"));
}

#undef _VDEDITWINDOW_CXX
