#define _VDFEATURE_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdFeature.cxx                                     *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdFeature.hxx>

#define AREALENGTH 256
#define AREATYPE short[AREALENGTH]

short * VdFeature::CmpStep(short * p)
{ switch (*p)
  { case VdFeature::ChCodeEnd:
      return NULL;
    case VdFeature::ChCodeAltB:
      while (*(p=CmpNextAlt(p))!=VdFeature::ChCodeAltE);
    default:
      p++; return p;
  }
}

short * VdFeature::CmpNextAlt(short * p)
{ if (*p==VdFeature::ChCodeAltE) return p;
  if ((*p==VdFeature::ChCodeAltB)||(*p==VdFeature::ChCodeAltS))
  { do{ p++; } while (*p==VdFeature::ChCodeAltS);
    return p;
  }
  while (  (*(p=CmpStep(p))!=VdFeature::ChCodeAltE)
         &&(*p!=VdFeature::ChCodeAltS)
        );
  return p;
}

short * VdFeature::CmpNext(short * p)
{ while(1)
  { switch (*p)
    { case VdFeature::ChCodeEnd:
        return NULL;
      case VdFeature::ChCodeAltS:
      case VdFeature::ChCodeAltB:
        while (*(p=CmpNextAlt(p))!=VdFeature::ChCodeAltE) p++;
      default: 
        p++;
        if ((*p==VdFeature::ChCodeAltS)||(*p==VdFeature::ChCodeAltE))
          continue;
        return p;
    }
  }
}

int VdFeature::CmpCompare(short * c1, short * c2)
{ if (!((c1)&&(c2))) return 0;
  if ((*c1==VdFeature::ChCodeStar)||(*c2==VdFeature::ChCodeStar))
  { if (CmpCompare(CmpNext(c1),c2)) return 1; 
    else return CmpCompare(c1,CmpNext(c2));
  }
  if ((*c1==VdFeature::ChCodeMark)||(*c2==VdFeature::ChCodeMark))
    return CmpCompare(CmpNext(c1),CmpNext(c2));
  if (*c1==VdFeature::ChCodeAltB)
  { while (*(c1=CmpNextAlt(c1))!=VdFeature::ChCodeAltE)
      if (CmpCompare(c1,c2)) return 1;
      else c1=CmpNextAlt(c1);
    return 0;
  }
  if (*c2==VdFeature::ChCodeAltB)
  { while (*(c2=CmpNextAlt(c2))!=VdFeature::ChCodeAltE)
      if (CmpCompare(c2,c1)) return 1;
      else c2=CmpNextAlt(c2);
    return 0;
  }
  if (*c1==*c2)
  { if (*c1==VdFeature::ChCodeEnd) return 1;
    else return CmpCompare(CmpNext(c1),CmpNext(c2));
  }
  else return 0;
}


void VdFeature::AnaAlternate(char* & cp, short* &pp, int& l)
{ while (1)
  { switch (*cp)
    { case '{' : cp++; *pp++=VdFeature::ChCodeAltB; l++; break;
      case '|' : cp++; *pp++=VdFeature::ChCodeAltS; l++; break;
      case '}' : cp++; *pp++=VdFeature::ChCodeAltE; l++; return;
      case '\0': *pp++=VdFeature::ChCodeAltE; l++; return;
    }
    AnaAnalize(cp,pp,l);
  }
}

void VdFeature::AnaIdentifier(char* & cp, short* &pp, int& l)
{ char buf[1024]; int bufptr;
  bufptr=0;
  while (1)
  switch (*cp)
  { case '{' : 
    case ':' :
    case '|' :
    case '}' :
    case '*' :
    case '?' :
    case '\0' :
      if (bufptr)
      { buf[bufptr]='\0';
        VdString idf(buf);
        if ((*pp=Identifier.idx(idf))<0) 
          Identifier[*pp=!Identifier]=idf;
        pp++; l++;
      }
      return;
    default : buf[bufptr++]=*cp++; continue;
  }
}

void VdFeature::AnaAnalize(char* & cp, short* &pp, int& l)
{ while(1)
  switch (*cp)
  { case '{' : AnaAlternate(cp,pp,l); continue;
    case ':' :  cp++; continue;
    case '|' :
    case '}' : return;
    case '*' : cp++; *pp++=VdFeature::ChCodeStar; l++; continue;
    case '?' : cp++; *pp++=VdFeature::ChCodeMark; l++; continue;
    case '\0' : return;
    default : AnaIdentifier(cp,pp,l); continue;
  }
}


VdFeature::VdFeature(void)
{ *(pattern=new AREATYPE)=ChCodeEnd; CodeLength=1;
}

VdFeature::VdFeature( const VdFeature & c )
{ memcpy(pattern=new AREATYPE, c.pattern, sizeof(AREATYPE));
  CodeLength=c.CodeLength;
}

VdFeature::VdFeature( const VdString & s )
{ char * cptr; short * pptr;
  cptr=s.charptr();
  CodeLength=0; 
  pptr=pattern=new AREATYPE;
  AnaAnalize(cptr,pptr,CodeLength);
  pattern[CodeLength++]=ChCodeEnd;
}

VdFeature::VdFeature( char * s )
{ char * cptr; short * pptr;
  cptr=s;
  CodeLength=0; 
  pptr=pattern=new AREATYPE;
  AnaAnalize(cptr,pptr,CodeLength);
  pattern[CodeLength++]=ChCodeEnd;
}

VdFeature & VdFeature::operator= ( const VdFeature & c )
{ memcpy(pattern, c.pattern, sizeof(AREATYPE));
  CodeLength=c.CodeLength;    // completed (JH, 26.10.92)
  return *this;
}

VdFeature & VdFeature::operator+= ( const VdFeature & c )
{ memcpy(pattern+CodeLength-1,c.pattern
        ,(AREALENGTH-CodeLength)*sizeof(short)
        );
  CodeLength+=c.CodeLength-1; pattern[CodeLength-1]=ChCodeEnd;
  return *this;
}

int VdFeature::operator! ( void ) const
{ int n = 0; short * pptr = pattern;
  while ( *pptr != ChCodeEnd ) { n++; pptr=CmpNext(pptr); }
  return n;
}
  
int VdFeature::length ( void ) const
{ int n = 0; short * pptr = pattern;
  while ( *pptr != ChCodeEnd ) { n++; pptr=CmpNext(pptr); }
  return n;
}
  
VdFeature VdFeature::operator+ ( const VdFeature & c) const
{ VdFeature h;
  memcpy(h.pattern,pattern,CodeLength*sizeof(short));
  memcpy(h.pattern+CodeLength-1,c.pattern
        ,(AREALENGTH-CodeLength)*sizeof(short)
        );
  h.CodeLength=CodeLength+c.CodeLength-1;
  h.pattern[h.CodeLength-1]=ChCodeEnd;
  return h;
}

void VdFeature::removeLast(int n)
{ short *p, *q; int i=n;
  p=pattern; while ((i--)&&(*p!=ChCodeEnd)) p=CmpNext(p);
  if (i>=0)
  { pattern[0]=ChCodeEnd; CodeLength=1;
  }
  else
  { q=pattern;
    while (*p!=ChCodeEnd) { p=CmpNext(p); q=CmpNext(q); }
    *q=ChCodeEnd; CodeLength-=n;
  }
}

VdFeature VdFeature::getLast( int n ) const
{ short *p, *q; int i=n;
  p=pattern; while ((i--)&&(*p!=ChCodeEnd)) p=CmpNext(p);
  if (i>=0) return *this;
  q=pattern;
  while (*p!=ChCodeEnd) { p=CmpNext(p); q=CmpNext(q); }
  VdFeature hf; i=0;
  while (q!=p) *(hf.pattern+i++)=*q++;
  *(hf.pattern+i++)=ChCodeEnd;
  hf.CodeLength=i;
  return hf;
}

VdFeature VdFeature::operator| ( const VdFeature & c ) const
{ VdFeature h;
  int a1, a2, l;
  short * pptr;
  pptr=pattern;
  a1 = ((*pptr == ChCodeAltB) && (*(CmpNext(pptr)) == ChCodeEnd));
  pptr=c.pattern;
  a2 = ((*pptr == ChCodeAltB) && (*(CmpNext(pptr)) == ChCodeEnd));
  pptr=h.pattern; l=0;
  if (!a1) *pptr++=ChCodeAltB;
  memcpy(pptr,pattern,CodeLength*sizeof(short));
  if (a1) l=CodeLength-2;
  else    l=CodeLength;
  *(pptr+l++)=ChCodeAltS;
  memcpy(pptr+l,a2?(c.pattern+1):c.pattern
        ,(AREALENGTH-l)*sizeof(short)
        );
  if (a2)
  { if ((l+=c.CodeLength-1)>AREALENGTH)
    { l=AREALENGTH-2;
      *(pptr+l++)=ChCodeAltE;
      *(pptr+l++)=ChCodeEnd;
    }
  }
  else
  { l+=c.CodeLength-1;
    if (l>AREALENGTH-2) l=AREALENGTH-2;
    *(pptr+l++)=ChCodeAltE;
    *(pptr+l++)=ChCodeEnd;
  }
  h.CodeLength=l;
  return h;
}

VdFeature & VdFeature::operator|= ( const VdFeature & c )
{ int a1, a2, l;
  short hpattern[AREALENGTH];
  short * pptr;
  pptr=pattern;
  a1 = ((*pptr == ChCodeAltB) && (*(CmpNext(pptr)) == ChCodeEnd));
  pptr=c.pattern;
  a2 = ((*pptr == ChCodeAltB) && (*(CmpNext(pptr)) == ChCodeEnd));
  pptr=hpattern; l=0;
  if (!a1) *pptr++=ChCodeAltB;
  memcpy(pptr,pattern,CodeLength*sizeof(short));
  if (a1) l=CodeLength-2;
  else    l=CodeLength;
  *(pptr+l++)=ChCodeAltS;
  memcpy(pptr+l,a2?(c.pattern+1):c.pattern
        ,(AREALENGTH-l)*sizeof(short)
        );
  if (a2)
  { if ((l+=c.CodeLength-1)>AREALENGTH)
    { l=AREALENGTH-2;
      *(pptr+l++)=ChCodeAltE;
      *(pptr+l++)=ChCodeEnd;
    }
  }
  else
  { l+=c.CodeLength-1;
    if (l>AREALENGTH-2) l=AREALENGTH-2;
    *(pptr+l++)=ChCodeAltE;
    *(pptr+l++)=ChCodeEnd;
  }
  memcpy(pattern,hpattern,sizeof(hpattern));
  CodeLength=l;
  return *this;
}


int VdFeature::operator== ( const VdFeature & c ) const
{ return CmpCompare(pattern,c.pattern);
}

int VdFeature::operator!= ( const VdFeature & c ) const
{ return !CmpCompare(pattern,c.pattern);
}

VdString VdFeature::getString ( void ) const
{ VdString hs; short * pptr = pattern; int test;
  static VdString asterisk("*"), 
                  question_mark("?"),
                  left_bracket("{"),
                  vertical_line("|"),
                  right_bracket("}"),
                  dp_asterisk(":*"),
                  dp_question_mark(":?"),
                  dp_left_bracket(":{"),
                  double_point(":");
  while(1)
  { switch (*pptr)
    {  case ChCodeStar : hs+=asterisk; break;
       case ChCodeMark : hs+=question_mark; break;
       case ChCodeAltB : hs+=left_bracket; break;
       case ChCodeAltS : hs+=vertical_line; break;
       case ChCodeAltE : hs+=right_bracket; break;
       case ChCodeEnd  : return hs;
       default :         hs+=Identifier[*pptr]; break;
    }
    pptr++; test=1;
    while (test)
    { 
      switch (*pptr)
      {  case ChCodeStar : hs+=dp_asterisk; break;
         case ChCodeMark : hs+=dp_question_mark; break;
         case ChCodeAltB : hs+=dp_left_bracket; test=0; break;
         case ChCodeAltS : hs+=vertical_line;  test=0; break;
         case ChCodeAltE : hs+= right_bracket; break;
         case ChCodeEnd  : return hs;
         default :         hs+=double_point+Identifier[*pptr]; break;
      }
      pptr++;
    }
  }
}

#undef _VDFEATURE_CXX
