#define _VDFRACTNUMBER_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdFractNumber.cxx                                 *
*  Date: 21-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/


#include <Vd/VdFractNumber.hxx>
#include <math.h>
#include <values.h>

#define MININT (~MAXINT)
static float halffloat=1.0/2.0;
static double epsilon=1.0/10000000000.0;

VdFractNumber::VdFractNumber(int i,int j)
       {
	if (j==0)
	   { denominator=1 ;
	     if (i>=0) numerator=MAXINT ;
             else numerator=-MAXINT;
	     return ;
	   }
	if (j<0)
	   {
	    denominator=-j ;
	    numerator=-i ;
	   }
        else
           {
   	    numerator=i;
	    denominator=j ;
           }
	}

VdFractNumber::VdFractNumber(double d)
{ int i = (d<0.0), k, w;
  if (d<0.0) d=-d;
  if (d>=MAXINT)
  { numerator=i?MININT:MAXINT;
    denominator=1;
    return;
  }
  w = (int)d;
  k=0;
  while(w) { w=w>>1; k++; }
  if (k>10)
  { numerator=i?-((int)d):((int)d);
    denominator=1;
    return;
  }
  d*=(1<<(12-k));
  numerator=i?-((int)d):((int)d);
  denominator=(1<<(12-k));
}

float  VdFractNumber::getFloatValue(void) const
       { return ((float)numerator)/((float)denominator) ;
       }

double  VdFractNumber::getDoubleValue(void) const
       { return ((double)numerator)/((double)denominator) ;
       }
int VdFractNumber::getIntValue(void) const
       { return numerator/denominator;
       }

int VdFractNumber::getBestIntValue(void) const
      { return (int)(getFloatValue()+halffloat);
      }

void VdFractNumber::setDenominator(const int d)
       {
	 if (d==0)
	    {
	     denominator=1 ;
	     numerator=MAXINT ;
	     return ;
	    }
	 if (d<0)
	    {
	     denominator=-d ;
	     numerator=-numerator ;
	     return ;
	    }
	 denominator=d ;
       }

int VdFractNumber::operator >(
		   const VdFractNumber & aFractNumber
			     ) const
       {
	 return(getDoubleValue() >
		aFractNumber.getDoubleValue()) ;
       }

int VdFractNumber::operator <(
		   const VdFractNumber & aFractNumber
			     ) const
       {
	 return(getDoubleValue() <
		aFractNumber.getDoubleValue()) ;
       }

int VdFractNumber::operator <=(
		   const VdFractNumber & aFractNumber
			      ) const
       {
	 return(getDoubleValue() <=
		aFractNumber.getDoubleValue()) ;
       }

int VdFractNumber::operator >=(
		   const VdFractNumber & aFractNumber
			      ) const
       {
	 return(getDoubleValue() >=
		aFractNumber.getDoubleValue()) ;
       }

int VdFractNumber::operator ==(
		   const VdFractNumber & aFractNumber
			      ) const
       {
	 return (fabs(getDoubleValue()-aFractNumber.getDoubleValue())
                   < epsilon
                );
       }

VdFractNumber VdFractNumber::operator - (void) const
{ return VdFractNumber(-numerator,denominator);
}

VdFractNumber VdFractNumber::operator + (
	       const VdFractNumber &aFractNumber
			   ) const
{ return VdFractNumber(getDoubleValue()+aFractNumber.getDoubleValue());
}

  VdFractNumber VdFractNumber::operator - (
	       const VdFractNumber &aFractNumber
			   ) const
{ return VdFractNumber(getDoubleValue()-aFractNumber.getDoubleValue());
}

VdFractNumber VdFractNumber::operator *(
	      const VdFractNumber & aFractNumber
				       ) const
       { VdFractNumber
		h(numerator*aFractNumber.numerator,
		  denominator*aFractNumber.denominator) ;
	 return h ;
       }

VdFractNumber VdFractNumber::operator /(
	      const VdFractNumber & aFractNumber
				       ) const
       { VdFractNumber
		h(numerator*aFractNumber.denominator,
		  denominator*aFractNumber.numerator) ;
	 return h ;
       }

VdFractNumber VdFractNumber::half (void) const
{ return VdFractNumber(numerator/2,denominator);
}

#undef _VDFRACTNUMBER_CXX
