#define _VDGEOMETRICCONTEXT_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdGeometricContext.cxx                            *
*  Date: 15-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: B. Miethig                                      *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Implementation of class VdGeometricContext           *
*                                                          *
*     Interface                                            *
*                                                          *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdGeometricContext.hxx>

extern "C" {
#include <stdio.h>
}


VdGeometricContext::VdGeometricContext(
		          const VdArea &aWindowArea ,
                          const VdWindowType aWindowType  
				      )
	{
	 displayedArea=aWindowArea ;
	 windowArea=aWindowArea ;
         worldArea=aWindowArea ;
         theWindowType=aWindowType ; 
	 zoomX.setDenominator(displayedArea.lr.x-displayedArea.ul.x) ;
	 zoomY.setDenominator(displayedArea.ul.y-displayedArea.lr.y) ;
	 zoomX.setNumerator(windowArea.lr.x-windowArea.ul.x) ;
	 zoomY.setNumerator(windowArea.ul.y-windowArea.lr.y) ;
	}



VdGeometricContext::VdGeometricContext(
		      const VdPoint &origin,
		      const VdFractNumber &aZoomX,
		      const VdFractNumber &aZoomY,
                      const  VdWindowType aWindowType

                     		      )
	{
	 VdArea hArea(VdNullPoint,aZoomX.getDenominator(),
		      aZoomY.getDenominator()) ;
                                      
                 	 VdPoint hPoint(aZoomX.getNumerator(),
			-aZoomY.getNumerator());
                      zoomX=aZoomX ;
	 zoomY=aZoomY ;
	 displayedArea=hArea ;
	 VdArea wArea(origin,hPoint+origin) ;
	 windowArea=wArea ;
         worldArea=displayedArea ; 
         theWindowType=aWindowType ;
	}



VdPoint VdGeometricContext::transPictToWind(
		       const VdPoint &aDisplayedPoint
					   )
       {
	VdPoint aWindowPoint ;
	aWindowPoint=aDisplayedPoint-displayedArea.ul ;
        aWindowPoint.x=(zoomX*aWindowPoint.x).getIntValue();
	aWindowPoint.y=-(zoomY*aWindowPoint.y).getIntValue();
	return aWindowPoint ;
       }


VdVector VdGeometricContext::transPictToWind(
		       const VdVector &aDisplayedVector
					    )
       {
	VdVector aWindowVector ;
	aWindowVector.x=(zoomX*aDisplayedVector.x).getIntValue() ;
	aWindowVector.y=-(zoomY*aDisplayedVector.y).getIntValue() ;
	return aWindowVector ;
       }


VdArea VdGeometricContext::transPictToWind(
		       const VdArea &aDisplayedArea
					  )
       {
	VdArea aWindowArea ;
	aWindowArea.ul=transPictToWind(aDisplayedArea.ul) ;
	aWindowArea.lr=transPictToWind(aDisplayedArea.lr) ;
	return aWindowArea ;
       }



VdPoint VdGeometricContext::transWindToPict(
		       const VdPoint &aWindowPoint
					   )
       {
	VdPoint aDisplayedPoint ;
	aDisplayedPoint=aWindowPoint;
	aDisplayedPoint.x=((VdFractNumber)aDisplayedPoint.x/zoomX).getIntValue();
	aDisplayedPoint.y=-((VdFractNumber)aDisplayedPoint.y/zoomY).getIntValue() ;
	aDisplayedPoint=aDisplayedPoint+displayedArea.ul ;
	return aDisplayedPoint ;
       }


VdVector VdGeometricContext::transWindToPict(
		       const VdVector &aWindowVector
					    )
       {
	VdVector aDisplayedVector ;
	aDisplayedVector.x=((VdFractNumber)aWindowVector.x/zoomX).getIntValue() ;
	aDisplayedVector.y=-((VdFractNumber)aWindowVector.y/zoomY).getIntValue();
	return aDisplayedVector ;
       }


VdArea VdGeometricContext::transWindToPict(
		       const VdArea &aWindowArea
					  )
       {
	VdArea aDisplayedArea ;
	aDisplayedArea.ul=transWindToPict(aWindowArea.ul) ;
	aDisplayedArea.lr=transWindToPict(aWindowArea.lr) ;
	return aDisplayedArea ;
       }


VdFractNumber VdGeometricContext::getZoomX() {return zoomX;}

VdFractNumber VdGeometricContext::getZoomY() {return zoomY;}



void VdGeometricContext::
	setZoomX(const VdFractNumber &aZoomX)
       {
	zoomX=aZoomX ;
	windowArea.lr.x=windowArea.ul.x
		      +(zoomX*
		       (displayedArea.lr.x-displayedArea.ul.x)).getIntValue() ;
       }



void VdGeometricContext::
	setZoomY(const VdFractNumber &aZoomY)
       {
	 zoomY=aZoomY ;
	 windowArea.lr.y=windowArea.ul.y
		       -(zoomY*
			(displayedArea.ul.y-displayedArea.lr.y)).getIntValue() ;
       }


void VdGeometricContext::
        setWorldArea(const VdArea &aWorldArea)
        {
         worldArea=aWorldArea ;
        }  

void VdGeometricContext::
	setWindowArea(const VdArea &aWindowArea)
       {
	windowArea=aWindowArea ;
	zoomX.setNumerator(windowArea.lr.x-windowArea.ul.x) ;
	zoomY.setNumerator(windowArea.ul.y-windowArea.lr.y) ;
       }


void VdGeometricContext::
	setDisplayedArea(const VdArea &aDisplayedArea)
       {
	displayedArea=aDisplayedArea ;
	zoomX.setDenominator(displayedArea.lr.x-displayedArea.ul.x) ;
	zoomY.setDenominator(displayedArea.ul.y-displayedArea.lr.y) ;
       }


VdArea VdGeometricContext::getWorldArea(){return worldArea;}
              
VdArea VdGeometricContext::getDisplayedArea(){return displayedArea;}

VdArea VdGeometricContext::getWindowArea(){return windowArea;}

void VdGeometricContext::
        changeWindowArea(VdArea &aWindowArea)
        {
         VdArea hArea,dArea ;
         VdFractNumber oldZoomX,oldZoomY ;

         switch (theWindowType){
            case Vd_DEFAULT_WINDOW:
		hArea=getWindowArea() ;
		oldZoomX=zoomX ;
		oldZoomY=zoomY ;
		setWindowArea(aWindowArea) ; 
		dArea = displayedArea;
		if (hArea.lr.y==windowArea.lr.y) { // obere Ausdehnung gendert
			dArea.ul.y = displayedArea.lr.y +
		                  ((VdFractNumber)(windowArea.ul.y-windowArea.lr.y)
						/oldZoomY).getIntValue();  
		} else {			   // untere Ausdehnung gendert
			dArea.lr.y = displayedArea.ul.y -
		                  ((VdFractNumber)(windowArea.ul.y-windowArea.lr.y)
						/oldZoomY).getIntValue();  
		}
		if (hArea.lr.x==windowArea.lr.x) { // linke Ausdehnung gendert
		        dArea.ul.x = displayedArea.lr.x -
		                 ((VdFractNumber)(windowArea.lr.x-windowArea.ul.x)
						/oldZoomX ).getIntValue();  
		} else {			   // rechte Ausdehnung gendert
		        dArea.lr.x = displayedArea.ul.x +
		                 ((VdFractNumber)(windowArea.lr.x-windowArea.ul.x)
						/oldZoomX ).getIntValue();  
		}
		setDisplayedArea(dArea); // update of zoom factors!!!
		break ; 

            case Vd_BUBBLE_GUM:
		setWindowArea(aWindowArea) ;
		break ; 
            } ;
         }


VdWindowType VdGeometricContext::getWindowType( void ) {
		return theWindowType; };

void VdGeometricContext::setWindowType( VdWindowType wt ) {
		theWindowType = wt; };


#undef _VDGEOMETRICCONTEXT_CXX
