#ifndef _VDGEOMETRICCONTEXT_HXX
#define _VDGEOMETRICCONTEXT_HXX


/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdGeometricContext.hxx                            *
*  Date: 15-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: B. Miethig                                      *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Declaration of class VdGeometricContext:             *
*                                                          *
*     Interface                                            *
*                                                          *
*                VdFractNumber.hxx                         *
*                VdArea.hxx                                *
*                VdVector.hxx                              *
*                VdPoint.hxx                               *
*					                   *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdGeometricContext;

/******************    dependences    *********************/

#include <Vd/VdFractNumber.hxx>
#include <Vd/VdArea.hxx>
#include <Vd/VdVector.hxx>
#include <Vd/VdPoint.hxx>


enum   VdWindowType {Vd_DEFAULT_WINDOW, 
                    Vd_BUBBLE_GUM
                   } ;


class VdGeometricContext
	   {
          VdArea worldArea ;
           
	  VdArea displayedArea ;

	  VdArea windowArea ;

	  VdFractNumber zoomX,zoomY ;
	       // the zoomfactors in both directions

          VdWindowType theWindowType ;  

public: VdGeometricContext(
		  const VdArea &aWindowArea,
                  const VdWindowType aWindowType=Vd_DEFAULT_WINDOW  
			     );

	   VdGeometricContext(
		  const VdPoint &displayedOrigin,
		  const VdFractNumber &aZoomX,
		  const VdFractNumber &aZoomY,
                  const VdWindowType aWindowType=Vd_DEFAULT_WINDOW  
			     ) ;

	   VdPoint transPictToWind(
		       const VdPoint &aDisplayedPoint
				  ) ;

	   VdVector transPictToWind(
		       const VdVector &aDisplayedVector
				   ) ;

	   VdArea transPictToWind(
		       const VdArea &aDisplayedArea
				 ) ;

	   VdPoint transWindToPict(
		       const VdPoint &aWindowPoint
				  ) ;

	   VdVector transWindToPict(
		       const VdVector &aWindowVector
				   );

	   VdArea transWindToPict(
		       const VdArea &aWindowArea
				 ) ;

	     VdFractNumber getZoomX();
	     VdFractNumber getZoomY();
	     void setZoomX(const VdFractNumber &aZoomX) ;
	     void setZoomY(const VdFractNumber &aZoomY) ;

             void setWorldArea(const VdArea &aWorldArea) ;
	     void setWindowArea(const VdArea &aWindowArea) ;
	     void setDisplayedArea(const VdArea &aDisplayedArea) ;

             VdArea getWorldArea();
	     VdArea getDisplayedArea();
	     VdArea getWindowArea();
             void changeWindowArea(VdArea &aWindowArea) ; 

	VdWindowType getWindowType( void );
	void setWindowType( VdWindowType wt ) ;

};

#endif /* _VDGEOMETRICCONTEXT_HXX */
