#define _VDGLOBALFUNCTIONS_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdGlobalFunctions.cxx                             *
*  Date: 07-04-93                                          *
*  Copyright (c) 1993                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     This modul contains global function which are        *
*     no methods of any classes                            *
*                                                          *
*     References:                                          *
* [XtRM] X Toolkit Intrinsics Reference Manual,            * 
*        Second Edition for X11 Release 4                  *
*        O'Reilly & Associates, Inc.                       *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

extern "C" {
#include <stdlib.h>
#include <stdio.h>
}

#include <Xm/MessageB.h>
#include <Vd/VdString.hxx>
#include <Vd/VdStringList.hxx>
#include <Vd/VdShell.hxx>
#include <Vd/VdFormular.hxx>
#include <Vd/VdGlobalFunctions.hxx>

#include <Xm/DialogS.h>
#include <Xm/Form.h>
#include <Xm/PushBG.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/SeparatoG.h>
#include <Xm/FileSB.h>
#include <Xm/MwmUtil.h>

//***********************************************************

void localMainLoop(int * stopFlag)
{
	XEvent ev;

	while (!*stopFlag) {
		XtAppNextEvent(VdShell::theAppContext,&ev);
		XtDispatchEvent(&ev);
	}
}

void VdRegenerateDialogShell()
{
	Arg al[15];
	Cardinal ac;
	Position x,y;

// The plan was to have a single dialog shell with persistent
// geometry data (XmNx, XmNy) for the certain dialogs (decision,
// formular, fileselection). But it seems impossible to remove
// the subwidget of a XmDialogShell well-timed. If the following
// action create a new subwidget of the dialog shell, a toolkit
// error occurs.
// Undependently from this a  XtUnmanageChild call should be
// inserted before any XtDestroyWidget call.

	ac=0;		// get old Positions
	XtSetArg(al[ac], XmNx, &x);
	ac++;
	XtSetArg(al[ac], XmNy, &y);
	ac++;
	XtGetValues(VdShell::theDialogShell,al,ac);
#ifdef TEST_JH
	fprintf(stderr,"VdRegenerateDialogShell: x=%d, y=%d\n",x,y);
#endif
			// destroy it
	XtDestroyWidget(VdShell::theDialogShell);
			// create new
	ac=0;
	XtSetArg(al[ac], XmNdeleteResponse, XmDO_NOTHING);
	ac++;
	XtSetArg(al[ac], XmNallowShellResize, True);
	ac++;
	XtSetArg(al[ac], XmNx, x); 
	ac++;
	XtSetArg(al[ac], XmNy, y);
	ac++;
	// XmNx, XmNy will only define shell position
	// if the subwidget defines XmNdefaultPosition as False
	// otherwise position will be overridden
        XtSetArg(al[ac], XmNmwmDecorations,
                MWM_DECOR_TITLE | MWM_DECOR_BORDER); 
	ac++;
	VdShell::theDialogShell = 
		XmCreateDialogShell(
			VdShell::theApplicationShell,
			"VisamadDialog",
			al,ac);
}

Widget VdGetActiveShell( void )
{
	Window hasFocus; int revert;
	Widget parent;

	XGetInputFocus(
		VdShell::theDisplay,
		&hasFocus,
		&revert
	);
		
	parent = XtWindowToWidget(
		VdShell::theDisplay,
		hasFocus
	);
	
	if ( 	!parent 
		|| hasFocus == PointerRoot 
		|| hasFocus == None) 
	    return VdShell::theMainShellPtr->getWidget();

	while ( !XtIsSubclass(parent,wmShellWidgetClass) )
		parent = XtParent(parent);

	return parent;
}

void VdSynchronize( void )
{
	XSync(
		VdShell::theDisplay,
		False
	);
}


//***********************************************************

typedef struct { int nrButton; 
		 int * adrFlag;
		 int *adrSelection; } DecisionData;

static void buttonCallback(Widget, 
		XtPointer clientData, 
		XtPointer)
{
	DecisionData * data = (DecisionData *)clientData;
	*(data->adrSelection) = data->nrButton;
	*(data->adrFlag) = 1;
}

int VdCreateAndHandleDecision(
		const VdString & dialogTitle,
		const VdString & messageText,
		VdStringList & buttonLabels)
{
	Arg al[20];
	Cardinal ac;
	int stop, selectedButton, i;

	Widget dialog = VdShell::theDialogShell,
		form,
		message,
		separator,
		buttons[20];
	VdStringList theButtonLabels(buttonLabels);
	if (!theButtonLabels==0) 
		theButtonLabels[0]=VdString("Ok");
	int nrLabels=!theButtonLabels;
	DecisionData data[20];
	
	ac=0;	
	XtSetArg(al[ac], XmNtitle, dialogTitle.charptr() );
	ac++;
	XtSetArg(al[ac], XmNtransientFor, VdGetActiveShell());
	ac++;
	XtSetValues(dialog,al,ac);
	ac=0;	
	XtSetArg(al[ac], XmNdialogStyle,
		XmDIALOG_APPLICATION_MODAL); ac++;
	XtSetArg(al[ac], XmNverticalSpacing, 5); ac++;
	XtSetArg(al[ac], XmNhorizontalSpacing, 5); ac++;
	XtSetArg(al[ac], XmNdefaultPosition, False); ac++;
	XtSetArg(al[ac], XmNautoUnmanage, False); ac++;
	form = XmCreateForm(dialog,
			"VisamadForm",
			al,ac);

	XmString messageString = XmStringCreateLtoR (
		messageText.charptr(), XmSTRING_DEFAULT_CHARSET
	);
	ac=0;	
	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM );
	ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM );
	ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM );
	ac++;
	XtSetArg(al[ac], XmNlabelString, messageString );
	ac++;
	message = XmCreateLabel(form,
			"VisamadMessageText",
			al,ac);
	XtManageChild(message);
	XmStringFree(messageString);

	ac=0;	
	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET );
	ac++;
	XtSetArg(al[ac], XmNtopWidget, message );
	ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM );
	ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM );
	ac++;
	separator = XmCreateSeparatorGadget(
			form,
			"VisamadSeparator",
			al,ac);
	XtManageChild( separator );

	for (i=0;i<nrLabels && i<20;i++) {
		ac=0;
		XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET );
		ac++;
		XtSetArg(al[ac], XmNtopWidget, separator );
		ac++;
		XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM );
		ac++;
		if (i==0) {
			XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM );  
			ac++;
		} else {
			XtSetArg(al[ac], XmNleftAttachment, XmATTACH_POSITION );  
			ac++;
			XtSetArg(al[ac], XmNleftPosition, 100*i/nrLabels+1 );  
			ac++;
		}
		if (i==nrLabels-1) {
			XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM );  
			ac++;
		} else {
			XtSetArg(al[ac], XmNrightAttachment, XmATTACH_POSITION );  
			ac++;
			XtSetArg(al[ac], XmNrightPosition, 100*(i+1)/nrLabels-1 );  
			ac++;
		}
		XtSetArg(al[ac], XmNalignment, XmALIGNMENT_CENTER); 
		ac++;
		buttons[i] = 
		    XmCreatePushButtonGadget(
			form,
			theButtonLabels[i].charptr(),
			al,ac);
		XtManageChild(buttons[i]);
		data[i].nrButton=i; data[i].adrFlag = &stop;
		data[i].adrSelection=&selectedButton;
		XtAddCallback(
			buttons[i], 
			XmNactivateCallback, 
			buttonCallback, 
			(XtPointer)&data[i] );
	}

	XtManageChild(form);
	XtManageChild(dialog);
	stop=0; selectedButton=-1;
	localMainLoop(&stop);
	XtUnmapWidget(dialog);
	XtUnmanageChild(form); 	// JH
	XtDestroyWidget(form);
	VdRegenerateDialogShell();
	return selectedButton;
}

//***********************************************************
typedef struct { int * adrFlag;
		 String result;
		 int canceled;
		 } FileSelectionData;

static void fileSelectionCallback(Widget, 
		XtPointer clientData, 
		XtPointer callData)
{
	FileSelectionData * data = (FileSelectionData *)clientData;
	XmSelectionBoxCallbackStruct * cbs =
		(XmSelectionBoxCallbackStruct *) callData;
	if (cbs->reason == XmCR_OK) {
		XmStringGetLtoR(
			cbs->value,
			(XmStringCharSet) 
			    XmSTRING_DEFAULT_CHARSET,
			&data->result);
		data->canceled = 0;
	} else {
		// cancel 
		data->canceled = 1;
	}
	*(data->adrFlag) = 1;
}

int VdCreateAndHandleFileSelection( 
		const VdString & dialogTitle,
		VdString & directory, VdString & fileName,
		int filter)
{
	Arg al[20];
	Cardinal ac;
	static int stop;
	static FileSelectionData data;
	XmString theDir, theFile;
	Widget theFileSelectionBox;

	stop = 0;
	data.adrFlag = &stop;

	ac = 0; 
	XtSetArg(al[ac], XmNtitle, dialogTitle.charptr() );
	ac++;
	XtSetArg(al[ac], XmNtransientFor, VdGetActiveShell());
	ac++;
	XtSetValues(VdShell::theDialogShell,al,ac);

	ac = 0; 
	if (filter) {
		theDir = XmStringCreateSimple (
			directory.charptr()
		);
		theFile = XmStringCreateSimple (
			fileName.charptr()
		);
		XtSetArg(al[ac], XmNdirectory, theDir); 
		ac++;
		XtSetArg(al[ac], XmNpattern, theFile); 
		ac++;
	}
	XtSetArg(al[ac], XmNdialogStyle, 
		XmDIALOG_APPLICATION_MODAL);
	ac++;
	XtSetArg(al[ac], XmNdefaultPosition, False); 
	ac++;
	XtSetArg(al[ac], XmNautoUnmanage, False);
	ac++;
	theFileSelectionBox=
	    XmCreateFileSelectionBox(
		VdShell::theDialogShell,
		"VisamadFileSelection",
		al, ac);
	XtAddCallback(
		theFileSelectionBox, 
		XmNokCallback, 
		fileSelectionCallback, 
		(XtPointer)&data );
	XtAddCallback(
		theFileSelectionBox, 
		XmNcancelCallback, 
		fileSelectionCallback, 
		(XtPointer)&data );
	if (filter) {
		XmStringFree(theDir);
		XmStringFree(theFile);
	}	
	XtManageChild(theFileSelectionBox);
	stop=0;
	localMainLoop(&stop);
	if (!data.canceled) {
		char *p = strrchr(data.result,'/');
		p++;
		// the last slash
		fileName = VdString(p);
		*p=0;
		directory = VdString(data.result);
		XtFree(data.result); data.result=NULL;
	}
	XtUnmapWidget(VdShell::theDialogShell);
	XtUnmanageChild(theFileSelectionBox); 	// JH
	XtDestroyWidget(theFileSelectionBox);
	VdRegenerateDialogShell();
	return data.canceled;
}
//***********************************************************
int VdCreateAndHandleFormular( 
		VdFormularInterface * interface)
{
	int res;
	VdFormular aFormular( interface );
	aFormular.map();
	localMainLoop(aFormular.getFlagPtr());
	res = aFormular.wasCanceled() ? 0 : 1;
	VdRegenerateDialogShell();
	return res;
}
//***********************************************************
#define A4BEGIN 0
#define A4TITLE 1
#define A4SCALE 10
#define ORBEGIN 11
#define ORTITLE 12
#define ORBOUND 15
#define ORTRANS 38
#define PCBEGIN 47
#define PCEND 48
#define OREOF 53
#define A4EOF 58
#define PAPERWIDTH 504
#define PAPERHEIGHT 720

static char * PSprefix[]=
{"%!%!PS-Adobe-2.0 EPSF-1.2"
,"%%Title: "
,"%%Creator: VISAMAD.Defgraph "
,"%%Pages: 1 (A4)"
,"%%BoundingBox: 39 55 543 775"
,"%%EndComments "
,"10 dict begin"
,"/showpage {} def"
,"save"
,"39 55 translate"
,"%d %d div %d %d div scale\n"
,"%!%!PS-Adobe-2.0 EPSF-1.2"
,"%%Title: "
,"%%Creator: VISAMAD.Defgraph "
,"%%Pages: 1 "
,"%%BoundingBox: "
,"%%EndComments "
,"/VISAMAD_dict 500 dict def "
,"VISAMAD_dict begin "
,"/SC { 64.0 div setgray } bind def"
,"/BC 1.0 def /SBC { /BC exch def } bind def /GBC { BC setgray } bind def"
,"/SLW { 10.0 div setlinewidth } bind def"
,"/SLS { pop } bind def"
,"/SAS { pop } bind def"
,"/FP 1 def /SFP { /FP exch def } bind def /GFP { } bind def"
,"/A { gsave newpath div dup 1 exch scale 1 exch div 1 sub mul 0"
       " exch translate arc stroke grestore } bind def"
,"/L { newpath moveto lineto stroke } bind def"
,"/AL { newpath moveto lineto lineto lineto lineto stroke } bind def"
,"/D { newpath 1 0 360 arc fill } bind def"
,"/R { newpath moveto 1 index 0 rlineto 0 exch rlineto neg 0 "
                 "rlineto closepath stroke } bind def"
,"/FR { newpath moveto 1 index 0 rlineto 0 exch rlineto neg 0 "
         "rlineto closepath gsave GFP fill grestore stroke } bind def"
,"/FA { gsave newpath div scale div mul sub neg translate arc gsave "
               "GFP fill grestore stroke grestore } bind def"
,"/F { findfont exch scalefont setfont }  bind def"
,"/T { gsave newpath moveto exch dup 3 1 roll stringwidth pop div 1 "
                 "scale show grestore } bind def"
,"end"
,"%%EndProlog "
,"%%Page: 1 1 "
,"save "
,"%d %d translate\n"
,"VISAMAD_dict begin"
,"0.0 SC"
,"1.0 SBC"
,"5 SFP"
,"0.0 SLW"
,"0 SLS"
,"0 SAS"
,"10 /Times-Roman F"
,"%% PictureBegin"
,"%% PictureEnd"
,"end"
,"restore "
,"showpage"
,"%%Trailer "
,"%End-of-file "
,"restore "
,"end"
,"showpage"
,"%%Trailer "
,"%End-of-file "
};

int VdPostScriptOriginal(VdString & filename
                        ,VdPicture * picture
                        ,VdPoint &leftlower
                        )
{ FILE* f;
  VdArea a; int i, w, h;
  if ((f=fopen(filename.charptr(),"w"))==NULL) return -1;
  if (picture==NULL) return 1;
  picture->prepare();
  a=picture->getAbsFrame();
  if (a.ul==a.lr) return 2;
  VdPoint ll(a.ul.x,a.lr.y);
  w=a.lr.x-a.ul.x; h=a.ul.y-a.lr.y;
  for (i=ORBEGIN; i<=OREOF; i++)
    switch(i)
    { case ORTITLE: fprintf(f,"%s %s\n",PSprefix[i],filename.charptr());
      break;
      case ORBOUND: fprintf(f,"%s %d %d %d %d\n"
                             ,PSprefix[i]
                             ,leftlower.x,leftlower.y
                             ,leftlower.x+w,leftlower.y+h
                           );
      break;
      case ORTRANS: fprintf(f,PSprefix[i],leftlower.x-ll.x,leftlower.y-ll.y);
      break;
      case PCBEGIN: fprintf(f,"%s\n",PSprefix[i]);
                    picture->printPostScript(f);
      break;
      case PCEND: fprintf(f,"%s\n",PSprefix[i]);
      break;
      case OREOF: fprintf(f,"%s %s\n",PSprefix[i],filename.charptr());
      break;
      default:    fprintf(f,"%s\n",PSprefix[i]);
      break;
    }
  fclose(f);
  return 0;
}

int VdPostScriptA4(VdString & filename
                  ,VdPicture * picture
                  )
{ FILE* f;
  VdArea a; int i, w, h; VdPoint leftlower;
  if ((f=fopen(filename.charptr(),"w"))==NULL) return -1;
  if (picture==NULL) return 1;
  picture->prepare();
  a=picture->getAbsFrame();
  if (a.ul==a.lr) return 2;
  VdPoint ll(a.ul.x,a.lr.y);
  w=a.lr.x-a.ul.x; h=a.ul.y-a.lr.y;
  for (i=A4BEGIN; i<=A4EOF; i++)
    switch(i)
    { case A4TITLE: fprintf(f,"%s %s\n",PSprefix[i],filename.charptr());
      break;
      case A4SCALE: if ((w<=PAPERWIDTH) && (h<=PAPERHEIGHT)) break;
                    if ((w<=PAPERHEIGHT) && (h<=PAPERWIDTH))
                    { fprintf(f,"90 rotate 0 -%d translate\n",PAPERWIDTH);
                      break;
                    }
                    if (w<=h)
                      fprintf(f,PSprefix[i],PAPERWIDTH,w,PAPERHEIGHT,h);
                    else
                    { fprintf(f,"90 rotate 0 -%d translate\n",PAPERWIDTH);
                      fprintf(f,PSprefix[i],PAPERHEIGHT,w,PAPERWIDTH,h);
                    }
      break;
      case ORTITLE: fprintf(f,"%s %s\n",PSprefix[i],filename.charptr());
      break;
      case ORBOUND: fprintf(f,"%s %d %d %d %d\n"
                             ,PSprefix[i]
                             ,leftlower.x,leftlower.y
                             ,leftlower.x+w,leftlower.y+h
                           );
      break;
      case ORTRANS: fprintf(f,PSprefix[i],leftlower.x-ll.x,leftlower.y-ll.y);
      break;
      case PCBEGIN: fprintf(f,"%s\n",PSprefix[i]);
                    picture->printPostScript(f);
      break;
      case PCEND: fprintf(f,"%s\n",PSprefix[i]);
      break;
      case OREOF: fprintf(f,"%s %s\n",PSprefix[i],filename.charptr());
      break;
      case A4EOF: fprintf(f,"%s %s\n",PSprefix[i],filename.charptr());
      break;
      default:    fprintf(f,"%s\n",PSprefix[i]);
      break;
    }
  fclose(f);
  return 0;
}

#undef _VDGLOBALFUNCTIONS_CXX
