#define _VDGRAPHICATTRIBUTES_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdGraphicAttributes.cxx                           *
*  Date: 04-11-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Benary, B. Miethig                           *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Implementation of classes                            *
*        VdColor                                           *
*        VdFillType                                        *
*        VdLineType                                        *
*        VdArcType                                         *
*        VdGraphicAttributes                               *
*                                                          *
*     Interface                                            *
*            VdGraphicAttributes.hxx, X11/Intrinsic.h,     *
*            X11/StringDefs.h, Xm/AtomMgr.h,               *
*            Xm/Protocols.h                                *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdGraphicAttributes.hxx>
#include <Vd/VdColor.hxx>
#include <Vd/VdFillType.hxx>
#include <Vd/VdLineType.hxx>
#include <Vd/VdArcType.hxx>
#include <Vd/VdShell.hxx>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include <strstream.h>

extern "C" {
#include <stdlib.h>
#include <stdio.h>
}


#define MAXNUMBERFONTS 50

static struct FontStruct {
	Font fontID;
	VdVector size;
	VdVector bias;
} VisamadFontList[MAXNUMBERFONTS];
static int VisamadFontListLength;

static int compareFontStruct(
	const void *pp1, const void *pp2)
{
	const FontStruct *p1 = (const FontStruct *)pp1,
                         *p2 = (const FontStruct *)pp2;
	if (p1->size.y < p2->size.y) return -1;
	if (p1->size.y > p2->size.y) return 1;
	if (p1->size.x < p2->size.x) return -1;
	if (p1->size.x > p2->size.x) return 1;
	return 0;
}

static int ColorTable[64];

void InitGraphicAttributes(Display *dp)
{ 

  Colormap cm ;
  int i,j ;
  XColor colors[64] ;

  cm=XDefaultColormap(dp,0) ;
  VdColor::MaxColors=64;
  VdColor::Highlight=0x20;

//--------------------- Colors ------------------------------
         colors[0].red=0x0f00; colors[0].green=0x0f00;
		colors[0].blue=0x0f00; // VdBLACK (dark)
          colors[16].red=0x1f00; colors[16].green=0x1f00;
		colors[16].blue=0x1f00; // VdBLACK (middle)
          colors[32].red=0x2f00; colors[32].green=0x2f00;
		colors[32].blue=0x2f00; // VdBLACK (light)
          colors[48].red=0x0000; colors[48].green=0x0000;
		colors[48].blue=0x0000; // VdBLACK (intensive)

          colors[1].red=0x2000; colors[1].green=0x0800;
		colors[1].blue=0x0800; // VdBROWN (dark)
          colors[17].red=0x4000; colors[17].green=0x1000;
		colors[17].blue=0x1000; // VdBROWN (middle)
          colors[33].red=0x6000; colors[33].green=0x1800;
		colors[33].blue=0x1800; // VdBROWN (light)
          colors[49].red=0xa500; colors[49].green=0x2a00;
		colors[49].blue=0x2a00; // VdBROWN (intensive)

          colors[2].red=0x4fff; colors[2].green=0x4fff;
		colors[2].blue=0x4fff; // VdGRAY (dark)
          colors[18].red=0x6fff; colors[18].green=0x6fff;
		colors[18].blue=0x6fff; // VdGRAY (middle)
          colors[34].red=0x8fff; colors[34].green=0x8fff;
		colors[34].blue=0x8fff; // VdGRAY (light)
          colors[50].red=0xafff; colors[50].green=0xafff;
		colors[50].blue=0xafff; // VdGRAY (intensive)

          colors[3].red=0x2fff; colors[3].green=0x5fff;
		colors[3].blue=0x5fff; // VdAZURE (dark)
          colors[19].red=0x47ff; colors[19].green=0x8fff;
		colors[19].blue=0x8fff; // VdAZURE (middle)
          colors[35].red=0x60ff; colors[35].green=0xbfff;
		colors[35].blue=0xbfff; // VdAZURE (light)
          colors[51].red=0x7fff; colors[51].green=0xffff;
		colors[51].blue=0xffff; // VdAZURE (intensive)

          colors[4].red=0x32ff; colors[4].green=0x4bff;
		colors[4].blue=0x11ff; // VdOLIVE (dark)
          colors[20].red=0x4aff; colors[20].green=0x70ff;
		colors[20].blue=0x1cff; // VdOLIVE (middle)
          colors[36].red=0x66ff; colors[36].green=0x9aff;
		colors[36].blue=0x27ff; // VdOLIVE (light)
          colors[52].red=0xafff; colors[52].green=0xffff;
		colors[52].blue=0x3fff; // VdOLIVE (intensive)

          colors[5].red=0x0000; colors[5].green=0x5fff;
		colors[5].blue=0x3fff; // VdTURQUOISE (dark)
          colors[21].red=0x0000; colors[21].green=0x7fff;
		colors[21].blue=0x5fff; // VdTURQUOISE (middle)
          colors[37].red=0x0000; colors[37].green=0x9fff;
		colors[37].blue=0x6fff; // VdTURQUOISE (light)
          colors[53].red=0x0000; colors[53].green=0xffff;
		colors[53].blue=0xbfff; // VdTURQUOISE (intensive)

          colors[6].red=0x0000; colors[6].green=0x5fff;
		colors[6].blue=0x0000; // VdGREEN (dark)
          colors[22].red=0x0000; colors[22].green=0x7fff;
		colors[22].blue=0x0000; // VdGREEN (middle)
          colors[38].red=0x0000; colors[38].green=0x9fff;
		colors[38].blue=0x0000; // VdGREEN (light)
          colors[54].red=0x0000; colors[54].green=0xffff;
		colors[54].blue=0x0000; // VdGREEN (intensive)

          colors[7].red=0x0000; colors[7].green=0x0000;
		colors[7].blue=0x5fff; // VdBLUE (dark)
          colors[23].red=0x0000; colors[23].green=0x0000;
		colors[23].blue=0x7fff; // VdBLUE (middle)
          colors[39].red=0x0000; colors[39].green=0x0000;
		colors[39].blue=0x9fff; // VdBLUE (light)
          colors[55].red=0x0000; colors[55].green=0x0000;
		colors[55].blue=0xffff; // VdBLUE (intensive)

          colors[8].red=0x5d00; colors[8].green=0x4700;
		colors[8].blue=0x8b00; // VdPURPLE (dark)
          colors[24].red=0x7400; colors[24].green=0x5800;
		colors[24].blue=0xaa00; // VdPURPLE (middle)
          colors[40].red=0x8a00; colors[40].green=0x6900;
		colors[40].blue=0xcb00; // VdPURPLE (light)
          colors[56].red=0xab00; colors[56].green=0x8200;
		colors[56].blue=0xff00; // VdPURPLE (intensive)

          colors[9].red=0x5fff; colors[9].green=0x0000;
		colors[9].blue=0x5fff; // VdVIOLETT (dark)
          colors[25].red=0x7fff; colors[25].green=0x0000;
		colors[25].blue=0x7fff; // VdVIOLETT (middle)
          colors[41].red=0x9fff; colors[41].green=0x0000;
		colors[41].blue=0x9fff; // VdVIOLETT (light)
          colors[57].red=0xffff; colors[57].green=0x0000;
		colors[57].blue=0xffff; // VdVIOLETT (intensive)

          colors[10].red=0x8fff; colors[10].green=0x4fff;
		colors[10].blue=0xc000; // VdPINK (dark)
          colors[26].red=0xafff; colors[26].green=0x5fff;
		colors[26].blue=0xc000; // VdPINK (middle)
          colors[42].red=0xcfff; colors[42].green=0x6fff;
		colors[42].blue=0xc000; // VdPINK (light)
          colors[58].red=0xffff; colors[58].green=0x8c00;
		colors[58].blue=0xc000; // VdPINK (intensive)

          colors[11].red=0x7fff; colors[11].green=0x0000;
		colors[11].blue=0x0000; // VdRED (dark)
          colors[27].red=0x9fff; colors[27].green=0x0000;
		colors[27].blue=0x0000; // VdRED (middle)
          colors[43].red=0xbfff; colors[43].green=0x0000;
		colors[43].blue=0x0000; // VdRED (light)
          colors[59].red=0xffff; colors[59].green=0x0000;
		colors[59].blue=0x0000; // VdRED (intensive)

          colors[12].red=0x8fff; colors[12].green=0x4fff;
		colors[12].blue=0x0000; // VdORANGE (dark)
          colors[28].red=0xafff; colors[28].green=0x5fff;
		colors[28].blue=0x0000; // VdORANGE (middle)
          colors[44].red=0xcfff; colors[44].green=0x6fff;
		colors[44].blue=0x0000; // VdORANGE (light)
          colors[60].red=0xffff; colors[60].green=0x8c00;
		colors[60].blue=0x0000; // VdORANGE (intensive)

          colors[13].red=0x8800; colors[13].green=0x6600;
		colors[13].blue=0x0000; // VdGOLD (dark)
          colors[29].red=0xaa00; colors[29].green=0x8200;
		colors[29].blue=0x0000; // VdGOLD (middle)
          colors[45].red=0xcc00; colors[45].green=0x9d00;
		colors[45].blue=0x0000; // VdGOLD (light)
          colors[61].red=0xff00; colors[61].green=0xc700;
		colors[61].blue=0x0000; // VdGOLD (intensive)

          colors[14].red=0xcfff; colors[14].green=0xcfff;
		colors[14].blue=0x0000; // VdYELLOW (dark)
          colors[30].red=0xdfff; colors[30].green=0xdfff;
		colors[30].blue=0x0000; // VdYELLOW (middle)
          colors[46].red=0xefff; colors[46].green=0xefff;
		colors[46].blue=0x0000; // VdYELLOW (light)
          colors[62].red=0xffff; colors[62].green=0xffff;
		colors[62].blue=0x0000; // VdYELLOW (intensive)

          colors[15].red=0xdaff; colors[15].green=0xdaff;
		colors[15].blue=0xdaff; // VdWHITE (dark)
          colors[31].red=0xe5ff; colors[31].green=0xe5ff;
		colors[31].blue=0xe5ff; // VdWHITE (middle)
          colors[47].red=0xefff; colors[47].green=0xefff;
		colors[47].blue=0xefff; // VdWHITE (light)
          colors[63].red=0xffff; colors[63].green=0xffff;
		colors[63].blue=0xffff; // VdWHITE (intensive)


	int allocColorError=0;
	for(i=48;i<64;i++)  { // intensive colors
          colors[i].flags=7; colors[i].pixel=i;
	  if ( !XAllocColor(dp,cm,&colors[i]) )
		  allocColorError=2;
	  ColorTable[i] = colors[i].pixel;
	}
	for(i=16;i<32;i++)  {  // middle colors
          colors[i].flags=7; colors[i].pixel=i;
	  ColorTable[i] =  ColorTable[i+32]; 
	  if ( !XAllocColor(dp,cm,&colors[i]) )
		  allocColorError=1;
	  else
		  ColorTable[i] = colors[i].pixel;
	}
	for(i=0;i<16;i++)  { // dark colors
          colors[i].flags=7; colors[i].pixel=i;
	  ColorTable[i] =  ColorTable[i+16];
	  if ( !XAllocColor(dp,cm,&colors[i]) )
		  allocColorError=1;
	  else
		  ColorTable[i] = colors[i].pixel;
	}
	for(i=32;i<48;i++)  { // light colors
          colors[i].flags=7; colors[i].pixel=i;
	  ColorTable[i] =  ColorTable[i-16];
	  if ( !XAllocColor(dp,cm,&colors[i]) )
		  allocColorError=1;
	  else
		  ColorTable[i] = colors[i].pixel;
	}
	if (allocColorError) {
		char message[100];
		ostrstream error(message,100);
		if (allocColorError==2)
		  error <<" - can not allocate base colors"
			<< "\n";
		else
		  error << " - can not allocate all colors" 
			<< "\n";
		XtWarning (message); allocColorError=1;
	}



// initialization of fonts
// primitive implementation

   {
	static char default_pattern[]="-misc-*-r-*-c-*-iso8859-1";
	// so ok?
	int i;
	char ** fontNameList;
	XCharStruct info;
	XFontStruct *fontInfo;

	char *pattern = XGetDefault(dp,
			VdShell::theClassName,"fontmask");
	if (!pattern) pattern = default_pattern;

	fontNameList = XListFonts(dp,pattern,
		MAXNUMBERFONTS,&VisamadFontListLength);
	if ( !fontNameList ) {
          	char message[100];
	    	ostrstream error(message,100);
	    	error 	<< " - can not load fonts for fontmask " 
			<< pattern << "\n";
		XtWarning (message); return;
	}
//	if (!fontNameList) VisamadFontListLength = 0;
	for (i=0;i<VisamadFontListLength;i++) {
		VisamadFontList[i].fontID = 
		   XLoadFont(dp, fontNameList[i]);
		fontInfo = 
		   XQueryFont(dp, VisamadFontList[i].fontID);
		info = fontInfo->max_bounds;
		VisamadFontList[i].size = 
			VdVector(info.width,
			         info.ascent + info.descent);
		VisamadFontList[i].bias = 
			VdVector(info.lbearing,-info.descent);
	}
	XFreeFontNames(fontNameList);
	qsort( (void *)VisamadFontList, 
			(size_t)VisamadFontListLength, 
			(size_t)sizeof(FontStruct),  
			(int (*)(const void *, const void *))
				&compareFontStruct );
   }  // end font initialization

}


int VdColor::MaxColors, 
    VdColor::Highlight ;

// constants for VdColor
const VdColor         VdBLACK(0x10) ,
                      VdBROWN(0x11),
                      VdGRAY(0x12), VdGREY(0x12),
                      VdAZURE(0x13),
                      VdOLIVE(0x14),
                      VdTURQUOISE(0x15),
                      VdGREEN(0x16),
                      VdBLUE(0x17),
                      VdPURPLE(0x18),
                      VdVIOLETT(0x19), VdMAGENTA(0x19),
                      VdPINK(0x1a),
                      VdRED(0x1b),
                      VdORANGE(0x1c),
                      VdGOLD(0x1d),
                      VdYELLOW(0x1e),
                      VdWHITE(0x1f),
                      VdCOLORUNDEF(-1) ;

int VdColor::Code(void) const 
{ if (code == -1) return 0; else return ColorTable[code]; 
}


VdColor VdColor::operator+(void) const
{ VdColor h;
  if (code==-1) h.code=code;
  else h.code=code|Highlight;
  return h;
}


VdColor VdColor::operator-(void) const
{ VdColor h;
  if (code==-1) h.code=code;
  else h.code=code&~Highlight;
  return h;
}

VdColor VdColor::operator!(void) const
{ VdColor h;
  if (code==-1) h.code=code;
  else h.code ^= Highlight;
  return h;
}


VdColor VdColor::dark(void) const
{ VdColor h;
  if (code==-1) { h.code=code; return h; }
  h.code=(code%16) ;
  return h ;
}


VdColor VdColor::middle(void) const
{ VdColor h;
  if (code==-1) { h.code=code; return h; }
  h.code=(code%16)+16;
  return h ;
}


VdColor VdColor::light(void) const
{ VdColor h;
  if (code==-1) { h.code=code; return h; }
  h.code=(code%16)+32;
  return h;
}

VdColor VdColor::operator+(const VdColor & c) const
{ VdColor h;
  if (code==-1) h.code=code;
  else if (c.code==-1) h.code=c.code;
       else h.code=code|c.code ;
  return h;
}

VdColor VdColor::operator-(const VdColor & c) const
{ VdColor h;
  if (code==-1) h.code=code;
  else if (c.code==-1) h.code=c.code;
       else h.code=code&c.code;
  return h;
}

VdColor VdColor::operator*(const VdColor & c) const
{ VdColor h;
  if (code==-1) h.code=code;
  else if (c.code==-1) h.code=c.code;
       else h.code=code^c.code;
  return h;
}

int VdColor::operator==(const VdColor & c) const
{ return code==c.code;
}

int VdColor::operator!=(const VdColor & c) const
{ return code!=c.code;
}


VdFillType::VdFillType(const int aCode)
{
 if((aCode>0)&&(aCode<5))  code=aCode ;
 else code=-1 ;
} 

// constants for VdFillType
const VdFillType VdFILLSOLID(FillSolid), 
                 VdFILLTILED(FillTiled), 
                 VdFILLSTIPPLED(FillStippled),
//  produce errormessage B.M. VdFILLOPAQUE(FillOpaqueStipple),
                 VdFILLUNDEF(-1);

int VdFillType::operator==(const VdFillType & c) const
{ return code==c.code;
}

int VdFillType::operator!=(const VdFillType & c) const
{ return code!=c.code;
}

VdLineType::VdLineType(const int aType)
{
 if((aType>0)&&(aType<4)) code=aType ;
 else code=-1 ;
 width=1 ;
}


// constants for VdLineType
const VdLineType VdLINESOLID(LineSolid),
                 VdLINEDASH(LineOnOffDash) ,
                 VdLINEDOUBLEDASH(LineDoubleDash),
                 VdLINEUNDEF; 

VdLineType VdLineType::operator()(int w) const
{ VdLineType h;
  h.code=code; h.width=w;
  return h;
}

int VdLineType::operator==(const VdLineType & c) const
{ return (code==c.code)&&(width==c.width);
}

int VdLineType::operator!=(const VdLineType & c) const
{ return (code!=c.code)||(width!=c.width);
}


VdArcType::VdArcType(const int aType)
{
 if((aType>0)&&(aType<3)) code=aType ;
 else code=-1 ;
}


// constants for VdArcType
const VdArcType VdARCPIESLICE(ArcPieSlice), 
                VdARCCHORD(ArcChord),
                VdARCUNDEF(-1);

int VdArcType::operator==(const VdArcType & c) const
{ return code==c.code;
}

int VdArcType::operator!=(const VdArcType & c) const
{ return code!=c.code;
}


int VdGraphicAttributes::isInitialized=0;

VdGraphicAttributes::VdGraphicAttributes
                   ( Display * theDisplay
                   , Drawable theDrawable
                   )
{ 
  if (!isInitialized) {
	isInitialized = 1;
	InitGraphicAttributes(theDisplay);
                                   }
  display=theDisplay; drawable=theDrawable;
  valuemask=0;
  values.function=GXcopy;
  values.plane_mask=0xff;
  values.foreground=VdBLACK.Code();
  values.background=VdWHITE.Code();
  values.line_width=1;
  values.line_style=LineSolid;
  values.cap_style=CapButt;
  values.join_style=JoinMiter; 

  values.font= fontID = None;
  fontSize = fontBias = VdNullVector;

  gc=XCreateGC(display,drawable,
               (GCFunction | GCPlaneMask | GCForeground | GCBackground
               | GCLineWidth | GCLineStyle | GCCapStyle | GCJoinStyle )
              ,&values
              );

  newGC='y';
}

VdGraphicAttributes::VdGraphicAttributes
                              ( const VdGraphicAttributes & g )
{ display=g.display; drawable=g.drawable;
  values=g.values;             /* mit memcpy() ? */
  valuemask=g.valuemask;
  gc=g.gc; newGC='\0';
  fontSize = g.fontSize;
  fontBias = g.fontBias;
  fontID =g.fontID;

}

VdGraphicAttributes & VdGraphicAttributes::operator=
                              ( const VdGraphicAttributes & g )
{ display=g.display; drawable=g.drawable;
  values=g.values; valuemask=g.valuemask;
  gc=g.gc; newGC='\0';
  return *this;
}

void VdGraphicAttributes::setForeground ( const VdColor & c )
{ if ((VdCOLORUNDEF==c)||(c.Code()==values.foreground)) return;
  values.foreground=c.Code(); valuemask|=GCForeground;
}

void VdGraphicAttributes::setBackground(const VdColor & c)
{ if ((VdCOLORUNDEF==c)||(c.Code()==values.background)) return;
  values.background=c.Code(); valuemask|=GCBackground;
}

void VdGraphicAttributes::setLineType(const VdLineType & c)
{ if ((c.Code()==-1)||(c.Code()==values.line_style));
  else
  { values.line_style=c.Code(); valuemask|=GCLineStyle;
  }
  if ((c.Width()==-1)||(c.Width()==values.line_width));
  else
  { values.line_width=c.Width(); valuemask|=GCLineWidth;
  }
}

void VdGraphicAttributes::setFillType(const VdFillType & c)
{ if ((VdFILLUNDEF==c)||(c.Code()==values.fill_style)) return;
  values.fill_style=c.Code(); valuemask|=GCFillStyle;
}

void VdGraphicAttributes::setArcType(const VdArcType & c)
{ if ((VdARCUNDEF==c)||(c.Code()==values.arc_mode)) return;
  values.arc_mode=c.Code(); valuemask|=GCArcMode;
}

void VdGraphicAttributes::setDrawingFunction(
	VdDrawingFunctionType drawType)
{
	if (values.function == drawType ) return;
	values.function = drawType;
	valuemask|=GCFunction;
}

GC VdGraphicAttributes::getGC(void)
{ if (valuemask)
  { if (newGC) 
	XChangeGC(display,gc,valuemask,&values);
    else {
	GC oldGC=gc;
	gc=XCreateGC(display,drawable,0,NULL);
	XCopyGC(display,oldGC,-1,gc);
	XChangeGC(display,gc,valuemask,&values);
    }
    newGC='n'; valuemask=0;
  }
  return gc;
}

VdGraphicAttributes::~VdGraphicAttributes(void)
{ if (newGC) XFreeGC(display,gc);
}

void VdGraphicAttributes::setFontSize(const VdVector &cell)
{
#ifdef TEST_JH
cerr << "VdGraphicAttributes::setFontSize cell = (" 
	<< cell.x << "," << cell.y << ")\n";
#endif
	int i,sign;
	VdVector cellAbs = cell;
	if (cellAbs.y < 0) { cellAbs.y = -cellAbs.y; sign = 1; }
	else sign = 0;
	for (i=VisamadFontListLength-1; i>=0; i--) {
		if ( (VisamadFontList[i].size.x <= cellAbs.x)
		     && (VisamadFontList[i].size.y <= cellAbs.y) ) {
			if (VisamadFontList[i].fontID 
				== values.font) return;
			fontSize = VisamadFontList[i].size;
			if (sign) fontSize.y = -fontSize.y;
			fontID = VisamadFontList[i].fontID;
			fontBias = VisamadFontList[i].bias;
			values.font =fontID;
			valuemask |= GCFont;
#ifdef TEST_JH
cerr << "VdGraphicAttributes::setFontSize real fontSize: (" 
	<< fontSize.x << "," << fontSize.y << ")\n";
#endif
			return;
		}
	}
#ifdef TEST_JH
cerr << "VdGraphicAttributes::setFontSize cell too small\n";
cerr << "VdGraphicAttributes::setFontSize smallest fontSize: (" 
	<< VisamadFontList[0].size.x << "," << VisamadFontList[0].size.y << ")\n";
cerr << "VdGraphicAttributes::setFontSize biggest fontSize: (" 
	<< VisamadFontList[VisamadFontListLength-1].size.x 
	<< "," << VisamadFontList[VisamadFontListLength-1].size.y << ")\n";
#endif
	// cell too small
	if (fontID == None) return;
	fontSize = VdNullVector;
	fontID = None;
	fontBias = VdNullVector;
}

VdVector VdGraphicAttributes::getFontSize ( void )
{
	return fontSize;
}

VdVector VdGraphicAttributes::getBias ( void )
{
	return fontBias;
}

Font VdGraphicAttributes::getFontID( void )
{
	return fontID;
}

#ifdef TEST_JH
void VdGraphicAttributes::printGC( void )
{
	fprintf(stderr,"VdGraphicAttributes::printGC(%c)\n",newGC);
	fprintf(stderr,"XGCValues: fg=%x, bg=%x\n",values.foreground, values.background);
}
#endif

#undef _VDGRAPHICATTRIBUTES_CXX
