#define _VDHELP_CXX

#include <Vd/VdHelp.hxx>
#include <Vd/VdHelpMemory.hxx>

#include <iostream.h>
extern "C" {
#include <stdlib.h>
#include <stdio.h>
}

VdHelpPtrList VdHelp::helpObjects;
static VdString noHelp("No help\n");

VdHelp::VdHelp( const VdString &title, 
		const VdFeature &theIdf,	
		VdTopicList &subTopics,
		int toUpdate)
{
	VdHelp *oldOne;
	theTitle=title;
	theIdent=theIdf;
	oldOne = getObject(theIdf);
	if (oldOne) delete oldOne;
	helpObjects.ins()=this;
	theSubTopics=subTopics;
	if(toUpdate) {
		// connecting to all subtopics
		updateSubTopics(); 
		// connecting from all mother topics
		updateFathers();
	}
}

VdHelp::~VdHelp() 
{
	VdHelp * p = this;
	int i=helpObjects.idx(p);
	if (i>=0 && i<!helpObjects) helpObjects.rem(i);
	remove();
}

void VdHelp::update( void )
{
	int nr = !helpObjects, i;
	for (i=0;i<nr;i++) {
		helpObjects[i]->updateSubTopics();
	}
}

void VdHelp::updateSubTopics( void )
{
	int nr,i;

	// connection to all subtopics 
	nr = !theSubTopics;
	for(i=0;i<nr;i++) {
		if (!theSubTopics[i].theHelp) {
		    theSubTopics[i].theHelp=
			getObject(theSubTopics[i].theIdent);
		}
	}

}	

void VdHelp::updateFathers( void )
{
	int nr,i,k;
	VdTopic topic;

	// connection from all fathers 
	topic.theIdent = theIdent;
	nr = !helpObjects;
	for(i=0;i<nr;i++) {
		k = helpObjects[i]->theSubTopics.idx(topic);
		if (k>=0) {
		    helpObjects[i]->theSubTopics[k].theHelp = this;
		}
	}
}	

VdHelpPtrList VdHelp::getAllFathers( void )
{
	int nr,i,k;

	VdHelpPtrList list;
	VdTopic topic;

	topic.theIdent = theIdent;
	nr = !helpObjects;
	for(i=0;i<nr;i++) {
		k = helpObjects[i]->theSubTopics.idx(topic);
		if (k>=0) {
		    list.ins() = helpObjects[i];
		}
	}
	return list;
}
	
void VdHelp::remove(void) 
{
	int i, j, nr;
	VdHelpPtrList list = getAllFathers();
	
	nr=!theSubTopics;
	for(i=0;i<nr;i++)
	    theSubTopics[i].theHelp = NULL;

	nr=!list;
	for(i=0;i<nr;i++) {
	    for (j=0;j<!(list[i]->theSubTopics);j++)
		if (list[i]->theSubTopics[j].theHelp == this) {
		    list[i]->theSubTopics[j].theHelp = NULL;
		    return;
		}
	}
}
		
	
void VdHelp::addSubTopic(const VdFeature &idf, 
			int update) 
{	
	VdTopic aTopic;

	aTopic.theHelp=getObject(idf);
	aTopic.theIdent=idf;
	theSubTopics.ins()=aTopic;
	if (update) updateFathers();
}


void VdHelp::remSubTopic(const VdFeature &idf)
{
	VdTopic topic;
	int i;

	topic.theIdent = idf;
	i = theSubTopics.idx(topic);
	if (i>=0) theSubTopics[i].theHelp = NULL;
}


VdString VdHelp::getTitle(void)
{
	return theTitle ;
}

VdHelp* VdHelp::getObject(const VdFeature &aFeature)
{
	int nr=!helpObjects, j=0;
	if (nr==0) return NULL;	
	while(j<nr) { 
		  if( (helpObjects[j]->theIdent)
			==aFeature) return helpObjects[j];
		  j++;
		}
	return NULL ;
}  


VdStringList VdHelp::getSubTopics(void)
{
	VdStringList list;
	VdString noHelp("");
	int i;
	
	for (i=0;i<!theSubTopics;i++)
		if (theSubTopics[i].theHelp) 
			list.ins()=theSubTopics[i].theHelp->theTitle;
		else
			list.ins()=noHelp;
	return list;
}
	

VdString VdHelp::getHelpText(void)
{
	return noHelp;
}

// =====================================================
// help file processing

static FILE* theHelpFile;
static int lineCount = 1;
static int lineend = 0;
static char buffer[129];

static void prepareNewLine( char * theLine )
{ int l=strlen(theLine);
  if (l==0) return;
  char * ptr = theLine+l-1;
  if (*ptr!='\n') return;
  *ptr='\0';
  if (l==1)
  { lineend=1; return;
  }
  if (*(ptr-1)=='\\')
  { *(ptr-1)='\0'; return;
  }
  lineend=1; return;
}

static int analizeLine(char** theLine)
{
	if ( !fgets(*theLine,128,theHelpFile) )
		return -1;
	if (lineend)
        { lineCount++; lineend=0;
    	}
	prepareNewLine(*theLine);
	if(!strncmp(*theLine,"<<<",3)) 	// Identifikator
	{
		(*theLine) += 3;
		return 1 ;
	}

	if(!strncmp(*theLine,">>>",3))	// SubTopicident.
	{
		(*theLine) += 3;
		return 2 ;
	}

	if(!strncmp(*theLine,"#",1))	// Kommentar 
	{
		return 3 ;
	}
	if (lineend)
	{ (*theLine)[strlen(*theLine)]='\n' ;
	  (*theLine)[strlen(*theLine)+1]='\0' ;
	}
	return 0 ;	// sonst Text bzw Titel
}

static int skipComment( char** theLine )
{ 
	int erg;
	do {
		erg=analizeLine(theLine);
	} while ( erg == 3 );
	return erg;
}

int VdHelp::parseHelpFile( const VdString & filename )
{
	int erg, isText;
	char * adrBuffer;
	VdTopicList topics ;
	VdTopic aTopic ;
	char title[128], aIdf[128];
	aTopic.theHelp=NULL ;
	lineCount=0;

	theHelpFile=fopen(filename.charptr(),"r") ;	
	if(!theHelpFile) return 0;
	   
	adrBuffer = buffer;
	erg = skipComment(&adrBuffer);

	while(erg != -1 ) {
		title[0] = aIdf[0] = 0;
	 	VdString theHelpString;

		if(erg!=1)  {
			cerr <<"Error in helpfile "
		     	<< filename.charptr()
		    	<< "   at line: "
		     	<< lineCount
		     	<< "\n<<< espected\n" ;
			exit(1) ;
		}
  		strcpy(aIdf,adrBuffer) ;

		adrBuffer = buffer;
		erg = skipComment(&adrBuffer);

		adrBuffer[strlen(adrBuffer)-1]='\0' ;
		if(erg!=0) {
			cerr <<"Error in helpfile "
		     	<< filename.charptr()
		    	 << "   at line: "
		     	<< lineCount
		     	<< "\nname for helpentry expected\n" ;
			exit(1) ;
		}
		strcpy(title,adrBuffer) ;

		do {
		    adrBuffer = buffer;
		    erg = skipComment(&adrBuffer);

		    if((erg!=0)&&(erg!=2)) {
			cerr <<"Error in helpfile "
		    	 << filename.charptr()
		     	<< "   at line: "
		     	<< lineCount
		     	<< "\n>>>SubtopicIdent or helptext expected \n" ;
			exit(1) ;
		    }
		    if(erg==2) {
			aTopic.theIdent=VdFeature(adrBuffer);
			topics.ins()=aTopic;
		    }
		} while(erg==2) ; 


		isText=0;
		while( erg==0 ) {
		    theHelpString+=VdString(adrBuffer);
		    isText=1;
		    adrBuffer = buffer;
	   	    erg = skipComment(&adrBuffer);
		}
		if(isText==0) {
			cerr <<"Error in helpfile "
		    	 << filename.charptr()
		     	<< "   at line: "
		     	<< lineCount
		     	<< "\nhelp text expected \n" ;
			exit(1) ;
		}
		VdHelpMemory *aHelp=
		    new VdHelpMemory(
			VdString(title),
			VdFeature(aIdf),
			topics,
			theHelpString,
			0);
		topics.reset() ;
	}

	fclose(theHelpFile);
	update();
	return 1;
}

#undef _VDHELP_CXX
