#define _VDHELPWINDOW_CXX

#include <Vd/VdHelpWindow.hxx>
#ifdef SIMULATION
#include <stdio.h>
#endif
#include <stdlib.h>

#define VdListClassName VdWidgetList
#define VdListClassElementType Widget

#include <Vd/VdListDefinition.mxx>

#undef VdListClassName
#undef VdListClassElementType

#include <Xm/MainW.h>
#include <Xm/Text.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/ScrollBar.h>
#include <Xm/Form.h>

VdHelpWindow::VdHelpWindow( VdHelpManager *manager )
{
	theManager = manager;
	theTopic = VdString("nothing");
	theHelpText = VdString("no help information");
	isMapped = 0;
#ifndef SIMULATION
	createMotifObjects();
	setMotifValues();
#endif
}

VdHelpWindow::VdHelpWindow(
		VdHelpManager *manager,
		VdString topic,
		VdStringList subTopics,
		VdString helpText)
{
	theManager = manager;
	theTopic = topic;
	theHelpText = helpText;
	theSubTopics = subTopics;
	isMapped = 0;
#ifndef SIMULATION
	createMotifObjects();
	setMotifValues();
#endif
}

VdHelpWindow::~VdHelpWindow( void )
{
//	XtUnmanageChild(theShell->getWidget());
//	XtDestroyWidget(theShell->getWidget());
	delete theShell;
}

void VdHelpWindow::set(	
		VdString topic,
		VdStringList subTopics,
		VdString helpText )
{
	theTopic = topic;
	theHelpText = helpText;
	theSubTopics = subTopics;
#ifdef SIMULATION
	if (isMapped) map();
	setMotifValues();
#else
	setMotifValues();
#endif
}

void VdHelpWindow::map( void )
{
#ifdef SIMULATION
	int i;
	printf("HelpFenster zu >>>%s<<<\n", theTopic.charptr());
	printf("Text:\n   %s\nEndeText\n", theHelpText.charptr());
	printf(" -3: close\n");
	printf(" -2: previous\n");
	printf(" -1: newWindow\n");
	for (i=0;i<!theSubTopics;i++)
	    printf("  %d: Hilfe zu >>>%s<<<\n",i,theSubTopics[i].charptr());
	printf("Auswahl: ");
	scanf("%d",&i);
	if (i == -3) { delete this; return; }
	if (theManager) theManager->handleSelection(i);
	map();
	
#else
	theShell->map();
#endif
}

void VdHelpWindow::unmap( void )
{
#ifdef SIMULATION
	printf("Fenster zu >>>%s<<< geschlossen\n", theTopic.charptr());
#else
	theShell->unmap();
#endif
}

void VdHelpWindow::createMotifObjects( void )
{
	Arg al[20];
	unsigned int ac;
	Widget aForm;

	theShell = new VdShell(*VdShell::theMainShellPtr,
			"VisamadHelp");
//	theShell->setGeometry(300,300,100,100);

	ac = 0;
	XtSetArg(al[ac], XmNcommandWindowLocation, 
		XmCOMMAND_BELOW_WORKSPACE); ac++;
	XtSetArg(al[ac], XmNshowSeparator, True); ac++;
	theMainWindow = XmCreateMainWindow( theShell->getWidget(),
			"VisamadHelpWindow", al, ac);

	theMenuBar = XmCreateMenuBar( theMainWindow,
			"VisamadMenuBar", NULL, 0);
	XtManageChild(theMenuBar);

	ac = 0;
	theCloseButton = XmCreateCascadeButton(theMenuBar, 
		"Close", al, ac);
	XtManageChild(theCloseButton);
	addCB(theCloseButton, activated);

	ac = 0;
	thePreviousButton = XmCreateCascadeButton(theMenuBar, 
		"Previous", al, ac);
	XtManageChild(thePreviousButton);
	addCB(thePreviousButton, activated);

	ac = 0;
	theWindowButton = XmCreateCascadeButton(theMenuBar, 
		"new Window", al, ac);
	XtManageChild(theWindowButton);
	addCB(theWindowButton, activated);

	theTopicBar = XmCreateMenuBar( theMainWindow,
			"VisamadMenuBar", NULL, 0);
	XtManageChild(theTopicBar);

	ac = 0;
	aForm = XmCreateForm(theMainWindow,
		"VisamadHelpForm",al,ac);
	XtManageChild(aForm);

	ac = 0;
	XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM );
	ac++;
	XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM );
	ac++;
	XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM );
	ac++;
	XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM ); 
	ac++;
	XtSetArg(al[ac], XmNeditable, False); ac++;
	XtSetArg(al[ac], XmNeditMode, XmMULTI_LINE_EDIT); ac++;
	XtSetArg(al[ac], XmNscrollHorizontal,False); ac++;
	XtSetArg(al[ac], XmNwordWrap, True); ac++;
	theHelpDisplay = XmCreateScrolledText(
		aForm,
		"VisamadHelpText",
		al, ac);

	XtManageChild(theHelpDisplay);


	XmMainWindowSetAreas( 
		theMainWindow, 
		theMenuBar,
		theTopicBar,
		NULL, NULL,
		aForm );

	XtManageChild(theMainWindow);

}

void VdHelpWindow::setMotifValues( void )
{
	Widget button;
	int i, isEmpty=1;
	Arg al[20];
	unsigned int ac;

	theShell->setTitle(theTopic.charptr());

	XtUnmanageChild(theMainWindow);
	XtUnmanageChild(theTopicBar);
	if (!theTopicButtons>0) {
	    for (i=0; i<!theTopicButtons; i++) {
		XtUnmanageChild(theTopicButtons[i]);
		XtDestroyWidget(theTopicButtons[i]);
	    }
	}
	theTopicButtons.reset();
	for (i=0; i<!theSubTopics; i++) {
		ac = 0;
		button = XmCreateCascadeButton(
		    theTopicBar, 
		    theSubTopics[i].charptr(), al, ac);
		theTopicButtons.ins() = button;
		if (theSubTopics[i].length()>0) {
		    XtManageChild(button);
		    addCB(button, activated);
		    isEmpty = 0;
		}
	}
	if (!isEmpty)
	    XtManageChild(theTopicBar);
	
	XmTextSetString(theHelpDisplay, 
		theHelpText.charptr());

	XtManageChild(theMainWindow);
}

void VdHelpWindow::handleActivated( void )
{
	Widget button = getLastWidgetOfEvent();
	int i;

	if (!theManager) return;
	if ( button == theCloseButton) {
		theManager->handleSelection(-3);
		delete this; return;
	} else if ( button == thePreviousButton ) {
		i = -2;
	} else if ( button == theWindowButton ) {
		i = -1;
	} else {
		i = theTopicButtons.idx(button);
	}
	theManager->handleSelection(i);
}

#undef _VDHELPWINDOW_CXX
