#define _VDHEXAGON_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdHexagon.cxx                                     *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdHexagon.hxx>


VdHexagon::VdHexagon(int aa, int bb )
{ a=(aa<0)?-aa:aa; b=(bb<0)?-bb:bb;
  include(&uleft);
  include(&dleft);
  include(&uright);
  include(&dright);
  include(&up);
  include(&down);
}

VdPicture* VdHexagon::duplicate(void)
{ VdHexagon* pic = new VdHexagon(a,b);
  *((VdPicture*)pic)=*((VdPicture*)this);
  return pic;
}

void VdHexagon::setAxisA( int aa )
{ a=(aa<0)?-aa:aa;
}

void VdHexagon::setAxisB( int bb )
{ b=(bb<0)?-bb:bb;
}

void VdHexagon::setRectangle( const VdArea & ar )
{ a=ar.lr.x-ar.ul.x+20;
  b=ar.ul.y-ar.lr.y;
}

int VdHexagon::getAxisA( void ) const
{ return a;
}

int VdHexagon::getAxisB( void ) const
{ return b;
}

VdPoint VdHexagon::getBorderPoint( VdPoint & p )
{ VdVector v;
  int x, y, r, xr, h1, h2;
  v = p-getFramePoint(VdXCENTRE|VdYCENTRE);
  if (a<=20) r=a>>1;
  else r=10;
  xr=(a>>1); x=xr-r;
  y=b>>1;
  if (y==0) return getFramePoint(VdXCENTRE|VdYCENTRE);
  if (v!=VdNullVector)
  { if ((v.x*y-v.y*x)<=0)
    { if ((v.x*y+v.y*x)<=0)
      { if (v.y>=0)  { h1=y*xr; h2=v.y*r-v.x*y; }
        else  { h1=-y*xr; h2=v.y*r+v.x*y; }
        v*=h1; v/=h2;
      }
      else { h2=v.y; v*=y; v/=h2; }
    }
    else
    { if ((v.x*y+v.y*x)>=0)
      { if (v.y<0)  { h1=-y*xr; h2=v.y*r-v.x*y; }
        else  { h1=y*xr; h2=v.y*r+v.x*y; }
        v*=h1; v/=h2;
      }
      else { h2=v.y; v*=-y; v/=h2; }
    }
  }
  return getFramePoint(VdXCENTRE|VdYCENTRE)+v;
}

void VdHexagon::manipulate( void )
{ int r, bl;
  if (a&1) a++;
  if (b&1) b++;
  if (a<=20) r=a>>1;
  else r=10;
  bl=b>>1;
  up.setStartPoint(VdPoint(r,b));
  up.setEndPoint(VdPoint(a-r,b));
  down.setStartPoint(VdPoint(r,0));
  down.setEndPoint(VdPoint(a-r,0));
  uleft.setStartPoint(VdPoint(r,b));
  uleft.setEndPoint(VdPoint(0,bl));
  dleft.setStartPoint(VdPoint(r,0));
  dleft.setEndPoint(VdPoint(0,bl));
  uright.setStartPoint(VdPoint(a-r,b));
  uright.setEndPoint(VdPoint(a,bl));
  dright.setStartPoint(VdPoint(a-r,0));
  dright.setEndPoint(VdPoint(a,bl));
}

#undef _VDHEXAGON_CXX
