#define _VDMAGNIFIERWINDOW_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdMagnifierWindow.cxx                             *
*  Date: 30-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: B. Miethig, J. Hutschenreiter                   *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Implementation of class VdMagnifierWindow:           *
*                                                          *
*     Interface                                            *
*          VdMagnifierWindow.hxx                           *
*                                                          *
***********************************************************/

/******************    dependences    *********************/


extern "C" {
#include <stdio.h>
#include <stdlib.h>
}
#include <iostream.h>
#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/DialogS.h>
#include <Vd/VdCBmanager.hxx>
#include <Vd/VdAbstractWindow.hxx>
#include <Vd/VdWindowConnection.hxx>
#include <Vd/VdMagnifierWindow.hxx> 
#include <Vd/VdGraphicAttributes.hxx>
#include <Vd/VdFractNumber.hxx>
#include <Vd/VdEventHandling.hxx>
#include <Vd/VdButton.hxx>
#include <Vd/VdPulldownOptions.hxx>
#include <Vd/VdMagnifierButton.hxx>
#include <Vd/VdRadioBoxInterface.hxx>

const Cardinal curPointer[10] = {   
   XC_crosshair,
   XC_fleur,
   XC_top_side,
   XC_top_right_corner,
   XC_right_side,
   XC_bottom_right_corner,
   XC_bottom_side,
   XC_bottom_left_corner,
   XC_left_side,
   XC_top_left_corner
};


//*** local VdMagnifier class implementation *************

static int changed=0; 

VdMagnifier::VdMagnifier(const VdPoint &aul,
                         const VdPoint &alr, 
                         int aBorderThick,
                         int aCornerThick,
                         const VdMagnifierMode aMode
                         ) 
 {
   ul=aul ;
   lr=alr ;
   borderThick=aBorderThick ;
   cornerThickW=cornerThickH=aCornerThick ;
   theMode=aMode;
   theHandling=Vd_MAGNIFIER_DRAGS ; 
   theWork=0 ;  
   position=10 ;
   cor.x=cor.y=borderThick>>1 ;
   widthToHeight.setNumerator(lr.x-ul.x+2*cor.x);
   widthToHeight.setDenominator(lr.y-ul.y+2*cor.y); 
 }


int VdMagnifier::getBorderThick(void) 
 {
   return borderThick;
 }

void VdMagnifier::setBorderThick(int aBorderThick)
 {
   borderThick=aBorderThick ;
   cor.x=cor.y= borderThick>1 ;
 }
       

unsigned int VdMagnifier::magnifierArea(VdPoint &aPoint)
{
 if (aPoint.x<=(ul.x-borderThick)||aPoint.x>=lr.x+borderThick   //out
    ||aPoint.y<=ul.y-borderThick||aPoint.y>=lr.y+borderThick) return 10 ;

 if (aPoint.x<=ul.x)  /*left Rand  */
   {   
    if (aPoint.y<=ul.y+cornerThickH) return 9 ;
    if (aPoint.y>=lr.y-cornerThickH) return 7 ;
    return 8 ;
   } ;
 if  (aPoint.x>=lr.x)  /* right Rand  */
   {
    if (aPoint.y<=ul.y+cornerThickH) return 3 ;
    if (aPoint.y>=lr.y-cornerThickH) return 5 ;
    return 4 ;
   } ;
 if  (aPoint.y>=lr.y)  /* lower Rand  */
   {    if (aPoint.x<=ul.x+cornerThickW) return 7 ;
    if (aPoint.x>=lr.x-cornerThickW) return 5 ;
    return 6 ;
   } ;
 if  (aPoint.y<=ul.y)  /* upper Rand  */
   {
    if (aPoint.x<=ul.x+cornerThickW) return 9 ;
    if (aPoint.x>=lr.x-cornerThickW) return 3 ;
    return 2 ;
   } ;
 return 1 ;
}

//******** VdMagnifierWindow methods *************************

VdMagnifierWindow::VdMagnifierWindow( VdShell &shell,
                Boolean withScrollbars):
	VdAbstractWindow(shell, withScrollbars)
{
        theConnection = NULL;
        cursor=XCreateFontCursor(getDisplay(),curPointer[0]);
        ownGC = NULL;
        theGrabWindow=NULL ;
	exitButtonPtr = NULL;
	modeButtonPtr = NULL;
	windowButtonPtr = NULL;
	theHm=NULL ;
	 
}

VdMagnifierWindow::~VdMagnifierWindow( void )
{
	if (theConnection) { 
		delete theConnection;
		theConnection = NULL;
	} 
}

void VdMagnifierWindow::createGC( void )
{
        if (ownGC) return;
        VdPoint p1,p2 ;
        VdPoint v(getDrawableSize()) ;
        p1.x=v.x>>2 ;
        p1.y=v.y>>2 ;
        p2.x=p1.x*3 ;
        p2.y=p1.y*3 ; 
   	theMagnifier.ul=p1+theMagnifier.cor;
   	theMagnifier.lr=p2-theMagnifier.cor;
   	theMagnifier.widthToHeight.setNumerator(theMagnifier.lr.x
            -theMagnifier.ul.x+2*theMagnifier.cor.x) ;
   	theMagnifier.widthToHeight.setDenominator(theMagnifier.lr.y
            -theMagnifier.ul.y+2*theMagnifier.cor.x) ;
	ownGC = XCreateGC( getDisplay(), 
			XtWindow(getDrawable()), 0, NULL);
	XCopyGC( getDisplay(), getGC(), 
			-1, ownGC);
   	XSetFunction(getDisplay(),ownGC,GXxor) ;
   	XSetForeground(getDisplay(),ownGC,-1) ;
	XSetPlaneMask(getDisplay(),ownGC,0x1fl);
        theGrabWindow=XCreateWindow(
                  getDisplay() ,
                  XtWindow(getDrawable()) , 
                  0,0,
 		  10,10,
                  0,0, 
                  InputOnly,
                  XDefaultVisual(getDisplay(),0) ,
                  0,NULL) ;
         XMapWindow(getDisplay(),theGrabWindow) ;
}


VdMagnifierMode VdMagnifierWindow::getMode(void) 
{
   return theMagnifier.theMode ;
}  


void VdMagnifierWindow::setMode(VdMagnifierMode aMode) 
{
   if(aMode<6) theMagnifier.theMode=aMode ;
}  

void VdMagnifierWindow::setWindowType(VdWindowType wt)
{
	VdAbstractWindow::setWindowType(wt);
	if (modeButtonPtr && theConnection) {
		switch (theConnection->getMode()) {
		case Vd_DEFAULT_DEFAULT:
			modeButtonPtr->setSelected(1);
			break;
		case Vd_DEFAULT_BUBBLE:
			modeButtonPtr->setSelected(2);
			break;
		case Vd_BUBBLE_DEFAULT: 
			modeButtonPtr->setSelected(3);
			break;
		case Vd_BUBBLE_BUBBLE:
			modeButtonPtr->setSelected(4);
			break;
		}
		modeButtonPtr->handleValueChanged(); 
	}
}

VdMagnifierHandling VdMagnifierWindow::getHandling(void)
{
   return theMagnifier.theHandling ;
}

void VdMagnifierWindow::setHandling(VdMagnifierHandling aHandling)
{
  if(aHandling<2) theMagnifier.theHandling=aHandling ;
}



void VdMagnifierWindow::setGrabWindow(VdPoint &aPoint) 
 {
   int x,y;
   unsigned int height,width ;  
   if(theMagnifier.position==10) return ;

   switch (theMagnifier.position) {
  case 1: x= aPoint.x-theMagnifier.ul.x 
                    +theMagnifier.cor.x;
          y= aPoint.y-theMagnifier.ul.y
                    +theMagnifier.cor.y;
          width=getDrawableSize().x
                -theMagnifier.lr.x+theMagnifier.ul.x
                -theMagnifier.getBorderThick ();
          height=getDrawableSize().y-theMagnifier.lr.y
                 +theMagnifier.ul.y-theMagnifier.getBorderThick(); 
                 break ;

  case 2:   x=0 ;
            y=aPoint.y-theMagnifier.ul.y
              +theMagnifier.cor.y;
            width=getDrawableSize().x ;
            height=theMagnifier.lr.y
                  -2*theMagnifier.cornerThickH
                 +theMagnifier.cor.y;
            break ;

  case 3:  x=theMagnifier.ul.x
                  +2*theMagnifier.cornerThickW
                  +aPoint.x-theMagnifier.lr.x
                  -theMagnifier.getBorderThick() ; 
             y=aPoint.y-theMagnifier.ul.y
                  +theMagnifier.cor.y ;
             width=getDrawableSize().x-x
                +aPoint.x-theMagnifier.lr.x 
                +theMagnifier.cor.x;
             height=theMagnifier.lr.y
                -2*theMagnifier.cornerThickH
                +theMagnifier.cor.y ;
        break ; 
             
 case 4:   x=theMagnifier.ul.x
             +2*theMagnifier.cornerThickW
             +aPoint.x-theMagnifier.lr.x
             -theMagnifier.getBorderThick() ; 
           y=0 ;
           width=getDrawableSize().x-x
                +aPoint.x-theMagnifier.lr.x
                +theMagnifier.cor.x ;
           height=getDrawableSize().y ;
       break ;

  case 5: x=theMagnifier.ul.x
              +2*theMagnifier.cornerThickW
              +aPoint.x-theMagnifier.lr.x
              -theMagnifier.getBorderThick() ; 
          y=theMagnifier.ul.y
              +2*theMagnifier.cornerThickH
              -theMagnifier.lr.y+aPoint.y
              -theMagnifier.getBorderThick() ; 
          width=getDrawableSize().x-x+aPoint.x
                -theMagnifier.lr.x+theMagnifier.cor.x ;
          height=getDrawableSize().y-y-theMagnifier.lr.y
               +aPoint.y+theMagnifier.cor.y ;
       break ;

  case 6: x=0 ;
          y=theMagnifier.ul.y
            +2*theMagnifier.cornerThickH
            -theMagnifier.lr.y+aPoint.y
            -theMagnifier.getBorderThick() ; 
          width=getDrawableSize().x ;
          height=getDrawableSize().y-y-theMagnifier.lr.y
            +aPoint.y+theMagnifier.cor.y ;
       break ;

  case 7: x=aPoint.x-theMagnifier.ul.x+theMagnifier.cor.x ;
              y=theMagnifier.ul.y
                 +2*theMagnifier.cornerThickH
                 -theMagnifier.lr.y+aPoint.y
                 -theMagnifier.getBorderThick() ; 
          width=theMagnifier.lr.x
                -2*theMagnifier.cornerThickW
                +theMagnifier.cor.x ;
          height=getDrawableSize().y-y-theMagnifier.lr.y
            +aPoint.y+theMagnifier.cor.y ;
       break ;

  case 8: x=aPoint.x-theMagnifier.ul.x+theMagnifier.cor.x ;
              y=0 ;
              width=theMagnifier.lr.x
                -2*theMagnifier.cornerThickW
                +theMagnifier.cor.x ;
              height=getDrawableSize().y ;
       break ;

  case 9: x=aPoint.x-theMagnifier.ul.x+theMagnifier.cor.x ;
              y=aPoint.y-theMagnifier.ul.y+theMagnifier.cor.y ;
             width=theMagnifier.lr.x
                -2*theMagnifier.cornerThickW
                +theMagnifier.cor.x ;
             height=theMagnifier.lr.y
                 -2*theMagnifier.cornerThickH
                +theMagnifier.cor.y ;
        break ;   
 }
//   if (width==0) width=1; if (height==0) height=1;
	width++; height++;

   XMoveResizeWindow(getDisplay(),theGrabWindow,
          x,y,width,height) ;
  theGrabWindowCor.x=x ;
  theGrabWindowCor.y=y ;
 } 


void VdMagnifierWindow::setPropGrabWindow(VdPoint &aPoint)
{
   int x,y;
   unsigned int height,width,corShort,corLong ; 
   if(theMagnifier.position==10) return ;
   VdFractNumber wtH,one(1,1) ; 
   corShort=20 ;
   corLong=((VdFractNumber)(20)*theMagnifier.widthToHeight).getIntValue() ;  
 
  switch (theMagnifier.position) {
  case 3:  
            y=theMagnifier.cor.y  ;
            wtH.setNumerator(getDrawableSize().x-theMagnifier.lr.x) ;
            wtH.setDenominator(theMagnifier.ul.y) ;
            if(theMagnifier.widthToHeight>one)
                     {
                      x=theMagnifier.ul.x+corLong           
                       -theMagnifier.getBorderThick() ; 
                     }

            if (theMagnifier.widthToHeight<=one)
                     {
                      x=theMagnifier.ul.x+corShort
                       -theMagnifier.getBorderThick() ; 
                      }

            width=getDrawableSize().x  ;    

             if(theMagnifier.widthToHeight>one)
                     {
                         height=theMagnifier.lr.y-corShort
                                 +theMagnifier.cor.y  ; 
                      }

           if (theMagnifier.widthToHeight<=one)
                      {
                        height=theMagnifier.lr.y-
                             corLong                        
                            +theMagnifier.cor.y-y  ; 
                       }

           if(wtH>=theMagnifier.widthToHeight)
                 {
                    width=((VdFractNumber)(theMagnifier.lr.y)
                          *theMagnifier.widthToHeight)
                          .getIntValue() ;
                 }
             else     
                       {      
                 y=theMagnifier.lr.y-((VdFractNumber)
                     (getDrawableSize().x-theMagnifier.ul.x-theMagnifier.getBorderThick())
                    /theMagnifier.widthToHeight).getIntValue() 
                    +theMagnifier.cor.y ;
                 height=theMagnifier.lr.y-y ;
                        }
            
           if(theMagnifier.widthToHeight>one)
                     {	
                      width-=corLong ;
                      height-=corShort ;
                     }
           else
                    { 	
                      width-=corShort ;
                      height-=corLong ;
                     }	
  
        break ; 
             
  case 5: if(theMagnifier.widthToHeight>one)
                     {
                      x=theMagnifier.ul.x+
                         corLong
                        -theMagnifier.cor.x ;
                      y=theMagnifier.ul.y+corShort
                        -theMagnifier.cor.y;
                       }
               if (theMagnifier.widthToHeight<=one)
                      {
                      x=theMagnifier.ul.x+corShort
                        -theMagnifier.cor.x;
                      y=theMagnifier.ul.y+
                         corLong
                         -theMagnifier.cor.y;
                      }
               wtH.setNumerator(getDrawableSize().x-x) ;
               wtH.setDenominator(getDrawableSize().y-y) ;
               width=getDrawableSize().x+theMagnifier.cor.x ;
               height=getDrawableSize().y+theMagnifier.cor.y ; 
               if(wtH>=theMagnifier.widthToHeight) 
                        width=(theMagnifier.widthToHeight*
                                    (getDrawableSize().y-theMagnifier.ul.y)) .
                                     getIntValue() ;           
               else height=( ((VdFractNumber)
                                     (getDrawableSize().x-theMagnifier.ul.x)) 
			    / theMagnifier.widthToHeight ).
                                        getIntValue() ;
               if(theMagnifier.widthToHeight>one)
                     {	
                      width-=corLong ;
                      height-=corShort;
                     }
               else
                    { 	
                      width-=corShort;
                      height-=corLong;
                     }	
       break ;

  case 7:
       x= theMagnifier.cor.x;
       wtH.setNumerator(theMagnifier.ul.x) ;
       wtH.setDenominator(getDrawableSize().y-theMagnifier.lr.y) ;
       if(theMagnifier.widthToHeight>one)
                     {
                           y=theMagnifier.ul.y+corShort
                      //       +aPoint.y-theMagnifier.lr.y 
                            -theMagnifier.getBorderThick() ; 
                      }

       if (theMagnifier.widthToHeight<=one)
                      {
                           y=theMagnifier.ul.y+
                          corLong
                      //    +aPoint.y-theMagnifier.lr.y 
                        -theMagnifier.getBorderThick() ; 
                      }

       height=getDrawableSize().y ;  

       if(theMagnifier.widthToHeight>one)
                     {
                    width=theMagnifier.lr.x-
                          corLong            
                          +theMagnifier.cor.x-x ;
                      }

        if (theMagnifier.widthToHeight<=one)
                      {
                      width=theMagnifier.lr.x-corShort
                        +theMagnifier.cor.x ; 
                       }

           if(wtH<theMagnifier.widthToHeight)
                 {
                   height=(((VdFractNumber)
                                  ( theMagnifier.lr.x))/theMagnifier.widthToHeight) .
                                     getIntValue()-20
                            +aPoint.y-theMagnifier.lr.y ;
                 }
             else     
                       {      
                 x=theMagnifier.lr.x-((VdFractNumber)
                     (getDrawableSize().y-theMagnifier.ul.y-theMagnifier.getBorderThick())
                   *theMagnifier.widthToHeight).getIntValue() 
                    +theMagnifier.cor.x;
                width=theMagnifier.lr.x-20-x ;
                        }
   
 
          break ;

   case 9: 
         wtH.setNumerator(theMagnifier.ul.x) ;
         wtH.setDenominator(theMagnifier.ul.y) ;
         x=0 ;
         y=0;
         width= theMagnifier.lr.x +theMagnifier.cor.x-x ;
         height= theMagnifier.lr.y+theMagnifier.cor.y  ; 
         if(theMagnifier.widthToHeight>one)
                     {
                    width=theMagnifier.lr.x-
                          corLong
                          +theMagnifier.cor.x-x ;
                      }

           if (theMagnifier.widthToHeight<=one)
                      {
                      width=theMagnifier.lr.x-corShort
                        +theMagnifier.cor.x ; 
                       }


           if(wtH>=theMagnifier.widthToHeight)
                 {
                 x=theMagnifier.lr.x-((VdFractNumber)
                     (theMagnifier.lr.y)
                   *theMagnifier.widthToHeight).getIntValue() 
                   +theMagnifier.cor.x ;
                width=theMagnifier.lr.x-x ;
                        }
           else
                    {      
                 y=theMagnifier.lr.y-((VdFractNumber)
                     (theMagnifier.lr.x)
                    /theMagnifier.widthToHeight).getIntValue() 
                     +theMagnifier.cor.y;
                 height=theMagnifier.lr.y-y ;
                    }

     if(theMagnifier.widthToHeight>one) 
              {
                 width-=corLong ;
                 height-=corShort ;
               } 
     else
               { 
                 width-=corShort ;
                 height-=corLong ;
                } 
  break ;   
 }


  XMoveResizeWindow(getDisplay(),theGrabWindow,
          x,y,width,height) ;
  theGrabWindowCor.x=x ;
  theGrabWindowCor.y=y ;

#ifdef  TEST_BM
  cerr <<" Grab Window daten:   "
           <<" links oben: " << x  <<"   " << y <<"\n"
           <<" width:  "  << width   <<"    height:  "  << height  <<"\n" 
           <<"rest :  "  <<  getDrawableSize().x-x  << "  "   << getDrawableSize().y-y  <<"\n" ;
#endif
}

void VdMagnifierWindow::getMagnifierData(VdPoint &aul, VdPoint &alr)
 {
  aul=theMagnifier.ul-theMagnifier.cor ;
  alr=theMagnifier.lr+theMagnifier.cor ;
 } 


void VdMagnifierWindow::setMagnifierData(const VdPoint aul
                        ,const VdPoint alr, int toRedraw) 
 {
    VdVector v(getDrawableSize()),
                       diag(alr.x-aul.x, alr.y-aul.y );
   int ok=1 ;
   if (toRedraw) drawMagnifier();
   if (0>aul.x) ok=0 ;
   if (0>aul.y) ok=0 ;
   if (diag.x<2*theMagnifier.cornerThickW) ok=0 ;
   if (diag.y<2*theMagnifier.cornerThickW) ok=0 ; 
   if ((aul+diag).x>v.x) ok=0 ;
   if ((aul+diag).y>v.y) ok=0 ;
   if (ok)  
        {
        theMagnifier.ul=aul+theMagnifier.cor;
        theMagnifier.lr=alr-theMagnifier.cor;
        } ;
  if(!ok)
      {
          theConnection->magnifierResized(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ;
#ifdef TEST_BM
          cerr << "incorrect magnifierdata \n" ;
#endif
      }
 
   theMagnifier.widthToHeight.setNumerator(theMagnifier.lr.x
          -theMagnifier.ul.x+2*theMagnifier.cor.x) ;
   theMagnifier.widthToHeight.setDenominator(theMagnifier.lr.y
          -theMagnifier.ul.y+2*theMagnifier.cor.x) ;  
   if (toRedraw) drawMagnifier();
 }


void VdMagnifierWindow::drawMagnifier(void)
 { int thick = theMagnifier.getBorderThick();
   int diff = thick >> 1;
   
   if(!XtWindow(getDrawable())) return ;
   createGC();
   XDrawRectangle(
	getDisplay(),XtWindow(getDrawable()),ownGC,
        theMagnifier.ul.x-diff,theMagnifier.ul.y-diff,
        theMagnifier.lr.x-theMagnifier.ul.x+thick,
        theMagnifier.lr.y-theMagnifier.ul.y+thick) ; 
 } 


void VdMagnifierWindow::realizeIt(const VdPoint &aPointer)
{
  int wi,he ;
  switch (theMagnifier.position)
          {
  case 3:
        wi= aPointer.x-theMagnifier.ul.x+theMagnifier.cor.x; 
        he=theMagnifier.lr.y-aPointer.y+theMagnifier.cor.y;
        break ;
  case 5:
        wi= aPointer.x-theMagnifier.ul.x+theMagnifier.cor.x; 
        he=aPointer.y-theMagnifier.ul.y+theMagnifier.cor.y;
        break ;
  case 7:
        wi=theMagnifier.lr.x-aPointer.x+theMagnifier.cor.x;
        he=aPointer.y-theMagnifier.ul.y+theMagnifier.cor.y; 
        break ;
  case 9:
        wi=theMagnifier.lr.x-aPointer.x+theMagnifier.cor.x;
        he=theMagnifier.lr.y-aPointer.y+theMagnifier.cor.y;
        break ;
         }

 VdFractNumber wiToHe(wi,he) ;

#ifdef TEST_BM
   cerr << "altes Verhaeltnis:"
        << theMagnifier.widthToHeight.getFloatValue() 
        << "\n" ;
       cerr << "    neues Verhaeltnis:"
        <<  wiToHe.getFloatValue()  
        << "    lang"  <<wi    
        << "   breit" << he 
        << "\n" ;
#endif

  if(wiToHe==theMagnifier.widthToHeight) return ;
  if(wiToHe>theMagnifier.widthToHeight)    // over diagonale
       {
#ifdef TEST_BM
       cerr <<" ber der Diagonale      neue Hhe  falsch    " ; 
#endif
       he=(((VdFractNumber)wi)/theMagnifier.widthToHeight).getIntValue() ;
#ifdef TEST_BM
       cerr  << "korigiere Hhe auf  "
             <<  he
             << "\n" ;  
#endif
       } 
  else
       {
#ifdef TEST_BM
       cerr << " Unter der Diagonale    neue Breite falsch:     ";            
#endif
       wi=(((VdFractNumber)he)
        *theMagnifier.widthToHeight).getIntValue() ;                       
     
#ifdef TEST_BM
             cerr  << "korigiere Breite auf  "
             <<  wi
             << "\n" ;     
#endif
       } 

 he-=theMagnifier.getBorderThick() ;
 wi-=theMagnifier.getBorderThick() ;

 switch (theMagnifier.position)
         {
 case  3: theMagnifier.ul.y=theMagnifier.lr.y-he ;
              theMagnifier.lr.x=theMagnifier.ul.x+wi ;
              break ;
 case 5:  theMagnifier.lr.x=theMagnifier.ul.x+wi ;
              theMagnifier.lr.y=theMagnifier.ul.y+he ;
              break ;
 case 7:  theMagnifier.lr.y=theMagnifier.ul.y+he ;
              theMagnifier.ul.x=theMagnifier.lr.x-wi ;
              break ;
 case 9:  theMagnifier.ul.x=theMagnifier.lr.x-wi ;
              theMagnifier.ul.y=theMagnifier.lr.y-he ;
               break ;
         }

}


void VdMagnifierWindow::manipulateMagnifier(
                              const VdVector &aVector,
                              const VdPoint &thePointer)
{
 switch (theMagnifier.position) {
 case 1:drawMagnifier() ;
	 theMagnifier.ul.x=theMagnifier.ul.x+aVector.x;
	 theMagnifier.ul.y=theMagnifier.ul.y+aVector.y;
	 theMagnifier.lr.x=theMagnifier.lr.x+aVector.x;
	 theMagnifier.lr.y=theMagnifier.lr.y+aVector.y;
             break ;
 case 2: drawMagnifier() ;
	 theMagnifier.ul.y=theMagnifier.ul.y+aVector.y;
         break ;
 case 3: drawMagnifier() ;
	 theMagnifier.ul.y=theMagnifier.ul.y+aVector.y;
	 theMagnifier.lr.x=theMagnifier.lr.x+aVector.x;
         break ;
 case 4: drawMagnifier() ;
	 theMagnifier.lr.x=theMagnifier.lr.x+aVector.x;
	 break ;
 case 5: drawMagnifier() ;
	 theMagnifier.lr.x=theMagnifier.lr.x+aVector.x ;
	 theMagnifier.lr.y=theMagnifier.lr.y+aVector.y;
	 break ;
 case 6: drawMagnifier() ;
	 theMagnifier.lr.y=theMagnifier.lr.y+aVector.y ;
	 break ;
 case 7: drawMagnifier() ;
	 theMagnifier.ul.x=theMagnifier.ul.x+aVector.x ;
	 theMagnifier.lr.y=theMagnifier.lr.y+aVector.y ;
	 break ;
 case 8: drawMagnifier() ;
	 theMagnifier.ul.x=theMagnifier.ul.x+aVector.x ;
	 break ;
 case 9: drawMagnifier() ;
	 theMagnifier.ul.x=theMagnifier.ul.x+aVector.x ;
	 theMagnifier.ul.y=theMagnifier.ul.y+aVector.y ;
	 break ;
 case 10:break;
    } ;
 

 if((theMagnifier.theMode==Vd_MAGNIFIER_PROPORTIONAL_MODE)
    &&(!(theMagnifier.position==1))) realizeIt(thePointer) ;
 if(! ((theMagnifier.position==1)||(theMagnifier.theMode==Vd_MAGNIFIER_PROPORTIONAL_MODE)))
    {
 theMagnifier.widthToHeight.setDenominator(theMagnifier.lr.y
         -theMagnifier.ul.y+theMagnifier.getBorderThick()) ;
 theMagnifier.widthToHeight.setNumerator(theMagnifier.lr.x
         -theMagnifier.ul.x+theMagnifier.getBorderThick()) ; 
       } ; 
 if(theMagnifier.position<10) drawMagnifier() ; 
}


void VdMagnifierWindow::setCursor(int which)
           {
             Window w;
             w=XtWindow(getDrawable()) ;
             XUndefineCursor(getDisplay(),w) ;
             XFreeCursor(getDisplay(),cursor) ; 
             cursor = XCreateFontCursor(getDisplay(),curPointer[which]);
             XDefineCursor(getDisplay(),w,cursor);
           } ;
            


void  VdMagnifierWindow::callbackButtonPressed(VdEvent *ev)
     {
       VdPoint poi,oldUl,oldLr,oldCorner ;
       Window w,root,child ;
       Display *display ;
       unsigned int mask ; 
       int  root_x,root_y,win_x,win_y,limit=0;
       if(ev->type==Vd_LEFT_MOUSE_BUTTON_PRESSED) 
               theMagnifier.theWork=0 ;
       else theMagnifier.theWork=1 ; 
       if(ev->type==Vd_RIGHT_MOUSE_BUTTON_PRESSED) 
		{
		theMagnifier.theWork=2 ;
		theMagnifier.theMode=Vd_MAGNIFIER_RESIZEABLE_MODE ;
		 }
       oldUl=theMagnifier.ul ;
       oldLr=theMagnifier.lr ;  
       display=getDisplay() ;
       poi=ev->location ;
       w=XtWindow(getDrawable()) ;
       oldCorner.x=theMagnifier.cornerThickW ; 
       oldCorner.y=theMagnifier.cornerThickH ;
       theMagnifier.position=theMagnifier.magnifierArea(poi) ;  
       setGrabWindow(poi) ;
       if ((theMagnifier.theMode==Vd_MAGNIFIER_PROPORTIONAL_MODE)
             && (theMagnifier.position>1)) 
             {
               theMagnifier.cornerThickW=1+(oldLr.x-oldUl.x)/2;
               theMagnifier.cornerThickH=1+(oldLr.y-oldUl.y)/2 ;
               theMagnifier.position=theMagnifier.magnifierArea(poi) ;
               setPropGrabWindow(poi) ;
             }
       theMagnifier.position=theMagnifier.magnifierArea(poi) ;
       poi=poi-theGrabWindowCor ;	
       if (theMagnifier.position<10)
          {
            if(theMagnifier.theMode==Vd_MAGNIFIER_NON_RESIZEABLE_MODE)
                      theMagnifier.position=1 ;
            setCursor(theMagnifier.position) ;
	    XGrabPointer(display,w,0,
                     PointerMotionMask|ButtonReleaseMask,
		     GrabModeAsync,GrabModeAsync,theGrabWindow
                                               ,None,CurrentTime) ;
	    do
	     {
	     XQueryPointer(display,theGrabWindow,&root,&child,&root_x,&root_y,
			 &win_x,&win_y,&mask) ;

	     if (poi.x!=win_x || poi.y!=win_y)
	      {
	      manipulateMagnifier(VdVector(-poi.x+win_x,-poi.y+win_y),
              		VdPoint(win_x,win_y)+theGrabWindowCor) ;
              limit+= abs(poi.x-win_x)+abs(poi.y-win_y) ;           
              poi.x=win_x ;             
              poi.y=win_y ;
               } 

              if((theMagnifier.theMode==Vd_MAGNIFIER_DRAGS)
                  &&(limit>=3))
               {
               limit=0 ;
               if(theConnection && theMagnifier.theWork==1)
               {
                if(theMagnifier.position==1) 
                theConnection->
                  worldMoved(
                       theMagnifier.ul-theMagnifier.cor,
                       theMagnifier.lr+theMagnifier.cor) ;
                else
                theConnection->
                 worldResized(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ;        
                }

                if(theConnection && theMagnifier.theWork==0)
                {
                 if(theMagnifier.position==1) 
                 theConnection->
                  magnifierMoved(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ;
                 else
                 theConnection->
                 magnifierResized(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ; 
                  } 

	        if(theConnection && theMagnifier.theWork==2)
                {
                 if(theMagnifier.position==1) 
                 theConnection->
                  displayMoved(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ;
                 else
                 theConnection->
                 displayResized(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ; 
                  } 
                 } 	      
            } while(mask) ;

            if((theMagnifier.ul!=oldUl)||(theMagnifier.lr!=oldLr))
               changed=1 ; 
                   };
        theMagnifier.cornerThickW=oldCorner.x ;
        theMagnifier.cornerThickH=oldCorner.y ; 
                } 



void VdMagnifierWindow::callbackButtonReleased(VdEvent *)
          {
           XUngrabPointer(getDisplay(),CurrentTime) ;
           setCursor(0) ;
           if(changed==0) return ; 
           if(theConnection && theMagnifier.theWork==1)
              {
               if(theMagnifier.position==1) 
                theConnection->
                  worldMoved(
                       theMagnifier.ul-theMagnifier.cor,
                       theMagnifier.lr+theMagnifier.cor) ;
               else
                theConnection->
                 worldResized(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ;        
               }
           if(theConnection && theMagnifier.theWork==0)
              {
               if(theMagnifier.position==1) 
                theConnection->
                  magnifierMoved(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ;
               else
                theConnection->
                 magnifierResized(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ; 
              } 
           if(theConnection && theMagnifier.theWork==2)
              {
               if(theMagnifier.position==1) 
                theConnection->
                  displayMoved(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ;
               else
                theConnection->
                 displayResized(
                      theMagnifier.ul-theMagnifier.cor,
                      theMagnifier.lr+theMagnifier.cor) ;
		if((theConnection->getMode()==Vd_DEFAULT_DEFAULT)||
		   (theConnection->getMode()==Vd_DEFAULT_BUBBLE))
		   theMagnifier.theMode=Vd_MAGNIFIER_PROPORTIONAL_MODE ;	
              } 

            changed=0 ;    
} 



void VdMagnifierWindow::handleDestroyed( void )
{
	if ( !theConnection ) return;
	theConnection->removeMagnifierWindow();
}

void VdMagnifierWindow::redrawPicture(int toClear)
{
	VdAbstractWindow::redrawPicture(toClear);
	drawMagnifier();
}

void VdMagnifierWindow::handleResized( void )
{
	VdAbstractWindow::handleResized();
	if (!theConnection) {
		drawMagnifier();
	} else {
		VdArea displayed, world;
		getDisplayOfWorld(displayed,world);
		setDisplayOfWorld(displayed,displayed);
		theConnection->worldResizedByBorder(
			theMagnifier.ul-theMagnifier.cor,
    			theMagnifier.lr+theMagnifier.cor) ;
	}
}

void VdMagnifierWindow::handleExposed( void )
{
	VdAbstractWindow::handleExposed();
}

VdFeature VdMagnifierWindow::setConnection( 
	VdWindowConnection *wc, VdFeature& prefix )
{
// This function will be only called by 
// VdWindowConnection::setMagnifierWindow
	theConnection = wc;
	if (wc)
		setIdent(prefix + VdFeature("MAGNIFIER"));
	else 
		setIdent( (getIdent()).getLast(1));
	return getIdent();
}

static void help (Widget parent);

void VdMagnifierWindow::handleHelp(void)
{
	// help means here that the magnifier centralize
	// a "lost" picture out of the world
	VdArea frame, world;
	if (getPicture()) {
		(getPicture())->prepare();
		frame = (getPicture())->getFrame();
	}
	getDisplayOfWorld(world,world);
	VdVector worldCentre( (world.lr.x + world.ul.x) /2 ,
			      (world.lr.y + world.ul.y) /2 );
	VdVector frameCentre( (frame.lr.x + frame.ul.x) /2 ,
			      (frame.lr.y + frame.ul.y) /2 );

	VdVector shift = frameCentre - worldCentre;
	if ( (shift.x*shift.x+shift.y*shift.y<10) 
		|| !getPicture()) {
		// gag
		VdShell::help(VdFeature("Visamad:Magnifier:Centralized"));
	} else {
		VdVector wshift = 
		   (getGeometricContext()).transPictToWind(shift);
		if (theConnection) {
		   theConnection->worldMoved(
			(theMagnifier.ul-theMagnifier.cor)-wshift,
    			(theMagnifier.lr+theMagnifier.cor)-wshift);
		   // move the world's centre to centre of frame
		   theConnection->magnifierMoved(
			(theMagnifier.ul-theMagnifier.cor),
    			(theMagnifier.lr+theMagnifier.cor));
		   // updating the EditWindow
		} else {
		   // only Magnifier update
	    		world += shift;
			setDisplayOfWorld(world,world);
			redrawPicture();
		}

	}   

}


//*****************************************************************

VdButton * VdMagnifierWindow::addEditWindowButton( 
				const VdFeature & ident )
{
	VdFeature id(ident);
	VdButton * button = 
		new VdMagnifierButton(this, id, theConnection);
	button->map();
	return button;
}

void VdMagnifierWindow::removeEditWindowButton( VdButton * button )
{
	if (button) button->remove();
}

VdButton * VdMagnifierWindow::renameEditWindowButton( VdButton * button,
				const VdFeature & ident )
{
	if (button) button->setLabel(ident.getString().charptr());
	return button;
}


/***** handler for Exit button *******************

class ExitHandler: public VdButtonInterface {
	int (* checkExit) (const VdFeature &);
   public:
	ExitHandler( int (* checkIt) (const VdFeature &) );
	void handleActivated( void );
}

ExitHandler::ExitHandler(  ) 
{   
	idf = VdString("Exit");
	checkExit = checkIt;
}

void ExitHandler::handleActivated( void )
{   
	if (checkExit()) ...
}

*/

//***** handler for Mode button *******************

class ModeMenu : public VdRadioBoxInterface {
	VdWindowConnection **theWC; 
   public:
	ModeMenu( VdWindowConnection **wc );
	virtual void activate( const VdFeature & );
};

ModeMenu::ModeMenu( VdWindowConnection **wc )
{
	theWC = wc;
	mask=1;
	idf = VdString("Mode");
	labels[0]=VdString("Default-Default");
	labels[1]=VdString("Default-Elastic");
	labels[2]=VdString("Elastic-Default");
	labels[3]=VdString("Elastic-Elastic");
}

void ModeMenu::activate( const VdFeature & )
{
	if (!*theWC) return;
	switch (mask) {
		case 1: (*theWC)->setMode(Vd_DEFAULT_DEFAULT);
			break;
		case 2: (*theWC)->setMode(Vd_DEFAULT_BUBBLE);
			break;
		case 4: (*theWC)->setMode(Vd_BUBBLE_DEFAULT);
			break;
		case 8: (*theWC)->setMode(Vd_BUBBLE_BUBBLE);
			break;
	}
}

//***** handler for Window button *******************

class WindowMenu : public VdRadioBoxInterface {
	VdWindowConnection **theWC;
	VdMagnifierWindow *theMW;
	
   public:
	WindowMenu( VdWindowConnection **wc,
			VdMagnifierWindow *mw );
	virtual void activate( const VdFeature & );
};

WindowMenu::WindowMenu( VdWindowConnection **wc,
			VdMagnifierWindow *mw )
{
	theWC = wc;
	theMW = mw;
	mask=0;
	idf = VdString("Window");
}

void WindowMenu::activate( const VdFeature & )
{
	// programmtechnisch einfache, aber umstaendliche
	// Version, besser Idents der EditWindows aufbewahren

	if (!*theWC) return;
	int i=-1;
	long h=mask;
	while (h) { h >>= 1; i++; }
	VdString newTitle(((*theWC)->getIdent()).getString());
	VdFeature ewIdent = VdFeature(newTitle)
			+VdFeature(labels[i]);
	(*theWC)->magnify(ewIdent);
}


void VdMagnifierWindow::installModeButton( void )
{
	if (modeButtonPtr) return;
	modeButtonPtr = new VdPulldownOptions(
		*(VdAbstractWindow *)this,
		"Mode",
		1,
		1,
		new ModeMenu(&theConnection)
	);
	modeButtonPtr->map();
}

void VdMagnifierWindow::installWindowButton( void )
{
	if (windowButtonPtr) return;
	windowButtonInterface = 
		new WindowMenu(&theConnection,this);
	windowButtonPtr = new VdPulldownOptions(
		*(VdAbstractWindow *)this,
		"Window",
		1,
		1,
		windowButtonInterface
	);
	windowButtonPtr->map();
}

void VdMagnifierWindow::addWindowButtonEntry( 
		const VdFeature & editWindow )
{
	VdFeature dummy;
	if (!windowButtonPtr) installWindowButton();
	windowButtonPtr->addOption(
	    (editWindow.getLast(1)).getString().charptr()
	);
	if (!windowButtonInterface->labels == 1)
		windowButtonInterface->activate(dummy);
}

void VdMagnifierWindow::removeWindowButtonEntry( 
		const VdFeature & editWindow )
{
	if (!windowButtonPtr) return;
	windowButtonPtr->removeOption(
	    (editWindow.getLast(1)).getString().charptr()
	);
}

void VdMagnifierWindow::renameWindowButtonEntry( 
		const VdFeature & oldEditWindow,
		const VdFeature & newEditWindow )
{
	if (!windowButtonPtr) return;
	windowButtonPtr->renameOption(
	    (oldEditWindow.getLast(1)).getString().charptr(),
	    (newEditWindow.getLast(1)).getString().charptr()
	);
}



#undef _VDMAGNIFIERWINDOW_CXX
