#ifndef _VDMAGNIFIERWINDOW_HXX
#define _VDMAGNIFIERWINDOW_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdMagnifierWindow.hxx                             *
*  Date: 30-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: B. Miethig, J. Hutschenreiter                   *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Declaration of class VdMagnifierWindow:              *
*                                                          *
*     Interface                                            *
*          VdAbstractWindow.hxx, VdWindowConnection.hxx    *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdMagnifier;
class VdMagnifierWindow;

/******************    dependences    *********************/

#include <X11/cursorfont.h>
#include <Vd/VdCBmanager.hxx>
#include <Vd/VdAbstractWindow.hxx>
#include <Vd/VdEventHandling.hxx>
#include <Vd/VdArea.hxx>
#include <Vd/VdPoint.hxx>
#include <Vd/VdVector.hxx>
#include <Vd/VdFractNumber.hxx>
#include <Vd/VdEvent.hxx>
#include <Vd/VdRadioBoxInterface.hxx>
#include <Vd/VdHelpManager.hxx>

class VdButton;
class VdPulldownOptions;

enum VdMagnifierMode { Vd_MAGNIFIER_RESIZEABLE_MODE, 
                       Vd_MAGNIFIER_PROPORTIONAL_MODE,
                       Vd_MAGNIFIER_NON_RESIZEABLE_MODE
                     } ; 

enum VdMagnifierHandling { Vd_MAGNIFIER_DRAGS,
                       Vd_MAGNIFIER_CHANGESVALUE
                      } ;


class VdMagnifier
     {
 public: int borderThick ;
         int cornerThickW,cornerThickH ;
         VdPoint ul,lr ;
         int position ;
         VdVector cor ;
         VdMagnifierMode  theMode ;
         VdMagnifierHandling theHandling;
         int theWork ; 
         VdFractNumber widthToHeight ;  

       VdMagnifier(const VdPoint & aul=VdNullPoint,
                   const VdPoint & alr=VdNullPoint, 
                   int aBorderThick=10,
                   int aCornerThick=10,
                   const VdMagnifierMode
                   aMode=Vd_MAGNIFIER_RESIZEABLE_MODE 
                      ) ;
              
       int getBorderThick(void) ;

       void  setBorderThick(int aBorderThick) ;
       
       unsigned int magnifierArea(VdPoint &aPoint) ;

     
   } ;  
  

        

class VdWindowConnection ;
           
class VdMagnifierWindow : public VdAbstractWindow {
// inheritance should be virtual but OASYS translator 
// contains an error generating virtual function tables
// for twice virtual public inheritances

   friend class VdWindowConnection;
 
	VdWindowConnection *theConnection;
	VdHelpManager *theHm ;
	VdFeature windowIdent;
        VdMagnifier theMagnifier ; 
        Cursor cursor ;
        Window  theGrabWindow ;
        VdVector theGrabWindowCor ; 
	VdFeature setConnection( VdWindowConnection *wc, 
		       VdFeature& prefix );
	GC ownGC;
	void createGC( void );
   protected:
     // 3 standard buttons
	VdPulldownOptions *modeButtonPtr;
	VdPulldownOptions *windowButtonPtr;
	VdRadioBoxInterface *windowButtonInterface;

   public:
        VdMagnifierWindow( VdShell &shell,
                Boolean withScrollbars = False);

        ~VdMagnifierWindow( void );

        void getMagnifierData(VdPoint &alu, VdPoint &alr) ;  
            
        void setMagnifierData(const VdPoint aul
                        ,const VdPoint alr, int toRedraw = 1) ;

        VdMagnifierMode getMode(void) ;

        void setMode(VdMagnifierMode aMode) ;
	void setWindowType(VdWindowType wt);

        VdMagnifierHandling getHandling(void) ;

        void setHandling(VdMagnifierHandling aHandling) ;

	virtual void redrawPicture( int toClear = 1);
        virtual void callbackButtonPressed(VdEvent *ev) ; 
        virtual void callbackButtonReleased(VdEvent *ev) ; 
	virtual void handleDestroyed( void );
	virtual void handleResized( void );
	virtual void handleExposed( void );
	virtual void handleHelp(void);

   private: 
	void drawMagnifier(void) ;
        void manipulateMagnifier(const VdVector &aVector,
			const VdPoint &thePointer) ;
        void setCursor(int which) ;
        void setGrabWindow(VdPoint &aPoint) ;
        void setPropGrabWindow(VdPoint &aPoint) ;   
        void realizeIt(const VdPoint &aPointer) ;

   public:
	virtual VdButton * addEditWindowButton( 
		const VdFeature & ident );
	virtual void removeEditWindowButton( VdButton * );
	virtual VdButton * renameEditWindowButton( VdButton *,
		const VdFeature & ident );
	void installModeButton( void );
	void installWindowButton( void );
   protected:
	void addWindowButtonEntry( 
		const VdFeature & editWindow );
	void removeWindowButtonEntry( 
		const VdFeature & editWindow );
	void renameWindowButtonEntry( 
		const VdFeature & oldEditWindow,
		const VdFeature & newEditWindow );
};

#endif  /* _VDMAGNIFIERWINDOW_HXX */
