#define _VDNET_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdNet.cxx                                         *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdNet.hxx>
#include <Vd/VdLine.hxx>
#include <Vd/VdintList.hxx>

#define LSQR 100
#define LSQR0 1000
#define L 10

VdNet::VdNet( VdVector & v1, VdVector & v2 )
{ if (v1==VdNullVector)
  { dist=L; or1=VdVector(L,0); }
  else
  { dist=!v1; or1=v1.norm(L); }
  if (v2==VdNullVector) or2=VdVector(0,L);
  else or2=v2.norm(L);
  if (or1||or2) { or2.x=or1.y; or2.y=-or1.x; }
}

void VdNet::clear(void)
{ pnt1.reset();
  pnt2.reset();
}

void VdNet::setSegmentOrientation(VdVector & v1 )
{ if (v1==VdNullVector)
  { dist=L; or1=VdVector(L,0); }
  else
  { dist=!v1; or1=v1.norm(L); }
  if (or1||or2) { or2.x=or1.y; or2.y=-or1.x; }
}

void VdNet::setLineOrientation(VdVector & v1 )
{ if (v1==VdNullVector) or2=VdVector(0,L);
  else or2=v1.norm(L);
  if (or1||or2) { or2.x=or1.y; or2.y=-or1.x; }
}

VdVector VdNet::getSegmentOrientation(void)
{ return or1;
}

VdVector VdNet::getLineOrientation(void)
{ return or2;
}

void VdNet::addPoint1( VdPoint & p )
{ pnt1.ins()=p;
}

void VdNet::addPoint2( VdPoint & p )
{ pnt2.ins()=p;
}

VdPoint VdNet::getBestPoint( VdPoint & p )
{ int l;
  VdPicture * ptr;
  VdPoint q1, q2;
  VdVector o1q;
  switch (!pnt1+!pnt2)
  { case 0 : return -1;
    case 1 : if (!pnt1) return pnt1[0];
             return pnt2[0];
    default:
      ptr=getReal(0);
      if (ptr==NULL)
      { prepare(); ptr=getReal(0);
        if (ptr==NULL) return VdNullPoint;
      }
      q1=((VdLine*)ptr)->getStartPoint();
      q2=((VdLine*)ptr)->getEndPoint();
      o1q=VdVector(ptr->getLocation());
      q1+=o1q; q2+=o1q;
      if (q1==q2) return q1;
      o1q.x=or1.y; o1q.y=-or1.x;
      l=(((p-q1)*o1q)<<8)/((q2-q1)*o1q);
      if (l<0) return q1;
      if (l>256) return q2;
      return q1+((q2-q1)*l)/256;
  }
}

void VdNet::manipulate( void )
{ VdintList mu1, mu2;
  int mmin1, mmin2,lmin, lmax, k, o1o2q /*, o2o1q */;
  VdVector o1, o2;
  VdPoint pa, pe;
  int i;
  clearReals();
  if (((!pnt1)+(!pnt2))<2)
  { if (!pnt1) moveTo(pnt1[0]);
    if (!pnt2) moveTo(pnt2[0]);
    return;
  }
  o1.x=or1.y; o1.y=-or1.x;
  o2.x=or2.y; o2.y=-or2.x;
  o1o2q=or1*o2; /* o2o1q=-o1o2q; */
  if (!pnt2)
  { mu2[0]=(VdVector(pnt2[0])*o2);
    mmin2=0;
    lmin=lmax=(VdVector(pnt2[0])*o1);
    for (i=1; i<!pnt2; i++)
    { k=mu2[i]=(VdVector(pnt2[i])*o2);
      if (k<mu2[mmin2]) mmin2=i;
      k=(VdVector(pnt2[i])*o1);
      if (k<lmin) lmin=k;
      else if (k>lmax) lmax=k;
    }
  }
  else
  { lmin=lmax=(VdVector(pnt1[0])*o1);
  }
  if (!pnt1)
  { mu1[0]=(-VdVector(pnt1[0])*o2);
    mmin1=0;
    k=(VdVector(pnt1[0])*o1);
    if (k<lmin) lmin=k;
    else if (k>lmax) lmax=k;
    for (i=1; i<!pnt1; i++)
    { k=mu1[i]=(-VdVector(pnt1[i])*o2);
      if (k<mu1[mmin1]) mmin1=i;
      k=(VdVector(pnt1[i])*o1);
      if (k<lmin) lmin=k;
      else if (k>lmax) lmax=k;
    }
  }
  if (!pnt1==0) k=mu2[mmin2]-(o1o2q*dist)/L;
  else
  { if (!pnt2==0) k=-mu1[mmin1]+(o1o2q*dist)/L;
    else k=(mu2[mmin2]-mu1[mmin1])>>1;
  }
  pa=(k*or1-lmin*or2)/o1o2q;
  pe=(k*or1-lmax*or2)/o1o2q;
  moveTo(VdNullPoint);
  VdLine * lptr;
  if (pa!=pe)
  { lptr=new VdLine(pa,pe);
    lptr->setAutoControl();
    include(lptr);
  }
  for (i=0; i<!pnt1; i++)
  { lptr=new VdLine(pnt1[i],pnt1[i]+((k+mu1[i])*or1)/o1o2q);
    lptr->setAutoControl();
    include(lptr);
  }
  for (i=0; i<!pnt2; i++)
  { lptr=new VdLine(pnt2[i],pnt2[i]-((mu2[i]-k)*or1)/o1o2q);
    lptr->setAutoControl();
    include(lptr);
  }
}

#undef _VDNET_CXX
