#define _VDNODE_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdNode.cxx                                        *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdNode.hxx>

#ifdef _HXX_INCLUDE_
#include <math.hxx>
#else
#include <math.h>
#include <stdio.h>
#endif

VdNode::VdNode(int nt )
{ text = new VdText;
  text->setAutoControl();
  switch (nodeType=nt)
  { case 1 : border = new VdArc; break;
    case 2 : border = new VdHexagon; break;
    case 3 : border = new VdOval; break;
    default: border = new VdRectangle; break;
  }
  border->setAutoControl();
  xSpace=ySpace=3;
  include(text);
  include(border);
}

VdNode::VdNode( VdNode & node )
: VdPicture(node)
{ nodeType=node.nodeType;
  text = (VdText*)getReal(0);
  border=getReal(1);
  xSpace=node.xSpace;
  ySpace=node.ySpace;
}

VdPicture* VdNode::duplicate(void)
{ VdNode * pic = new VdNode;
  *((VdPicture*)pic)=*((VdPicture*)this);
  pic->nodeType=nodeType;
  pic->text = (VdText*)(text->duplicate());
  pic->text->setAutoControl();
  switch (pic->nodeType)
  { case 1 : pic->border = new VdArc; break;
    case 2 : pic->border = new VdHexagon; break;
    case 3 : pic->border = new VdOval; break;
    default: pic->border = new VdRectangle; break;
  }
  pic->border->setAutoControl();
  pic->xSpace=xSpace;
  pic->ySpace=ySpace;
  pic->include(pic->text);
  pic->include(pic->border);
  pic->setAutoControl();
  return pic;
}

void VdNode::setXSpace( int aa )
{ xSpace=(aa<0)?-aa:aa;
}

void VdNode::setYSpace( int aa )
{ ySpace=(aa<0)?-aa:aa;
}

int VdNode::getXSpace( void ) const
{ return xSpace;
}

int VdNode::getYSpace( void ) const
{ return ySpace;
}

void VdNode::setNodeType( int nt )
{ VdPoint p;
  border->prepare();
  p = border->getFramePoint(VdXCENTRE|VdYCENTRE);
  delete border;
  switch (nodeType=nt)
  { case 1 : border = new VdArc; break;
    case 2 : border = new VdHexagon; break;
    case 3 : border = new VdOval; break;
    default: border = new VdRectangle; break;
  }
  border->setAutoControl();
  border->prepare();
  border->moveTo(p+( border->getLocation()
                    -border->getFramePoint(VdXCENTRE|VdYCENTRE)
                )  );
  include(border);
}

int VdNode::getNodeType( void ) const
{ return nodeType;
}

void VdNode::setText( VdString & s )
{ text->setString(s);
}

VdText & VdNode::getText( void )
{ return *text;
}

VdPoint VdNode::getBorderPoint( VdPoint & p )
{ VdVector v = p-getFramePoint(VdXCENTRE|VdYCENTRE);
  int x, y, h1, h2;
  VdArea a;
  switch (nodeType)
  { case 1 :
      a=border->getFrame();
      x=a.lr.x-a.ul.x; y=a.ul.y-a.lr.y;
      h1=sqrt(v.x*v.x*y*y+v.y*v.y*x*x); h2=(x*y);
      if (h2&&h1) { v*=h2; v/=h1<<1; } else v=VdNullVector;
      return getFramePoint(VdXCENTRE|VdYCENTRE)+v;
    case 2 :
      return ((VdHexagon*)border)->getBorderPoint(p);
    case 3 :
      return ((VdOval*)border)->getBorderPoint(p);
    default:
      x=(border->getFramePoint(VdXRIGHT|VdYTOP)-border->getLocation()).x;
      y=(border->getFramePoint(VdXRIGHT|VdYTOP)-border->getLocation()).y;
      if ((x==0)||(y==0)) return getFramePoint(VdXCENTRE|VdYCENTRE);
      if (v!=VdNullVector)
      { if ((v.x*y-v.y*x)<=0)
        { if ((v.x*y+v.y*x)<0) { v.y=-(v.y*x)/(v.x<<1); v.x=-(x>>1); }
          else { v.x=(v.x*y)/(v.y<<1); v.y=y>>1; }
        }
        else
        { if ((v.x*y+v.y*x)>=0) { v.y=(v.y*x)/(v.x<<1); v.x=x>>1; }
          else { v.x=-(v.x*y)/(v.y<<1); v.y=-(y>>1); }
        }
      }
      return getFramePoint(VdXCENTRE|VdYCENTRE)+v;
  }
}
 
void VdNode::manipulate( void )
{ text->prepare();
  VdArea a = text->getFrame();
  int x = a.lr.x-a.ul.x+(xSpace<<1);
  int y = a.ul.y-a.lr.y+(ySpace<<1);
  int hx, hy;
  VdPoint p = border->getFramePoint(VdXCENTRE|VdYCENTRE);
  switch (nodeType)
  { case 1 :
      VdArc * aptr;
      aptr=(VdArc*)border;
      hx=(x*7+5)/10; hy=(y*7+5)/10;
      aptr->setAxisA(hx);
      aptr->setAxisB(hy);
      aptr->setCentre(VdPoint(hx,hy));
      break;
    case 2 :
      ((VdHexagon*)border)->setRectangle(VdArea(VdNullPoint,x,y));
      break;
    case 3:
      ((VdOval*)border)->setRectangle(VdArea(VdNullPoint,x,y));
      break;
    default:
      ((VdRectangle*)border)->setRectangle(VdArea(VdNullPoint,x,y));
      break;
  }
  border->prepare();
  border->moveTo(p+( border->getLocation()
                    -border->getFramePoint(VdXCENTRE|VdYCENTRE)
                )  );
}

void VdNode::layout( void )
{ text->shift( border->getFramePoint(VdXCENTRE|VdYCENTRE)
              -text->getFramePoint(VdXCENTRE|VdYCENTRE)
             );
}

#undef _VDNODE_CXX
