#define _VDOVAL_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdOval.cxx                                        *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdOval.hxx>

#ifdef _HXX_INCLUDE_
#include <math.hxx>
#else
#include <math.h>
#endif

VdOval::VdOval(int aa, int bb )
  : left(VdNullPoint,aa,bb,90,180),
    right(VdNullPoint,aa,bb,90,-180)
{ a=(aa<0)?-aa:aa; b=(bb<0)?-bb:bb;
  if (a&1) a++;
  if (b&1) b++;
  include(&left);
  include(&right);
  include(&up);
  include(&down);
}

VdPicture* VdOval::duplicate(void)
{ VdOval * pic = new VdOval(a,b);
  *((VdPicture*)pic)=*((VdPicture*)this);
  return pic;
}

void VdOval::setAxisA( int aa )
{ a=(aa<0)?-aa:aa;
  if (a&1) a++;
}

void VdOval::setAxisB( int bb )
{ b=(bb<0)?-bb:bb;
  if (b&1) b++;
}

void VdOval::setRectangle( const VdArea & ar )
{ a=ar.lr.x-ar.ul.x+20;
  b=ar.ul.y-ar.lr.y;
}

int VdOval::getAxisA( void ) const
{ return a;
}

int VdOval::getAxisB( void ) const
{ return b;
}

VdPoint VdOval::getBorderPoint( VdPoint & p )
{ VdVector v;
  int x, y, r;
  double h1, h2, h3, a2, b2, vx, vy;
  v = p-getFramePoint(VdXCENTRE|VdYCENTRE);
  vx=v.x; vy=v.y;
  if ((a<=20)||(b<=20)) r=((a<b)?a:b)>>1;
  else r=10;
  x=(a>>1)-r;
  y=b>>1;
  if (y==0) return getFramePoint(VdXCENTRE|VdYCENTRE);
  if (v!=VdNullVector)
  { if ((vx*y-vy*x)<=0)
    { if ((vx*y+vy*x)<=0)
      { b2=y*y;
        a2=r*r;
        h2=vx*vx*b2+vy*vy*a2;
        h3=vx*x*b2;
        h1=-h3+sqrt(h3*h3 - b2*(x*x-a2)*h2);
        vx*=h1; vx/=h2; vy*=h1; vy/=h2;
      }
      else { vx=(vx*y)/(vy); vy=y; }
    }
    else
    { if ((vx*y+vy*x)>=0)
      { b2=y*y;
        a2=r*r;
        h2=vx*vx*b2+vy*vy*a2;
        h3=vx*x*b2;
        h1=h3+sqrt(h3*h3 - b2*(x*x-a2)*h2);
        vx*=h1; vx/=h2; vy*=h1; vy/=h2;
      }
      else { vx=-(vx*y)/(vy); vy=-(y); }
    }
  }
  v.x=(int)vx; v.y=(int)vy;
  return getFramePoint(VdXCENTRE|VdYCENTRE)+v;
}

void VdOval::manipulate( void )
{ int r, bl;
  if ((a<20)||(b<20)) r=((a<b)?a:b)>>1;
  else r=10;
  bl=b>>1;
  up.setStartPoint(VdPoint(r,b));
  up.setEndPoint(VdPoint(a-r,b));
  down.setStartPoint(VdPoint(r,0));
  down.setEndPoint(VdPoint(a-r,0));
  left.setAxisA(r);
  left.setAxisB(bl);
  right.setAxisA(r);
  right.setAxisB(bl);
}

void VdOval::layout( void )
{ int r, bl;
  if ((a<20)||(b<20)) r=((a<b)?a:b)>>1;
  else r=10;
  bl=b>>1;
  left.moveTo(down.getLocation()+VdVector(r,bl));
  right.moveTo(down.getLocation()+VdVector(a-r,bl));
}

#undef _VDOVAL_CXX
