#ifndef _VDPICTURE_HXX
#define _VDPICTURE_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdPicture.hxx                                      *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdPicture;

/******************    dependences    *********************/


#include <Vd/VdArea.hxx>
#include <Vd/VdPoint.hxx>
#include <Vd/VdVector.hxx>

#ifdef _HXX_INCLUDE_
#include <stddef.hxx>
#include <stdio.hxx>
#else
#include <stddef.h>
#include <stdio.h>
#endif

#include <Vd/VdGeometricContext.hxx>
#include <Vd/VdGraphicAttributes.hxx>
#include <Vd/VdPicturePtrList.hxx>
#include <Vd/VdHandlePtrList.hxx>
#include <Vd/VdEvent.hxx>

class VdHandle;
class VdAllSelectedHandles;

#define VdXCENTRE 1
#define VdXRIGHT  2
#define VdXLEFT   3
#define VdYCENTRE 4
#define VdYTOP    8
#define VdYBOTTOM 12


class VdPicture{ 

    friend class VdHandle;

   protected :           // instance managment

     static int detectCreationWithNew;
     
     int createdWithNew, autoControl;

  public :

     static void* operator new ( size_t );

     static void operator delete( void * ptr );

     VdPicture(void);

     VdPicture( VdPicture & );

     VdPicture & operator = ( VdPicture & );

     virtual VdPicture * duplicate ( void );

     virtual ~VdPicture(void);

     void setAutoControl(void);

     int operator == ( const VdPicture & ) const;
     int operator != ( const VdPicture & ) const;

  protected :            // may be partial Picture

     VdPicturePtrList insertedIn;

     void addInsertion( VdPicture * );
     void remInsertion( VdPicture * );
  public:
     void clearInsertions( void );

  protected :             // may contain partial Pictures

     VdPicturePtrList realPictures;    // as real Pictures
     VdPicturePtrList virtPictures;    // as virtual Pictures

     void addReal( VdPicture * );
     void remReal( VdPicture * );

     void addVirtual( VdPicture * );
     void remVirtual( VdPicture * );

   public :

     void include ( VdPicture * );
     void includeAt ( VdPicture * , int );
     void exclude ( VdPicture * );
     int numberOfReals ( void ) const;
     int idxOfReal ( VdPicture * );
     VdPicture * getReal ( int );
     void clearReals( void );
     
     void refer ( VdPicture * );
     void derefer ( VdPicture * );
     int numberOfVirtuals ( void ) const;
     int idxOfVirtual ( VdPicture * );
     VdPicture * getVirtual ( int );
     void clearVirtuals( void );

  protected:           // related handles

    VdHandlePtrList handles;

    void addHandle (VdHandle * e);
    void remHandle (VdHandle * e);

  public:

    virtual void collectHandles ( VdAllSelectedHandles * );

    void include ( VdHandle * );
    void includeAt ( VdHandle * , int );
    void exclude ( VdHandle * );
    int numberOfHandles ( void ) const;
    int idxOfHandle ( VdHandle * );
    VdHandle * getHandle ( int );
    void clearHandles( void );

    virtual const VdString & getSelectionIdentifier( void );

    virtual void enterHandleSelection( const VdEvent * theEvent );
    virtual void leaveHandleSelection( const VdEvent * theEvent );

    virtual void setSelectionColor( const VdColor & color );
    virtual VdColor getSelectionColor( void );

   protected:

     VdPoint location; 
     VdArea frame; 
     int autoFrame;
     VdColor foreground, background, selection, forewhileselect;
     VdLineType  lineType; 
     VdFillType fillType;
     VdArcType arcType;
     VdVector textSize;

   public :

     void setAutoFrame  ( int i );
     void setFrame      ( const VdArea & );
     void setForeground ( const VdColor & );
     void setBackground ( const VdColor & );
     void setLineType   ( const VdLineType & );
     void setFillType   ( const VdFillType & );
     void setArcType    ( const VdArcType & );
     void setTextSize   ( const VdVector & );
     void setLocation   ( const VdPoint & p );
     void moveTo        ( const VdPoint & p );
     void shift         ( const VdVector & v );

     int        isAutoFrame   (void) const;
     VdArea     getFrame      (void);
     VdArea     getAbsFrame   (void);
     VdPoint    getFramePoint ( int ) const;
     VdColor    getForeground (void) const;
     VdColor    getBackground (void) const;
     VdLineType getLineType   (void) const;
     VdFillType getFillType   (void) const;
     VdArcType  getArcType    (void) const;
     VdVector   getTextSize   (void) const;
     VdPoint    getLocation   (void) const;


     virtual void prepare(void);

     virtual void manipulate(void);
     virtual void layout(void);
     virtual void computeFrame ( void );

     virtual void draw( VdGeometricContext * G
                      , VdGraphicAttributes  GA
                      );

    void clearPicture ( void );
     virtual void printPostScript(FILE* f);

    virtual void print( int );

};

/******************     constants     *********************/ 
 
#ifdef _VDPICTURESUMMARY_CXX

   int VdPicture::detectCreationWithNew=0;

#endif

#endif /* _VDPICTURE_HXX */
