#define _VDPICTURESUMMARY_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdPictureSummary.cxx                              *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdPictureSummary.hxx>

#ifdef _HXX_INCLUDE_
#include <math.hxx>
#include <stdio.hxx>
#else
#include <math.h>
#include <stdio.h>
#endif


#define VdListClassName VdPicturePtrList
#define VdListClassElementType VdPicture* 
#include <Vd/VdListDefinition.mxx>
#undef VdListClassName
#undef VdListClassElementType

void* VdPicture::operator new ( size_t s )
{ detectCreationWithNew=1; return ::operator new(s);
}

void VdPicture::operator delete( void * ptr)
{ ::operator delete(ptr);
}

static void replaceReal ( void * where
                            , void * ptr
                            , void * env
                            , int isNew
                            )
{ if (isNew); else return;
  int idx = (int)where;
  VdPicture * real = (VdPicture *)ptr;
  VdPicture * pict = (VdPicture *)env;
  pict->exclude(pict->getReal(idx));
  pict->includeAt(real,idx);
}

static void replaceVirtual ( void * where
                               , void * ptr
                               , void * env
                               , int isNew
                               )
{ if (isNew); else return;
  int idx = (int)where;
  VdPicture * virt = (VdPicture *)ptr;
  VdPicture * pict = (VdPicture *)env;
  pict->derefer(pict->getVirtual(idx));
  pict->refer(virt);
}

static void replaceHandle ( void * where
                              , void * ptr
                              , void * env
                              , int isNew
                              )
{ if (isNew); else return;
  int idx = (int)where;
  VdHandle * hand = (VdHandle *)ptr;
  VdPicture * pict = (VdPicture *)env;
  pict->exclude(pict->getHandle(idx));
  pict->includeAt(hand,idx);
}

VdPicture::VdPicture(void)
{ createdWithNew=detectCreationWithNew; detectCreationWithNew=0;
  autoControl=0; autoFrame=1;
  selection=VdRED;
}

VdPicture::VdPicture( VdPicture & p )
 : location(p.location),
   frame(p.frame),
   autoFrame(p.autoFrame),
   foreground(p.foreground),
   background(p.background),
   selection(p.selection),
   lineType(p.lineType),
   fillType(p.fillType),
   arcType(p.arcType),
   textSize(p.textSize)
{ int i;
  createdWithNew=detectCreationWithNew; detectCreationWithNew=0;
  autoControl=0;
  VdConsistence::registerInstance(&p,this);
  handles.reset();
  for (i=0; i<!p.handles; i++) 
  { include(p.handles[i]);
    VdConsistence::registerReplacement((void*)i
                                      ,(void*)p.handles[i]
                                      ,(void*)this
                                      ,&replaceHandle
                                      );
  }
  realPictures.reset();
  for (i=0; i<!p.realPictures; i++) 
  { if (p.realPictures[i]->createdWithNew)
    { include(p.realPictures[i]->duplicate());
    }
    else
    { include(p.realPictures[i]);
      VdConsistence::registerReplacement((void*)i
                                        ,(void*)p.realPictures[i]
                                        ,(void*)this
                                        ,&replaceReal
                                      );
    }
  }
  virtPictures.reset();
  for (i=0; i<!p.virtPictures; i++) 
  { if (p.virtPictures[i]->createdWithNew)
    { refer(p.virtPictures[i]->duplicate());
    }
    else
    { refer(p.virtPictures[i]);
      VdConsistence::registerReplacement((void*)i
                                        ,(void*)p.virtPictures[i]
                                        ,(void*)this
                                        ,&replaceVirtual
                                      );
    }
  }
}

VdPicture & VdPicture::operator = ( VdPicture & p )
{ if (!(autoFrame=p.autoFrame)) frame=p.frame;
  foreground=p.foreground;
  background=p.background;
  selection=p.selection;
  lineType=p.lineType; 
  fillType=p.fillType;
  arcType=p.arcType;
  textSize=p.textSize;
  return *this;
}

VdPicture * VdPicture::duplicate( void )
{ int i;
  VdPicture * pict = new VdPicture;
  pict->autoControl=1;
  *pict = *((VdPicture*)this);
  for (i=0; i<!handles; i++) 
  { pict->include(handles[i]);
  }
  for (i=0; i<!realPictures; i++) 
  { pict->include(realPictures[i]->duplicate());
  }
  for (i=0; i<!virtPictures; i++) 
  { pict->refer(virtPictures[i]->duplicate());
  }
  return pict;
}

VdPicture::~VdPicture(void)
{ autoControl=0;
  clearInsertions();
  clearReals();
  clearVirtuals();
  clearHandles();
}

void VdPicture::setAutoControl(void)
{ if (createdWithNew) autoControl=1;
}

int VdPicture::operator == ( const VdPicture & p ) const
{ return this == &p;
}

int VdPicture::operator != ( const VdPicture & p ) const
{ return this != &p;
}

void VdPicture::addInsertion( VdPicture * p )
{ if (insertedIn.idx(p)<0) insertedIn.ins()=p;
}

void VdPicture::remInsertion( VdPicture * p )
{ insertedIn.rem(insertedIn.idx(p));
}

void VdPicture::clearInsertions( void )
{ int i;
  for (i=0; i<!insertedIn; i++)
  { insertedIn[i]->remReal(this);
    insertedIn[i]->remVirtual(this);
  }
  insertedIn.reset();
}

void VdPicture::addReal( VdPicture * p )
{ if (realPictures.idx(p)<0)
  { realPictures.ins()=p;
  }
}

void VdPicture::remReal( VdPicture * p )
{ int i;
  if ((i=realPictures.idx(p))>=0)
  { if (   (p->autoControl)
        && (!(p->insertedIn)==0)
       )
      delete p;
    realPictures.rem(i);
  }
}

void VdPicture::include ( VdPicture * p )
{ addReal(p);
  p->addInsertion(this);
}

void VdPicture::includeAt ( VdPicture * p , int idx)
{ if (realPictures.idx(p)>=0) return;
  if (idx>=!realPictures) realPictures.ins()=p;
  else realPictures.ins(idx)=p;
  p->addInsertion(this);
}

void VdPicture::exclude ( VdPicture * p )
{ p->remInsertion(this);
  remReal(p);
}

int VdPicture::numberOfReals ( void ) const
{ return !realPictures;
}

int VdPicture::idxOfReal ( VdPicture * p )
{ return realPictures.idx(p);
}

VdPicture * VdPicture::getReal ( int i )
{ if ((i<0)||(i>=!realPictures)) return NULL;
  return realPictures[i];
}

void VdPicture::clearReals( void )
{ while (!realPictures) exclude(realPictures[0]);
}

void VdPicture::addVirtual( VdPicture * p )
{ if (virtPictures.idx(p)<0)
  { virtPictures.ins()=p;
  }
}

void VdPicture::remVirtual( VdPicture * p )
{ int i;
  if ((i=virtPictures.idx(p))>=0)
  { if (   (p->autoControl)
        && (!(p->insertedIn)==0)
       )
      delete p;
    virtPictures.rem(i);
  }
}

void VdPicture::refer ( VdPicture * p )
{ addVirtual(p);
  p->addInsertion(this);
}

void VdPicture::derefer ( VdPicture * p )
{ p->remInsertion(this);
  remVirtual(p);
}

int VdPicture::numberOfVirtuals ( void ) const
{ return !virtPictures;
}

int VdPicture::idxOfVirtual ( VdPicture * p )
{ return virtPictures.idx(p);
}

VdPicture * VdPicture::getVirtual ( int i )
{ if ((i<0)||(i>=!virtPictures)) return NULL;
  return virtPictures[i];
}

void VdPicture::clearVirtuals( void )
{ while (!virtPictures) derefer(virtPictures[0]);
}

void VdPicture::addHandle ( VdHandle * h )
{ if (handles.idx(h)<0) handles.ins()=h;
}

void VdPicture::remHandle ( VdHandle * h )
{ handles.rem(handles.idx(h));
}

void VdPicture::collectHandles ( VdAllSelectedHandles * pll )
{ int i; VdSelectedHandles * hptr;
  for (i=0; i<!handles; i++)
    if ( handles[i]->test(&(pll->theEvent),this) ) 
    { hptr=&(pll->selections.ins());
      hptr->thePicture=this;
      hptr->handles.ins()=handles[i];
      break;
    }
  for (i++; i<!handles; i++)
    if ( handles[i]->test(&(pll->theEvent),this) ) 
      hptr->handles.ins()=handles[i];
  for (i=0; i<!realPictures; i++)
    realPictures[i]->collectHandles(pll);
  for (i=0; i<!virtPictures; i++)
    virtPictures[i]->collectHandles(pll);
}

void VdPicture::include ( VdHandle * h )
{ addHandle(h);
  h->addPicture(this);
}

void VdPicture::includeAt ( VdHandle * h , int idx)
{ if (handles.idx(h)>=0) return;
  if (idx>=!handles) handles.ins()=h;
  else handles.ins(idx)=h;
  h->addPicture(this);
}

void VdPicture::exclude ( VdHandle * h )
{ remHandle(h);
  h->remPicture(this);
}

int VdPicture::numberOfHandles ( void ) const
{ return !handles;
}

int VdPicture::idxOfHandle ( VdHandle * h )
{ return handles.idx(h);
}

VdHandle * VdPicture::getHandle ( int i )
{ if ((i<0)||(i>=!handles)) return NULL;
  return handles[i];
}

void VdPicture::clearHandles( void )
{ while (!handles) exclude(handles[0]);
}

const VdString & VdPicture::getSelectionIdentifier( void )
{ static VdString ident("");
  return ident;
}

void VdPicture::enterHandleSelection( const VdEvent * theEvent )
{ forewhileselect=foreground;
  foreground=selection;
}

void VdPicture::leaveHandleSelection( const VdEvent * theEvent )
{ if (foreground!=forewhileselect)
  {  foreground=forewhileselect; 
  }
}

void VdPicture::setSelectionColor( const VdColor & color )
{ selection=color;
}

VdColor VdPicture::getSelectionColor( void )
{ return selection;
}

void VdPicture::setAutoFrame ( int i )
{ autoFrame = (i?1:0);
}

void VdPicture::setFrame ( const VdArea & a )
{ if (autoFrame); 
  else
  { frame=a;
  }
}

void VdPicture::setForeground ( const VdColor & c )
{ foreground = c;
  if (selection==foreground)
  { if (selection==VdGREEN) selection=VdRED;
    else selection=VdGREEN;
  }
}

void VdPicture::setBackground ( const VdColor & c )
{ background = c;
}

void VdPicture::setLineType ( const VdLineType &  l )
{ lineType=l;
}

void VdPicture::setFillType ( const VdFillType & f )
{ fillType=f;
}

void VdPicture::setArcType ( const VdArcType & a )
{ arcType=a;
}

void VdPicture::setTextSize   ( const VdVector & t )
{ textSize=t;
  if (textSize.x<0) textSize.x=-textSize.x;
  if (textSize.y<0) textSize.y=-textSize.y;
}

void VdPicture::setLocation ( const VdPoint & p )
{ location = p;
}

void VdPicture::moveTo ( const VdPoint & p )
{ VdVector d = p-location;
  location = p;
  int i;
  for (i=0; i<!realPictures; i++ ) realPictures[i]->shift(d);
  for (i=0; i<!virtPictures; i++ ) virtPictures[i]->shift(d);
}

void VdPicture::shift ( const VdVector & v )
{ location += v;
  int i;
  for (i=0; i<!realPictures; i++ ) realPictures[i]->shift(v);
  for (i=0; i<!virtPictures; i++ ) virtPictures[i]->shift(v);
}

int VdPicture::isAutoFrame (void) const
{ return autoFrame;
}

VdArea VdPicture::getFrame (void)
{ return frame;
}
  
VdArea VdPicture::getAbsFrame (void)
{ return frame+VdVector(location);
}

VdPoint VdPicture::getFramePoint ( int kind ) const
{ VdVector v;
  switch (kind&0x3)
  { case VdXCENTRE : v.x=(frame.ul.x+frame.lr.x)>>1; break;
    case VdXRIGHT  : v.x=frame.lr.x; break;
    default       : v.x=frame.ul.x; break;
  }
  switch (kind&0xC)
  { case VdYCENTRE : v.y=(frame.ul.y+frame.lr.y)>>1; break;
    case VdYTOP    : v.y=frame.ul.y; break;
    default        : v.y=frame.lr.y; break;
  }
  return location+v;
}

VdColor VdPicture::getForeground (void) const
{ return foreground;
}

VdColor VdPicture::getBackground (void) const
{ return background;
}

VdLineType VdPicture::getLineType (void) const
{ return lineType;
}

VdFillType VdPicture::getFillType (void) const
{ return fillType;
}

VdArcType VdPicture::getArcType (void) const
{ return arcType;
}

VdVector VdPicture::getTextSize (void) const
{ return textSize;
}

VdPoint VdPicture::getLocation (void) const
{ return location;
}

void VdPicture::prepare ( void )
{ int i;
  manipulate();
  for (i=0; i<!realPictures; i++ ) realPictures[i]->prepare();
  for (i=0; i<!virtPictures; i++ ) virtPictures[i]->prepare();
  layout();
  computeFrame();
}

void VdPicture::manipulate ( void )
{
}

void VdPicture::layout ( void )
{
}

void VdPicture::computeFrame ( void )
{ if (autoFrame)
  { int i;
    frame=VdEmptyArea;
    for (i=0; i<!realPictures; i++ ) frame|=realPictures[i]->getAbsFrame();
    for (i=0; i<!virtPictures; i++ ) frame|=virtPictures[i]->getAbsFrame();
    frame-=(VdVector)location;
  }
}

void VdPicture::clearPicture ( void )
{ while (!realPictures) exclude(realPictures[0]);
  while (!virtPictures) derefer(virtPictures[0]);
  while (!handles) exclude(handles[0]);
}


void VdPicture::draw( VdGeometricContext * G
                    , VdGraphicAttributes GA
                    ) 
{ int i; 
  GA.setForeground(foreground);
  GA.setBackground(background);
  GA.setLineType(lineType);
  GA.setFillType(fillType);
  GA.setArcType(arcType);
  if ((textSize.x)&&(textSize.y))
    GA.setFontSize(G->transPictToWind(textSize));
  for( i=0; i<!realPictures; i++) realPictures[i]->draw(G,GA); 
}

void VdPicture::printPostScript(FILE* f)
{ int i, s=0; 
  if(  (lineType!=VdLINEUNDEF)
     ||(fillType!=VdFILLUNDEF)
     ||((textSize.x)&&(textSize.y))
     ||(arcType!=VdARCUNDEF)
     ||(background!=VdCOLORUNDEF)
    )
  { fprintf(f,"save\n"); s=1; }
  else
  if (foreground!=VdCOLORUNDEF)
  { fprintf(f,"gsave\n"); s=2; }
  if (s)
  { if (lineType!=VdLINEUNDEF) fprintf(f,"%d SLS ",2);
    if (fillType!=VdFILLUNDEF)
      fprintf(f,"%d SFP "
             ,2
             );
    if ((textSize.x)&&(textSize.y))
      fprintf(f,"%d /Times-Roman F ",textSize.y);
    if (arcType!=VdARCUNDEF) fprintf(f,"%d SAS ",2);
    if (background!=VdCOLORUNDEF)
      fprintf(f,"%d SBC "
             ,(background.code>>4)+((background.code&0xf)<<2)
             );
    if (foreground!=VdCOLORUNDEF)
      fprintf(f,"%d SC "
             ,(foreground.code>>4)+((foreground.code&0xf)<<2)
             );
    fprintf(f,"\n");
  }
  for( i=0; i<!realPictures; i++) realPictures[i]->printPostScript(f); 
  switch (s) { case 1 : fprintf(f,"restore\n"); break;
               case 2 : fprintf(f,"grestore\n"); break;
             }
}

void VdPicture::print( int i)
{ int j;
  fprintf(stderr,
          " VdPicture"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
  for (j=0; j<!realPictures; j++)
  { fprintf(stderr,"%*dR->",i+3,j);
    realPictures[j]->print(i+3);
  }
  for (j=0; j<!virtPictures; j++)
  { fprintf(stderr,"%*dV->",i+3,j);
    virtPictures[j]->print(i+3);
  }
}

VdDot::VdDot ( const VdPoint & p )
{ dot=p;
}
 
VdDot::VdDot(int x, int y)
{ VdPoint p(x,y); dot=p;
}

VdPicture * VdDot::duplicate(void)
{ VdDot * pict = new VdDot;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->dot = dot;
  return pict;
}

void VdDot::setDot (const VdPoint & p)
{ dot=p;
}

void VdDot::setDot(int x, int y)
{ VdPoint p(x,y); dot=p;
}

VdPoint VdDot::getDot(void)
{ return dot;
}

void VdDot::prepare ( void )
{ VdDot::computeFrame();
}

void VdDot::computeFrame ( void )
{ VdArea a(dot,dot);
  if (autoFrame) frame=a;
}

void VdDot::draw( VdGeometricContext * G
                , VdGraphicAttributes GA
                ) 
{ 
  VdPoint p = G->transPictToWind( dot+(VdVector)location ); 
  GA.setForeground(foreground); 
 
  XDrawPoint( GA.getDisplay(), GA.getDrawable(), GA.getGC(), 
              p.x, p.y 
            ); 
} 

void VdDot::printPostScript(FILE* f)
{ VdPoint p = dot+(VdVector)location; 
  if (foreground!=VdCOLORUNDEF)
  { fprintf(f,"gsave\n");
    fprintf(f,"%d SC\n"
           ,(foreground.code>>4)+((foreground.code&0xf)<<2)
           );
  }
  fprintf(f,"%d %d D\n",p.x,p.y);
  if (foreground!=VdCOLORUNDEF) fprintf(f,"grestore\n");
}

void VdDot::print( int i)
{ fprintf(stderr,
          " VdDot(%d,%d)"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,dot.x,dot.y
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}

 VdRectangle::VdRectangle( const VdPoint & p1
                         , const VdPoint & p2
                         )
{ VdArea a(p1,p2); rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
VdRectangle::VdRectangle( const VdPoint & p1
                        , int width, int height
                        )
{ VdArea a(p1,width,height); rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
VdRectangle:: VdRectangle( const VdArea & a ) 
{ rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
VdPicture * VdRectangle::duplicate(void)
{ VdRectangle * pict = new VdRectangle;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->rectangle=rectangle;
  return pict;
}

void VdRectangle::setRectangle( const VdPoint & p1
                              , const VdPoint & p2
                              )
{ VdArea a(p1,p2); rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
void VdRectangle::setRectangle( const VdPoint & p1
                              , int width, int height
                              )
{ VdArea a(p1,width,height); rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
void VdRectangle::setRectangle( const VdArea & a ) 
{ rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
VdArea VdRectangle::getRectangle ( void ) const
{ return rectangle;
}

void VdRectangle::prepare ( void )
{ VdRectangle::computeFrame();
}

void VdRectangle::computeFrame ( void )
{ 
}

void VdRectangle::draw( VdGeometricContext * G
                      , VdGraphicAttributes GA
                      ) 
{ VdPoint p = G->transPictToWind(rectangle.ul+(VdVector)location); 
  VdPoint q = G->transPictToWind(rectangle.lr+(VdVector)location); 
  int width = q.x - p.x; 
  int height = q.y - p.y; 
  GA.setForeground(foreground);
  GA.setBackground(background);
  GA.setLineType(lineType); 

  XDrawRectangle( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
                  p.x, p.y, width, height 
                );

} 

void VdRectangle::printPostScript(FILE* f)
{ int s=0; 
  VdPoint p = rectangle.ul+(VdVector)location; 
  VdPoint q = rectangle.lr+(VdVector)location; 
  int width = q.x - p.x; 
  int height = p.y - q.y; 
   if(  (lineType!=VdLINEUNDEF)
     ||(background!=VdCOLORUNDEF)
    )
  { fprintf(f,"save\n"); s=1; }
  else
  if (foreground!=VdCOLORUNDEF)
  { fprintf(f,"gsave\n"); s=2; }
  if (s)
  { if (lineType!=VdLINEUNDEF) fprintf(f,"%d SLS ",2);
    if (background!=VdCOLORUNDEF)
      fprintf(f,"%d SBC "
             ,(background.code>>4)+((background.code&0xf)<<2)
             );
    if (foreground!=VdCOLORUNDEF)
      fprintf(f,"%d SC "
             ,(foreground.code>>4)+((foreground.code&0xf)<<2)
             );
    fprintf(f,"\n");
  }
  fprintf(f,"%d %d %d %d R\n",width,height,p.x,q.y 
         );
  switch (s) { case 1 : fprintf(f,"restore\n"); break;
               case 2 : fprintf(f,"grestore\n"); break;
             }
} 

void VdRectangle::print( int i)
{ fprintf(stderr,
          " VdRectangle((%d,%d)(%d,%d))"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,rectangle.ul.x,rectangle.ul.y,rectangle.lr.x,rectangle.lr.y
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}



VdFilledRectangle::VdFilledRectangle( const VdPoint & p1
                                    , const VdPoint & p2
                                    )
{ VdArea a(p1,p2); rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
VdFilledRectangle::VdFilledRectangle( const VdPoint & p1
                                    , int width, int height
                                    )
{ VdArea a(p1,width,height); rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
VdFilledRectangle:: VdFilledRectangle( const VdArea & a ) 
{ rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
VdPicture * VdFilledRectangle::duplicate(void)
{ VdFilledRectangle * pict = new VdFilledRectangle;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->rectangle=rectangle;
  return pict;
}

void VdFilledRectangle::setRectangle( const VdPoint & p1
                                    , const VdPoint & p2
                                    )
{ VdArea a(p1,p2); rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
void VdFilledRectangle::setRectangle( const VdPoint & p1
                                    , int width, int height
                                    )
{ VdArea a(p1,width,height); rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
void VdFilledRectangle::setRectangle( const VdArea & a ) 
{ rectangle=a;
  if (autoFrame)
  { frame = a;
  }
}
 
VdArea VdFilledRectangle::getRectangle ( void ) const
{ return rectangle;
}

void VdFilledRectangle::prepare ( void )
{ VdFilledRectangle::computeFrame();
}

void VdFilledRectangle::computeFrame( void )
{
}

void VdFilledRectangle::draw( VdGeometricContext * G
                            , VdGraphicAttributes GA
                            ) 
{ VdPoint p = G->transPictToWind(rectangle.ul+(VdVector)location); 
  VdPoint q = G->transPictToWind(rectangle.lr+(VdVector)location); 
  int width = q.x - p.x; 
  int height = q.y - p.y; 
  GA.setForeground(foreground);
  GA.setBackground(background);
  GA.setLineType(lineType); 
  GA.setFillType(fillType); 

  XFillRectangle( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
                  p.x, p.y, width+1, height+1 
                );
} 
         
void VdFilledRectangle::printPostScript(FILE* f)
{ int s=0; 
  VdPoint p = rectangle.ul+(VdVector)location; 
  VdPoint q = rectangle.lr+(VdVector)location; 
  int width = q.x - p.x; 
  int height = p.y - q.y; 
   if(  (lineType!=VdLINEUNDEF)
     ||(fillType!=VdFILLUNDEF)
     ||(background!=VdCOLORUNDEF)
    )
  { fprintf(f,"save\n"); s=1; }
  else
  if (foreground!=VdCOLORUNDEF)
  { fprintf(f,"gsave\n"); s=2; }
  if (s)
  { if (lineType!=VdLINEUNDEF) fprintf(f,"%d SLS ",2);
    if (fillType!=VdFILLUNDEF)
      fprintf(f,"%d SFP "
             ,2
             );
    if (background!=VdCOLORUNDEF)
      fprintf(f,"%d SBC "
             ,(background.code>>4)+((background.code&0xf)<<2)
             );
    if (foreground!=VdCOLORUNDEF)
      fprintf(f,"%d SC "
             ,(foreground.code>>4)+((foreground.code&0xf)<<2)
             );
    fprintf(f,"\n");
  }
  fprintf(f,"%d %d %d %d FR\n",width,height,p.x,q.y
         );
  switch (s) { case 1 : fprintf(f,"restore\n"); break;
               case 2 : fprintf(f,"grestore\n"); break;
             }
} 

void VdFilledRectangle::print( int i)
{ fprintf(stderr,
          " VdFilledRectangle((%d,%d)(%d,%d))"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,rectangle.ul.x,rectangle.ul.y,rectangle.lr.x,rectangle.lr.y
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}

 
 
VdLine::VdLine( const VdPoint & from
              , const VdPoint & to
              ) 
{ startPoint = from;  endPoint = to;
}

VdLine::VdLine( const VdPoint  &  from
              , const VdVector &  v
              , int betrag
              ) 
{ startPoint = from; endPoint = from + v.norm(betrag); 
}
 
VdPicture * VdLine::duplicate(void)
{ VdLine * pict = new VdLine;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->startPoint=startPoint;
  pict->endPoint=endPoint;
  return pict;
}

void VdLine::setStartPoint ( const VdPoint & p )
{ startPoint=p;
}

void VdLine::setEndPoint ( const VdPoint & p )
{ endPoint=p;
}

VdPoint VdLine::getStartPoint ( void ) const
{ return startPoint;
}

VdPoint    VdLine::getEndPoint ( void ) const
{ return endPoint;
}

void VdLine::prepare ( void )
{ VdLine::computeFrame();
}

void VdLine::computeFrame( void )
{ VdArea a(startPoint,endPoint);
  if (autoFrame) frame=a;
}

void VdLine::draw( VdGeometricContext * G
                 , VdGraphicAttributes GA 
                 ) 
{  VdPoint p = G->transPictToWind(startPoint+(VdVector)location); 
   VdPoint q = G->transPictToWind(endPoint+(VdVector)location); 
   GA.setForeground(foreground); 
   GA.setBackground(background); 
   GA.setLineType(lineType); 
 
   XDrawLine( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
              p.x, p.y, q.x, q.y
            );
} 

void VdLine::printPostScript(FILE* f)
{ int s=0; 
  VdPoint p = startPoint+(VdVector)location; 
  VdPoint q = endPoint+(VdVector)location; 
   if(  (lineType!=VdLINEUNDEF)
     ||(background!=VdCOLORUNDEF)
    )
  { fprintf(f,"save\n"); s=1; }
  else
  if (foreground!=VdCOLORUNDEF)
  { fprintf(f,"gsave\n"); s=2; }
  if (s)
  { if (lineType!=VdLINEUNDEF) fprintf(f,"%d SLS ",2);
    if (background!=VdCOLORUNDEF)
      fprintf(f,"%d SBC "
             ,(background.code>>4)+((background.code&0xf)<<2)
             );
    if (foreground!=VdCOLORUNDEF)
      fprintf(f,"%d SC "
             ,(foreground.code>>4)+((foreground.code&0xf)<<2)
             );
    fprintf(f,"\n");
  }
   fprintf(f,"%d %d %d %d L\n",p.x,p.y,q.x,q.y
          );
  switch (s) { case 1 : fprintf(f,"restore\n"); break;
               case 2 : fprintf(f,"grestore\n"); break;
             }
}
 
void VdLine::print( int i)
{ fprintf(stderr,
          " VdLine"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}

 
 
VdArc::VdArc( const VdPoint & center,         
              int     aa          ,              // Hauptscheitel
              int     bb          ,              // Nebenscheitel
              int   alpha1        ,              // Winkel bzgl. pos. Achse 
              int   beta1         ,              // Winkel des Bogens 
              char    grad                       // Masseinheit der Winkel 
            )  
{  a = (aa>0?aa:-aa); b = (bb>0?bb:-bb);
   if (a==0)
   { if (b==0) a=1;
     else a=b;
   }
   if (b==0) b=a;
   switch (grad)
   { case 'g': case 'G': alpha = alpha1*64; beta = beta1*64; break;
     case 'r': case 'R': alpha = (int)(VdRadToGrad*alpha1*64);
                         beta  = (int)(VdRadToGrad*beta1*64);
                         break;
     default:            alpha = alpha1; beta = beta1; break;
   }
   if (alpha>23040) alpha%=23040;
   if (alpha<0) alpha=23040-((-alpha)%23040);
   if (alpha==23040) alpha=0;
   if (beta>23040) beta=23040;
   if (beta<-23040) beta=-23040;
   centre = center;
} 

VdPicture * VdArc::duplicate(void)
{ VdArc * pict = new VdArc;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->alpha=alpha;
  pict->beta=beta;
  pict->centre=centre;
  pict->a=a;
  pict->b=b;
  return pict;
}

void VdArc::setStartPoint ( const VdPoint & p )
{ centre+=p-getStartPoint();
}

void VdArc::setEndPoint ( const VdPoint & p )
{ centre+=p-getEndPoint();
}

void VdArc::setCentre ( const VdPoint & p )
{ centre = p; 
}

void VdArc::setAlpha( int alpha1 , char grad )
{ switch (grad)
  { case 'g': case 'G': alpha = alpha1*64; break;
    case 'r': case 'R': alpha = (int)(VdRadToGrad*alpha1*64);
                        break;
    default:            alpha = alpha1; break;
  }
  if (alpha>23040) alpha%=23040;
  if (alpha<0) alpha=23040-((-alpha)%23040);
  if (alpha==23040) alpha=0;
}

void VdArc::setBeta( int beta1 , char grad )
{ switch (grad)
  { case 'g': case 'G': beta = beta1*64; break;
    case 'r': case 'R': beta  = (int)(VdRadToGrad*beta1*64);
                        break;
    default:            beta = beta1; break;
  }
  if (beta>23040) beta=23040;
  if (beta<-23040) beta=-23040;
}

void VdArc::setAxisA ( int aa )
{ if (aa) a=(aa<0)?-aa:aa;
}

void VdArc::setAxisB ( int bb )
{ if (bb) b=(bb<0)?-bb:bb;
}

VdPoint VdArc::getStartPoint ( void ) const
{ int c = cos((VdGradToRad*alpha)/64)*a; 
  int s = sin((VdGradToRad*alpha)/64)*b;
  VdVector v(c,s);
  return centre+v;
}

VdPoint VdArc::getEndPoint ( void ) const
{ int c = cos((VdGradToRad*(alpha+beta))/64)*a; 
  int s = sin((VdGradToRad*(alpha+beta))/64)*b;
  VdVector v(c,s);
  return centre+v;
}

VdPoint VdArc::getCentre ( void ) const
{ return centre;
}

int VdArc::getAlpha ( char grad ) const
{ switch (grad)
  { case 'g': case 'G': return (alpha+32)/64;
    case 'r': case 'R': return VdGradToRad*((alpha+32)/64);
    default:            return alpha;
  }
}

int VdArc::getBeta ( char grad ) const
{ switch (grad)
  { case 'g': case 'G': return (beta+32)/64;
    case 'r': case 'R': return VdGradToRad*((beta+32)/64);
    default:            return beta;
  }
}

int VdArc::getAxisA ( void ) const
{ return a;
} 

int VdArc::getAxisB ( void ) const
{ return b;
} 

void VdArc::prepare ( void )
{ VdArc::computeFrame();
}

void VdArc::computeFrame( void )
{ 
  if (autoFrame)
  { int rot, aw, ew, aa, bb, xmin, xmax, ymin, ymax;
    VdPoint p1, p2;
    if (beta>=0)
    { aw=alpha; ew=beta; }
    else
    { if ((aw=alpha+beta)<0) aw+=23040; ew=-beta; }
    rot=aw/5760;
    aw -= rot*5760;
    if (rot&1)
    { aa=b; bb=a; }
    else
    { aa=a,bb=b; }
    int awx = cos((VdGradToRad*aw)/64)*aa; 
    int awy = sin((VdGradToRad*aw)/64)*bb;
    int ewx = cos((VdGradToRad*ew)/64)*aa; 
    int ewy = sin((VdGradToRad*ew)/64)*bb;
    switch ((aw+ew)/5760)
    { case 0: xmin=ewx; xmax=awx;
              ymin=awy; xmax=ewy;
              break;
      case 1: xmin=ewx; xmax=awx;
              ymin=(awy<ewy)?awy:ewy; ymax=bb;
              break;
      case 2: xmin=-aa; xmax=awx;
              ymin=ewy; ymax=bb;
              break;
      case 3: xmin=-aa; xmax=(awx<ewx)?ewx:awx;
              ymin=-bb; ymax=bb;
              break;
      case 4: xmin=-aa; xmax=aa;
              ymin=-bb; ymax=bb;
              break;
    }
    switch (rot)
    { case 0: p1.x=xmin; p1.y=ymin; p2.x=xmax; p2.y=ymax; break;
      case 1: p1.x=-ymin; p1.y=xmin; p2.x=-ymax; p2.y=xmax; break;
      case 2: p1.x=-xmin; p1.y=-ymin; p2.x=-xmax; p2.y=-ymax; break;
      case 3: p1.x=ymin; p1.y=-xmin; p2.x=ymax; p2.y=-xmax; break;
    }
    VdArea ha(p1+VdVector(centre),p2+VdVector(centre));
    frame=ha;
  }
}

void VdArc::draw( VdGeometricContext * G
                , VdGraphicAttributes GA
                )
{ VdVector v(-a,b);
  VdPoint p = G->transPictToWind(centre+v+(VdVector)location); 
  VdPoint q = G->transPictToWind(centre-v+(VdVector)location); 
  int width = q.x - p.x; 
  int height = q.y - p.y; 

  GA.setForeground(foreground); 
  GA.setBackground(background); 
  GA.setLineType(lineType); 

  XDrawArc( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
            p.x, p.y, width, height,alpha,beta
          ); 
}

void VdArc::printPostScript(FILE* f)
{ int s=0; 
  VdVector v(a,b);
  VdPoint q1 = centre-v+(VdVector)location; 
  VdPoint q2 = centre+v+(VdVector)location; 
  int width  = (q2.x - q1.x)>>1; 
  int height = (q2.y - q1.y)>>1; 
  int aw, ew, hw;
  VdPoint p = centre+(VdVector)location; 
   if(  (lineType!=VdLINEUNDEF)
     ||(background!=VdCOLORUNDEF)
    )
  { fprintf(f,"save\n"); s=1; }
  else
  if (foreground!=VdCOLORUNDEF)
  { fprintf(f,"gsave\n"); s=2; }
  if (s)
  { if (lineType!=VdLINEUNDEF) fprintf(f,"%d SLS ",2);
    if (background!=VdCOLORUNDEF)
      fprintf(f,"%d SBC "
             ,(background.code>>4)+((background.code&0xf)<<2)
             );
    if (foreground!=VdCOLORUNDEF)
      fprintf(f,"%d SC "
             ,(foreground.code>>4)+((foreground.code&0xf)<<2)
             );
    fprintf(f,"\n");
  }  
  aw=alpha>>6; ew=(alpha+beta)>>6;
  if (aw>ew) { hw=ew; ew=aw; aw=hw; }
  fprintf(f,"%d %d %d %d %d %d %d %d A\n"
           ,p.x,p.y,width,aw,ew,p.y,height,width
          ); 
  switch (s) { case 1 : fprintf(f,"restore\n"); break;
               case 2 : fprintf(f,"grestore\n"); break;
             }
}

void VdArc::print( int i)
{ fprintf(stderr,
          " VdArc((%d,%d)[%d,%d]<%d,%d>)"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,centre.x,centre.y,a,b,alpha,beta
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}

 
 
VdFilledArc::VdFilledArc( const VdPoint & center,         
              int     aa          ,              // Hauptscheitel
              int     bb          ,              // Nebenscheitel
              int   alpha1        ,              // Winkel bzgl. pos. Achse 
              int   beta1         ,              // Winkel des Bogens 
              char    grad                       // Masseinheit der Winkel 
            )  
{  a = (aa>0?aa:-aa); b = (bb>0?bb:-bb);
   if (a==0)
   { if (b==0) a=1;
     else a=b;
   }
   if (b==0) b=a;
   switch (grad)
   { case 'g': case 'G': alpha = alpha1*64; beta = beta1*64; break;
     case 'r': case 'R': alpha = (int)(VdRadToGrad*alpha1*64);
                         beta  = (int)(VdRadToGrad*beta1*64);
                         break;
     default:            alpha = alpha1; beta = beta1; break;
   }
   if (alpha>23040) alpha%=23040;
   if (alpha<0) alpha=23040-((-alpha)%23040);
   if (alpha==23040) alpha=0;
   if (beta>23040) beta=23040;
   if (beta<-23040) beta=-23040;
   centre = center;
} 

VdPicture * VdFilledArc::duplicate(void)
{ VdFilledArc * pict = new VdFilledArc;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->alpha=alpha;
  pict->beta=beta;
  pict->centre=centre;
  pict->a=a;
  pict->b=b;
  return pict;
}

void VdFilledArc::setStartPoint ( const VdPoint & p )
{ centre+=p-getStartPoint();
}

void VdFilledArc::setEndPoint ( const VdPoint & p )
{ centre+=p-getEndPoint();
}

void VdFilledArc::setCentre ( const VdPoint & p )
{ centre = p; 
}

void VdFilledArc::setAlpha( int alpha1 , char grad )
{ switch (grad)
  { case 'g': case 'G': alpha = alpha1*64; break;
    case 'r': case 'R': alpha = (int)(VdRadToGrad*alpha1*64);
                        break;
    default:            alpha = alpha1; break;
  }
  if (alpha>23040) alpha%=23040;
  if (alpha<0) alpha=23040-((-alpha)%23040);
  if (alpha==23040) alpha=0;
}

void VdFilledArc::setBeta( int beta1 , char grad )
{ switch (grad)
  { case 'g': case 'G': beta = beta1*64; break;
    case 'r': case 'R': beta  = (int)(VdRadToGrad*beta1*64);
                        break;
    default:            beta = beta1; break;
  }
  if (beta>23040) beta=23040;
  if (beta<-23040) beta=-23040;
}

void VdFilledArc::setAxisA ( int aa )
{ if (aa) a=(aa<0)?-aa:aa;
}

void VdFilledArc::setAxisB ( int bb )
{ if (bb) b=(bb<0)?-bb:bb;
}

VdPoint VdFilledArc::getStartPoint ( void ) const
{ int c = cos((VdGradToRad*alpha)/64)*a; 
  int s = sin((VdGradToRad*alpha)/64)*b;
  VdVector v(c,s);
  return centre+v;
}

VdPoint VdFilledArc::getEndPoint ( void ) const
{ int c = cos((VdGradToRad*(alpha+beta))/64)*a; 
  int s = sin((VdGradToRad*(alpha+beta))/64)*b;
  VdVector v(c,s);
  return centre+v;
}

VdPoint VdFilledArc::getCentre ( void ) const
{ return centre;
}

int VdFilledArc::getAlpha ( char grad ) const
{ switch (grad)
  { case 'g': case 'G': return (alpha+32)/64;
    case 'r': case 'R': return VdGradToRad*((alpha+32)/64);
    default:            return alpha;
  }
}

int VdFilledArc::getBeta ( char grad ) const
{ switch (grad)
  { case 'g': case 'G': return (beta+32)/64;
    case 'r': case 'R': return VdGradToRad*((beta+32)/64);
    default:            return beta;
  }
}

int VdFilledArc::getAxisA ( void ) const
{ return a;
} 

int VdFilledArc::getAxisB ( void ) const
{ return b;
} 

void VdFilledArc::prepare ( void )
{ VdFilledArc::computeFrame();
}

void VdFilledArc::computeFrame( void )
{
  if (autoFrame)
  { int rot, aw, ew, aa, bb, xmin, xmax, ymin, ymax;
    VdPoint p1, p2;
    if (beta>=0)
    { aw=alpha; ew=beta; }
    else
    { if ((aw=alpha+beta)<0) aw+=23040; ew=-beta; }
    rot=aw/5760;
    aw -= rot*5760;
    if (rot&1)
    { aa=b; bb=a; }
    else
    { aa=a,bb=b; }
    int awx = cos((VdGradToRad*aw)/64)*aa; 
    int awy = sin((VdGradToRad*aw)/64)*bb;
    int ewx = cos((VdGradToRad*ew)/64)*aa; 
    int ewy = sin((VdGradToRad*ew)/64)*bb;
    switch ((aw+ew)/5760)
    { case 0: xmin=ewx; xmax=awx;
              ymin=awy; xmax=ewy;
              break;
      case 1: xmin=ewx; xmax=awx;
              ymin=(awy<ewy)?awy:ewy; ymax=bb;
              break;
      case 2: xmin=-aa; xmax=awx;
              ymin=ewy; ymax=bb;
              break;
      case 3: xmin=-aa; xmax=(awx<ewx)?ewx:awx;
              ymin=-bb; ymax=bb;
              break;
      case 4: xmin=-aa; xmax=aa;
              ymin=-bb; ymax=bb;
              break;
    }
    switch (rot)
    { case 0: p1.x=xmin; p1.y=ymin; p2.x=xmax; p2.y=ymax; break;
      case 1: p1.x=-ymin; p1.y=xmin; p2.x=-ymax; p2.y=xmax; break;
      case 2: p1.x=-xmin; p1.y=-ymin; p2.x=-xmax; p2.y=-ymax; break;
      case 3: p1.x=ymin; p1.y=-xmin; p2.x=ymax; p2.y=-xmax; break;
    }
    VdArea ha(p1+VdVector(centre),p2+VdVector(centre));
    frame=ha;
  }
}

void VdFilledArc::draw( VdGeometricContext * G
                      , VdGraphicAttributes GA
                      )
{ VdVector v(-a,b);
  VdPoint p = G->transPictToWind(centre+v+(VdVector)location); 
  VdPoint q = G->transPictToWind(centre-v+(VdVector)location); 
  int width = q.x - p.x; 
  int height = q.y - p.y; 

  GA.setForeground(foreground); 
  GA.setBackground(background); 
  GA.setLineType(lineType); 
  GA.setFillType(fillType);
  GA.setArcType(arcType);

  XFillArc( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
            p.x, p.y, width, height,alpha,beta
          ); 
}

void VdFilledArc::printPostScript(FILE* f)
{ int s=0; 
  VdVector v(a,b);
  VdPoint q1 = centre-v+(VdVector)location; 
  VdPoint q2 = centre+v+(VdVector)location; 
  int width  = (q2.x - q1.x)>>1; 
  int height = (q2.y - q1.y)>>1; 
  int aw, ew, hw;
  VdPoint p = centre+(VdVector)location; 
   if(  (lineType!=VdLINEUNDEF)
     ||(fillType!=VdFILLUNDEF)
     ||(arcType!=VdARCUNDEF)
     ||(background!=VdCOLORUNDEF)
    )
  { fprintf(f,"save\n"); s=1; }
  else
  if (foreground!=VdCOLORUNDEF)
  { fprintf(f,"gsave\n"); s=2; }
  if (s)
  { if (lineType!=VdLINEUNDEF) fprintf(f,"%d SLS ",2);
    if (fillType!=VdFILLUNDEF)
      fprintf(f,"%d SFP "
             ,2
             );
    if (background!=VdCOLORUNDEF)
      fprintf(f,"%d SBC "
             ,(background.code>>4)+((background.code&0xf)<<2)
             );
    if (arcType!=VdARCUNDEF) fprintf(f,"%d SAS ",2);
    if (foreground!=VdCOLORUNDEF)
      fprintf(f,"%d SC "
             ,(foreground.code>>4)+((foreground.code&0xf)<<2)
             );
    fprintf(f,"\n");
  }  
  aw=alpha>>6; ew=(alpha+beta)>>6;
  if (aw>ew) { hw=ew; ew=aw; aw=hw; }
  fprintf(f,"%d %d %d %d %d %d %d %d FA\n"
           ,p.x,p.y,width,aw,ew,p.y,height,width
          ); 
  switch (s) { case 1 : fprintf(f,"restore\n"); break;
               case 2 : fprintf(f,"grestore\n"); break;
             }
}

void VdFilledArc::print( int i)
{ fprintf(stderr,
          " VdFilledArc((%d,%d)[%d,%d]<%d,%d>)"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,centre.x,centre.y,a,b,alpha,beta
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}

static void VdPrepareTextString(VdString & theString,
                                int & lines, int & width
                               )
{ char *optr; int w;
  lines=1; width=w=0;
  optr=theString.charptr();
  if (optr == NULL) return;
  while(1)
  { switch(*optr++)
    { case '\0' : if (w>width) width=w;
                  return;
      case '\n' : lines++;
                  if (w>width) width=w;
                  w=0;
                  break;
      default   : w++;
    }
  }
}
         


VdText::VdText ( const VdString & str
               , const VdVector & sz
               )
{ string=str;
  VdPrepareTextString(string,lines,width);
  textSize=sz;
  if (textSize.x<0) textSize.x=-textSize.x;
  if (textSize.y<0) textSize.y=-textSize.y;
}

VdText::VdText ( const VdText & t )
{ string=t.string; textSize=t.textSize;
  lines=t.lines; width=t.width;
}

VdPicture * VdText::duplicate(void)
{ VdText * pict = new VdText;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->string=string;
  pict->lines=lines;
  pict->width=width;
  return pict;
}

void VdText::setString ( const VdString & s )
{ string=s;
  VdPrepareTextString(string,lines,width);
}

VdString VdText::getString (void) const
{  return string;
}

void VdText::prepare ( void )
{ VdText::computeFrame();
}

void VdText::computeFrame(void)
{ VdArea a(VdNullPoint,(VdPoint)textSize);
  a.lr.x*=width;
  a.lr.y=-(a.ul.y+2)*(lines-1);
  if (autoFrame)
    frame=a;
}

void VdText::draw( VdGeometricContext * G
                 , VdGraphicAttributes GA
                 )
{ char * sptr=string.charptr();
  if (sptr==NULL) return;
  VdVector realsize =G->transPictToWind(textSize);
  GA.setFontSize(realsize);
  GA.setForeground(foreground);
  VdVector fontsize = GA.getFontSize();
  VdPoint loc = location;
  VdLineType fontLineType = VdLINESOLID;
  VdVector offset(0,textSize.y>>1);
  VdPoint realbase, base;
  int i, j, w, m;
  char * hptr;
  if (fontsize==VdNullVector)
  { fontLineType((realsize.y)?(-(realsize.y)-1)|1:1);
    GA.setLineType(fontLineType);
    for (j=0; j<lines; j++)
    { realbase =G->transPictToWind(loc+offset);
      if (realsize.x>2)
      { w=0; while((*sptr)&&(*sptr++!='\n')) w++;
        for (i=0; i<w; i++)
          XDrawLine( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
                     realbase.x+i*realsize.x+1, realbase.y,
                     realbase.x+(i+1)*realsize.x-1, realbase.y
                   );
      }
      else
      { w=0; while((*sptr)&&(*sptr++!='\n')) w++;
        XDrawLine( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
                   realbase.x,realbase.y,
                   realbase.x+realsize.x*w,realbase.y
                 ); 
      }
      loc.y-=textSize.y+2;
    }
  }
  else
  { for (j=0; j<lines; j++)
    { w=0; hptr=sptr; while((*sptr)&&(*sptr!='\n')) { w++; sptr++; }
      if (*sptr) { m=1; *sptr='\0'; } else m=0;
      base =  G->transPictToWind(frame.ul+(VdVector)loc)
             +
              VdVector( (((realsize.x-fontsize.x)*!string)>>1)
                      , ((-fontsize.y-realsize.y)>>1)
                        + GA.getBias().y
                      );
      XDrawString( GA.getDisplay(), GA.getDrawable(), GA.getGC(),
                   base.x,base.y,
                   hptr, w
                 );
      if (m) *sptr++='\n';
      loc.y-=textSize.y+2;
    }
  }
}

void VdText::printPostScript(FILE* f)
{ char * sptr=string.charptr();
  if ((sptr==NULL)||(*sptr=='\0')) return;
  if ((textSize.x==0)||(textSize.y==0)) return;
  fprintf(f,"save\n");
  fprintf(f,"%d /Times-Roman F\n",textSize.y);
  if (foreground!=VdCOLORUNDEF)
    fprintf(f,"%d SC\n"
           ,(foreground.code>>4)+((foreground.code&0xf)<<2)
           );
  VdPoint loc = location+VdVector(0,(textSize.y+2)/5);
  int i, j, w;
  char * hptr, buffer[2048];
  for (j=0; j<lines; j++)
  { hptr=buffer; w=0;
    while((*sptr)&&(*sptr!='\n'))
    { if ((*sptr=='(')||(*sptr==')')) *hptr++='\\';
      if (*sptr=='\\') *hptr++='\\';
      *hptr++=*sptr++; w++;
    }
    *hptr='\0'; sptr++;
    if (w)
      fprintf(f,"(%s) %d %d %d T\n",buffer,w*textSize.x,loc.x,loc.y
           );
    loc.y-=textSize.y+2;
  }
  fprintf(f,"restore\n");
}

void VdText::print( int i)
{ char buffer[2048], *bptr, *sptr;
  bptr=buffer; sptr=string.charptr();
  do { if (*sptr=='\n')
       { *bptr++='\\'; *bptr++='n'; }
       else *bptr++=*sptr;
     } while (*sptr++);
  fprintf(stderr,
          " VdText(%s)"
          " Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,buffer
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}
 
#undef _VDPICTURESUMMARY_CXX
