#ifndef _VDPOINT_HXX
#define _VDPOINT_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdPoint.hxx                                       *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*     definition of points of the plane                    *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdPoint;

/******************    dependences    *********************/

#include <Vd/VdVector.hxx> 


class VdPoint{ 
   public : 
     int x, y; 

     VdPoint ( void ) { x = y = 0; } 
     VdPoint ( int x1 , int y1 = 0 ) { x = x1; y = y1;} 
     VdPoint ( const VdPoint & p ) { x = p.x; y = p.y; } 
     VdPoint ( const VdVector & v );        // Punkt zum Ortsvektor v 

     VdPoint operator+ ( const VdVector & v ) const;     // p = q + v 
     VdPoint operator- ( const VdVector & v ) const;     // p = q - v 
     VdPoint& operator+= ( const VdVector & v );         // p = p + v 
     VdPoint& operator-= ( const VdVector & v );         // p = q - v 

     VdVector operator- ( const VdPoint & ) const; 
 
     int operator== ( const VdPoint & ) const;        // Gleichheit 
     int operator!= ( const VdPoint & ) const;        // Ungleichheit 
 
     VdPoint rotate( int arc , char we = 'G' ) const; 

}; 

/*********************** constants *************************/ 
 
#ifdef _VDPOINT_CXX

   VdPoint  VdNullPoint;                          // Nullpunkt 
   float VdGradToRad = 0.017453294444;  // von Grad zu Radiant 
   float VdRadToGrad = 57.29577319532;  // von Radiant zu Grad 

#else

   extern const VdPoint  VdNullPoint;             // Nullpunkt 
   extern float VdGradToRad;            // von Grad zu Radiant 
   extern float VdRadToGrad;            // von Radiant zu Grad 

#endif

#endif /* _VDPOINT_HXX */
