#define _VDPOLYARROW_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdPolyArrow.C                                         *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdPolyArrow.hxx>
#include <Vd/VdArrow.hxx>
#include <Vd/VdLine.hxx>

#ifdef _HXX_INCLUDE_
#include <stdio.hxx>
#else
#include <stdio.h>
#endif

VdPolyArrow::VdPolyArrow( const VdPoint & from
                        , const VdPoint & to
                        ) 
{ points[0] = from;  points[1] = to;
  isModified=1;
}

VdPicture * VdPolyArrow::duplicate(void)
{ VdPolyArrow * pict = new VdPolyArrow;
  *((VdPicture*)pict) = *((VdPicture*)this);
  pict->autoControl=1;
  pict->points=points;
  pict->isModified=1;
  return pict;
}

void VdPolyArrow::setStartPoint ( const VdPoint & p )
{ points[0]=p;
  isModified=1;
}

void VdPolyArrow::setEndPoint ( const VdPoint & p )
{ points[!points-1]=p;
  isModified=1;
}

void VdPolyArrow::setIntermediate( int i, const VdPoint & p )
{ int l=!points-1;
  if (++i<l) points[i]=p;
  else
  { points[i+1]=points[l];
    if (i!=l) points[l]=VdNullPoint;
    points[i]=p;
  }
  isModified=1;
}

void VdPolyArrow::remIntermediate( int i )
{ if (i<0)
  { points[1]=points[!points-1];
    points.reset(2);
  }
  else
  { if (++i>=!points-1) return;
    points.rem(i);
  }
  isModified=1;
}

int VdPolyArrow::hasIntermediate (void) const
{ return !points-2;
}

VdPoint VdPolyArrow::getStartPoint ( void )
{ return points[0];
}

VdPoint VdPolyArrow::getEndPoint ( void )
{ return points[!points-1];
}

VdPoint VdPolyArrow::getIntermediate( int i )
{ if (++i>=!points-1) return VdNullPoint;
  return points[i];
}

void VdPolyArrow::reverse(void)
{ VdPoint h;
  int i = 0, j = !points-1;
  while (i<j)
  { h=points[i]; points[i]=points[j]; points[j]=h;
  }
  isModified=1;
}

static int simpleDist( const VdPoint & p1, const VdPoint & p2, const VdPoint & p )
{ int la, my;
  VdVector v=p2-p1;
  VdVector n(-(v.y),v.x);
  int w=v*v;
  if (w==0) return (int)!(p-p1);
  la=(p-p1)*v;
  if (la<=0) return (int)!(p-p1);
  if (la>=w) return (int)!(p-p2);
  return ((p1-p)*n+(w>>1))/w;
}

int VdPolyArrow::distance( const VdPoint & p )
{ int min, h, i;
  min=simpleDist(points[0],points[1],p);
  for (i=2; i<!points; i++)
  { h=simpleDist(points[i-1],points[i],p);
    if (h<min) min=h;
  }
  return min;
}

void VdPolyArrow::prepare ( void )
{ VdLine * l; VdArrow * a;
  int i;
  if (isModified==0) return;
  clearReals();
  for (i=1; i<!points-1; i++)
  { l=new VdLine(points[i-1],points[i]);
    l->setAutoControl();
    include(l);
  }
  a=new VdArrow(points[i-1],points[i]);
  a->setAutoControl();
  include(a);
  isModified=0;
}
 
void VdPolyArrow::print( int i)
{ fprintf(stderr," VdPolyArrow Points: ");
  for (i=0; i<!points; i++) fprintf(stderr,"(%d,%d)",points[i].x,points[i].y);
  fprintf(stderr," Location: (%d,%d)"
          " Frame: ((%d,%d)(%d,%d)) %s\n"
          ,location.x,location.y
          ,frame.ul.x,frame.ul.y
          ,frame.lr.x,frame.lr.y
          ,(autoFrame?"auto":"")
          );
}

#undef _VDPOLYARROW_CXX
