#define _VDPULLDOWNOPTIONS_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdPulldownOptions.cxx                             *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdPulldownOptions.hxx>
#include <Vd/VdRadioBoxInterface.hxx>
#include <Vd/VdCheckBoxInterface.hxx>
#include <X11/Intrinsic.h>

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleBG.h>
#include <Xm/CascadeB.h>

#ifdef _HXX_INCLUDE_
#include <iostream.hxx>
#else
#include <iostream.h>
#endif

VdPulldownOptions::VdPulldownOptions( 
		VdAbstractWindow & window, 
		String aLabelString,
		int onlyOne,
		long initialSelectedMask,
		VdButtonInterface * handle): 
	VdButton(window,aLabelString,handle), oneOfMany(onlyOne)

{
	selectedMask = initialSelectedMask;
	theToggleList = NULL;
}

VdPulldownOptions::~VdPulldownOptions( void )
{
	// nothing to do
	// *theToggleList is a self destroying object!
}

void VdPulldownOptions::map ( void )
{
	Arg al[10];
	Cardinal ac;

        if (!isCreated) {
                isCreated = True;

		// Erzeugen des MenuPanes
		ac=0;
		if (oneOfMany) {
		    XtSetArg(al[ac], XmNradioBehavior, True); ac++;
		    XtSetArg(al[ac], XmNradioAlwaysOne, True); ac++;
		} else {
		    XtSetArg(al[ac], XmNradioBehavior, False); ac++;
		}
		menuPane = XmCreatePulldownMenu(
			parentWidget,
			"VisamadPulldownOptions", al, ac);
		VdFeature temp = parentWindow->getIdent();
		theToggleList = new VdToggleList(menuPane,
			&temp, 
			theHandle,
			oneOfMany, selectedMask);
		if (theHandle) 
		    for (ac=0; ac<theToggleList->getNrToggles(); ac++)
			addCB( theToggleList->getWidget(ac), valueChanged ); 
		ac=0;
		XtSetArg(al[ac], XmNsubMenuId, menuPane); ac++;
                theButton = XmCreateCascadeButton(
                        parentWidget,
                        theLabelString, al, ac);
                XtManageChild( theButton );
                XtSetSensitive(theButton, isSensitive);
		addCB(theButton, cascaded);
		handleValueChanged();
        } else {
                XtMapWidget(theButton);
	}
}

void VdPulldownOptions::unmap ( void )
{
        VdButton::unmap();
}

void VdPulldownOptions::remove( void )
{
	XtDestroyWidget(menuPane);
	VdButton::remove();
}

void VdPulldownOptions::update( void )
{
	int i;
	if (!theHandle) return;
	// nur etwas zu tun, wenn
	// theToggleList->getMask() != theHandle->mask oder
	// theToggleList->optionLabels != theHandle->labels 
	if (theHandle->getType()==1) {
		selectedMask = ((VdRadioBoxInterface *)theHandle)->mask;
	} else if (theHandle->getType()==2) {
		selectedMask = ((VdCheckBoxInterface *)theHandle)->mask;
	}
	if (theToggleList) {
		if (theHandle) 
		    for (i=0; i<theToggleList->getNrToggles(); i++) {
			XtUnmanageChild( theToggleList->getWidget(i) );
			XtDestroyWidget( theToggleList->getWidget(i) );
		} 
		delete ((VdDestruction *)theToggleList);
	}
	if (isCreated) {
		VdFeature temp = parentWindow->getIdent();
		setLabel(theHandle->idf.charptr());
		theToggleList = new VdToggleList(menuPane,
			&temp, theHandle,
			oneOfMany, selectedMask);
		if (theHandle) 
		    for (i=0; i<theToggleList->getNrToggles(); i++)
			addCB( theToggleList->getWidget(i), valueChanged ); 
	}
	handleValueChanged();
}


int VdPulldownOptions::getSelected( void )
{
	if (theToggleList)
		return theToggleList->getSelected();
	else return 0;
}

void VdPulldownOptions::setSelected( int nr )
{
	if (theToggleList)
		theToggleList->setSelected( nr );
}

void VdPulldownOptions::addOption( String newOption )
{
	if (theToggleList)
		theToggleList->addOption( newOption );
	else if (theHandle) {
	    // if (theToggleList) this is executed by
	    // VdToggleList
	    VdStringList *lp;
	    if (theHandle->getType()==1) {
		lp = & (((VdRadioBoxInterface *)theHandle)->labels);
	    } else if (theHandle->getType()==2) {
		lp = & (((VdCheckBoxInterface *)theHandle)->labels);
	    }
	    lp->ins() = VdString(newOption);
	}
	if (theHandle) handleValueChanged();
	if (theToggleList)
	    	addCB( theToggleList->getWidget(
			   theToggleList->getNrToggles()-1), 
			valueChanged ); 
}

void VdPulldownOptions::removeOption( String oldOption )
{
	VdString oldOS(oldOption);
	if (theToggleList)
		theToggleList->removeOption( oldOption );
	else if (theHandle) {
	    // if (theToggleList) this is executed by
	    // VdToggleList
	    VdStringList *lp;
	    long *mp;
	    if (theHandle->getType()==1) {
		lp = & (((VdRadioBoxInterface *)theHandle)->labels);
		mp = & (((VdRadioBoxInterface *)theHandle)->mask);
	    } else if (theHandle->getType()==2) {
		lp = & (((VdCheckBoxInterface *)theHandle)->labels);
		mp = & (((VdCheckBoxInterface *)theHandle)->mask);
	    }
	    int i = lp->idx( oldOS );
	    if (i<0) return;
	    *mp = 
		(( (1<<i) - 1) & *mp)
	      + ( (*mp >> (i+1)) << i );
	    lp->rem(i); 
	}
	if (theHandle) {
		handleValueChanged();
		long *m=NULL;
	        if (theHandle->getType()==1) {
		   m = &((VdRadioBoxInterface *)theHandle)
			->mask;
	        } else if (theHandle->getType()==2) {
		   m = &((VdCheckBoxInterface *)theHandle)
			->mask;
	    	}
		if (m && !*m) setLabel(theLabelString);
	}
}	

void VdPulldownOptions::renameOption( 
	String oldOption, String newOption )
{
	VdString oldOS(oldOption);
	VdStringList *lp;
	long *mp;
	if (theHandle->getType()==1) {
		lp = & (((VdRadioBoxInterface *)theHandle)->labels);
		mp = & (((VdRadioBoxInterface *)theHandle)->mask);
	} else if (theHandle->getType()==2) {
		lp = & (((VdCheckBoxInterface *)theHandle)->labels);
		mp = & (((VdCheckBoxInterface *)theHandle)->mask);
	}
	int i = lp->idx( oldOS );
	if (i<0) return;
	(*lp)[i] = VdString(newOption);
	update();
}

void VdPulldownOptions::setSensitivity( int nr, int state )
{
	if (theToggleList)
		theToggleList->setSensitivity( nr, state );
} 

void VdPulldownOptions::handleCascaded( void )
{
	if (!theHandle) return;
	if (theHandle->getType()==1) {
		((VdRadioBoxInterface *)theHandle)
			->pullDown();
	} else if (theHandle->getType()==2) {
		((VdCheckBoxInterface *)theHandle)
			->pullDown();
	}	
}

void VdPulldownOptions::handleValueChanged(void )
{
	Arg al[10];
	Cardinal ac;
	char label[100];
	XmString string;

	// updates the menubar button label to display
	// the state of the radio box
	if (!theHandle) return;
	if (theHandle->getType()!=1) return;
	ac = getSelected();
	if (ac==0 || ac>theToggleList->getNrToggles()) return;
	
	strcpy(label, theLabelString);
	strcat(label, ":");
	strcat(label, 
		(((VdRadioBoxInterface *)theHandle)->
		  labels[ac-1]).charptr());
	string = XmStringCreate(label, 
		   XmSTRING_DEFAULT_CHARSET);
	ac = 0;
	XtSetArg(al[ac], XmNlabelString, string ); 
	ac++;
	XtUnmanageChild(XtParent(XtParent(theButton)));
	XtSetValues(theButton, al, ac);
	XtManageChild(XtParent(XtParent(theButton)));
	XmStringFree(string);
}

#undef _VDPULLDOWNOPTIONS_CXX

