#define _VDREFERENCE_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdReference.cxx                                   *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdReference.hxx>
#include <Vd/VdSublanguage.hxx>
#include <Vd/VdConsistence.hxx>

#ifdef _HXX_INCLUDE_
#include <stddef.hxx>
#else
#include <stddef.h>
#endif

static void VdReplaceReference( void* where
                                  , void* newPtr
                                  , void* env
                                  , int isNew
                                  )
{ if (isNew)
  { VdSublanguage * slgPtr = (VdSublanguage *)newPtr;
    VdReference * refPtr = (VdReference *)env;
    refPtr->VdSetReference(slgPtr);
  }
} 

VdReference::VdReference (void)
{ VdSublanguagePointer=NULL;
}

VdReference::VdReference( VdReference & x)
{ VdSublanguagePointer=x.VdSublanguagePointer;
  VdReferenceName=x.VdReferenceName;
  if (VdSublanguagePointer)
  { VdSublanguagePointer->VdAddReference(this);
    VdConsistence::registerReplacement(NULL
                                      ,(void*)VdSublanguagePointer
                                      ,(void*)this
                                      ,&VdReplaceReference);
  }
}

VdReference::~VdReference ( void )
{ VdRemReference();
}

void VdReference::VdSetReference( VdSublanguage * x)
{ if (VdSublanguagePointer) VdSublanguagePointer->VdRemReference(this);
  if (x)
  { VdSublanguagePointer=x; VdSublanguagePointer->VdAddReference(this);
  }
  else VdSublanguagePointer=NULL;
}

void VdReference::VdRemReference( void )
{ if (VdSublanguagePointer)
  { VdSublanguagePointer->VdRemReference(this);
    VdSublanguagePointer=NULL;
  }
}

VdLanguageInterface * VdReference::VdGetRealSlangPtr(void) const
{ return VdSublanguagePointer?
          VdSublanguagePointer
             ->VdRealSlangPtr
                  ->VdGetLanguageInterface(VdReferenceName)
         :NULL;
}

int VdReference::operator! (void)
{ return VdGetRealSlangPtr()?1:0;
}

void VdReference::operator = ( VdReference & r )
{ VdSetReference(r.VdSublanguagePointer);
}

void VdReference::operator = ( VdSublanguage & s )
{ VdSetReference(&s);
}

void VdReference::refer( VdReference & r )
{ VdSetReference(r.VdSublanguagePointer);
}

void VdReference::refer( VdReference * r )
{ VdSetReference(r->VdSublanguagePointer);
}

void VdReference::refer( VdSublanguage & s )
{ VdSetReference(&s);
}

void VdReference::refer( VdSublanguage * s )
{ VdSetReference(s);
}

void VdReference::refer( VdLanguageInterface & l )
{ VdSetReference(l.VdOriginPtr);
}

void VdReference::refer( VdLanguageInterface * l )
{ VdSetReference(l->VdOriginPtr);
}

void VdReference::derefer(void)
{ VdRemReference();
}

int VdReference::operator == ( const VdReference & r ) const
{ return VdSublanguagePointer == r.VdSublanguagePointer;
}

int VdReference::operator != ( const VdReference & r ) const
{ return VdSublanguagePointer != r.VdSublanguagePointer;
}

int VdReference::operator == ( const VdSublanguage & r ) const
{ return VdSublanguagePointer == &r;
}

int VdReference::operator != ( const VdSublanguage & r ) const
{ return VdSublanguagePointer != &r;
}

int VdReference::operator == ( VdLanguageInterface & r ) const
{ return VdGetRealSlangPtr() == &r;
}

int VdReference::operator !=  ( VdLanguageInterface & r ) const
{ return VdGetRealSlangPtr() != &r;
}


int VdReference::identically ( const VdReference & r ) const
{ return this == &r;
}


#undef _VDREFERENCEANDSUBLANGUAGE_CXX
