#define _VDSHELL_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdShell.cxx                                       *
*  Date: 06-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Implementation of class VdShell:                     *
*        A VdShell is a "MainWindowShell" , the parent for *
*        any VdAbstractWindow object and an                *
*        OSF/Motif decoration border for this object when  *
*        displaying it on the screen.                      *
*        The common behavior is to have                    *
*        - a top level widget as a parent for              *
*          VdAbstractWindow's main window widget           *
*        - the methods setGeometry, getGeometry,           *
*          getDisplay, getGC, getColormap,                 *
*        There are 3 principal variants to construct       *
*        a VdShell object:                                 *
*        - construct a 'main shell' with application       *
*          context, opening a display and initialize       *
*          some X11 internals (GC, colormap), parsing      *
*          command line parameters                         *
*        - construct a VdShell object via copy constructor *
*          to have a further top level shell (iconifyable, *
*          destroyable with ALT/F4), connected with the    *
*          main shell by the common application context    *
*          and display                                     *
*        - construct a transient shell iconifyable         *
*          only together with its top level shell          *
*        There can be only one VdShell object in an        *
*        application and all other shell objects get this  *
*        object as a parameter for the constructor.        *
*                                                          *
*     Interface                                            *
*        X11/Intrinsic.h, X11/Shell.h                      *
*                                                          *
*     References:                                          *
*        [XtRM] X Toolkit Intrinsics Reference Manual,     *
*        Second Edition for X11 Release 4                  *
*        O'Reilly & Associates, Inc.                       * 
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdShell.hxx>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/StringDefs.h>
#include <Xm/MwmUtil.h>

#include <Xm/MainW.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include <Xm/DialogS.h>
#include <Xm/RowColumn.h>
#include <Vd/VdAbstractWindow.hxx>
#include <Vd/VdHelpManager.hxx>

#include <strstream.h> 

extern "C" {
#include <stdlib.h>
}

#define icon_width 50
#define icon_height 50
static char icon_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xe0, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8,
   0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfc, 0x7f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x3e, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0xe0, 0x01, 0x00,
   0x00, 0x00, 0x00, 0x07, 0xc0, 0x01, 0x00, 0x00, 0x00, 0x80, 0x07, 0xc0,
   0x03, 0x00, 0x00, 0x00, 0x80, 0x03, 0x80, 0x03, 0x00, 0x00, 0x00, 0x80,
   0x03, 0x80, 0x03, 0x00, 0x00, 0x00, 0x80, 0x03, 0x80, 0x03, 0x00, 0x00,
   0x00, 0x80, 0x03, 0x80, 0x03, 0x00, 0x00, 0x00, 0x80, 0x03, 0x80, 0x03,
   0x00, 0x00, 0x00, 0x80, 0x07, 0xc0, 0x03, 0x00, 0x00, 0x00, 0x00, 0x07,
   0xc0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x0f, 0xe0, 0x01, 0x00, 0x00, 0x00,
   0x00, 0x3e, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x00, 0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x80, 0xe7, 0xcf,
   0x01, 0x00, 0x00, 0x00, 0x80, 0x03, 0x80, 0x03, 0x00, 0x00, 0x00, 0xc0,
   0x03, 0x80, 0x07, 0x00, 0x00, 0x00, 0xc0, 0x01, 0x00, 0x07, 0x00, 0x00,
   0x00, 0xe0, 0x01, 0x00, 0x0f, 0x00, 0x00, 0x00, 0xe0, 0x00, 0x00, 0x1e,
   0x00, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x70, 0x00,
   0x00, 0x3c, 0x00, 0x00, 0x00, 0x78, 0x00, 0x00, 0x38, 0x00, 0x00, 0x00,
   0x38, 0x00, 0x00, 0x70, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x00, 0xf0, 0x00,
   0x00, 0xfe, 0xff, 0x7f, 0xfc, 0xff, 0x7f, 0x00, 0x06, 0x00, 0x60, 0xac,
   0xaa, 0x6a, 0x00, 0xfe, 0xff, 0x7f, 0xac, 0xaa, 0x6a, 0x00, 0x06, 0x00,
   0x60, 0xac, 0xaa, 0x6a, 0x00, 0xfe, 0xff, 0x7f, 0xac, 0xaa, 0x6a, 0x00,
   0x06, 0x00, 0x60, 0xac, 0xaa, 0x6a, 0x00, 0xfe, 0xff, 0x7f, 0xfc, 0xff,
   0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x04, 0xc5, 0x41, 0x08, 0x42, 0x78, 0x00, 0x04,
   0x25, 0x42, 0x18, 0x43, 0x88, 0x00, 0x88, 0x24, 0xa0, 0x18, 0xa3, 0x08,
   0x01, 0x88, 0xc4, 0xa1, 0xa8, 0xa2, 0x08, 0x01, 0x88, 0x04, 0x12, 0xa9,
   0x12, 0x09, 0x01, 0x50, 0x24, 0xf2, 0x49, 0xf2, 0x09, 0x01, 0x50, 0x24,
   0x0a, 0x4a, 0x0a, 0x8a, 0x00, 0x20, 0xc4, 0x09, 0x4a, 0x0a, 0x7a, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00};
static Pixmap icon_pixmap;

//******************************************** global X variables

XtAppContext VdShell::theAppContext;
Widget 	VdShell::theApplicationShell;
Widget 	VdShell::theDialogShell; 
String 	VdShell::theDisplayString;
String	VdShell::theClassName;
Display * VdShell::theDisplay;
GC 	VdShell::theGC;
Colormap VdShell::theColormap;
VdShell * VdShell::theMainShellPtr = NULL;
VdHelpManager * VdShell::theHelpSystem = NULL;

//********************************************* private methods

void VdShell::altF4CB(Widget /*widget*/, 
		XtPointer clientData, XtPointer /*callData*/ )
{
	VdShell *shellPtr 
		= (VdShell *)clientData;
	int toClose = 1;
	if (shellPtr->theWindowPtr)
		toClose = shellPtr->theWindowPtr->checkClose();
	if (toClose) {
	    shellPtr->handleDestroyed();
	    switch (shellPtr->theShellType) {
	    case VdShellMain: 
			exit(0);
		    	break;
	    case VdShellTopLevel: 
	    case VdShellTransient: 
			XtDestroyWidget(shellPtr->getWidget());
			break;
	    }
	}
}

void VdShell::setWindow(VdAbstractWindow *aw)
{
	theWindowPtr = aw;
}

void VdShell::init(VdShellType type, Widget parentShellWidget)
{
	Arg al[15];
	Cardinal ac;
	theShellType = type;

	ac=0;
	XtSetArg(al[ac], XmNmappedWhenManaged, False);
	ac++;
	XtSetArg(al[ac], XmNdeleteResponse, XmDO_NOTHING);
	ac++;

	if ( (type != VdShellTransient ) && icon_pixmap) {
	   	XtSetArg(al[ac], XmNiconPixmap, icon_pixmap); ac++;
	}

	XtSetArg(al[ac], XmNminHeight,150);
	ac++;
	XtSetArg(al[ac], XmNminWidth,200);
	ac++;
	XtSetArg(al[ac], XmNheight,400);
	ac++;
	XtSetArg(al[ac], XmNwidth,500);
	ac++;
	XtSetArg(al[ac], XmNx,100);
	ac++;
	XtSetArg(al[ac], XmNy,100);
	ac++;

	switch (type) {
	case VdShellMain:
		theShellWidget = XtAppCreateShell ( 
			theProgramName, theClassName,
			topLevelShellWidgetClass, 
			theDisplay, al, ac);
		XtRealizeWidget(theShellWidget);  
		break;
	case VdShellTopLevel:
		theShellWidget = XtAppCreateShell ( 
			theProgramName, theProgramName,
			topLevelShellWidgetClass, 
			theDisplay, al, ac);
		XtRealizeWidget(theShellWidget);  
		break;
	case VdShellTransient:
		XtSetArg(al[ac], XmNwindowGroup, 
			XtWindow(parentShellWidget));
		ac++;
		XtSetArg(al[ac], XmNtransient, True);
		ac++;
		theShellWidget = XtAppCreateShell ( 
			theProgramName, theProgramName,
			topLevelShellWidgetClass, 
			theDisplay, al, ac);
		XtRealizeWidget(theShellWidget);
/* Here an open problem exists:
	If for some Shell more than one transient shells are
	created, only the last (!) of these shells are iconified
	together with the common ancestor shell.
	In earlier versions in this case a XmDialogShell or a
	TransientShell was generated but there were aborts closing
	these transient shells via <ALT/F4> or the system close button.
	The "iconify problem" was the same.
*/
		break;
	}

	XmAddWMProtocolCallback(theShellWidget,
		XmInternAtom(theDisplay,
			"WM_DELETE_WINDOW",True),
		(XtCallbackProc)&altF4CB, 
		(caddr_t)this );
}

//**************************************************  public methods

VdShell::VdShell(int *argc, char **argv, 
			char *displayString,
			char *applicationName,
			char *applicationClass)
{
	Arg al[15];
	Cardinal ac;
	XIconSize *size_list; int count;

	if (theApplicationShell) {
		char message[100];
		ostrstream error(message,100);
		error << theProgramName 
			<< " - can not open a second main shell " 
			<< "\n";
		XtWarning (message);
		exit(1);
	} else
		theMainShellPtr = this;

	XtToolkitInitialize ();
	theAppContext = XtCreateApplicationContext();
	if (displayString) {
	   theDisplayString = new char[strlen(displayString)+1];
	   strcpy(theDisplayString, displayString);
	} else {
	   theDisplayString = displayString;
	}
	if (applicationName) {
	   theProgramName = new char[strlen(applicationName)+1];
	   strcpy(theProgramName, applicationName);
	} else {
	   theProgramName = new char[1];
	   *theProgramName = 0;
	}
	if (applicationClass) {
	   theClassName = new char[strlen(applicationClass)+1];
	   strcpy(theClassName, applicationClass);
	} else {
	    theClassName= new char[1];
	   *theClassName = 0;
	}

	theDisplay = XtOpenDisplay (      // [XtRM, p.266]
			theAppContext, theDisplayString, 
			theProgramName, theClassName, 
			(XrmOptionDescRec *)NULL, 0,
#if XtSpecificationRelease==4
			(Cardinal *) argc, 
#else
			argc,
#endif
			argv);

	if (!theDisplay) {
		char message[100];
		ostrstream error(message,100);
		error << theProgramName 
			<< " - can not open display " 
			<< "\n";
		XtWarning (message);
		exit(1);
	}

	icon_pixmap = XCreateBitmapFromData(
		theDisplay,
		RootWindow(theDisplay,0),
		icon_bits, icon_width, icon_height);
	XGetIconSizes(theDisplay,
		RootWindow(theDisplay,0),
		&size_list,&count);
	if (icon_width<=size_list[0].max_width 
		&& icon_height<=size_list[0].max_height)
	    icon_pixmap = XCreateBitmapFromData(theDisplay,
		RootWindow(theDisplay,0),
		icon_bits, icon_width, icon_height);
	XFree((char *)size_list);

	isTheMainShell = True;
	theDisplayString = XDisplayString(theDisplay);
	theGC = XDefaultGC(theDisplay, 0);
	theColormap = XDefaultColormap(theDisplay, 0);
	theWindowPtr = NULL;

	// creating a (not visible) appliaction shell and a
	// dialog shell for all widgets
	ac=0;
	XtSetArg(al[ac], XmNmappedWhenManaged, False);
	ac++;
	XtSetArg(al[ac], XmNheight,100);
	ac++;
	XtSetArg(al[ac], XmNwidth,100);
	ac++;
	theApplicationShell = XtAppCreateShell (  // [XtRM, p.90]
			theProgramName, theClassName,
			applicationShellWidgetClass, 
			theDisplay, al, ac);
	XtRealizeWidget(theApplicationShell);    // [XtRM, p.287]
	ac=0;
	XtSetArg(al[ac], XmNdeleteResponse, XmDO_NOTHING);
	ac++;
	XtSetArg(al[ac], XmNallowShellResize, True);
	ac++;
	XtSetArg(al[ac], XmNx,100);
	ac++;
	XtSetArg(al[ac], XmNy,100);
	ac++;
        XtSetArg(al[ac], XmNmwmDecorations,
                MWM_DECOR_TITLE | MWM_DECOR_BORDER); 
	ac++;
	theDialogShell = XmCreateDialogShell(
			theApplicationShell,
			"VisamadDialog",
			al,ac);
	init(VdShellMain);
	theHelpSystem = new VdHelpManager();
}


VdShell::VdShell( VdShell& shell, String title )
{
	isTheMainShell = False;
	theWindowPtr = NULL;
	theGC = shell.theGC;
	theProgramName = new char[strlen(title)+1];
	strcpy(theProgramName, title);

	init(VdShellTopLevel);
}


VdShell::VdShell(String title, VdShell& shell)
{	
	isTheMainShell = False;
	theWindowPtr = NULL;
	theProgramName = new char[strlen(title)+1];
	strcpy(theProgramName, title);

	init(VdShellTransient,shell.getWidget());	
}


VdShell::~VdShell( void )
{
	theWindowPtr = NULL;
	XtDestroyWidget(theShellWidget);
	if (isTheMainShell)
		XtDestroyWidget(theApplicationShell);
}


void VdShell::mainLoop( void )
{
	if (!isTheMainShell) {
		XtWarning("MainLoop must be started by main shell");
		return;
	} else {
		XtMapWidget(theShellWidget);
		XtAppMainLoop(theAppContext);  // [XtRM, p.102]
	}
}

void VdShell::map( void )
{
	XtMapWidget(theShellWidget);
}

void VdShell::unmap( void )
{
	XtUnmapWidget(theShellWidget);
}

Widget VdShell::getWidget( void) 
{ 
	return theShellWidget; 
}

void VdShell::setGeometry(unsigned int width, 
			unsigned int height, 
			unsigned int x, unsigned int y)
{
	Arg al[10];
	Cardinal ac;
	Position xx,yy; Dimension w,h;

	w = width; h = height; xx = x; yy = y;
	ac=0;
	XtSetArg(al[ac], XtNwidth, w); ac++;
	XtSetArg(al[ac], XtNheight, h); ac++;
	XtSetArg(al[ac], XtNx, xx); ac++;
	XtSetArg(al[ac], XtNy, yy); ac++;

	XtSetValues(theShellWidget, al, ac);
}

void VdShell::getGeometry(unsigned int *width, 
			unsigned int *height, 
			unsigned int *x, unsigned int* y)
{
	Arg al[10];
	Cardinal ac;
	Position xx,yy; Dimension w,h;

	ac=0;
	XtSetArg(al[ac], XtNwidth, &w); ac++;
	XtSetArg(al[ac], XtNheight, &h); ac++;
	XtSetArg(al[ac], XtNx, &xx); ac++;
	XtSetArg(al[ac], XtNy, &yy); ac++;

	XtGetValues(theShellWidget, al, ac);
	*width = w; *height = h; *x = xx; *y = yy;
}

void VdShell::setTitle( char * title)
{
	Arg al[4];
	Cardinal ac;
        char * ptr;
        ptr=title;
        while (*ptr) { if (*ptr++ == ':') break; }
        if (ptr); else ptr=title;
	ac=0;
	XtSetArg(al[ac], XmNtitle, title); ac++;
	if (theShellType != VdShellTransient ) {
	   	XtSetArg(al[ac], XmNiconName, ptr); ac++;
	}
	XtSetValues(theShellWidget, al, ac);	
}

char *VdShell::getTitle( void )
{
	Arg al[2];
	Cardinal ac;
	char *title;

	ac=0;
	XtSetArg(al[ac], XmNtitle, &title); ac++;
	XtGetValues(theShellWidget, al, ac);	

	return title;
}

Boolean VdShell::isMainShell( void ) 
{
	return isTheMainShell; 
}

//**********************************************************

Display *VdShell::getDisplay( void ) 
{ 
	return theDisplay; 
}

GC VdShell::getGC( void ) 
{
	return theGC; 
}

Colormap VdShell::getColormap( void ) 
{ 
	return theColormap; 
}

//**********************************************************

void VdShell::handleDestroyed( void )
{
	if (theWindowPtr) {
		theWindowPtr->handleDestroyed();
		theWindowPtr = NULL;
	}
}

void VdShell::help( const VdFeature & helpIdent )
{
	theHelpSystem->handleHelp(helpIdent);
}

#undef _VDSHELL_CXX
