#ifndef _VDSHELL_HXX
#define _VDSHELL_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdShell.hxx                                       *
*  Date: 06-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Implementation of class VdShell:                     *
*        A VdShell is a "MainWindowShell" , the parent for *
*        any VdAbstractWindow object and an                *
*        OSF/Motif decoration border for this object when  *
*        displaying it on the screen.                      *
*        The common behavior is to have                    *
*        - a top level widget as a parent for              *
*          VdAbstractWindow's main window widget           *
*        - the methods setGeometry, getGeometry,           *
*          getDisplay, getGC, getColormap,                 *
*        There are 3 principal variants to construct       *
*        a VdShell object:                                 *
*        - construct a 'main shell' with application       *
*          context, opening a display and initialize       *
*          some X11 internals (GC, colormap), parsing      *
*          command line parameters                         *
*        - construct a VdShell object via copy constructor *
*          to have a further top level shell (iconifyable, *
*          destroyable with ALT/F4), connected with the    *
*          main shell by the common application context    *
*          and display                                     *
*        - construct a transient shell iconifyable         *
*          only together with its top level shell          *
*        There can be only one VdShell object in an        *
*        application and all other shell objects get this  *
*        object as a parameter for the constructor.        *
*                                                          *
*     Interface                                            *
*        X11/Intrinsic.h, X11/Shell.h                      *
*                                                          *
*     References:                                          *
*        [XtRM] X Toolkit Intrinsics Reference Manual,     *
*        Second Edition for X11 Release 4                  *
*        O'Reilly & Associates, Inc.                       * 
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdShell;

/******************    dependences    *********************/

#include <X11/Intrinsic.h>
#include <X11/Shell.h>

class VdAbstractWindow;
class VdHelpManager;
#include <Vd/VdFeature.hxx>

typedef enum { 
	   VdShellMain=1, VdShellTopLevel, VdShellTransient }
	VdShellType;

class VdShell {
	friend class VdAbstractWindow;
   public: 
	// global values for window programming
	static XtAppContext theAppContext;
	static Widget theApplicationShell;
		// a non mapped widget
	static Widget theDialogShell; 
		// for Formulars, Decisions and other dialogs
	static VdShell * theMainShellPtr;
	static String 
		theDisplayString,
		theClassName;
	String	theProgramName;
	static Display *theDisplay;
	static GC theGC;
	static Colormap theColormap;

   private:
	Widget theShellWidget;

	Boolean isTheMainShell;
	VdShellType theShellType;
	VdAbstractWindow *theWindowPtr;
	static VdHelpManager * theHelpSystem;
	
	static void altF4CB(Widget widget, 
		XtPointer clientData, XtPointer callData );
	void setWindow(VdAbstractWindow *aw);
	void init(VdShellType type, Widget parentShellWidget=0);

   public:
	VdShell( int *argc = 0, 	// application shell
		char **argv = NULL, 
		char *displayString = NULL,
		char *applicationName 
			= "VisamadApplication", 
		char *applicationClass 
			= "VISAMAD");
	VdShell( VdShell& shell, 	// top level shell
		String title 
			= "VisamadWindow");
	VdShell( char *title, 		// transient shell
		VdShell& shell);
	~VdShell( void );

	void mainLoop( void );
	void map( void );
	void unmap( void );
	Widget getWidget( void );
	void setGeometry(unsigned int width, 
		unsigned int height, 
		unsigned int x, unsigned int y);
	void getGeometry(unsigned int* width, 
		unsigned int* height, 
		unsigned int* x, unsigned int* y);
	void setTitle( char * );
	char *getTitle( void );

	Boolean isMainShell( void );

	Display *getDisplay( void );  	// rudimentary 
	GC getGC( void );  		// rudimentary 
	Colormap getColormap( void );  	// rudimentary 

	virtual void handleDestroyed( void );
	static void help(const VdFeature & helpIdent);
};

#endif   /* _VDSHELL_HXX */
