/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdSimpleWindowManager.cxx                         *
*  Date: 03-12-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author:  J. Hutschenreiter                              *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*                                                          *
************************************************************
*                                                          *
*     Interface                                            *
*         VdSimpleWindowManager.hxx                        *
*                                                          *
***********************************************************/

#include <Vd/VdSimpleWindowManager.hxx>

#include <Vd/VdWindowConnection.hxx>
#include <Vd/VdMagnifierWindow.hxx>
#include <Vd/VdDecoratedMagnifierWindow.hxx>
#include <Vd/VdEditWindow.hxx>
#include <Vd/VdDecoratedEditWindow.hxx>
#include <Vd/VdGlobalFunctions.hxx>


extern "C" {
#include <stdlib.h>
#include <stdarg.h>
}

VdSimpleWindowManager::VdSimpleWindowManager( 
				VdShell & applicationShell )
{
	theIdent = VdFeature(applicationShell.theProgramName);
	theShPtr = & applicationShell;
	theMWPtr = new VdDecoratedMagnifierWindow(applicationShell,0);
	theWCPtr = new VdWindowConnection(theIdent,
		(VdWindowManagerInterface *)this);
	theWCPtr->setMagnifierWindow(*theMWPtr);
	theShPtr->map();
} 


VdSimpleWindowManager::VdSimpleWindowManager(
		int *argc, 	
		char **argv, 
		char *displayString,
		char *applicationName, 
		char *applicationClass)
{
	theIdent = VdFeature(applicationName);
	theShPtr = new VdShell(argc, argv, displayString,
				applicationName, applicationClass);
	theMWPtr = new VdDecoratedMagnifierWindow(*theShPtr,0);
	theWCPtr = new VdWindowConnection(theIdent,
		(VdWindowManagerInterface *)this);
	theWCPtr->setMagnifierWindow(*theMWPtr);
	theShPtr->map();
}

void VdSimpleWindowManager::run ( void )
{
	theShPtr->mainLoop();
}

VdFeature VdSimpleWindowManager::createWindow
               ( const VdFeature & aWindow)
{
// theWindowType ist vorlufig ein zweistufiger Bezeichner
// aus Connection Ident und Fenster Ident (also genau das,
// was durch WindowConnection erzeugt wird
	VdShell *shPtr = new VdShell(
		*theShPtr,
		((aWindow.getLast(1)).getString()).charptr());
	VdEditWindow * ewPtr = new VdDecoratedEditWindow(*shPtr);
	return theWCPtr->addEditWindow( *ewPtr );
}

void VdSimpleWindowManager::destroyWindows
               ( const VdFeature & someWindows )
{
	theWCPtr->removeEditWindow(someWindows);
}

void VdSimpleWindowManager::renameWindow
		( const VdFeature & oldWindow,
		  const VdFeature & newWindow)
{
	theWCPtr->renameWindow(oldWindow,newWindow);
}

void VdSimpleWindowManager::mapWindow	
               ( const VdFeature & someWindows )
{
	theWCPtr->map( someWindows );
}

void VdSimpleWindowManager::unmapWindow
               ( const VdFeature & someWindows )
{
	theWCPtr->unmap( someWindows );
}

void VdSimpleWindowManager::setPicture 
               ( const VdFeature & someWindows
               , VdPicture * aPicture )
{
	theWCPtr->setPicture(someWindows, aPicture);
}

VdPicture * VdSimpleWindowManager::getPicture
               ( VdFeature & aWindow )
{
	return theWCPtr->getPicture(aWindow);
}

void VdSimpleWindowManager::movePicture( 
		const VdFeature & aWindow, 
		VdPicture * toDraw, VdPicture * toMove,
		VdPoint * worldLocation)
{
	theWCPtr->movePicture( aWindow, toDraw, 
			toMove, worldLocation);
}

VdFeature VdSimpleWindowManager::getWindowsOfPicture
               ( VdPicture * aPicture )
{
	return theWCPtr->
		     getWindowsWithPicture(aPicture);
}

void VdSimpleWindowManager::setWorldArea 	
	       ( const VdFeature & someWindows,
		 const VdArea & worldArea )
{
	theWCPtr->setWorldArea( someWindows, worldArea );
}

VdArea VdSimpleWindowManager::getWorldArea
	       ( const VdFeature & window )
{
	return theWCPtr->getWorldArea( window );
}

void VdSimpleWindowManager::setDisplayArea 
	      	( const VdFeature & someWindows,
		  const VdArea & displayedArea )
{
	theWCPtr->setDisplayArea( someWindows, displayedArea );
}

VdArea VdSimpleWindowManager::getDisplayArea
	       ( const VdFeature & aWindow )
{
	return theWCPtr->getDisplayArea( aWindow );
}

void VdSimpleWindowManager::setView(
		const VdFeature & someWindows,
		VdFractNumber editWindowUnitsByPixel,
		VdPoint editWindowCentre,
		VdFractNumber magnifierSizeInWindow,
		VdFractNumber magnifierXPosition,
		VdFractNumber magnifierYPosition)
{
	theWCPtr->setView(
		someWindows,
		editWindowUnitsByPixel,
		editWindowCentre,
		magnifierSizeInWindow,
		magnifierXPosition,
		magnifierYPosition);
}

void VdSimpleWindowManager::getView(
		const VdFeature & aWindow,
		VdFractNumber & editWindowUnitsByPixel,
		VdPoint & editWindowCentre,
		VdFractNumber & magnifierSizeInWindow,
		VdFractNumber & magnifierXPosition,
		VdFractNumber & magnifierYPosition)
 {
	theWCPtr->getView(
		aWindow,
		editWindowUnitsByPixel,
		editWindowCentre,
		magnifierSizeInWindow,
		magnifierXPosition,
		magnifierYPosition);
}

void VdSimpleWindowManager::drawPicture
               ( VdPicture * aPicture )
{
	theWCPtr->redrawPicture(aPicture,0);
}

void VdSimpleWindowManager::redrawPicture
               ( VdPicture * aPicture )
{
	theWCPtr->redrawPicture(aPicture,1);
}

void VdSimpleWindowManager::setEventManager	
               ( const VdFeature & someWindows
               , VdEventManagerInterface * aManager)
{
	theWCPtr->setEventManager(
			someWindows, aManager );
}

void VdSimpleWindowManager::removeEventManager
               ( const VdFeature & someWindows )
{
	theWCPtr->removeEventManager(someWindows);
}

void VdSimpleWindowManager::addButtons 
               ( const VdFeature & someWindows 
               , VdButtonInterfacePtrList * buttons )
{
	theWCPtr->addButtons( someWindows, buttons );
}

void VdSimpleWindowManager::removeButtons
               ( const VdFeature & someWindows
               , VdButtonInterfacePtrList * buttons )
{
	theWCPtr->removeButtons( someWindows, buttons );
}

void VdSimpleWindowManager::getImage
               ( const VdFeature & aWindow
               , VdPixMap & aPixmap )
{
	theWCPtr->getImage( aWindow, aPixmap );
}

void VdSimpleWindowManager::createAndHandleMenu(
		VdPopupMenuInterface * aPopupMenuInterface)
{
	theWCPtr->createAndHandleMenu(aPopupMenuInterface);
}

void VdSimpleWindowManager::setExitChecker(
		int (* checkingFunction)
		   (const VdFeature & window) )
{
	theWCPtr->setExitChecker( checkingFunction );
}

void VdSimpleWindowManager::parseHelpFile(
		const VdString & fileName)
{
	VdHelp::parseHelpFile(fileName);
}

void VdSimpleWindowManager::createHelpWindow(
		const VdFeature & helpIdent)
{
	(new VdHelpManager(1))->handleHelp(helpIdent);
}

int VdSimpleWindowManager::VdCreateAndHandleFormular(char * name, char * group, ... )
{ va_list ap;
  va_start(ap,group);
  VdFormularInterface frm;
  frm.formularName=VdString(name);
  VdFeature grp(group);
  int i; char * ptr;
  while(1)
  { ptr=va_arg(ap,char*);
    if (ptr==NULL) break;
    VdString mem(ptr);
    for (i=0; i<!VdEntries; i++)
      if (  ( mem == VdEntries[i]->name )
         && ( grp == VdGroup[i] )
         )
      { frm.entries.ins()=VdEntries[i];
        break;
      }
  }
  va_end(ap);
  if (!frm.entries)
  { return ::VdCreateAndHandleFormular(&frm);
  }
  return 0;
}

void VdSimpleWindowManager::VdRegisterFormular(char * grp, VdFormularEntry * e )
{ VdEntries.ins(0)=e;
  VdGroup.ins(0)=VdFeature(grp);
}

