#ifndef _VDSIMPLEWINDOWMANAGER_HXX
#define _VDSIMPLEWINDOWMANAGER_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdSimpleWindowManager.hxx                         *
*  Date: 03-12-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter                               *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Declaration of class VdSimpleWindowManager           *
*                                                          *
*     Interface                                            *
*                                                          *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdSimpleWindowManager;

/******************    dependences    *********************/

#include <Vd/VdWindowManagerInterface.hxx>
#include <Vd/VdShell.hxx>
#include <Vd/VdFormularInterface.hxx>
#include <Vd/VdFormularEntry.hxx>
#include <Vd/VdFormularEntryPtrList.hxx>
#include <Vd/VdFeatureList.hxx>

class VdWindowConnection;
class VdMagnifierWindow;
class VdHelpManager ;


class VdSimpleWindowManager: public VdWindowManagerInterface {
  VdFeature theIdent;
  VdShell * theShPtr;
  VdWindowConnection *theWCPtr;
  VdMagnifierWindow *theMWPtr;

  VdFormularEntryPtrList VdEntries;
  VdFeatureList VdGroup;

 public:
  VdSimpleWindowManager( VdShell & applicationShell );
  VdSimpleWindowManager(
		int *argc = 0, 	
		char **argv = NULL, 
		char *displayString = NULL,
		char *applicationName 
			= "VisamadApplication", 
		char *applicationClass 
			= "VISAMAD");
  void run ( void );
  virtual VdFeature createWindow
               ( const VdFeature & aWindow);
  virtual void destroyWindows
               ( const VdFeature & someWindows );
  virtual void renameWindow
		( const VdFeature & oldWindow,
		  const VdFeature & newWindow );

  virtual void mapWindow			// visibilty
               ( const VdFeature & someWindows );
  virtual void unmapWindow
               ( const VdFeature & someWindows );

  virtual void setPicture 	       // pictures & windows
               ( const VdFeature & someWindows
               , VdPicture * aPicture );
  virtual VdPicture * getPicture
               ( VdFeature & aWindow );
  virtual void movePicture( const VdFeature & aWindow, 
			VdPicture * toDraw, VdPicture * toMove,
			VdPoint * worldLocation);
  virtual VdFeature getWindowsOfPicture
               ( VdPicture * thePicture );
  virtual void setWorldArea 		 // displaying world
	       ( const VdFeature & someWindows,
		 const VdArea & worldArea );
  virtual VdArea getWorldArea( const VdFeature & aWindow );
  virtual void setDisplayArea 
	      ( const VdFeature & someWindowsindows,
		const VdArea & displayedArea );
  virtual VdArea getDisplayArea( const VdFeature & aWindow );
  void setView(const VdFeature & someWindows,
		VdFractNumber editWindowUnitsByPixel,
		VdPoint editWindowCentre,
		VdFractNumber magnifierSizeInWindow,
		VdFractNumber magnifierXPosition,
		VdFractNumber magnifierYPosition);
  void getView(const VdFeature & aWindow,
		VdFractNumber & editWindowUnitsByPixel,
		VdPoint & editWindowCentre,
		VdFractNumber & magnifierSizeInWindow,
		VdFractNumber & magnifierXPosition,
		VdFractNumber & magnifierYPosition);
 
  virtual void drawPicture		   	// draw only
               ( VdPicture * aPicture );
  virtual void redrawPicture		   // clear and draw
               ( VdPicture * aPicture );

  virtual void setEventManager		   // event managing
               ( const VdFeature & someWindows
               , VdEventManagerInterface * aManager );
  virtual void removeEventManager
               ( const VdFeature & someWindows );

  virtual void addButtons 		  // button managing
               ( const VdFeature & someWindows 
               , VdButtonInterfacePtrList * buttons );
  virtual void removeButtons
               ( const VdFeature & someWindows
               , VdButtonInterfacePtrList * buttons );
  virtual void getImage
               ( const VdFeature & aWindow
               , VdPixMap & aPixmap );

  virtual void createAndHandleMenu(
		VdPopupMenuInterface * aPopupMenuInterface);

  virtual void setExitChecker(
		int (* checkingFunction)
		   (const VdFeature & window) );

  virtual void parseHelpFile(const VdString & fileName);
  virtual void createHelpWindow(const VdFeature & helpIdent);

  int VdCreateAndHandleFormular(char * name, char * grp, ... );
  void VdRegisterFormular(char * grp, VdFormularEntry * e );

};

#endif /* _VDSIMPLEWINDOWMANAGER_HXX */
