#define _VDSTRING_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdString.cxx                                      *
*  Date: 29-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Benary, J. Hutschenreiter                    *
*  State of Realization: experimental version              *
************************************************************
*                                                          *
*     Implementation of the class VdString of strings with *
*     variable length                                      *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdString.hxx>
 
char VdString::dummy; 

const BUFFERLENGTH=1024;
static char buffer[BUFFERLENGTH];

char * VdString::strdup(char const * s)
{ 
   if (s)
      return strcpy(new char[strlen(s)+1],s);
   else
      return NULL;
}

VdString::VdString( char const * s )
{ 
   ptr = strdup(s); 
}

VdString::VdString( const VdString & s)
{ 
   ptr=strdup(s.ptr);
} 

VdString::VdString( int i )
{
  sprintf(buffer,"%d",i); ptr=strdup(buffer); 
}

VdString::VdString( const VdPoint & p )
{ 
  sprintf(buffer,"P(%d,%d)",p.x,p.y); ptr=strdup(buffer); 
}

VdString::VdString( const VdVector & v )
{ 
  sprintf(buffer,"V(%d,%d)",v.x,v.y); ptr=strdup(buffer); 
}

VdString::VdString( const VdArea & a )
{ 
  sprintf(buffer,"A((%d,%d),(%d,%d))",a.ul.x,a.ul.y,a.lr.x,a.lr.y);
  ptr=strdup(buffer); 
}

VdString & VdString::operator= ( const VdString & s ) 
{ 
  if (this == &s) return *this; 
                      // correctness of s=s, JH, 28.10.92
  if (ptr) delete ptr; 
  ptr = strdup(s.ptr); 
  return  *this; 
} 
 

 
VdString::~VdString ( void ) 
{
  if (ptr) { delete ptr; ptr=NULL; /*JH*/} 
} 
 

 
VdString VdString::operator+ ( const VdString & s ) const
{
  VdString h; 
  h.ptr = new char[! *this+ ! s +1]; 
  if (ptr) strcpy(h.ptr,ptr); else *(h.ptr) = '\0'; 
  if (s.ptr) strcat(h.ptr, s.ptr); 
  return h; 
} 
 
VdString & VdString::operator+= ( const VdString & s ) 
{ 
  return *this = *this + s; 
} 


 
int VdString::operator! ( void ) const
{ return ptr ? (int)strlen(ptr)  : 0; 
} 
 
int VdString::length ( void ) const
{ return ptr ? (int)strlen(ptr)  : 0; 
} 


 
VdString VdString::operator << (unsigned int i) const
{ return (*this)(i, ! *this); 
} 


 
VdString VdString::operator() ( int a, int e ) const
{ VdString h; 
  int lenght = !*this-1; 
  if (a<0) a=0; 
  if (e>lenght) e = lenght; 
  if (e<a) return h; 
  h.ptr = new char[e-a+2]; 
  strncpy(h.ptr,ptr+a,e-a+1); 
  h.ptr[e-a+1] = '\0'; 
  return h; 
} 

/* If the string is empty, the reference of VdString::dummy *
*  will be the result                                      */
 
char & VdString::operator[] ( int i ) const
{ if ((i<0)||(i>=!*this)) return dummy;
  return ptr[i]; 
} 


 
int VdString::operator <  (const VdString & s) const 
{  return (strcmp(ptr?ptr:"", s.ptr?s.ptr:"") < 0) ; 
} 
 
int VdString::operator <= (const VdString & s) const 
{ return ( strcmp(ptr?ptr:"", s.ptr?s.ptr:"") <= 0 ); 
} 
 
int VdString::operator >  (const VdString & s) const 
{  return (strcmp(ptr?ptr:"", s.ptr?s.ptr:"") > 0); 
} 
 
int VdString::operator >=  (const VdString & s) const 
{ return (strcmp(ptr?ptr:"", s.ptr?s.ptr:"") >= 0); 
} 
 
int VdString::operator == (const VdString & s) const 
{  return ( strcmp(ptr?ptr:"", s.ptr?s.ptr:"")  == 0 ) ; 
} 

int VdString::operator != (const VdString & s) const 
{ return ( strcmp(ptr?ptr:"", s.ptr?s.ptr:"")  != 0 ) ; 
} 

#undef _VDSTRING_CXX
