#ifndef _VDSTRING_HXX
#define _VDSTRING_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdString.hxx                                      *
*  Date: 29-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Benary, J. Hutschenreiter                    *
*  State of Realization: experimental version              *
************************************************************
*                                                          *
*     Definition of the class VdString of strings with     *
*     variable length                                      *
*                                                          *
*     Interface                                            *
*          string.hxx                                      *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdString;

/******************    dependences    *********************/

#ifdef _HXX_INCLUDE_
#include <string.hxx> 
#include <stdio.hxx>
#else 
#include <string.h> 
#include <stdio.h>
#endif
#include <Vd/VdPoint.hxx>
#include <Vd/VdVector.hxx>
#include <Vd/VdArea.hxx>
 
class VdString{ 
      char * ptr;  
      static char dummy;  // dummy reference for operator[]
      char * strdup(char const * s);

   public: 

// constructors  
 
     VdString ( void ) { ptr=NULL; } 
     VdString ( char const * s ); 
     VdString ( const VdString & s ); 
     VdString( int i );
     VdString( const VdPoint & p );
     VdString( const VdVector & v );
     VdString( const VdArea & a );
     VdString & operator= ( const VdString & s ); 
 
// destructor 
 
     ~VdString ( void );
 
// concatenating 
 
     VdString operator+ ( const VdString & s ) const;   
                                              // s1 = s2 + s 
     VdString & operator+= ( const VdString & s ); 
                                              // s1 = s1 + s 
 
// length 
 
     int operator! ( void ) const;
     int length ( void ) const;
 
// left shift 
 
     VdString operator << ( unsigned int i ) const; 
 
// substring 
 
     VdString operator() ( int a , int e ) const; 
 
// reference to the i-th character 
  
     char & operator[] ( int i ) const;  

// type conversion 
 
     char * charptr ( void ) const { return ptr; } 
  
// string comparing *
 
     int operator<  ( const VdString & s ) const; 
     int operator<= ( const VdString & s ) const; 
     int operator>  ( const VdString & s ) const; 
     int operator>= ( const VdString & s ) const; 
     int operator== ( const VdString & s ) const; 
     int operator!= ( const VdString & s ) const; 
  
};

#endif  /* _VDSTRING_HXX */
