#define _VDSTRINGPATTERN_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdStringPattern.cxx                               *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdStringPattern.hxx>

#ifdef _HXX_INCLUDE_
#include <string.hxx>
#else
#include <string.h>
#endif

static void * strdup( void * s )
{ if (s) return strcpy(new char[strlen((char*)s)+1],(char*)s);
  else return NULL;
}

#define MARK '\273'
#define STACKSIZE 8

static int maxlen;
static struct{ unsigned char * cptr
             , *rptr; int idx;
             } stack[STACKSIZE];
static int stackptr;

static unsigned char * findcont( unsigned char * ptr )
{ int d=0;
  while(1)
  { if (*ptr++==255)
    { switch (*ptr)
      { case '\0' : return ptr-1;
        case '{'  : d++; ptr+=3; continue;
        case '}'  : d--; if (d==0) return ptr+1;
                    ptr++; continue;
        default   : ptr+=3; continue;
      }
    }
  }
}

int VdStringPattern::analyse
         ( unsigned char * apptr, char * asptr , int len ) const
{ int i, min, max, code;
  unsigned char * rptr, * cptr;
  int idx;
  if (!apptr) return 1;
  if (!*apptr) return 1;
  while (*apptr!=255)
    if (*apptr!=*asptr)
    { if (maxlen<len) maxlen=len;
      return 0;
    }
    else
    { apptr++; asptr++; len++;
    }
  switch (*++apptr)
  { case '\0': if (*asptr=='\0') return 1;
               if (maxlen<len) maxlen=len;
               return 0;
    case '{' : rptr=apptr-1;
               if ((stackptr)&&(stack[stackptr-1].rptr==rptr))
               { cptr=stack[stackptr-1].cptr;
                 idx=stack[stackptr-1].idx;
               }
               else
               { stack[stackptr].rptr=rptr;
                 cptr=stack[stackptr].cptr=findcont(rptr);
                 idx=stack[stackptr++].idx=0;
               }
               code=*apptr++; min=*apptr++-1; max=*apptr++-1;
               if (idx>=max) return analyse(cptr,asptr,len);
               if (analyse(apptr,asptr,len))
               { if (idx==0) stackptr--;
                 return 1;
               }
               if (idx<min)
               { if (idx==0) stackptr--;
                 return 0;
               }
               if (idx==0) stackptr--;
               return analyse(cptr,asptr,len);
    case '}' : stack[stackptr-1].idx++;
               return analyse(stack[stackptr-1].rptr,asptr,len);
    default  : code=*apptr++; min=*apptr++-1; max=*apptr++-1;
               for (i=0; i<min; i++ )
               if (!((*asptr)&&(types[code].contains(*asptr))))
               { if (maxlen<len) maxlen=len;
                 return 0;
               }
               else
               { asptr++; len++;
               }
               for ( ; i<max; i++)
               { if (analyse(apptr,asptr,len)) return 1;
                 if (!((*asptr)&&(types[code].contains(*asptr))))
                 { if (maxlen<len) maxlen=len;
                   return 0;
                 }
                 asptr++; len++;
               }
               if (analyse(apptr,asptr,len)) return 1;
               if (maxlen<len) maxlen=len;
               return 0;
  }
}

VdStringPattern::VdStringPattern( void )
{ pattern=NULL;
  setType('l',VdCharSet('a','z'));
  setType('L',VdCharSet('A','Z'));
  setType('d',VdCharSet('0','9'));
  setType('o',VdCharSet('0','7'));
  setType('x',VdCharSet('a','f')|VdCharSet('A','F')
                                           |VdCharSet('0','9'));
  setType('a',VdCharSet('a','z')|VdCharSet('A','Z')
                                           |VdCharSet('0','9'));
  setType('c',VdCharSet('\t')|VdCharSet(' ','\176'));
  setType('C',VdCharSet('\t')|VdCharSet('\n')|VdCharSet(' ','\176'));
  setType('W',VdCharSet('\001',' ')|VdCharSet('\177'));
  setType('w',(VdCharSet('\001',' ')|VdCharSet('\177')).remove('\n'));
  setType('f',VdCharSet("*?\\{[]}$~\"'`,;<>()",18)
                           |VdCharSet('\001',' ')|VdCharSet('\177'));
  setType('p',VdCharSet("\\{[]}$\"'`,;<>()",15)
                           |VdCharSet('\001',' ')|VdCharSet('\177'));
}

VdStringPattern::VdStringPattern( const VdStringPattern & p )
{ int i;
  for (i=1; i<53; i++) types[i]=p.types[i];
  pattern=( unsigned char *)strdup(p.pattern);
}

VdStringPattern & VdStringPattern::operator =
                                 ( const VdStringPattern & p )
{ int i;
  for (i=1; i<53; i++) types[i]=p.types[i];
  pattern=( unsigned char *)strdup(p.pattern);
  return *this;
}

int VdStringPattern::setType( char c , const VdCharSet & s )
{ if ((c>='a')&&(c<='z'))
  { types[c-('a'-27)]=s;
    return 1;
  }
  if ((c>='A')&&(c<='Z'))
  { types[c-('A'-1)]=s;
    return 1;
  }
  return 0;
}

VdCharSet VdStringPattern::getType( char c ) const
{ if ((c>='a')&&(c<='z')) return types[c-('a'-27)];
  if ((c>='A')&&(c<='Z')) return types[c-('A'-1)];
  VdCharSet null;
  return null;
}

void VdStringPattern::setPattern( const VdString & s)
{ char * ptr = s.charptr();
  unsigned char buf[1024];
  int bufptr = 0;
  int depth=0;
  while (*ptr)
  { if (*ptr == '%')
    { ptr++;
      if (*ptr=='%')
      { buf[bufptr++]='%';
        ptr++;
        continue;
      }
      if (*ptr=='\0')
      { buf[bufptr++]='%';
        continue;
      }
      if (*ptr=='}')
      { ptr++; 
        if (depth)
        { if (depth<=STACKSIZE)
          { buf[bufptr++]=255; buf[bufptr++]='}';
          }
          depth--;
        }
        continue;
      }
      int min = -1, max = -1;
      if ((*ptr>='0')&&(*ptr<='9'))
      { min=0;
        while ((*ptr>='0')&&(*ptr<='9')) min+=10*min+ *ptr++ -'0';
        min++; if (min>254) min=254;
      }
      if (*ptr=='.')
      { ptr++;
        max=0;
        while ((*ptr>='0')&&(*ptr<='9')) max+=10*max+ *ptr++ -'0';
        max++; if (max>254) max=254;
      }
      if (max<0) { if (min<0) max=254; else max=min; }
      if (min<0) min=1;
      if ((*ptr>='a')&&(*ptr<='z'))
      { buf[bufptr++]=255; buf[bufptr++]=*ptr-('a'-27);
        buf[bufptr++]=min; buf[bufptr++]=max;
        ptr++;
        continue;
      }
      if ((*ptr>='A')&&(*ptr<='Z'))
      { buf[bufptr++]=255; buf[bufptr++]=*ptr-('A'-1);
        buf[bufptr++]=min; buf[bufptr++]=max;
        ptr++;
        continue;
      }
      if (*ptr=='{')
      { depth++;
        if (depth<=STACKSIZE)
        { buf[bufptr++]=255; buf[bufptr++]='{';
          buf[bufptr++]=min; buf[bufptr++]=max;
        }
        ptr++;
        continue;
      }
    }
    else buf[bufptr++]=*ptr++;
  }
  buf[bufptr++]=255;
  buf[bufptr++]='\0';
  pattern=(unsigned char *)strdup(buf);
}

int VdStringPattern::test( VdString & s ) const
{ maxlen=0; stackptr=0;
  if (analyse(pattern,s.charptr(),0)) return 0;
  return maxlen+1;
}

int VdStringPattern::testAndMark( VdString & s ) const
{ maxlen=0; stackptr=0;
  if (analyse(pattern,s.charptr(),0)) return 0;
  int len=strlen(s.charptr());
  char * ptr = new char[len+2];
  if (maxlen>len) maxlen=len;
  strncpy(ptr,s.charptr(),maxlen);
  *(ptr+maxlen)=MARK;
  *(ptr+maxlen+1)='\0';
  if (len>=maxlen) strcat(ptr,s.charptr()+maxlen);
  s=VdString(ptr);
  delete ptr;
  return maxlen+1;
}

VdStringPattern::~VdStringPattern( void )
{ delete pattern;
}

#undef _VDSTRINGPATTERN_CXX
