#define _VDSUBLANGUAGE_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdSublanguage.cxx                                 *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdSublanguage.hxx>
#include <Vd/VdReference.hxx>
#include <Vd/VdConsistence.hxx>

#ifdef _HXX_INCLUDE_
#include <stddef.hxx>
#else
#include <stddef.h>
#endif


void VdSublanguage::VdAddReference( VdReference * r )
{ if (VdReferencePtrs.idx(r)<0)
    VdReferencePtrs.ins()=r;
}

void VdSublanguage::VdRemReference( VdReference * r )
{ VdReferencePtrs.rem(VdReferencePtrs.idx(r));
}

void VdSublanguage::VdClearReferences( void )
{ while(!VdReferencePtrs)
    VdReferencePtrs[0]->VdRemReference();
}

VdSublanguage::VdSublanguage ( void )
{ 
}

VdSublanguage::VdSublanguage ( VdSublanguage & x )
{ VdConsistence::registerInstance(&x,this);
  VdLanguageInterface * hptr;
  VdSublanguageName=x.VdSublanguageName;
  hptr=x.VdRealSlangPtr->VdDuplicateLanguage();
  VdRealSlangPtr=hptr->VdGetLanguageInterface(VdSublanguageName);
  VdRealSlangPtr->VdOriginPtr=this;
}

VdSublanguage::~VdSublanguage (void )
{ VdClearReferences();
  delete VdRealSlangPtr;
}

void VdSublanguage::VdExchangeSlg( VdSublanguage * x )
{ VdLanguageInterface * hptr;
  hptr=VdRealSlangPtr; VdRealSlangPtr=x->VdRealSlangPtr;
  x->VdRealSlangPtr=hptr;
  VdRealSlangPtr->VdOriginPtr=this;
  x->VdRealSlangPtr->VdOriginPtr=x;
}

void VdSublanguage::VdExchangeRefPtrs( VdSublanguage * x )
{ VdReferencePtrList ptrlist = VdReferencePtrs;
  int i;
  VdClearReferences();
  for (i=0; i<!x->VdReferencePtrs; i++ )
    x->VdReferencePtrs[i]->VdSetReference(this);
  x->VdClearReferences();
  for (i=0; i<!ptrlist; i++ )
    ptrlist[i]->VdSetReference(x);
}

int VdSublanguage::operator ! (void) const
{ return !VdReferencePtrs;
}

void VdSublanguage::operator = ( VdSublanguage & s )
{ VdLanguageInterface * hptr;
  if (s.VdRealSlangPtr->VdGetLanguageInterface(VdSublanguageName))
  { hptr=s.VdRealSlangPtr->VdDuplicateLanguage();
    delete VdRealSlangPtr;
    VdRealSlangPtr=hptr->VdGetLanguageInterface(VdSublanguageName);
    VdRealSlangPtr->VdOriginPtr=this;
  }
}

void VdSublanguage::operator = ( VdReference & r )
{ VdLanguageInterface * hptr;
  if (r.VdSublanguagePointer
         ->VdRealSlangPtr
            ->VdGetLanguageInterface(VdSublanguageName)
     )
  { hptr=r.VdSublanguagePointer->VdRealSlangPtr->VdDuplicateLanguage();
    delete VdRealSlangPtr;
    VdRealSlangPtr=hptr->VdGetLanguageInterface(VdSublanguageName);
    VdRealSlangPtr->VdOriginPtr=this;
  }
}

void VdSublanguage::obtain( VdSublanguage & s )
{ VdLanguageInterface * hptr;
  if (s.VdRealSlangPtr->VdGetLanguageInterface(VdSublanguageName))
  { hptr=s.VdRealSlangPtr->VdDuplicateLanguage();
    delete VdRealSlangPtr;
    VdRealSlangPtr=hptr->VdGetLanguageInterface(VdSublanguageName);
    VdRealSlangPtr->VdOriginPtr=this;
  }
}

void VdSublanguage::obtain( VdReference & r )
{ VdLanguageInterface * hptr;
  if (r.VdSublanguagePointer
         ->VdRealSlangPtr
            ->VdGetLanguageInterface(VdSublanguageName)
     )
  { hptr=r.VdSublanguagePointer->VdRealSlangPtr->VdDuplicateLanguage();
    delete VdRealSlangPtr;
    VdRealSlangPtr=hptr->VdGetLanguageInterface(VdSublanguageName);
    VdRealSlangPtr->VdOriginPtr=this;
  }
}

int VdSublanguage::operator == ( const VdSublanguage & s ) const
{ return this == &s;
}

int VdSublanguage::operator != ( const VdSublanguage & s ) const
{ return this != &s;
}

int VdSublanguage::operator == ( const VdReference & r ) const
{ return this == r.VdSublanguagePointer;
}

int VdSublanguage::operator != ( const VdReference & r ) const
{ return this != r.VdSublanguagePointer;
}

int VdSublanguage::operator == ( VdLanguageInterface & l ) const
{ return VdRealSlangPtr == l.VdGetLanguageInterface(VdSublanguageName);
}

int VdSublanguage::operator != ( VdLanguageInterface & l ) const
{ return VdRealSlangPtr != l.VdGetLanguageInterface(VdSublanguageName);
}

#undef _VDSUBLANGUAGE_CXX
