#define _VDTOGGLELIST_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdToggleList.cxx                                  *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdToggleList.hxx>
#include <Vd/VdRadioBoxInterface.hxx>
#include <Vd/VdCheckBoxInterface.hxx>
#include <X11/Intrinsic.h>

#include <Xm/Xm.h>
#include <Xm/ToggleBG.h>

extern "C" {
#include <stdio.h>
#include <stdlib.h>
}

VdToggleList::VdToggleList( Widget parent, 
		VdFeature * parentWindow,
		VdButtonInterface * handle,
		int onlyOne,
		long initialSelectedMask ): 
	oneOfMany(onlyOne)
{
	Arg al[10];
	Cardinal ac;
	Widget toggleButton;
	int i; long h=1;
	nrToggles=0;
	if (!handle || !handle->getType()) {
		fprintf(stderr,
		   "VISAMAD-Fatal: illegal interface for VdToggleList\n");
		exit(1);
	}
	menuPane = parent;
	selectedMask = initialSelectedMask;
	if (onlyOne)
	    optionLabels = ((VdRadioBoxInterface *)handle)->labels;
	else
	    optionLabels = ((VdCheckBoxInterface *)handle)->labels;
	theParentWindow = * parentWindow;
	theHandle = handle;
	for (i=0;i<!optionLabels && i<32; i++) {
		// Einfgen der toggleButtons
		ac=0;
		if (oneOfMany) {
		    XtSetArg(al[ac], XmNindicatorType, 
		        XmONE_OF_MANY); ac++;
		} else {
		    XtSetArg(al[ac], XmNindicatorType, 
		        XmN_OF_MANY); ac++;
		}
		if (selectedMask & h) {
		    XtSetArg(al[ac], XmNset, True); ac++;
		}
		h <<= 1;
		toggleButton = 
		    XmCreateToggleButtonGadget(
			menuPane,
			optionLabels[i].charptr(),
			al,ac);
		XtManageChild(toggleButton);
		addCB(toggleButton, valueChanged);
		toggles[nrToggles++]=toggleButton;
	}
	watchDestroying(parent);
}

VdToggleList::~VdToggleList( void )
{
} 


int VdToggleList::getSelected( void )
{
	if (!oneOfMany) return -1;
	int r=0;
	long h=selectedMask;
	while (h) { h >>= 1; r++; }
	return r;
}

void VdToggleList::setSelected( int nr )
{
	Arg al[10];
	Cardinal ac;
	long h=1; int selected;

	selected = getSelected();
	ac = 0;
	XtSetArg(al[ac], XmNset, False); ac++;
	XtSetValues(toggles[selected-1], al, ac);

	selected = nr;
	while (--selected) h <<= 1;
	if (oneOfMany)
		selectedMask = h;
	else
		selectedMask ^= h;
	ac = 0;
	XtSetArg(al[ac], XmNset, True); ac++;
	XtSetValues(toggles[nr-1], al, ac);
	if (theHandle->getType()==1) {
		((VdRadioBoxInterface *)theHandle)
			->mask=selectedMask;
	} else if (theHandle->getType()==2) {
		((VdCheckBoxInterface *)theHandle)
			->mask=selectedMask;
	}
}

long VdToggleList::getMask( void )
{
	return selectedMask;
}

void VdToggleList::setMask( long mask )
{
	Arg al[10];
	Cardinal ac;
	selectedMask = mask;
	int i, h=1;
	for (i=0;i<nrToggles;i++) {
		ac = 0;
		XtSetArg(al[ac], XmNset, (h&mask ? True : False) ); 
		ac++;
		XtSetValues(toggles[i], al, ac);
		h <<= 1;
	}
}

Widget VdToggleList::getWidget( int nr)
{
	if (nr>=0 && nr<nrToggles) return toggles[nr];
	else return 0;
}

int VdToggleList::getNrToggles( void )
{
	return nrToggles;
}

void VdToggleList::addOption( String newOption )
{
	Arg al[10];
	Cardinal ac;
	Widget toggleButton;
	VdStringList *lp;
	
	if (!optionLabels >= 32) return;
	optionLabels[!optionLabels]=VdString(newOption);
	ac=0;
	if (theHandle->getType()==1) {
	    lp = & (((VdRadioBoxInterface *)theHandle)->labels);
	    XtSetArg(al[ac], XmNindicatorType, 
	        XmONE_OF_MANY); ac++;
	} else {
	    XtSetArg(al[ac], XmNindicatorType, 
	        XmN_OF_MANY); ac++;
	    lp = & (((VdCheckBoxInterface *)theHandle)->labels);
	}
	lp->ins() = VdString(newOption);
	if (selectedMask & (1 << nrToggles)) {
		XtSetArg(al[ac], XmNset, True); ac++;
	}
	toggleButton = 
	    XmCreateToggleButtonGadget(
		menuPane,
		newOption,
		al,ac);
	XtManageChild(toggleButton);
	addCB(toggleButton, valueChanged);
	toggles[nrToggles++]=toggleButton;
}

void VdToggleList::removeOption( String oldOption )
{
	int i;
	VdString oldOS(oldOption);
	if (nrToggles==0) return;
	i=optionLabels.idx(oldOS);
	if (i<0) return;
	optionLabels.rem(i);
	selectedMask = 
		(((1<<i)-1)&selectedMask)
	      + ((selectedMask>>(i+1))<<i);
	XtUnmanageChild(menuPane);
	XtDestroyWidget(toggles[i]);
	XtManageChild(menuPane);
	for (;i<nrToggles-1;i++) toggles[i]=toggles[i+1];
	nrToggles--;
	if (theHandle) {
	    VdStringList *lp;
	    if (theHandle->getType()==1) {
		((VdRadioBoxInterface *)theHandle)
			->mask=selectedMask;
		lp = & (((VdRadioBoxInterface *)theHandle)->labels);
	    } else if (theHandle->getType()==2) {
		((VdCheckBoxInterface *)theHandle)
			->mask=selectedMask;
		lp = & (((VdCheckBoxInterface *)theHandle)->labels);
	    }
	    int i = lp->idx( oldOS );
	    if (i<0) return;
	    lp->rem(i); 
	}
}

void VdToggleList::setSensitivity( int nr, int state )
{
	if (nr>=nrToggles || nr<0) return;
	XtSetSensitive(toggles[nr],state);
} 

void VdToggleList::handleValueChanged( void )
{
	int i; long h=1;
	Widget widget = getLastWidgetOfEvent();
	for (i=0;i<nrToggles;i++) {
		if (widget==toggles[i]) 
			selectedMask ^= h;
		h <<= 1;
	}
	if (oneOfMany) {
	   if (selectedMask) handleActivated();
	   // This is correct if at first the switch off toggle is
	   // handled and after this the switch on toggle. 
	   // handleActivated will be executed once.
	} else {
		handleActivated();
	}
}


void VdToggleList::handleActivated( void )
{
	if (!theHandle) return;
	if (theHandle->getType()==1) {
		((VdRadioBoxInterface *)theHandle)
			->mask=selectedMask;
	} else if (theHandle->getType()==2) {
		((VdCheckBoxInterface *)theHandle)
			->mask=selectedMask;
	}
	theHandle->activate( theParentWindow );
}

#undef _VDTOGGLELIST_CXX
