#define _VDVECTOR_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdVector.cxx                                      *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdVector.hxx>

#ifdef _HXX_INCLUDE_
#include <math.hxx> 
#else
#include <math.h> 
#endif

VdVector::VdVector ( const VdPoint & p ) { x = p.x; y = p.y; }
 
VdVector VdVector::operator+ ( const VdVector & v ) const
{ VdVector h; 
  h.x = x + v.x; h.y = y + v.y; 
  return h; 
} 
 
VdVector & VdVector::operator+= ( const VdVector & v ) 
{ x += v.x; y += v.y;
  return *this; 
} 
 
VdVector VdVector::operator- ( const VdVector & v ) const
{ VdVector h; 
  h.x = x - v.x; h.y = y - v.y; 
  return h; 
} 
 
VdVector & VdVector::operator-= ( const VdVector & v ) 
{ x -= v.x; y -= v.y;
  return *this; 
} 
 

VdVector VdVector::operator* ( int f ) const
{ VdVector h; 
  h.x = f*x; h.y = y*f; 
  return h; 
} 
 
VdVector operator* ( int f, const VdVector & v ) 
{ return v*f; 
} 
 
VdVector & VdVector::operator*= ( int f ) 
{  x *= f; y *= f; 
   return *this; 
} 
 

VdVector VdVector::operator- ( void ) const
{ VdVector v(-x, -y); 
  return v; 
} 
 
 
 
VdVector VdVector::operator/ ( int f ) const
{ VdVector h; 
  if (f) { h.x = x/f; h.y = y/f; } 
  return h; 
} 
 
VdVector & VdVector::operator/= ( int f ) 
{ if (f) {x /=  f; y /= f;} 
  return *this; 
} 

 
int VdVector::operator* ( const VdVector & v ) const
{ return x*v.x + y*v.y; 
} 

 
float VdVector::operator! ( void ) const
{ return sqrt(x*x+y*y); 
} 
 
float VdVector::length ( void ) const
{ return sqrt(x*x+y*y); 
} 

 
int VdVector::operator== ( const VdVector & v ) const
{ return ((x == v.x)&&(y == v.y)); 
} 

int VdVector::operator!= ( const VdVector & v ) const
{ return ((x != v.x)||(y != v.y)); 
} 
 
int VdVector::operator|| ( const VdVector & v ) const
{ return (x * v.y - y * v.x)  == 0; 
} 

 
VdVector VdVector::norm ( int l ) const
{ VdVector h; int i = x*x+y*y;
  if (i==0) return h;
  float ll=sqrt(i);
  h.x = ((float)(x * l))/ll; 
  h.y = ((float)(y * l))/ll; 
  return h; 
} 

 
VdVector VdVector::rotate ( int arc , char we ) const
{ float arc1; 
  if ((we == 'G') || (we == 'g')) 
    arc1 = arc * VdGradToRad; 
  else if ( ( we == 'R') || (we == 'r')) 
              arc1 = arc; 
       else if ( ( we == 'X') || (we == 'x')) 
                  arc1 = (arc<<6)*VdGradToRad; 
  float alpha = cos(arc1), beta = sin(arc1); 
  VdVector h; 
  h.x = alpha * x + beta * y; 
  h.y = -beta * x + alpha * y; 
  return h; 
} 

#undef _VDVECTOR_CXX
