#ifndef _VDVECTOR_HXX
#define _VDVECTOR_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdVector.hxx                                      *
*  Date: 29-09-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdVector;

/******************    dependences    *********************/

#include <Vd/VdPoint.hxx>

class VdVector { 
   public : 
     int x, y; 
 
 
     VdVector ( void ) { x = y = 0; } 
     VdVector ( int x1 , int y1 = 0 ) { x = x1; y = y1; } 
     VdVector ( const VdVector & v ) { x = v.x; y = v.y; } 
     VdVector ( const VdPoint & p );              // Ortsvektor von p 
 
     VdVector operator+ ( const VdVector & v ) const;    // y = x + v 
     VdVector & operator+= ( const VdVector & v );       // y = y + v 
     VdVector operator- ( const VdVector & v ) const;    // y = x - v 
     VdVector & operator-= ( const VdVector & v );       // y = y - v 
 
     VdVector  operator* ( int f ) const;               // y = x * f 
     friend VdVector operator* ( int f, const VdVector & v );
                                                        // y = f * v 
     VdVector & operator*= ( int f );                   // y *= f 
     VdVector  operator- ( void ) const;                // y = -x 
     VdVector operator/ ( int f ) const;                // y = x / f 
     VdVector & operator/= ( int f );                   // y /= f 
 
     int operator * ( const VdVector & v ) const;       // s = x * v 
     float operator! ( void ) const;                    // Betrag
     float length ( void ) const;                       // Betrag

     int operator== ( const VdVector & v ) const;     // Gleichheit 
     int operator!= ( const VdVector & v ) const;     // Ungleichheit 
     int operator|| ( const VdVector & v ) const;     // Parallelit. 

     VdVector norm ( int l = 1 ) const; 
              // Vektor x mit Richtungsvektor von v und Laenge l
     VdVector rotate ( int arc , char we = 'G' ) const; 
              // Vektor wird um Winkel arc rotiert 
};

/******************     constants     *********************/ 

#ifdef _VDVECTOR_CXX

   VdVector VdNullVector;          // Nullvektor 

#else

   extern VdVector VdNullVector;          // Nullvektor 

#endif
 
   extern float VdGradToRad;          // von Grad zu Radiant 
   extern float VdRadToGrad;          // von Radiant zu Grad 

#endif /* _VDVECTOR_HXX */
