#define _VDWINDOWCONNECTION_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdWindowConnection.cxx                            *
*  Date: 09-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter, B.Miethig                    *
*  State of Realization:  first version                    *
*                                                          *
************************************************************
* changes:                                                 *
* 07-May-93   addButton/removeButton also for Magnifier    *
*             map/unmap also for Magnifier                 *
***********************************************************/

/******************    dependences    *********************/

#include <Vd/VdWindowConnection.hxx>
#include <Vd/VdMagnifierWindow.hxx>
#include <Vd/VdMagnifierButton.hxx>
#include <Vd/VdEditWindow.hxx>
#include <Vd/VdWindowManagerInterface.hxx>
#include <Vd/VdEventManagerInterface.hxx>
#include <Vd/VdPopupMenuInterface.hxx>
#include <Vd/VdPulldownOptions.hxx>
#include <Vd/VdRadioBoxInterface.hxx>
#include <Vd/VdCheckBoxInterface.hxx>
#include <Vd/VdPopupMenu.hxx>
#include <Vd/VdFormular.hxx>
#include <Vd/VdFormularInterface.hxx>

#ifdef _HXX_INCLUDE_
#include <iostream.hxx>
#else
#include <iostream.h>
#endif

extern "C" {
#include <stdlib.h>
}


// internal list of edit windows 

static int operator== ( 
	const VdEditWindowListEntry le1,
	const VdEditWindowListEntry le2)
{
	if ( (le1.ewPtr==le2.ewPtr)
	     && (le1.ewId==le2.ewId)
	     && (le1.pic==le2.pic))
		return 1;
	else
		return 0;
}

static int operator!= ( 
	const VdEditWindowListEntry le1,
	const VdEditWindowListEntry le2)
{
	return !operator==(le1,le2);

}

#define VdListClassName VdEditWindowList
#define VdListClassElementType VdEditWindowListEntry

#include <Vd/VdListDefinition.mxx>

#undef VdListClassName
#undef VdListClassElementType

// internal list of buttons 

static int operator== ( 
	VdButtonListEntry le1,
	VdButtonListEntry le2)
{
	if ( (le1.bPtr==le2.bPtr)
	     && (le1.handle==le2.handle)
	     && (le1.awPtr==le2.awPtr) )
		return 1;
	else
		return 0;
}

static int operator!= ( 
	VdButtonListEntry le1,
	VdButtonListEntry le2)
{
	return 1 - operator==(le1,le2);

}

#define VdListClassName VdButtonList
#define VdListClassElementType VdButtonListEntry

#include <Vd/VdListDefinition.mxx>

#undef VdListClassName
#undef VdListClassElementType
 


VdWindowConnection::VdWindowConnection( void )
{
	theIdentification = VdFeature("VisamadInterface");
	theMagnifierPtr = NULL;
	theMagnifiedPicturePtr = NULL;
	activeEditWindow = NULL;
	theEventManager = NULL;
	theWindowManager = NULL;
	theMode=Vd_DEFAULT_DEFAULT ;
	theMagnifierIdent = VdFeature("*:MAGNIFIER");
	theCheckingFunction = NULL;
}

VdWindowConnection::VdWindowConnection( const VdFeature ident )
{
	theIdentification = ident;
	theMagnifierPtr = NULL;
	theMagnifiedPicturePtr = NULL;
	activeEditWindow = NULL;
	theEventManager = NULL;
	theWindowManager = NULL;
	theMode=Vd_DEFAULT_DEFAULT ;
	theMagnifierIdent = VdFeature("*:MAGNIFIER");
	theCheckingFunction = NULL;
}

VdWindowConnection::VdWindowConnection( const VdFeature ident,
		VdWindowManagerInterface *interface )
{
	theIdentification = ident;
	theMagnifierPtr = NULL;
	theMagnifiedPicturePtr = NULL;
	activeEditWindow = NULL;
	theEventManager = NULL;
	theWindowManager = interface;
	theMode=Vd_DEFAULT_DEFAULT ;
	theMagnifierIdent = VdFeature("*:MAGNIFIER");
	theCheckingFunction = NULL;
}

VdFeature VdWindowConnection::getIdent( void )
{
	return theIdentification;
}


void VdWindowConnection::magnifierMoved( 
		VdPoint ul, VdPoint lr )
{
	if (!activeEditWindow) return;
	VdArea displayed, world;
	VdVector magnSize = theMagnifierPtr->getDrawableSize();
	activeEditWindow->getDisplayOfWorld(displayed,world);

	displayed = displayOfWorldByMagnifier(
		world, ul, lr, magnSize);

	activeEditWindow->setDisplayOfWorld(displayed,world);
	activeEditWindow->redrawPicture();
}

void VdWindowConnection::magnifierResized( 
		VdPoint ul, VdPoint lr )
{
	VdArea displayed, world;
	if ( !activeEditWindow ) return;
	activeEditWindow->getDisplayOfWorld(displayed,world);
	VdVector magnSize = theMagnifierPtr->getDrawableSize();

	if (( activeEditWindow->getWindowType()==Vd_BUBBLE_GUM )
	    || ( theMagnifierPtr->getMode()
		== Vd_MAGNIFIER_PROPORTIONAL_MODE )) {
		displayed = displayOfWorldByMagnifier(
			world, ul, lr, magnSize);
		activeEditWindow->
			setDisplayOfWorld(displayed,world);
		activeEditWindow->redrawPicture();
	} else {
		// reset magnifier to old values (resize not allowed)
		VdPoint magnUl, magnLr;
		theMagnifierPtr->
			getMagnifierData(magnUl, magnLr);
		magnifierByDisplayOfWorld(
			displayed, world, magnSize, 
			&magnUl, &magnLr);
		theMagnifierPtr->
			setMagnifierData(magnUl, magnLr);
	}
}


void VdWindowConnection::worldMoved( 
		VdPoint ul, VdPoint lr )
{
	VdArea displayed, world;
	VdPoint p; int diffx, diffy;
	if (!activeEditWindow) return;
	VdVector magnSize = theMagnifierPtr->getDrawableSize();
	activeEditWindow->
		getDisplayOfWorld(displayed,world);
	p = (displayOfWorldByMagnifier(
			world, ul, lr, magnSize)).ul;
	diffx = p.x - displayed.ul.x;
	diffy = p.y - displayed.ul.y;
	// p would be the fictive displayed.ul if the magnifier
	// would be moved, the diffs are the differences in
	// world coordinates between old and this "new" value

	// EditWindow's moving:
	world.ul.x -= diffx;
	world.ul.y -= diffy;
	world.lr.x -= diffx;
	world.lr.y -= diffy;
	activeEditWindow->setDisplayOfWorld(displayed,world);
	activeEditWindow->updateScrollbars();

	// magnifier's world moving:
	theMagnifierPtr->
		getDisplayOfWorld(displayed,world);
	world.ul.x -= diffx;
	world.ul.y -= diffy;
	world.lr.x -= diffx;
	world.lr.y -= diffy;
	theMagnifierPtr->setDisplayOfWorld(world,world);
	theMagnifierPtr->redrawPicture();
}

void VdWindowConnection::worldResized( 
		VdPoint ul, VdPoint lr )
// world resized by magnifier manipulation
{
	VdArea displayed, world;
	VdVector magnSize = theMagnifierPtr->getDrawableSize();
	if (!activeEditWindow) return;
	activeEditWindow->
		getDisplayOfWorld(displayed,world);
	world = worldByDisplayAndMagnifier(
			displayed, ul, lr, magnSize);
	activeEditWindow->setDisplayOfWorld(displayed,world);
	activeEditWindow->updateScrollbars();
	theMagnifierPtr->setDisplayOfWorld(world,world);
	theMagnifierPtr->redrawPicture();
}

void VdWindowConnection::worldResizedByBorder( 
		VdPoint , VdPoint  )
// world resized by OSF/Motif border manipulation
{
	VdArea magnDisplayed, magnWorld, 
	       editDisplayed, editWorld;
	VdVector magnSize = theMagnifierPtr->getDrawableSize();
	VdPoint magnUl, magnLr;

	theMagnifierPtr->
		getDisplayOfWorld(magnDisplayed,magnWorld);
	if (theMagnifierPtr->getWindowType()==Vd_BUBBLE_GUM) {
		// Vergrern Magnifier
		if (activeEditWindow) {
			activeEditWindow->
		   		getDisplayOfWorld(
					editDisplayed,editWorld);
			magnifierByDisplayOfWorld(
			   editDisplayed, editWorld, 
			   magnSize, &magnUl, &magnLr
			);
			theMagnifierPtr->setMagnifierData(
				magnUl, magnLr);
		} else {
			// default values
			theMagnifierPtr->setMagnifierData(
			   VdPoint(magnSize.x/4,3*magnSize.x/4),
			   VdPoint(magnSize.y/4,3*magnSize.y/4)
			);
		}
	} else {
		// Vergrern World im EditWindow
		if (!activeEditWindow) return;
		activeEditWindow->
		   getDisplayOfWorld(editDisplayed,editWorld);
		if (editDisplayed < magnWorld) {
		   activeEditWindow->
		      setDisplayOfWorld(editDisplayed,magnWorld);
		   activeEditWindow->updateScrollbars();
		} else {
		   // MgnifierWindow resizing cutted the magnifier
	VdPoint magnUl, magnLr;
	VdVector magnSize = theMagnifierPtr->getDrawableSize();
	editWorld = theMagnifierPtr->putAreaInWindow(editWorld);
	activeEditWindow->setDisplayOfWorld(editDisplayed,editWorld);
	activeEditWindow->updateScrollbars();
	theMagnifierPtr->setDisplayOfWorld(editWorld,editWorld);

	magnifierByDisplayOfWorld(
		editDisplayed, editWorld, magnSize, &magnUl, &magnLr);
	theMagnifierPtr->setMagnifierData(magnUl, magnLr, 0);
	theMagnifierPtr->redrawPicture();
	}
   }
}


void VdWindowConnection::displayMoved( 
		VdPoint ul, VdPoint lr )
{
	magnifierMoved(ul,lr);
}

void VdWindowConnection::displayResized( 
		VdPoint ul, VdPoint lr )
{
	// function does the same as magnifierResized but not
	// only if window is in Vd_BUBBLE_GUM mode
	VdArea displayed, world;
	if ( !activeEditWindow ) return;
	activeEditWindow->getDisplayOfWorld(displayed,world);
	VdVector magnSize = theMagnifierPtr->getDrawableSize();

	displayed = displayOfWorldByMagnifier(
			world, ul, lr, magnSize);
	activeEditWindow->
			setDisplayOfWorld(displayed,world);
	activeEditWindow->redrawPicture();
	return;
}

VdEditWindow * VdWindowConnection::getActiveEditWindow( void )
{
	return activeEditWindow;
}

int VdWindowConnection::isActiveEditWindow( 
	VdEditWindow * ewp )
{
	return ( ewp == activeEditWindow );
}

void VdWindowConnection::movedDisplayOfWorld( 
			VdArea displayed, VdArea world )
{
// should be called only after moving scrollbars
	if ( !theMagnifierPtr ) return;
	VdPoint magnUl, magnLr;
	VdVector magnSize = theMagnifierPtr->getDrawableSize();
	theMagnifierPtr->getMagnifierData(magnUl, magnLr);

	magnifierByDisplayOfWorld(
		displayed, world, magnSize, &magnUl, &magnLr);

	theMagnifierPtr->setMagnifierData(magnUl, magnLr);
}

void VdWindowConnection::changedDisplayOfWorld(
			VdArea displayed, VdArea world)
{
// will  be called after resizing EditWindow
	if ( !theMagnifierPtr) return;

	VdPoint magnUl, magnLr;
	VdVector magnSize = theMagnifierPtr->getDrawableSize();

	// adapt magnifier's world to edit's world
	// if the editwindow a defaultwindow

	if(activeEditWindow->getWindowType()==Vd_DEFAULT_WINDOW)
	{
	world = theMagnifierPtr->putAreaInWindow(world);
	activeEditWindow->setDisplayOfWorld(displayed,world);
	activeEditWindow->updateScrollbars();
	theMagnifierPtr->setDisplayOfWorld(world,world);

	magnifierByDisplayOfWorld(
		displayed, world, magnSize, &magnUl, &magnLr);
	theMagnifierPtr->setMagnifierData(magnUl, magnLr, 0);
	theMagnifierPtr->redrawPicture();
	 }

	// for a bubblegum only displayed will be changed
	if(activeEditWindow->getWindowType()==Vd_BUBBLE_GUM)
	 {
	   theMagnifierPtr->getMagnifierData(magnUl, magnLr) ; 	
	   activeEditWindow->getDisplayOfWorld(displayed,world);
	   displayed=displayOfWorldByMagnifier(
		world,magnUl,magnLr,magnSize);	
	   activeEditWindow->setDisplayOfWorld(displayed,world);
	  }
}


VdFeature VdWindowConnection::setMagnifierWindow( 
	VdMagnifierWindow& mw)
{
	int i;
	VdEditWindowListEntry *le;

	theMagnifierPtr = &mw;
	theMagnifiedPicturePtr = mw.getPicture();
	theMagnifierPtr->setConnection(
			this,theIdentification);
	
	for ( i=0; 
	      i < theEditWindows.length(); 
	      i++) {
		le = & theEditWindows[i];
		le->button = theMagnifierPtr->
			addEditWindowButton(le->ewId);
	}
	mw.setTitle(theIdentification.getString());
	mw.setMode(Vd_MAGNIFIER_PROPORTIONAL_MODE);
	return mw.getIdent();
}

VdMagnifierWindow * VdWindowConnection::removeMagnifierWindow(  
		int toDestroy )
{
	VdMagnifierWindow * temp = theMagnifierPtr;
	VdFeature magId("*:MAGNIFIER");
	if (!theMagnifierPtr) return temp;
	theMagnifierPtr->setConnection(
				NULL,theIdentification);
	theMagnifierPtr->setTitle(VdString("VISAMAD magnifier"));
	temp->setExitChecker(NULL);
	if (toDestroy) {
		temp->destroy();
		temp=NULL;
	} else {
		removeButtons(magId,NULL);
	}
	theMagnifierPtr = NULL;
	return temp;
}

void VdWindowConnection::magnify( const VdFeature& ew )
{
	VdEditWindowListEntry * le;
	VdString newTitle;  // of MagnifierWindow
	le = getListEntry(ew);
	if (!le) return;

	VdEditWindow *wp = le->ewPtr;
	if (wp && theMagnifierPtr) {
		VdPoint ul,lr;
		VdArea editDisplayed, editWorld;
	   	activeEditWindow = wp;
		setMode(wp,le->cm) ;
		theMagnifiedPicturePtr = 
			activeEditWindow->getPicture();

	     // the worldAreas are to be adapted
		activeEditWindow->getDisplayOfWorld(
			editDisplayed, editWorld);
		editWorld = theMagnifierPtr->
			putAreaInWindow(editWorld);
		activeEditWindow->setDisplayOfWorld(
				editDisplayed, editWorld);
		theMagnifierPtr->setDisplayOfWorld(
				editWorld,editWorld);
	     // display the corresponding magnifier
		magnifierByDisplayOfWorld(
			editDisplayed, editWorld,
			theMagnifierPtr->getDrawableSize(),
			&ul, &lr);
		theMagnifierPtr->setMagnifierData(ul,lr,0);
		theMagnifierPtr->setPicture(
				theMagnifiedPicturePtr);
		newTitle = getIdent().getString();
		newTitle += VdString(" >");
		newTitle += (activeEditWindow->
			getIdent()).getLast(1).getString();
		newTitle += VdString("<");
		theMagnifierPtr->setTitle(newTitle);
	}
}

void VdWindowConnection::setWindowManagerInterface( 
		VdWindowManagerInterface * wmInterface)
{
	theWindowManager = wmInterface;
}

VdWindowManagerInterface * 
	VdWindowConnection::getWindowManagerInterface( void )
{
	return theWindowManager;
}

VdFeature VdWindowConnection::addEditWindow( 
	VdEditWindow& aWindow)
{
	VdEditWindowListEntry * le;

	if ( !( le=getListEntry(&aWindow) ) ) {
		le = &(theEditWindows[!theEditWindows]);
		le->ewPtr = &aWindow;
		(le->ewPtr)->setConnection(
				this,theIdentification);
		le->ewId = aWindow.getIdent();
		le->pic = aWindow.getPicture();
		le->evIf = aWindow.getEventManagerInterface();
		aWindow.setTitle((le->ewId).getString());
		if (theMagnifierPtr) {
		   le->button = theMagnifierPtr->
			addEditWindowButton(le->ewId);
		} else
		   le->button = NULL;
		setMode(&aWindow,Vd_DEFAULT_DEFAULT);
		aWindow.setExitChecker(theCheckingFunction);
		setView(le->ewId, VdONE, VdNullPoint, 
			VdFractNumber(1,2), VdZERO, VdZERO);
	} 
	return le->ewId;
}

VdEditWindow * VdWindowConnection::removeEditWindow( 
	const VdFeature & someWindows, int toDestroy )
{
	VdEditWindow * temp;
	int i,j;
	VdEditWindowListEntry ** list = 
			getAllListEntries(someWindows);
	VdEditWindowListEntry * le;
	VdEditWindow *ewPtr;

	for (i=0; le = list[i]; i++) {
		ewPtr = le->ewPtr;
		ewPtr->setConnection(
				NULL,theIdentification);
		if ( ewPtr 
		     == activeEditWindow ) {
			activeEditWindow = NULL;
			if (theMagnifierPtr) {
			   theMagnifierPtr->setPicture(NULL);
			   theMagnifiedPicturePtr=NULL;
			}
		}
		// remove the WindowConnection Id from
		// the Edit Windo Id
		j = theEditWindows.idx(*le);
		VdFeature oldId = (ewPtr->getIdent()).getLast(1);
		ewPtr->setIdent( oldId );
		ewPtr->setTitle( oldId.getString());
		if (theMagnifierPtr)
		    theMagnifierPtr->
			removeEditWindowButton(le->button);
		if (toDestroy) {
			ewPtr->destroy();
		} else {
			removeButtons(ewPtr->getIdent(),NULL);
		}
		theEditWindows.rem(j);
	}
	if (toDestroy || i==0) temp=NULL; 
	else temp = list[0]->ewPtr;
	delete list;
	return temp;
}

void VdWindowConnection::renameWindow(
		const VdFeature & oldWindowId,
		const VdFeature & newWindowId )
{
	VdEditWindowListEntry * le = getListEntry(oldWindowId);
	VdEditWindowListEntry * le2 = getListEntry(newWindowId);
	if (!le || le2) return;
	VdEditWindow *ewPtr = le->ewPtr;

	ewPtr->setIdent( newWindowId );
	ewPtr->setTitle( newWindowId.getString());
	le->ewId = newWindowId;
	if (theMagnifierPtr)
	    le->button = theMagnifierPtr->
		renameEditWindowButton(le->button,newWindowId);
	if (ewPtr==activeEditWindow) magnify(newWindowId);
		// for consistence of title bar
}

void VdWindowConnection::map( const VdFeature & someWindows )
{
	int i;

	if ( theMagnifierPtr 
	     && someWindows==theMagnifierIdent)
		theMagnifierPtr->map();
	VdEditWindowListEntry ** list = 
			getAllListEntries(someWindows);
	VdEditWindowListEntry * le;

	for (i=0; le = list[i]; i++) {
		(le->ewPtr)->map();
	}
	delete list;
}

void VdWindowConnection::unmap( const VdFeature & someWindows )	
{
	int i;
	if ( theMagnifierPtr 
	     && someWindows==theMagnifierIdent)
		theMagnifierPtr->unmap();
	VdEditWindowListEntry ** list = 
			getAllListEntries(someWindows);
	VdEditWindowListEntry * le;

	for (i=0; le = list[i]; i++) {
		(le->ewPtr)->unmap();
	}
	delete list;
}

void VdWindowConnection::setPicture( 
	const VdFeature & someWindows, VdPicture* pic )
{
	int i;
	VdEditWindowListEntry ** list = 
			getAllListEntries(someWindows);
	VdEditWindowListEntry * le;
	VdEditWindow *ewPtr;

	for (i=0; le = list[i]; i++) {
		ewPtr = le->ewPtr;
		if ( ewPtr 
		     == activeEditWindow ) {
	    	  if (theMagnifierPtr) {
		     theMagnifierPtr->setPicture(pic);
		     theMagnifiedPicturePtr = pic;	
	  	  }
		}
		le->pic = pic;
		ewPtr->setPicture(pic);
	}
	delete list;
}

VdPicture * VdWindowConnection::getPicture( 
			const VdFeature & aWindow )
{
        VdEditWindowListEntry * le;

        le=getListEntry(aWindow);
	if (!le) return NULL;
	return le->pic;
}

VdFeature VdWindowConnection::getWindowsWithPicture(
				VdPicture * thePicture)
{
        int i;
        VdEditWindowListEntry * le;
	VdFeature windows;
        for (i=0; i<!theEditWindows; i++) {
	   le = & theEditWindows[i];
           if (le->pic == thePicture) {
		windows |= le->ewId;
           }
        }
	return windows;
}

void VdWindowConnection::setWorldArea( const VdFeature & windows, 
				VdArea worldArea )
{
	int i;
	VdEditWindowListEntry ** list = 
			getAllListEntries(windows);
	VdEditWindowListEntry * le;
	VdEditWindow *ewPtr;
	VdArea displayed, world;

	for (i=0; le = list[i]; i++) {
		ewPtr = le->ewPtr;
		ewPtr->getDisplayOfWorld(displayed, world);
		ewPtr->setDisplayOfWorld(displayed, worldArea);
		if (ewPtr==activeEditWindow)
			magnify(le->ewId);
		else
			ewPtr->redrawPicture();
	}
	delete list;
}

VdArea VdWindowConnection::getWorldArea( const VdFeature & window )
{
        VdEditWindowListEntry * le;
	VdArea displayed, world;

        le=getListEntry(window);
	if (!le) return VdEmptyArea;
	(le->ewPtr)->getDisplayOfWorld(displayed, world);
	return world;
}

static VdArea stretchArea(VdArea oldArea, 
		VdPoint centre,
		VdFractNumber multiplier)
{
	VdVector shift(centre);
	int n = multiplier.getNumerator(),
	    d = multiplier.getDenominator();
	if (d==0) return VdEntireArea;
	VdArea newArea = oldArea - shift;
	newArea.ul.x = newArea.ul.x * n / d;
	newArea.ul.y = newArea.ul.y * n / d;
	newArea.lr.x = newArea.lr.x * n / d;
	newArea.lr.y = newArea.lr.y * n / d;
	shift.x = shift.x * n / d;
	shift.y = shift.y* n / d;
	newArea = newArea + shift;
	return newArea;
}

void VdWindowConnection::setView(
		const VdFeature & someWindows,
		VdFractNumber editWindowUnitsByPixel,
		VdPoint editWindowCentre,
		VdFractNumber magnifierSizeInWindow,
		VdFractNumber magnifierXPosition,
		VdFractNumber magnifierYPosition)
{
	int i;
	VdEditWindowListEntry ** list = 
			getAllListEntries(someWindows);
	VdEditWindowListEntry * le;
	VdEditWindow *ewPtr;
	VdArea displayed, world; VdPoint size;
	int width, height, width2, height2;

	for (i=0; le = list[i]; i++) {
		ewPtr = le->ewPtr;
		size = ewPtr->getDrawableSize();
		ewPtr->getDisplayOfWorld(displayed, world);
		// displayedArea:
		width = size.x * editWindowUnitsByPixel.getNumerator()
				/ editWindowUnitsByPixel.getDenominator();
		height = size.y * editWindowUnitsByPixel.getNumerator()
				/ editWindowUnitsByPixel.getDenominator();
		displayed.ul.x = editWindowCentre.x
			- width/2;
		displayed.lr.y = editWindowCentre.y
			- height/2;
		displayed.lr.x = displayed.ul.x + width;
		displayed.ul.y = displayed.lr.y + height;

		// worldArea:
		if (magnifierSizeInWindow >= VdONE )
			world=displayed;
		else  { 
		    if (magnifierSizeInWindow < VdFractNumber(1,10)) {
			width2 = width * 10;
			height2 = height * 10;
		    } else {
			width2 = width * 
			    magnifierSizeInWindow.getDenominator()
			    / magnifierSizeInWindow.getNumerator();
			height2 = height * 
			    magnifierSizeInWindow.getDenominator()
			    / magnifierSizeInWindow.getNumerator();
		    }
		    VdVector halfDia( (width-width2)/2,
				  (height-height2)/2 );
		    halfDia.x = (magnifierXPosition.getDenominator()
				+ magnifierXPosition.getNumerator())
				* halfDia.x
				/ magnifierXPosition.getDenominator();
		    halfDia.y = (magnifierYPosition.getDenominator()
				+ magnifierYPosition.getNumerator())
				* halfDia.y
				/ magnifierYPosition.getDenominator();
		    world = VdArea(VdPoint(displayed.ul.x,displayed.lr.y),
				width2, height2);
		    world += halfDia;
		}
		ewPtr->setDisplayOfWorld(displayed, world);
		if (ewPtr == activeEditWindow) 
		    magnify(le->ewId);
	}
	delete list;
}

void VdWindowConnection::getView(
		const VdFeature & aWindow,
		VdFractNumber & editWindowUnitsByPixel,
		VdPoint & editWindowCentre,
		VdFractNumber & magnifierSizeInWindow,
		VdFractNumber & magnifierXPosition,
		VdFractNumber & magnifierYPosition)
{
        VdEditWindowListEntry * le;
	VdArea displayed, world; VdPoint size;
	int width, height, width2, height2;

        le=getListEntry(aWindow);
	if (!le) {
		editWindowUnitsByPixel = VdONE;
		editWindowCentre = VdNullPoint;
		magnifierSizeInWindow = VdFractNumber(1,2);
		magnifierXPosition = VdZERO;
		magnifierYPosition = VdZERO;
		return;
	}
	(le->ewPtr)->getDisplayOfWorld(displayed, world);
	size = (le->ewPtr)->getDrawableSize();
	width = displayed.lr.x-displayed.ul.x;
	height = displayed.ul.y-displayed.lr.y;
	width2 = world.lr.x-world.ul.x;
	height2 = world.ul.y-world.lr.y;
	editWindowUnitsByPixel = 
		VdFractNumber( width, size.x);
	editWindowCentre.x = 
		(displayed.ul.x+displayed.lr.x)/2;
	editWindowCentre.y = 
		(displayed.ul.y+displayed.lr.y)/2;
	magnifierSizeInWindow = 
		VdFractNumber(width, width2);

	VdVector halfDia( (width-width2)/2,
				  (height-height2)/2 );
	if (halfDia.x==0) 
	    magnifierXPosition=VdZERO;
	else
	    magnifierXPosition = 
		VdFractNumber( 
			displayed.ul.x-world.ul.x+halfDia.x,
			halfDia.x );
	if (halfDia.y==0) 
	    magnifierYPosition=VdZERO;
	else
	    magnifierYPosition = 
		VdFractNumber( 
			world.ul.y-displayed.ul.y+halfDia.y,
			halfDia.y );
}

void VdWindowConnection::setDisplayArea( const VdFeature & windows, 
				VdArea displayedArea )
{
	int i;
	VdEditWindowListEntry ** list = 
			getAllListEntries(windows);
	VdEditWindowListEntry * le;
	VdEditWindow *ewPtr;
	VdArea displayed, world;

	for (i=0; le = list[i]; i++) {
		ewPtr = le->ewPtr;
		ewPtr->getDisplayOfWorld(displayed, world);
	     // Falls Verzerrung verboten, Anpassung von displayedArea
	     // notwendig:
		displayedArea = ewPtr->putAreaInWindow(displayedArea);
		ewPtr->setDisplayOfWorld(displayedArea, world);
		if (ewPtr==activeEditWindow)
			magnify(le->ewId);
		else
			ewPtr->redrawPicture();
	}
	delete list;
}

VdArea VdWindowConnection::getDisplayArea( const VdFeature & window )
{
        VdEditWindowListEntry * le;
	VdArea displayed, world;

        le=getListEntry(window);
	if (!le) return VdEmptyArea;
	(le->ewPtr)->getDisplayOfWorld(displayed, world);
	return displayed;
}

void VdWindowConnection::redrawPicture( VdPicture *pic, int toClear )
{
	int i;
	for (i=0; i < !theEditWindows; i++)
	   if (pic == theEditWindows[i].pic)
		(theEditWindows[i].ewPtr)->redrawPicture(toClear);
	if ( (pic == theMagnifiedPicturePtr) 
	     && theMagnifierPtr ) {
		theMagnifierPtr->redrawPicture();
	}
}

void VdWindowConnection::movePicture(
		const VdFeature & aWindow, 
		VdPicture * toDraw, VdPicture * toMove,
		VdPoint * worldLocation)
{
        VdEditWindowListEntry * le;

        le=getListEntry(aWindow);
	if (!le) return;
	le->ewPtr->movePicture( toDraw, toMove, worldLocation );
}

void VdWindowConnection::setEventManager( 
		const VdFeature & someWindows, 
		VdEventManagerInterface * aManager )
{
	int i;
	VdEditWindowListEntry ** list = 
			getAllListEntries(someWindows);
	VdEditWindowListEntry * le;
	VdEditWindow *ewPtr;

	for (i=0; le = list[i]; i++) {
		ewPtr = le->ewPtr;
		ewPtr->setEventManagerInterface(aManager);
	}
	delete list;
}

void VdWindowConnection::removeEventManager( 
		const VdFeature & someWindows)
{
	int i;
	VdEditWindowListEntry ** list = 
			getAllListEntries(someWindows);
	VdEditWindowListEntry * le;
	VdEditWindow *ewPtr;

	for (i=0; le = list[i]; i++) {
		ewPtr = le->ewPtr;
		ewPtr->setEventManagerInterface(NULL);
	}
	delete list;
}

void VdWindowConnection::addButtons( 
		const VdFeature & theWindows, 
		VdButtonInterfacePtrList * buttonHandles )
{
	int i,j;
	VdButton *aButton;
	VdEditWindowListEntry ** list = 
			getAllListEntries(theWindows);
	VdEditWindowListEntry * le;
	VdButtonListEntry *be;
	VdAbstractWindow *awPtr;
	int andMagnifier=0;

	int length=0;
	while (list[length]) length++;

	if ( theMagnifierPtr 
	     && theWindows==theMagnifierIdent)
		andMagnifier=1;
	for (i=0; i < length+andMagnifier; i++) {
		if (i<length) {
			le = list[i];
			awPtr = le->ewPtr;
		} else
			awPtr = theMagnifierPtr;
		for (j=0;j<!(*buttonHandles);j++) {
			switch ((*buttonHandles)[j]->getType()) {
			    case 0:
			   	aButton = new VdButton(
				   *awPtr,
				   ((*buttonHandles)[j]->idf).charptr(),
				   (*buttonHandles)[j]
			        );
			        break;
			    case 1:
			   	aButton = new VdPulldownOptions(
				   *awPtr,
				   ((*buttonHandles)[j]->idf).charptr(),
				   1, 
				   ((VdRadioBoxInterface *)
					((*buttonHandles)[j]))->mask,
				   (*buttonHandles)[j]
			        );
			        break;
			    case 2:
			   	aButton = new VdPulldownOptions(
				   *awPtr,
				   ((*buttonHandles)[j]->idf).charptr(),
				   0,
				   ((VdCheckBoxInterface *)
					((*buttonHandles)[j]))->mask,
				   (*buttonHandles)[j]
			        );
			        break;
			}
			aButton->map();
			be = &theButtons[!theButtons];
			be->bPtr = aButton;
			be->awPtr = awPtr;
			be->handle = (*buttonHandles)[j];
		}
	}
	delete list;
}

void VdWindowConnection::removeButtons( 
		const VdFeature & theWindows, 
		VdButtonInterfacePtrList * buttonHandles )
{
	int i,j,k;
	VdEditWindowListEntry ** list = 
			getAllListEntries(theWindows);
	VdEditWindowListEntry * le;
	VdAbstractWindow *awPtr;
	int andMagnifier=0;

	int length=0;
	while (list[length]) length++;

	if ( theMagnifierPtr 
	     && theWindows==theMagnifierIdent)
		andMagnifier=1;
	for (i=0; i < length+andMagnifier; i++) {
	    if (i<length) {
		le = list[i];
		awPtr = le->ewPtr;
	    } else
		awPtr = theMagnifierPtr;
	    for (j=0;j<!(*buttonHandles);j++) 
		for (k=0;k<!theButtons;k++) 
		    if (theButtons[k].handle==(*buttonHandles)[j]
			   && theButtons[k].awPtr==awPtr) {
				theButtons[k].bPtr->remove();
				theButtons.rem(k); k--;
		    }
	}
	delete list;
}

void VdWindowConnection::getImage( 
		const VdFeature & aWindow,
		VdPixMap & aPixmap)
{
        VdEditWindowListEntry * le;

        le=getListEntry(aWindow);
	if (!le) return;
	le->ewPtr->getImage(aPixmap);;
} 

void VdWindowConnection::createAndHandleMenu(
		VdPopupMenuInterface * aPopupMenuInterface )
{
	VdAllSelectedHandles * allSelectedHandles =
		aPopupMenuInterface->selectedHandles;
	VdFeature *window = &(allSelectedHandles->theEvent.window);
        VdEditWindowListEntry * le;
        le=getListEntry(*window);
	if (!le) return;

	VdPopupMenu *popup = new
		VdPopupMenu(
			le->ewPtr->getDrawable(),
			*window,
			aPopupMenuInterface,
			this);
	// allSelectedHandles is the private member
	// of the EditWindow from where the event comes
	
	popup->map();
}

int VdWindowConnection::createAndHandleFormular ( 
		VdFormularInterface * aFormularInterface)
{
	VdFormular * aFormularPtr = 
		new VdFormular( aFormularInterface );
	aFormularPtr->map();
	return 0;
}



VdEditWindowListEntry* VdWindowConnection::getListEntry( 
		const VdFeature & id )
{
	int i;
	for (i=0; i < !theEditWindows; i++) {
	   if ( id == theEditWindows[i].ewId )
		return &theEditWindows[i];
	}
	return NULL;

} 

VdEditWindowListEntry* VdWindowConnection::getListEntry( 
		VdEditWindow *ewPtr )
{
	int i;
	for (i=0; i < !theEditWindows; i++)
	   if (ewPtr == theEditWindows[i].ewPtr)
		return &(theEditWindows[i]);
	return NULL;

} 

// generating of a list of these EditWindow list entries
// which window id corresponds to a (possibly group) of
// window ids
VdEditWindowListEntry** VdWindowConnection::getAllListEntries( 
		const VdFeature & id )
{
	VdEditWindowListEntry** list =
		 new VdEditWindowListEntry*[!theEditWindows+1];
	int i,j;
	for (i=0, j=0; i < !theEditWindows; i++) {
	   if ( id == theEditWindows[i].ewId )
		list[j++] = &theEditWindows[i];
	}
	list[j] = NULL;
	return list;

} 

VdArea VdWindowConnection::displayOfWorldByMagnifier(
		const VdArea & world,
		VdPoint ul, VdPoint lr, VdVector size)
{
	VdArea displayed;
	displayed.ul.x = world.ul.x + 
			 ul.x * (world.lr.x-world.ul.x) 
			 / size.x;
	displayed.ul.y = world.ul.y - 
			 ul.y * (world.ul.y-world.lr.y) 
			 / size.y;
	displayed.lr.x = world.ul.x + 
			 lr.x * (world.lr.x-world.ul.x) 
			 / size.x;
	displayed.lr.y = world.ul.y - 
			 lr.y * (world.ul.y-world.lr.y) 
			 / size.y;
	if (lr.x==size.x) displayed.lr.x = world.lr.x;
	if (lr.y==size.y) displayed.lr.y = world.lr.y;
	return displayed;
}

VdArea VdWindowConnection::worldByDisplayAndMagnifier(
		const VdArea & displayed,
		VdPoint ul, VdPoint lr, VdVector size)
{
	// Inverse formulas of displayOfWorldByMagnifier:
	VdArea world;
	world.ul.x = (displayed.ul.x * lr.x 
			- displayed.lr.x * ul.x)
		     / (lr.x-ul.x);
	world.ul.y = (displayed.ul.y * lr.y 
			- displayed.lr.y * ul.y)
		     / (lr.y-ul.y);
	world.lr.x = ( (size.x-lr.x) * displayed.ul.x 
			- (size.x-ul.x) * displayed.lr.x)
		     / (ul.x-lr.x);
	world.lr.y = ( (size.y-lr.y) * displayed.ul.y 
			- (size.y-ul.y) * displayed.lr.y)
		     / (ul.y-lr.y);
	if (lr.x==size.x) world.lr.x = displayed.lr.x;
	if (lr.y==size.y) world.lr.y = displayed.lr.y;
	return world;
}

void VdWindowConnection::magnifierByDisplayOfWorld(
		const VdArea & displayed,
		const VdArea & world, const VdVector & size, 
		VdPoint * ul, VdPoint * lr)
{

	ul->x = size.x * ( displayed.ul.x - world.ul.x )
				 / (world.lr.x - world.ul.x);
	ul->y = size.y * ( world.ul.y - displayed.ul.y )
				/ (world.ul.y - world.lr.y);
	lr->x = size.x * ( displayed.lr.x - world.ul.x) 
				 / (world.lr.x - world.ul.x);
	lr->y = size.y * ( world.ul.y - displayed.lr.y )
				/ (world.ul.y - world.lr.y);
	if (displayed.lr.x==world.lr.x) lr->x=size.x;
	if (displayed.lr.y==world.lr.y) lr->y=size.y;
}

/****************** Mode-Functions ***/

void VdWindowConnection::setMode(const VdFeature &someWindows,
				 VdConnectionMode cm)
{
	int i ;
	VdEditWindowListEntry ** list = 
			getAllListEntries(someWindows);
	VdEditWindowListEntry * le;
	VdEditWindow *ewPtr;

	for (i=0; le = list[i]; i++) {
		ewPtr = le->ewPtr;
		setMode(ewPtr,cm) ;
	}
	delete list ;
}

void VdWindowConnection::setMode(VdConnectionMode aMode)
{
	setMode( activeEditWindow, aMode );
}

void VdWindowConnection::setMode(VdEditWindow * ew, VdConnectionMode aMode)
{
	VdEditWindowListEntry * le = getListEntry(ew);
	if (le) le->cm=aMode;
	if (ew==activeEditWindow) theMode=aMode ;
	switch (aMode) {
	case Vd_BUBBLE_BUBBLE:
		if(theMagnifierPtr && ew==activeEditWindow) 
			theMagnifierPtr->setWindowType(Vd_BUBBLE_GUM) ;
		if (ew) ew->setWindowType(Vd_BUBBLE_GUM) ;
		break;
	case Vd_DEFAULT_BUBBLE:
		if(theMagnifierPtr && ew==activeEditWindow) 
			theMagnifierPtr->setWindowType(Vd_BUBBLE_GUM) ;
		if (ew) ew->setWindowType(Vd_DEFAULT_WINDOW) ;
		break;
	case Vd_BUBBLE_DEFAULT:
		if(theMagnifierPtr && ew==activeEditWindow) 
			theMagnifierPtr->setWindowType(Vd_DEFAULT_WINDOW) ;
		if (ew) ew->setWindowType(Vd_BUBBLE_GUM) ;
		break;
	case Vd_DEFAULT_DEFAULT:
		if(theMagnifierPtr && ew==activeEditWindow) 
			theMagnifierPtr->setWindowType(Vd_DEFAULT_WINDOW) ;
		if (ew) ew->setWindowType(Vd_DEFAULT_WINDOW) ;
		break;
	}
}	


VdConnectionMode VdWindowConnection::getMode(void)
{
	 return theMode ;
}

void VdWindowConnection::setExitChecker(
		int (* checkingFunction)
		   (const VdFeature & window) )
{
	theCheckingFunction = checkingFunction;
	int i;
	for (i=0; i<!theEditWindows; i++) {
		theEditWindows[i].ewPtr
		->setExitChecker(checkingFunction);
	}
	if (theMagnifierPtr)
		theMagnifierPtr
		->setExitChecker(checkingFunction);
}



#undef _VDWINDOWCONNECTION_CXX
