#ifndef __VDWINDOWCONNECTION_HXX
#define __VDWINDOWCONNECTION_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdWindowConnection.hxx                            *
*  Date: 09-10-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter, B.Miethig                    *
*  State of Realization:  second version                   *
************************************************************
*                                                          *
***********************************************************/

enum VdConnectionMode{
	Vd_BUBBLE_BUBBLE ,
	Vd_BUBBLE_DEFAULT ,
	Vd_DEFAULT_BUBBLE ,
	Vd_DEFAULT_DEFAULT } ;

class VdAbstractWindow;
class VdEditWindow;
class VdMagnifierWindow;
class VdMagnifierButton;
class VdEventManagerInterface;
class VdButtonHandle;
class VdButtonInterfacePtrList;
class VdPopupMenuInterface;
class VdFormularInterface;
class VdWindowManagerInterface;

#include <Vd/VdFeature.hxx>
#include <Vd/VdArea.hxx>
#include <Vd/VdPicture.hxx>
#include <Vd/VdPoint.hxx>
#include <Vd/VdButton.hxx>
#include <Vd/VdPixMap.hxx>
#include <Vd/VdFractNumber.hxx>

struct VdEditWindowListEntry {
	VdEditWindow *ewPtr;
	VdFeature ewId;
	VdPicture *pic;
	VdButton *button;
	VdEventManagerInterface * evIf;
	VdConnectionMode cm;
	};

#define VdListClassName VdEditWindowList
#define VdListClassElementType VdEditWindowListEntry

#include <Vd/VdListDeclaration.mxx>

#undef VdListClassName
#undef VdListClassElementType

struct VdButtonListEntry {
	VdButton *bPtr;
	VdButtonInterface *handle;
	VdAbstractWindow *awPtr;
};

#define VdListClassName VdButtonList
#define VdListClassElementType VdButtonListEntry

#include <Vd/VdListDeclaration.mxx>

#undef VdListClassName
#undef VdListClassElementType


class VdWindowConnection {

	VdMagnifierWindow* theMagnifierPtr;
	VdPicture *theMagnifiedPicturePtr;

	VdFeature theIdentification;
	VdFeature theMagnifierIdent;

	VdEditWindowList theEditWindows;
	VdButtonList theButtons;
	VdEditWindow* activeEditWindow;
	VdEventManagerInterface * theEventManager;
	VdWindowManagerInterface *theWindowManager;
	
	VdConnectionMode theMode ;
	int (* theCheckingFunction)
		   (const VdFeature & window);
   public:
	VdWindowConnection( void );
	VdWindowConnection( const VdFeature ident );
	VdWindowConnection( const VdFeature ident,
		VdWindowManagerInterface *interface );

	VdFeature getIdent( void );

	// Interface for a VdMagnifierWindow
	void magnifierMoved( VdPoint ul, VdPoint lr);
	void magnifierResized( VdPoint ul, VdPoint lr );
	void worldMoved( VdPoint ul, VdPoint lr);
	void worldResized( VdPoint ul, VdPoint lr );
	void worldResizedByBorder( VdPoint ul, VdPoint lr );
	void displayMoved( VdPoint ul, VdPoint lr);
	void displayResized( VdPoint ul, VdPoint lr );
	VdEditWindow * getActiveEditWindow( void );

	// Interface for a VdEditWindow
	int isActiveEditWindow( VdEditWindow * ewp );
	void movedDisplayOfWorld( 
			VdArea displayed, VdArea world );
	void changedDisplayOfWorld(
			VdArea displayed, VdArea world );

	// interface for a WindowManager
	VdFeature setMagnifierWindow( VdMagnifierWindow& mw);
	VdMagnifierWindow * removeMagnifierWindow( 
		int toDestroy = 1 );
	void magnify( const VdFeature& ew );
	void setWindowManagerInterface( 
		VdWindowManagerInterface * wmInterface);
	VdWindowManagerInterface * 
			 getWindowManagerInterface( void );
	VdFeature addEditWindow( VdEditWindow& aWindow);
	VdEditWindow * removeEditWindow( 
		const VdFeature& someWindows,
		int toDestroy = 1 );
	void renameWindow( const VdFeature & oldWindowId,
			   const VdFeature & newWind);
	void map( const VdFeature & someWindows );
	void unmap( const VdFeature & someWindows );	
	void setPicture( const VdFeature & someWindows, 
			 VdPicture* pic );
	VdPicture * getPicture( const VdFeature & aWindow );
	VdFeature getWindowsWithPicture(VdPicture * thePicture);
	void setWorldArea( const VdFeature & windows, 
				VdArea worldArea );
	VdArea getWorldArea( const VdFeature & window );
  	void setView(const VdFeature & someWindows,
		VdFractNumber editWindowUnitsByPixel,
		VdPoint editWindowCentre,
		VdFractNumber magnifierSizeInWindow,
		VdFractNumber magnifierXPosition,
		VdFractNumber magnifierYPosition);
  	void getView(const VdFeature & aWindow,
		VdFractNumber & editWindowUnitsByPixel,
		VdPoint & editWindowCentre,
		VdFractNumber & magnifierSizeInWindow,
		VdFractNumber & magnifierXPosition,
		VdFractNumber & magnifierYPosition);
 	void setDisplayArea( const VdFeature & windows, 
				VdArea displayedArea );
	VdArea getDisplayArea( const VdFeature & window );
	void redrawPicture( VdPicture * pic, int toClear=1);
		// redraw a picture in all windows where
		// it is displayed
	void movePicture( const VdFeature & aWindow, 
			VdPicture * toDraw, VdPicture * toMove,
			VdPoint * worldLocation);
	void setEventManager( const VdFeature & someWindows, 
			VdEventManagerInterface * aManager );
	void removeEventManager( const VdFeature & someWindows);
	void addButtons( const VdFeature & theWindows, 
			VdButtonInterfacePtrList * buttonHandles );
	void removeButtons( const VdFeature & theWindows, 
			VdButtonInterfacePtrList * buttonHandles );
 	void getImage( const VdFeature & aWindow,
			VdPixMap & aPixmap); 

	void createAndHandleMenu(
		VdPopupMenuInterface * aPopupMenuInterface);
	int createAndHandleFormular ( 
		VdFormularInterface * aFormularInterface);

	VdConnectionMode getMode(void) ;
	void setMode(const VdFeature &someWindows, 
			VdConnectionMode cm) ;
	void setMode( VdConnectionMode cm) ;
	void setExitChecker(
		int (* checkingFunction)
		   (const VdFeature & window) );

   private:
	void setMode(VdEditWindow * ew, VdConnectionMode aMode) ;

	VdEditWindowListEntry* getListEntry( 
		const VdFeature& id );
	VdEditWindowListEntry* getListEntry( 
		VdEditWindow *ewPtr );
	VdEditWindowListEntry ** getAllListEntries(
		const VdFeature& id );

	VdArea displayOfWorldByMagnifier(const VdArea & world,
		VdPoint ul, VdPoint lr, VdVector size);
	VdArea worldByDisplayAndMagnifier(const VdArea & displayed,
		VdPoint ul, VdPoint lr, VdVector size);
	void magnifierByDisplayOfWorld(const VdArea & displayed,
		const VdArea & world, const VdVector & size, 
		VdPoint * ul, VdPoint * lr);
	
};

#endif   // __VDWINDOWCONNECTION_HXX
