#define _VDWINDOWINTERFACE_CXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdWindowInterface.cxx                             *
*  Date: 22-04.93                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/


/******************    dependences    *********************/

#include <Vd/VdWindowInterface.hxx>
#include <Vd/VdWindowManagerInterface.hxx>
#include <stdio.h>
#include <stdarg.h>

#define VdListClassName VdWindowInterfacePtrList
#define VdListClassElementType VdWindowInterface*
#include <Vd/VdListDefinition.mxx>
#undef VdListClassName
#undef VdListClassElementType

void VdWindowStringForm::handleOkOrCancel(int result)
{ theResult= (result||isMisspelled);
}

int VdWindowInterface::exitAndCloseWindows( const VdFeature & windows, int close )
{ int i, result=1;
  for (i=0; i<!theWindows; i++)
  { if (windows==theWindows[i]->theIdentifier)
    { if (theWindows[i]->exitWindow())
      { if (close) theWindowSystemInterfacePtr->destroyWindows(theWindows[i]->theIdentifier);
        delete theWindows[i]; 
        theWindows.rem(i);
        i--;
      }
      else result=0;
    }
  }
  return result;
}

int VdWindowInterface::getNumberOfWindows(void)
{ return !theWindows;
}

VdWindowInterface::VdWindowInterface(void)
: theFileName("empty")
{ makeIdentifier();
  theWindows.ins()=(VdWindowInterface*)this;
  hasFileAndDirectoryName=0;
  theFormular.entries.ins()=&theFormEntry;
  theWindowType=-1;
}

VdWindowInterface::~VdWindowInterface(void)
{
}

void VdWindowInterface::makeIdentifier(void)
{ int i; char buffer[64];
  theNumber=0;
  for (i=0; i<!theWindows; i++)
    if (  (theWindows[i]!=this)
        &&(theWindows[i]->theFileName==theFileName)
        &&(theWindows[i]->theNumber>=theNumber)
       )
      theNumber=theWindows[i]->theNumber+1;
  sprintf(buffer," (%d)",theNumber);
  if (theNumber)
    theIdentifier=VdFeature(theMagnifierName+":"+theFileName+buffer);
  else
    theIdentifier=VdFeature(theMagnifierName+":"+theFileName);
}

void VdWindowInterface::makeWindow(void)
{ theWindowSystemInterfacePtr->createWindow(theIdentifier);
  configureWindow();
  theWindowSystemInterfacePtr->mapWindow(theIdentifier);
}

void VdWindowInterface::configureWindow(void)
{ theWindowSystemInterfacePtr->addButtons(getIdentifier(),&theButtons);
  openConnections();
}

void VdWindowInterface::openConnections(void)
{ }

void VdWindowInterface::clearWindowContents(void)
{ }

void VdWindowInterface::closeConnections(void)
{ }

int VdWindowInterface::exitWindow(void)
{ if (theWindowType<0) return 1;
  if (theWindowType>0) return exitFileWindow();
  return exitNameWindow();
}

int VdWindowInterface::exitFileWindow(void)
{ FILE* f; int val;
  VdString fname=theDirectoryName+theFileName;
  VdString title("Exit"),
           message1("Save the File\n"),
           message2("Can't Open File\n");
  VdStringList messbuttons;
  messbuttons.ins()=VdString("yes");
  messbuttons.ins()=VdString("no"); 
  messbuttons.ins()=VdString("cancel");
  if ((val=VdCreateAndHandleDecision(title,message1+fname,messbuttons))==0)
  { if ((f=fopen(fname.charptr(),"w"))==NULL)
    { messbuttons.reset();
      messbuttons.ins()=VdString("don't close");
      messbuttons.ins()=VdString("don't save and close"); 
      if (VdCreateAndHandleDecision(title,message2+fname,messbuttons))
      { closeConnections();
        return 1;
      }
      else return 0;
    }
    writeToFile(f);
  }
  else
  { if (val==2) return 0;
  }
  closeConnections();
  return 1;
}

int VdWindowInterface::exitNameWindow(void)
{ VdString fname=theDirectoryName+theFileName;
  VdString title("Exit"),
           message1("Save the Window\n"),
           message2("Can't Save Window\n");
  VdStringList messbuttons;
  int val;
  messbuttons.ins()=VdString("yes");
  messbuttons.ins()=VdString("no"); 
  messbuttons.ins()=VdString("cancel");
  if ( (val=VdCreateAndHandleDecision(title,message1+fname,messbuttons)) ==0)
  { if ( saveName() == 0 )
    { messbuttons.reset();
      messbuttons.ins()=VdString("don't close");
      messbuttons.ins()=VdString("don't save and close"); 
      if (VdCreateAndHandleDecision(title,message2+fname,messbuttons))
      { closeConnections();
        return 1;
      }
      else return 0;
    }
  }
  else
  { if (val==2) return 0;
  }
  closeConnections();
  return 1;
}

void VdWindowInterface::setWindowPicture(VdPicture * p)
{ theWindowSystemInterfacePtr->setPicture(getIdentifier(),p);
}

void VdWindowInterface::setWindowEventManager(VdEventManagerInterface * e)
{ theWindowSystemInterfacePtr->setEventManager(getIdentifier(),e);
}

VdFeature VdWindowInterface::getIdentifier(void) const
{ return theIdentifier;
}

VdString VdWindowInterface::getFileName(void) const
{ return theFileName;
}

VdString VdWindowInterface::getDirectoryName(void) const
{ return theDirectoryName;
}

void VdWindowInterface::openFileAndDirectoryName
                           ( const VdString & fname
                           , const VdString & dname
                           )
{ if (theWindowType<0) return;
  if (hasFileAndDirectoryName)
  { if ((theWindowType?closeFile():deleteName())==0) return;
  }
  hasFileAndDirectoryName=0;
  theFileName=fname; theDirectoryName=dname;
  VdFeature theOldIdent = theIdentifier;
  makeIdentifier();
  theWindowSystemInterfacePtr->renameWindow(theOldIdent,theIdentifier);
  if ((theWindowType?openFile():nameName())==0)
  { theFileName=VdString("empty");
    theOldIdent = theIdentifier;
    makeIdentifier();
    theWindowSystemInterfacePtr->renameWindow(theOldIdent,theIdentifier);
    return;
  }
  hasFileAndDirectoryName=1;
}

void VdWindowInterface::saveAsFileAndDirectoryName
                           ( const VdString & fname
                           , const VdString & dname
                           )
{ VdString theOldFileName=theFileName,
           theOldDirectoryName=theDirectoryName;
  if (theWindowType<0) return;
  theFileName=fname; theDirectoryName=dname;
  VdFeature theOldIdent = theIdentifier;
  makeIdentifier();
  theWindowSystemInterfacePtr->renameWindow(theOldIdent,theIdentifier);
  if ((theWindowType?saveFile():saveName())==0)
  { theFileName=theOldFileName;
    theDirectoryName=theOldDirectoryName;
    theWindowSystemInterfacePtr->renameWindow(theIdentifier,theOldIdent);
    theIdentifier=theOldIdent;
    return;
  }
  hasFileAndDirectoryName=1;
}

int VdWindowInterface::openFile(void)
{ FILE* f;
  VdString fname=theDirectoryName+theFileName;
  if ((f=fopen(fname.charptr(),"r"))==NULL)
  { if ((f=fopen(fname.charptr(),"w"))==NULL) return 0;
    else return 1;
  }
  readFromFile(f);
  fclose(f);
  return 1;
}

int VdWindowInterface::saveFile(void)
{ FILE* f;
  VdString fname=theDirectoryName+theFileName;
  VdString title("Save File"),message("Can't Open File\n");
  VdStringList messbuttons;
  if ((f=fopen(fname.charptr(),"w"))==NULL)
  { VdCreateAndHandleDecision(title,message+fname,messbuttons);
    return 1;
  }
  writeToFile(f);
  fclose(f);
  return 1;
}

int VdWindowInterface::closeFile(void)
{ FILE* f;
  VdFeature hf;
  VdString fname=theDirectoryName+theFileName;
  VdString title("Close File"),
           message1("Save the File\n"),
           message2("Can't Open File\n");
  VdStringList messbuttons;
  messbuttons.ins()=VdString("yes");
  messbuttons.ins()=VdString("no"); 
  if (VdCreateAndHandleDecision(title,message1+fname,messbuttons)==0)
  { if ((f=fopen(fname.charptr(),"w"))==NULL)
    { messbuttons.reset();
      messbuttons.ins()=VdString("don't close");
      messbuttons.ins()=VdString("don't save and close"); 
      if (VdCreateAndHandleDecision(title,message2+fname,messbuttons))
      { clearWindowContents();
        theWindowSystemInterfacePtr
          ->redrawPicture(theWindowSystemInterfacePtr
                            ->getPicture(hf=getIdentifier())
                         );
        return 1;
      }
      else return 0;
    }
    writeToFile(f);
    fclose(f);
  }
  clearWindowContents();
  theWindowSystemInterfacePtr
    ->redrawPicture(theWindowSystemInterfacePtr->getPicture(hf=getIdentifier()));
  return 1;
}

void VdWindowInterface::writeToFile(FILE * f)
{ }

void VdWindowInterface::readFromFile(FILE * f)
{ }

void VdWindowInterface::getAndWriteWindowLayout(FILE * f)
{ VdPoint centre;
  VdFractNumber editscale,  magscale, magxcentre, magycentre;
  theWindowSystemInterfacePtr->getView(getIdentifier()
                              ,editscale
                              ,centre
                              ,magscale
                              ,magxcentre
                              ,magycentre
                              );
  fprintf(f,"#WindowLayout#\n%d %d %d %d %d %d %d %d %d %d\n#\n"
           ,editscale.denominator,editscale.numerator
           ,centre.x,centre.y
           ,magscale.denominator,magscale.numerator
           ,magxcentre.denominator,magxcentre.numerator
           ,magycentre.denominator,magycentre.numerator
         );
}

void VdWindowInterface::readAndSetWindowLayout(FILE * f)
{ VdPoint centre;
  VdFractNumber editscale,  magscale, magxcentre, magycentre;
  char buf1[16], buf2[4];
  fscanf(f,"%s %d %d %d %d %d %d %d %d %d %d %s"
          ,buf1
          ,&editscale.denominator,&editscale.numerator
          ,&centre.x,&centre.y
          ,&magscale.denominator,&magscale.numerator
          ,&magxcentre.denominator,&magxcentre.numerator
          ,&magycentre.denominator,&magycentre.numerator
          ,buf2
        );
  theWindowSystemInterfacePtr->setView(getIdentifier()
                              ,editscale
                              ,centre
                              ,magscale
                              ,magxcentre
                              ,magycentre
                              );
}

int VdWindowInterface::nameName(void)
{ return 1;
}

int VdWindowInterface::deleteName(void)
{ return 1;
}

int VdWindowInterface::saveName(void)
{ return 1;
}

int VdWindowInterface::VdCreateAndHandleFormular(char * name, char * group, ... )
{ va_list ap;
  va_start(ap,group);
  VdFormularInterface frm;
  frm.formularName=VdString(name);
  VdFeature grp(group);
  int i; char * ptr;
  while(1)
  { ptr=va_arg(ap,char*);
    if (ptr==NULL) break;
    VdString mem(ptr);
    for (i=0; i<!VdEntries; i++)
      if (  ( mem == VdEntries[i]->name )
         && ( grp == VdGroup[i] )
         )
      { frm.entries.ins()=VdEntries[i];
        break;
      }
  }
  va_end(ap);
  if (!frm.entries)
  { return ::VdCreateAndHandleFormular(&frm);
  }
  return 0;
}

void VdWindowInterface::VdRegisterFormular(char * grp, VdFormularEntry * e )
{ VdEntries.ins(0)=e;
  VdGroup.ins(0)=VdFeature(grp);
}

VdWindowFileButton::VdWindowFileButton(void)
{ idf=VdString("File"); mask=1;
  labels.ins()=VdString("Open");
  labels.ins()=VdString("Close");
  labels.ins()=VdString("Save");
  labels.ins()=VdString("Save As");
  VdWindowInterface::theWindowType=1;
};

static VdString filepattern("*"),
                emptystring(""),
                openfile("Open A File"),
                defaultname("empty"),
                savefileas("Save The File As"),
                namewindow("Name The Window"),
                renamewindow("Rename The Window");

void VdWindowFileButton::activate( const VdFeature & window )
{ VdString file, dir;
  VdFeature theOldIdent;
  switch (mask)
  { case 1: file=filepattern;
            if (hasFileAndDirectoryName) dir=theDirectoryName;
            else dir=emptystring;
            if (VdCreateAndHandleFileSelection
                   ( openfile, dir, file, 1
                   ) 
               )
            return;
            if (file==emptystring) return;
            openFileAndDirectoryName(file,dir);
            return;
    case 2: if (hasFileAndDirectoryName)
            { if (closeFile()==0) return;
            }
            hasFileAndDirectoryName=0;
            theFileName=defaultname;
            theOldIdent = theIdentifier;
            makeIdentifier();
            theWindowSystemInterfacePtr->renameWindow(theOldIdent,theIdentifier);
            return;
    case 4: if (hasFileAndDirectoryName)
            { saveFile();
              return;
            }
    case 8: file=filepattern;
            if (hasFileAndDirectoryName) dir=theDirectoryName;
            else dir=emptystring;
            if (VdCreateAndHandleFileSelection
                   ( savefileas, dir, file, 1
                   ) 
               )
            return;
            if (file==emptystring) return;
            saveAsFileAndDirectoryName(file,dir);
            return;
  }
}

VdWindowStringButton::VdWindowStringButton(void)
{ idf=VdString("Info"); mask=1;
  labels.ins()=VdString("Name");
  labels.ins()=VdString("Delete");
  labels.ins()=VdString("Save");
  labels.ins()=VdString("Change");
  VdWindowInterface::theWindowType=0;
}

void VdWindowStringButton::activate( const VdFeature & window )
{ VdFeature theOldIdent;
  switch (mask)
  { case 1: theFormEntry.contents=theFileName;
            theFormular.formularName=namewindow;
            if (::VdCreateAndHandleFormular(&theFormular)==0)
            return;
            if (theFormEntry.theResult
                || (theFormEntry.contents==emptystring)
               )
            return;
            openFileAndDirectoryName(theFormEntry.contents,emptystring);
            return;
    case 2: if (hasFileAndDirectoryName)
            { if ( deleteName()==0) return;
            }
            hasFileAndDirectoryName=0;
            theFileName=defaultname;
            theOldIdent = theIdentifier;
            makeIdentifier();
            theWindowSystemInterfacePtr->renameWindow(theOldIdent,theIdentifier);
            return;
    case 4: if (hasFileAndDirectoryName)
            { saveName();
              return;
            }
    case 8: theFormEntry.contents=theFileName;
            theFormular.formularName=renamewindow;
            if (::VdCreateAndHandleFormular(&theFormular))
            return;
            if (theFormEntry.theResult
                || (theFormEntry.contents==emptystring)
               )
            return;
            saveAsFileAndDirectoryName(theFormEntry.contents,emptystring);
            return;
  }
}

#undef _VDWINDOWINTERFACE_CXX
