#ifndef _VDWINDOWINTERFACE_HXX
#define _VDWINDOWINTERFACE_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdWindowInterface.hxx                             *
*  Date: 22-04.93                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Hutschenreiter , J.Benary                    *
*  State of Realization:                                   *
*   ( experimental version )                               *
************************************************************
*                                                          *
*  Comment                                                 *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdWindowInterfacePtrList;
class VdWindowStringForm;
class VdWindowInterface;
class VdWindowFileButton;
class VdWindowStringButton;

/******************    dependences    *********************/

#include <Vd/VdString.hxx>
#include <Vd/VdFormularInterface.hxx>
#include <Vd/VdFormularInput.hxx>
#include <Vd/VdFormularEntry.hxx>
#include <Vd/VdFormularEntryPtrList.hxx>
#include <Vd/VdFeature.hxx>
#include <Vd/VdFeatureList.hxx>
#include <Vd/VdButtonInterface.hxx>
#include <Vd/VdRadioBoxInterface.hxx>
#include <Vd/VdGlobalFunctions.hxx>
#include <Vd/VdWindowManagerInterface.hxx>

#define VdListClassName VdWindowInterfacePtrList
#define VdListClassElementType VdWindowInterface*
#include <Vd/VdListDeclaration.mxx>
#undef VdListClassName
#undef VdListClassElementType

class VdWindowStringForm
: public VdFormularInput
{  friend class VdWindowInterface;
   friend class VdWindowStringButton;
  public:
    int theResult;
    void handleOkOrCancel(int result);
};


class VdWindowInterface
{
    friend class VdWindowFileButton;
    friend class VdWindowStringButton;

    static VdWindowInterfacePtrList theWindows;

    VdFormularEntryPtrList VdEntries;
    VdFeatureList VdGroup;

  public:
    static VdString theMagnifierName;
    static VdWindowManagerInterface* theWindowSystemInterfacePtr;
    static int exitAndCloseWindows( const VdFeature & windows, int close );
    static int getNumberOfWindows(void);

protected:
    VdString theDirectoryName;
    VdString theFileName;
    int theNumber;
    VdFeature theIdentifier;
    VdButtonInterfacePtrList theButtons;
    int hasFileAndDirectoryName;
    int theWindowType;

    VdWindowStringForm theFormEntry;
    VdFormularInterface theFormular;

    void makeIdentifier(void);

  public: 

    VdString VdWindowIdentifier;
    VdWindowInterface(void);
    ~VdWindowInterface(void);
    void makeWindow(void);

    virtual void configureWindow(void);
    virtual void openConnections(void);
    virtual void clearWindowContents(void);
    virtual void closeConnections(void);
    virtual int exitWindow(void); 
    virtual int exitFileWindow(void); 
    virtual int exitNameWindow(void); 

    void setWindowPicture(VdPicture * p);
    void setWindowEventManager(VdEventManagerInterface * e);

    VdFeature getIdentifier(void) const;
    VdString getFileName(void) const;
    VdString getDirectoryName(void) const;
    void openFileAndDirectoryName( const VdString & fname
                                 , const VdString & dname
                                 );
    void saveAsFileAndDirectoryName( const VdString & fname
                                   , const VdString & dname
                                   );

    virtual int openFile(void);
    virtual int closeFile(void);
    virtual int saveFile(void);
  
    virtual void writeToFile(FILE * f);
    virtual void readFromFile(FILE * f);

    void getAndWriteWindowLayout(FILE * f);
    void readAndSetWindowLayout(FILE * f);

    virtual int nameName(void);
    virtual int deleteName(void);
    virtual int saveName(void);

   int VdCreateAndHandleFormular(char * name, char * grp, ... );
   void VdRegisterFormular(char * grp, VdFormularEntry * e );
};


class VdWindowFileButton
: public VdRadioBoxInterface
, public virtual VdWindowInterface
{
  public:
    VdWindowFileButton(void);
    void activate(const VdFeature&);
};

class VdWindowStringButton
: public VdRadioBoxInterface
, public virtual VdWindowInterface
{
  public:
    VdWindowStringButton(void);
    void activate(const VdFeature&);
};

#ifdef _VDWINDOWINTERFACE_CXX
    VdWindowInterfacePtrList VdWindowInterface::theWindows;
    VdString VdWindowInterface::theMagnifierName;
    VdWindowManagerInterface* VdWindowInterface::theWindowSystemInterfacePtr=NULL;
#endif
  
#endif /* _VDWINDOWINTERFACE_HXX */
