#ifndef _VDWINDOWMANAGERINTERFACE_HXX
#define _VDWINDOWMANAGERINTERFACE_HXX

/***********************************************************
*                    VISAMAD.defgraph                      *
*               ==========================                 *
*  File: VdWindowManagerInterface.hxx                      *
*  Date: 03-12-92                                          *
*  Copyright (c) 1992                                      *
*  Technical University Dresden, Department of Mathematics *
*  Author: J. Benary, J. Hutschenreiter                    *
*  State of Realization:  experimental version             *
************************************************************
*                                                          *
*     Declaration of class VdWindowManagerInterface as a   *
*        base class for application specific window        *
*        manager objects                                   *
*                                                          *
*     Interface                                            *
*        VdFeature.hxx,  VdPicture.hxx,                    *
*        VdEventManagerInterface.hxx,                      *
*        VdButtonInterface.hxx, VdButtonInterface.hxx      *
*        VdFormularInterface.hxx                           *
*                                                          *
***********************************************************/

/******************  defined classes  *********************/

class VdWindowManagerInterface;

/******************    dependences    *********************/

#include <Vd/VdFeature.hxx>
#include <Vd/VdPicture.hxx>
#include <Vd/VdEventManagerInterface.hxx>
#include <Vd/VdButtonInterface.hxx>
#include <Vd/VdFormularInterface.hxx>
#include <Vd/VdPixMap.hxx>
#include <Vd/VdAllSelectedHandles.hxx>
#include <Vd/VdPopupMenuInterface.hxx>
#include <Vd/VdFractNumber.hxx>

class VdWindowManagerInterface{

 public:

  virtual VdFeature createWindow
               ( const VdFeature & aWindow);
  virtual void destroyWindows
               ( const VdFeature & someWindows );
  virtual void renameWindow
		( const VdFeature & oldWindow,
		  const VdFeature & newWindow );

  virtual void mapWindow			// visibilty
               ( const VdFeature & someWindows );
  virtual void unmapWindow
               ( const VdFeature & someWindows );

  virtual void setPicture 	       // pictures & windows
               ( const VdFeature & someWindows
               , VdPicture * aPicture );
  virtual VdPicture * getPicture
               ( VdFeature & aWindow );
  virtual VdFeature getWindowsOfPicture
               ( VdPicture * thePicture );
  virtual void movePicture( const VdFeature & aWindow, 
			VdPicture * toDraw, VdPicture * toMove,
			VdPoint * worldLocation);

  virtual void setWorldArea 		 // displaying world
	       ( const VdFeature & someWindows,
		 const VdArea & worldArea );
  virtual VdArea getWorldArea( const VdFeature & aWindow );
  virtual void setDisplayArea 
	      ( const VdFeature & someWindowsindows,
		const VdArea & displayedArea );
  virtual VdArea getDisplayArea( const VdFeature & aWindow );
  virtual void setView(const VdFeature & someWindows,
		VdFractNumber editWindowUnitsByPixel,
		VdPoint editWindowCentre,
		VdFractNumber magnifierSizeInWindow,
		VdFractNumber magnifierXPosition,
		VdFractNumber magnifierYPosition);
  virtual void getView(const VdFeature & aWindow,
		VdFractNumber & editWindowUnitsByPixel,
		VdPoint & editWindowCentre,
		VdFractNumber & magnifierSizeInWindow,
		VdFractNumber & magnifierXPosition,
		VdFractNumber & magnifierYPosition);
  virtual void drawPicture		   	// draw only
               ( VdPicture * aPicture );
  virtual void redrawPicture		   // clear and draw
               ( VdPicture * aPicture );

  virtual void setEventManager		   // event managing
               ( const VdFeature & someWindows
               , VdEventManagerInterface * aManager );
  virtual void removeEventManager
               ( const VdFeature & someWindows );

  virtual void addButtons 		  // button managing
               ( const VdFeature & someWindows 
               , VdButtonInterfacePtrList * buttons );
  virtual void removeButtons
               ( const VdFeature & someWindows
               , VdButtonInterfacePtrList * buttons );

  virtual void getImage( const VdFeature & aWindow
               , VdPixMap & aPixmap); 

  virtual void createAndHandleMenu(
		VdPopupMenuInterface * interface);

  virtual void setExitChecker(
		int (* checkingFunction)
		   (const VdFeature & window) );

  virtual void parseHelpFile(const VdString & fileName);
  virtual void createHelpWindow(const VdFeature & helpIdent);

};

#endif /* _VDWINDOWMANAGERINTERFACE_HXX */
